C     Matrix multiply - unoptimized example program
C
C     First, declare variables for the array dimensions, and arrays for the
C     input data (b,c), and the result (a).
C
         integer Size1, Size2
         real a(Size1,Size1), b(Size2,Size1), c(Size2,Size1)
C
C     Declare the arrays as SCATTER arrays to reduce memory contention.
C
         scatter a, b, c
         common Size1, Size2, a, b, c
C
C     Declare various routines to be used as arguments.
C
         external InitPerRun, Body, PrintAnswer
C
C     Get the matrix dimensions from the user's terminal.
C
         print 900
900      format(/, 'Build Matrix Multiply Array', /, 'Matrix Size:')
         read *, Size1, Size2
C
C     Propagate their values to the other processors.
C
         call share(%ptrloc(size1))
         call share(%ptrloc(size2))
C
C     and call initialization
C
         call InitProblemOnce()
C
C     Use timetest to check the routine's efficiency.
C
         call timetest(initperrun, body, printanswer)
C
       end
C
         subroutine InitProblemOnce()
C
         integer Size1, Size2
         real a(Size1,Size1), b(Size2,Size1), c(Size2,Size1)
C
         scatter a, b, c
         common Size1, Size2, a, b, c
C
         integer i,j
C
C     Allocate the three matrices.
C
         a = usallocscattermatrix(size1, size1, 4)
         b = usallocscattermatrix(size2, size1, 4)
         c = usallocscattermatrix(size2, size1, 4)
C
C     Propagate their values to the other processors.
C
         call sharescattermatrix(%ptrloc(a), size1)
         call share(%ptrloc(b))
         call share(%ptrloc(c))
C
C     Put some dummy data into the two arrays.
C
         do 110 i=1,Size1
            do 100 j=1,Size2
              if (i .eq. j) then
                b(j,i) = 3.0
              else
                b(j,i) = 0.0
              endif
              c(j,i) = Size2 * (i-1) + (j-1)
100          continue
110         continue
C
        end
C
C     At the start of each timed test, zero out the answer array.
C
         subroutine InitPerRun()
C
         integer Size1, Size2
         real a(Size1,Size1), b(Size2,Size1), c(Size2,Size1)
         scatter a, b, c
         common Size1, Size2, a, b, c
C
         integer i,j
C
         print *,''
         print *, ' Starting run with ',ProcsInUse(), ' processors.'

         do 210 i=1,Size1
            do 200 j=1,Size1
              a(i,j) = 0.0
200            continue
210          continue
         end
C
C     This is the part of the program that you want timed.
C
         subroutine Body()
C
         integer Size1, Size2
         real a(Size1,Size1), b(Size2,Size1), c(Size2,Size1)
         scatter a, b, c
         common Size1, Size2, a, b, c
C
         external DotProduct
C
C     Generate a task to compute each element of the output array.
C
         call genona(dotproduct, size1, size1)
         end
C
C     Worker routine to compute one element of the answer, by 
C     calculating the dot product of a row of b and a column of c. 
C
         subroutine DotProduct(dummy,i,j)
C
         integer Size1, Size2
         real a(Size1,Size1), b(Size2,Size1), c(Size2,Size1)
         scatter a, b, c
         common Size1, Size2, a, b, c
C
         integer dummy,i,j
C
         integer k
         real temp
            real row(Size2), col(Size2)
            allocatable row, col
C
         temp = 0.0
C
C     Create local pointers to the row and column being operated on.
C
         row = %loc(b(1,i))
         col = %loc(c(1,j))
C
         do 400 k=1,Size2
          temp = temp + row(k) * col(k)
400         continue
C
         a(i,j) = temp
C
         end
C
C     Routine to print results at the end of each test.
C
          subroutine PrintAnswer(time,procs,speedup)
C
          integer Size1, Size2
          real a(Size1,Size1), b(Size2,Size1), c(Size2,Size1)
          scatter a, b, c
          common Size1, Size2, a, b, c
C
          integer time,procs
          real speedup
C
          integer i
C
C     Print out the upper left corner (6X6) of the answer.
C
         print 1510, Size1, Size2, Size2, Size1, Size1, Size1
1510         format(/, 'Multiplying ', I3, ' X ', I3,
     1          ' matrix by ', I3, ' X ', I3,
     2          ' matrix, yields ', I3, ' X ', I3, ' matrix.',/)
C
         do 500 i=1,6
          print 1520,i,a(i,1), a(i,2), a(i,3), a(i,4), a(i,5), a(i,6)
1520          format('a row ', I2, ':', 6F6.0)
500      continue
C
         print *,''
C
C     Use a Uniform System routine to print out the timing results.
C
         call timetestprint(time,procs,speedup)
C
         print 1530
1530        format (/)
C
         end
