
/*ACI:  $Header: a.out.h,v 1.13 90/02/07 08:16:42 fwebb Exp $       */
/*
 *      File:   a.out.h
 *
 *      Copyright (C) 1989, BBN Advanced Computers
 *
*/

/* Version for new object file format with multiple, named segments. */

/*
 * Definitions of the a.out header, magic numbers, etc. are in a seperate header file, 
 * since they are used by the kernel.
 */


#ifdef	KERNEL
#include "../h/exec.h"
#else	KERNEL
#include <sys/exec.h>
#endif	KERNEL

/* memory management parameters */

#ifdef A_OUT_SUN3

#define PAGSIZ		(0x2000)
#define SEGSIZ		(0x20000)
#define OLD_PAGSIZ	(0x800)
#define OLD_SEGSIZ	(0x8000)

#define	N_PAGSIZ(x) \
	((x).a_machtype == M_OLDSUN2? OLD_PAGSIZ : PAGSIZ)
#define	N_SEGSIZ(x) \
	((x).a_machtype == M_OLDSUN2? OLD_SEGSIZ : SEGSIZ)

#else A_OUT_SUN3

/* avoid redefining this if someone else already has.  This is particularly
 * noticable in the nX kernel.  We really shouldn't be changing the value of
 * this define in real time, so this should help us avoid getting out of
 * sync.
 */
#ifndef	PAGSIZ

#ifdef A_OUT_BF
#define	PAGSIZ		(1024)
#else A_OUT_BF
#ifdef A_OUT_GENERIC
#define PAGSIZ		(1024)
#else
#define PAGSIZ		(8192)
#endif A_OUT_GENERIC
#endif	A_OUT_BF

#endif PAGSIZ

#define N_PAGSIZ(x)	PAGSIZ
#define N_SEGSIZ(x)	PAGSIZ

#endif A_OUT_SUN3

/* test for magic number */

#define	N_BADMAG(x) \
  (((x).a_magic != OMAGIC) && ((x).a_magic != NMAGIC) && ((x).a_magic != ZMAGIC))

/* Segment related definitions */

/*
 * segment count (at beginning of text area)
 */

typedef unsigned	num_segs_type;

#define MAX_SEGS	256

/* pre-defined segment numbers */

#define ABS_SEGMENT	(0)
#define TEXT_SEGMENT	(1)
#define DATA_SEGMENT	(2)	/* contains BSS too */

/* first real segment number */
#define FIRST_SEGMENT	(1)

#define SEG_NAME_LENGTH	8

/*
 * segment header (after segment count)
 */   

typedef union {
  struct {
    unsigned	text_data :	1; /* attributes only */
    unsigned	isshared :	1;
    unsigned	local_global :	1;
    unsigned	local_read :	1;
    unsigned	local_write :	1;
    unsigned	local_exec :	1;
    unsigned	remote_read :	1;
    unsigned	remote_write :	1;
    unsigned	remote_exec :	1;
    unsigned	interleaved :	1;
    unsigned	caching :	2;
    unsigned	wired :		1;
    unsigned	paged :		1;
    unsigned	load :		1;
    unsigned	pic :		1;
    unsigned	inherit :	2;
    unsigned	mach_mem :	2;
    unsigned	global_named :	1;
    unsigned    pglobal_named : 1;
    unsigned	import_export :	1;
    unsigned	obey_vaddr :	1; /* enables only */
    unsigned	obey_u_vaddr :	1; /* enables only */
  } bits;			/* total: 25 */
  unsigned long	word;
} attr_type;

struct segment_header {
  char		name [SEG_NAME_LENGTH]; /* null padded name */
  unsigned int	offset;		/* file offset */
  unsigned int	size;		/* length in bytes */
  unsigned int	u_size;		/* uninitialized length */
  unsigned int	r_offset;	/* relocation file offset */
  unsigned int	r_size;		/* relocation size */
  unsigned int	vaddr;		/* virtual start address */
  unsigned int	paddr;		/* physical start address */
  unsigned int	u_vaddr;	/* uninitialized virtual s.a. */
  unsigned int	u_paddr;	/* uninitialized physical s.a. */
  unsigned int	align1;		/* intra-seg. alignment */
  unsigned int	align2;		/* inter-seg. alignment */
  attr_type	attributes;	/* standard attributes */
  attr_type	enables;	/* enable bits */
  unsigned int	a_size;		/* extra attr. size */
  unsigned int	a_offset;	/* extra attr. offset */
};

/* segment table size */

#define SEG_TABLE_SIZE(n)	(sizeof (num_segs_type) + \
				 (n) * sizeof (struct segment_header))

#define NULL_ADDR	0xffffffff	/* null address */
#define NULL_ALIGN	0	/* default alignment */

/* attribute values */

/* attr_bits.text_data */
#define A_TEXT_SEG	0		/* text-type segment */
#define A_DATA_SEG	1		/* data-type segment */

/* attr_bits.isshared */
#define A_PRIVATE	0		/* private address space */
#define A_SHARED	1		/* shared address space */

/* attr_bits.local_global */
#define A_LOCAL		0		/* local memory */
#define A_GLOBAL	1		/* global (round-robined) memory */

/* attr_bits.local_read .. attr_bits.remote_exec are booleans */

/* attr_bits.interleaved */
#define A_CONTIGUOUS	0		/* contiguous */
#define A_INTERLEAVED	1		/* Bfly-2 hardware interleave */

/* attr_bits.caching */
#define A_NO_CACHE	0		/* not cacheable */
#define A_COPY_BACK	1		/* copy-back policy */
#define A_WRITE_THRU	2		/* write-through policy */
#define A_POLICY_3	3		/* third policy */

/* attr_bits.wired */
#define A_NOT_WIRED	0		/* not wired-down */
#define A_WIRED		1		/* wired-down */

/* attr_bits.paged */
#define A_PAGED		0		/* demand-paged */
#define A_PRELOAD	1		/* load all at once */

/* attr_bits.load */
#define A_DO_LOAD	0		/* load immediately */
#define A_DONT_LOAD	1		/* defer loading (overlays) */

/* attr_bits.pic */
#define A_NON_PIC	0		/* non-position-independent code */
#define A_PIC		1		/* position-independent code */

/* attr_bits.inherit */
#define A_INHERIT_NONE  0		/* VM_INHERIT_NONE */
#define A_INHERIT_SHARE	1		/* VM_INHERIT_SHARE */
#define A_INHERIT_COPY	2		/* VM_INHERIT_COPY */

/* attr_bits.mach_mem */
#define A_MACH_DEFAULT	0		/* default */
#define A_MACH_COW	1		/* copy-on-write */
#define A_MACH_COR	2		/* copy-on-ref */
#define A_MACH_OTHER	3		/* other? */

/* attr_bits.global_named, attr_bits.pglobal_named are boolean */

/* attr_bits.import_export */
#define A_IMPORT	0		/* import global named memory */
#define A_EXPORT	1		/* export global named memory */
					/* auto-allocate if not enabled */

/* attr_bits.obey_vaddr is boolean */

/* attr_bits.obey_u_vaddr is boolean */

/* offset of text area */

#define	N_TXTOFF(x) \
  ( (((x).a_machtype == M_SUN2) || ((x).a_machtype == M_SUN3)) \
        ? ((x).a_magic==ZMAGIC ? 0 : sizeof (struct exec)) \
        : ((x).a_magic==ZMAGIC ? N_PAGSIZ(x) : sizeof (struct exec)) )

/* offset of segment table (if any) */

#define N_SEGOFF(x)	\
  ( (((x).a_machtype == M_SUN2) || ((x).a_machtype == M_SUN3)) \
        ? sizeof (struct exec) : N_TXTOFF(x) )

/* offset of data area */

#define N_DATOFF(x) \
  (N_TXTOFF(x) + (x).a_text)

/* offset of text relocation area */

#define N_TROFF(x) \
  (N_DATOFF(x) + (x).a_data)

/* offset of data relocation area */

#define N_DROFF(x) \
  (N_TROFF(x) + (x).a_trsize)

/* offset of symbol table */

#define N_SYMOFF(x) \
  (N_DROFF(x) + (x).a_drsize)

/* offset of string table */

#define	N_STROFF(x) \
  (N_SYMOFF(x) + (x).a_syms)

/* virtual start address of text */

#ifdef A_OUT_SUN3

#define N_TXTADDR(x) \
  ((x).a_magic  == ZMAGIC \
   ? ((x).a_machtype == M_OLDSUN2 ? N_SEGSIZ(x) : N_PAGSIZ(x)) \
   : 0)

#else

#define N_TXTADDR(x)	(0)

#endif

/* virtual start address of data */

#define N_DATADDR(x) \
  (((x).a_magic == OMAGIC) \
   ? (N_TXTADDR(x) + (x).a_text) \
   : (N_SEGSIZ(x) + ((N_TXTADDR(x) + (x).a_text - 1) & ~(N_SEGSIZ(x) - 1))))

/* virtual start address of bss */

#define N_BSSADDR(x)  (N_DATADDR(x) + (x).a_data)

/*
 * relocation information (includes 88000 relocation)
 */

struct relocation_info {
  int		r_address;	/* offset in data portion */
#ifndef A_OUT_BF
  unsigned	r_symbolnum:24,	/* symbol or segment */
#else
  unsigned	r_symbolnum,	/* Brain damaged bf-linker */
#endif
 		r_pcrel:1,	/* PC-relative */
		r_length:2,	/* 0=8-bit, 1=16-bit, 2=32-bit */
		r_extern:1,	/* 0=segment, 1=symbol */
#ifdef NS32KREL
		 r_bsr:1, /* OVE: used on ns32k based systems, if you want */
		 r_disp:1, /* OVE: used on ns32k based systems, if you want */
		 nuthin:2;
#else
		r_reltyp:3,	/* Valid only when r_length == 1, r_pcrel == 0 */
		nuthin:1;
#endif
};

/* if r_extern==1, then r_symbolnum is symbol index,
   if r_extern==0, then r_symbolnum is segment.
   Old-style segment is N_TYPE value (N_ABS, N_TEXT, N_DATA, or N_BSS).
   New-style segment is segment number shifted left with BSS bit in LSB.
   */

#define R_BSS(x)	(((x) & 1) == 1)
#define R_SEGNUM(x)	((x) >> 1)

/* Values for r_reltyp field */

#define R_NORM		0		/* Normal 16 bit relocation */
#define R_LO16		1		/* Low 16 bits of 32 bit quantity */
#define R_HI16		2		/* High 16 bits of 32 bit quantity */
#define R_V16		3		/* 16 bit relocation, subject to literal synthesis */
#define R_CNST		4		/* Constant value for R_HI16 or R_V16 relocation */
#define R_CHI16		(R_CNST | R_HI16) /* HI16, accompanied by CNST value */
#define R_CV16		(R_CNST | R_V16) /* V16, accompanied by CNST value */

/*
 * symbol table entry
 */

struct nlist {
  union {
    char	*n_name;	/* used by linker */
    struct nlist *n_next;	/* used by assembler */
    long	n_strx;		/* index into file string table */
  } n_un;
  unsigned char n_type;		/* type flag */
  char		n_segnum;	/* index into segment table (used to be n_other) */
  short		n_desc;		/* other STAB info */
  unsigned	n_value;	/* symbol value */
};

#define	n_hash	n_desc		/* used internally by ld */
#define n_other n_segnum	/* old name, defined for backwards compatibility */

/* n_type values */

#define N_TYPE		0x1e		/* mask for type bits */
#define N_EXT   	0x01		/* external bit */

#define N_UNDF		0x00		/* undefined */
#define N_ABS		0x02		/* absolute */
#define N_TEXT		0x04		/* text */
#define N_DATA		0x06		/* data */
#define N_BSS		0x08		/* bss */

#define N_COMM  	0x12		/* common (internal to ld) */
#define N_FN		0x1f		/* file name symbol */

#define N_STAB  	0xe0		/* mask for STAB bits */

/*
 * Format for namelist values.
 */

#define N_FORMAT        "%08x"
