/*
 ****************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 ****************************************************************
 */

/*
 * Accent message calls arguments and what-not.
 *
 * HISTORY
 * 14-Nov-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Made routine declarations external.
 *
 *  4-Nov-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Changed conditional re-declaration of GeneralReturn once more.
 *
 * 23-Sep-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Removed declaration InternalCall unless MACH_ACC is on.
 *
 *  1-Sep-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Added routine declarations, mostly for LINTLIBRARY use.
 *
 * 28-Jul-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Created for short life.
 */

#ifndef	_ACCENT_CALLS_
#define	_ACCENT_CALLS_	1

#ifdef	KERNEL
#include "mach_acc.h"
#endif	KERNEL

#define	Wait                  0
#define	DontWait              1
#define	Reply                 2
#define Send_Rec_DefaultPts   5
#define Send_Rec_AllPts       6
#define Send_Rec_LocalPt      7

typedef	short         SendOption;             /* Wait..Reply */

#define	ReceiveIt               1
#ifdef	KERNEL
#if	MACH_ACC
#define InternalCall		3
#endif	MACH_ACC
#endif	KERNEL

typedef	short         ReceiveOption;       /* Preview..ReceiveWait */

#define	DefaultPts            0
#define	AllPts                1
#define	LocalPt               2

typedef	short         PortOption;           /* DefaultPts..LocalPt */


#define NoTimeout		0L
#define ImmediateTimeout	(-1L)

typedef	long	MicroSeconds;

#ifdef	KERNEL
#include "../accent/acc_errors.h"
#else	KERNEL
#include <accent/acc_errors.h>
#endif	KERNEL

#if	LINTLIBRARY
/*
 *	In order to avoid sucking in every Accent header file around,
 *	we cheat by saying that Send and EReceive have first arguments
 *	that are msg_header_t-type pointers, rather than Msg-type pointers.
 *	Lint users will just have to live with a "first argument type
 *	mismatch"; if there's much fuss, they'll become "varargs" things.
 */
#ifdef	KERNEL
#include "../h/message.h"
#else	KERNEL
#include <sys/message.h>
#endif	KERNEL
#endif	LINTLIBRARY

#ifdef	KERNEL
#if	defined(_SYSCALL_SW_) || defined(_MACH_IPC_)
#define	GeneralReturn	int
#endif	defined(_SYSCALL_SW_) || defined(_MACH_IPC_)
#endif	KERNEL

extern
GeneralReturn	Send
#if	LINTLIBRARY
			(msgptr, MaxWait, Option)
	msg_header_t	*msgptr;
	long		MaxWait;
	SendOption	Option;
	{ return(Success); }
#else	LINTLIBRARY
			();
#endif	LINTLIBRARY

extern
GeneralReturn	EReceive
#if	LINTLIBRARY
			(msgptr, MaxWait, PtOption, Option)
	msg_header_t	*msgptr;
	long		MaxWait;
	PortOption	PtOption;
	ReceiveOption	Option;
	{ return(Success); }
#else	LINTLIBRARY
			();
#endif	LINTLIBRARY

extern
GeneralReturn	Receive
#if	LINTLIBRARY
			(msgptr, MaxWait, PtOption, Option)
	msg_header_t	*msgptr;
	long		MaxWait;
	PortOption	PtOption;
	ReceiveOption	Option;
	{ return(Success); }
#else	LINTLIBRARY
			();
#endif	LINTLIBRARY

extern
GeneralReturn Cheat(/* lp */);
extern
GeneralReturn InitPort(/* prt, MaxcBackLog */);
extern
GeneralReturn SoftEnable(/* NormalOrEmergency, EnableOrDisable */);
extern
GeneralReturn LockPorts (/* LockDoLock, LockPortPtr, LockPortCnt */);
extern
GeneralReturn MessagesWaiting(/* MsgWType, MsgWPortPtr, MsgWPortCnt, ActualMsgWPortCnt */);

#ifdef	KERNEL
#undef	GeneralReturn
#endif	KERNEL

#endif	_ACCENT_CALLS_
