#ifndef _HEAP_H
#define _HEAP_H
/*****************************************************************************
File:     heap.h
Contents: This file contains header information for heap allocation.
System:   BBNACI Programming Environments.
Created:  28-Nov-1989
Author:   ksedgwic

Remarks:

Copyright 1989 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

/* $Header: /nfs/europa/usr/BF2USRSRC/include/RCS/heap.h,v 1.3 91/04/01 09:03:58 bthomas Exp $ */

#define HEAP_UNCACHED     0x0000
#define HEAP_INTERLEAVED  0x0001
#define HEAP_COPY_BACK    0x0002
#define HEAP_WRITE_THRU   0x0004
#define HEAP_ANYWHERE     0x0008

#define HEAP_LOCAL_NODE   (-1)

/****************************************************************
* Defines for heapsetup
****************************************************************/

typedef struct				/* Structure for heapsetup. */
{
    unsigned	shmalloc_type;
    int		shmalloc_node;
    unsigned	flags;
# define HEAP_BORROWOK	0x1
    unsigned	heap_base_address;
    int		region_alloc_size;
    int		max_regions;
    int		mapmem_size;
} heap_params_t;

typedef struct				/* Structure for shmemregister */
{
    char *	name;			/* Name of the registered item */
    unsigned	type;
# define SHMEM_HEAP	1
# define SHMEM_MAPMEM	2
    void *	root;			/* For type = REGISTRY_HEAP, allocated */
                                        /* by shmemregister init routine. */
                                        /* Intended to be ptr to user-defined */
                                        /* data structure. */
    unsigned	base_addr;		/* For type = REGISTRY_MAPMEM, set by */
                                        /* shmemregister alloction action. */
                                        /* Intended to be base address of the */
                                        /* address space region used by the */
                                        /* registered shared memory service. */
    unsigned	size;			/* For type = REGISTRY_MAPMEM, set from */
                                        /* shmemregister size parameter, rounded */
                                        /* up to next page sized chunk. */
                                        /* Intended to be size of the */
                                        /* address space region used by the */
                                        /* registered shared memory service. */
    int		fd;			/* For type = REGISTRY_MAPMEM, the file */
                                        /* descriptor of the shared memory file. */
    int		file_offset;		/* For type = REGISTRY_MAPMEM, the offset */
                                        /* within the shared memory file to be */
                                        /* used by the caller for mapping the */
                                        /* registered item's onto the file */
} registry_ent_t;


#if defined(c_plusplus) || defined(__STDC__)
char  *shsbrk(int incr);
char  *shmalloc(unsigned size);
void  shfree(char* ptr);
void  shfile(char* name);
char  *heapsbrk(int flags, int node, int incr);
char  *heapmalloc(int flags, int node, unsigned size);
void  heapfree(char *ptr);
void  heapsetparams (heap_params_t * spec);
void  heapgetparams (heap_params_t * spec);
void  heapfile(char *name);
void  heapsync();
void  heapsize(int flags, int nodearg, unsigned size);
registry_ent_t * shmemregister (char * name,
				int type,
				int size,
				int (* init)(registry_ent_t *, void *),
				void * arg);
int   shmemgetregistry (int maxents,
			int * numents,
			registry_ent_t * * ents);
#else
char  *shsbrk();
char  *shmalloc();
void  shfree();
void  shfile();
char  *heapsbrk();
char  *heapmalloc();
void  heapfree();
void  heapsetparams ();
void  heapgetparams ();
void  heapfile();
void  heapsync();
void  heapsize();
registry_ent_t * shmemregister ();
int   shmemgetregistry ();
#endif		  

/* ----- end of file marker ----- */
#endif _HEAP_H
