/* $Header: /nfs/europa/usr/BF2USRSRC/include/RCS/icc.h,v 1.3 90/11/26 14:57:38 tdonahue Exp $ */

#ifndef ICC_H
#define ICC_H

#include <bbn_errors.h>
#include <butterflydev/icc_ringbuf.h>

/* 
  Version ID is a field in every ICC structure.
  This field is checked on every ICC operation.
  Change this when an ICC structure has changed
  to prevent incompatable versions.
*/
#define	ICC_VERSION	2

typedef enum icc_state_s {
  ICC_ACTIVE = 0,
  ICC_INACTIVE,
  ICC_DEAD,
} ICC_STATE;

typedef struct icc_bufhdr_s {
  unsigned int 	next;
  unsigned int	pnn;
  unsigned int	physaddr;
  unsigned int	version;
} ICC_BUFHDR;

typedef struct icc_buf_s {
  ICC_BUFHDR	header;
  unsigned char	user_data[4];
} ICC_BUF;

typedef struct icc_event_s  {
  unsigned int	next;
  unsigned int	priority;
  unsigned int	data;
  ICC_OSDATA	os_data;	/* os specific data */
} ICC_EVENT;


typedef struct icc_cbhdr_s {
  unsigned int	version;	/* version ID */
  unsigned int	magic;		/* magic number to validate ICC */
  unsigned int	refcnt;		/* cur number of references to ICC */
  unsigned int	pnn;		/* pnn on which memory lives */
  unsigned int	physaddr;	/* physical location/offset of ICC */
  unsigned int	memsize;	/* size of memory region */
  ICC_STATE	icc_state;	/* state of icc (ICC_INACTIVE | ICC_ACTIVE) */
  /* Q control */
  unsigned int	q_type;		/* ICC_QFIFO | ICC_QPRIORITY */
  unsigned int	q_state;	/* state of q: >0-event, 0-data */
  int		q_max;		/* max length of q */
  int		q_cnt;		/* cur length of q */
  unsigned int	q_lock;		/* must be long word aligned */
  unsigned int	q_head;		/* head of queued ICC buffers */
  unsigned int	q_tail;		/* tail "    "     "    "     */
  /* event buffers control */
  int		e_max;		/* max number of blocked events */
  int		e_cnt;		/* number of free events buffers left */
  unsigned int	e_start;	/* start offset of event buffer area */
  unsigned int	e_lock;
  unsigned int	e_head;		/* head of link list of free buffers */
  unsigned int	e_tail;		/* tail of link list "   "    "      */
  /* data buffers control */
  int		b_max;		/* max number of data buffers */
  int		b_len;		/* size of buffers */
  int		b_cnt;		/* number of free buffers left */
  unsigned int	b_start;	/* start offset of data buffer area */
  unsigned int	b_lock;
  unsigned int	b_head;		/* head of link list of free buffers */
  unsigned int	b_tail;		/* tail of link list "   "    "      */
} ICC_CBHDR;

typedef struct icc_cb_s {
  ICC_CBHDR	hdr;
  /* remaining memory where event and data buffers are carved out */
  unsigned char	b_area[4];
} ICC_CB;

typedef struct icc_statbuf_s {
  int	version;
  int	nattached;
  int	icc_state;
  int	nblocked;
  int	nqueued;
  int	nbufs;
  int	blength;
  int	qlength;
} ICC_STATBUF;

#define ICC_MEMSIZE(npends, nbufs, buflen) \
  		   (sizeof(ICC_CBHDR) + \
		    (nbufs*(buflen+sizeof(ICC_BUFHDR))) + \
		    (npends*sizeof(ICC_EVENT)))

#define ICC_NBUFS(memsize, npends, buflen) \
                    ((memsize - sizeof(ICC_CBHDR) - \
		    (npends*sizeof(ICC_EVENT))) / (buflen+sizeof(ICC_BUFHDR)))

#define ICC_MAGIC	0xDEAD1234
#define ICC_QPRIORITY	0x0
#define ICC_QFIFO	0x1
#define ICC_QDATA	0xdd
#define ICC_QEVENT	0xee

#define ICC_OFFSET(icc, addr)	((unsigned int)addr-(unsigned int)icc)
#define ICC_ADDR(icc, offset)	((ICC_BUFHDR *)((unsigned int)icc+(unsigned int)offset))
#define ICC_BASIC_CHECK(icc) \
  if (icc->version != ICC_VERSION) return(ICC_EBADVERSION); \
  if (icc->magic != ICC_MAGIC) return(ICC_EBADICC); \
  if (icc->icc_state != ICC_ACTIVE) return(ICC_EINACTIVE);

#define ICC_COMPLETE(n) { if (n) *n += 1; }
    
/* OS Types */
#define MACH_OSTYPE	0x1001
#define PSOS_OSTYPE	0x1002

#ifdef NX_KERNEL
#define xmeml	xmemsupword
#endif NX_KERNEL

#endif
