/*
 * Copyrighted as an unpublished work.
 * (c) Copyright 1989 INTERACTIVE Systems Corporation
 * All rights reserved.
 *
 * RESTRICTED RIGHTS
 *
 * These programs are supplied under a license.  They may be used,
 * disclosed, and/or copied only as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this restricted rights notice.  Use, copying, and/or disclosure
 * of the programs is strictly prohibited unless otherwise provided
 * in the license agreement.
 */


/*
 *	limits.h
 *
 *	developed from POSIX and X3J11 standards
 */

#ifndef _LIMITS_H
#define _LIMITS_H

/*
 * POSIX minimum values; Table 2-2 POSIX 1003.1-1988 Std.
 *
 * These are the minimum values for POSIX systems. The limits in
 * a given implementation may be higher.
 */
#define	_POSIX_ARG_MAX		4096	/* exec argument/environment length */
#define	_POSIX_CHILD_MAX	6	/* processes per real UID */
#define	_POSIX_LINK_MAX		8	/* file link count */
#define	_POSIX_MAX_CANON	255	/* tty canonical input queue size */
#define	_POSIX_MAX_INPUT	255	/* tty input queue size */
#define	_POSIX_NAME_MAX		14	/* number of bytes in a filename */
#define	_POSIX_NGROUPS_MAX	0	/* supplementary GIDs per process */
#define	_POSIX_OPEN_MAX		16	/* open files per process */
#define	_POSIX_PATH_MAX		255	/* number of bytes in a pathname */
#define	_POSIX_PIPE_BUF		512	/* atomic pipe write size */


/***************************************************************************
 *	WARNING/NOTE: Some of system limits described here were taken
 *	from the sys/param.h file. Any changes over there must be
 *	reflected here and vice versa.
 ***************************************************************************/

/* Run-Time Increasable Values; Table 2-3 POSIX 1003.1-1988 Std. */
#define	NGROUPS_MAX	16		/* supplementary GIDs per process */


/* Run-Time Invariant Values; Table 2-4 POSIX 1003.1-1988 Std. */
#define	ARG_MAX		20480		/* max length of arguments to exec */
#define	CHILD_MAX	40		/* max # of processes per user id */
#define	OPEN_MAX	64		/* max # of files a process can have open */

/* Pathname Variable Values; Table 2-5 POSIX 1003.1-1988 Std. */
#define	LINK_MAX	32767		/* max # of links to a single file */
#define	MAX_CANON	255		/* >>> tty canonical queue input size */
#define	MAX_INPUT	255		/* >>> tty input queue size */
#define	NAME_MAX	255		/* max # of characters in a file name */
#define	PATH_MAX	1023		/* max # of characters in a path name */
#define	PIPE_BUF	4096		/* max # bytes atomic in write to a pipe */
/* dropped by POSIX; CLEANUP */
#define	PIPE_MAX	4096		/* max # bytes written to a pipe in a write */

/* other non-POSIX required defines but used by system */
#define	PID_MAX		30000		/* max value for a process ID */
#define	UID_MAX		60000		/* max value for a user or group ID */

/* FIPS 151-1 required define */
#define	CLK_TCK		100		/* # of clock ticks per second */


/* ANSI X3J11 required defines; sec 2.9.1 POSIX 1003.1-1988 Std. */
#define	CHAR_BIT	8		/* # of bits in a char */
#define	CHAR_MAX	127		/* max integer value of a char */
#define	CHAR_MIN	-128		/* min integer value of a char */
#define	INT_MAX		2147483647	/* max value of an int */
#define	INT_MIN		-2147483648	/* min value of an int */
#define	LONG_MAX	2147483647	/* max value of a long */
#define	LONG_MIN	-2147483648	/* min value of a long */
#define MB_LEN_MAX	1		/* max # of byes in muti-byte char */
#define	SCHAR_MAX	127		/* max value of signed char */
#define SCHAR_MIN	-127		/* min value of signed char */
#define	SHRT_MAX	32767		/* max value of a short */
#define	SHRT_MIN	-32768		/* min value of a short */
#define UCHAR_MAX	255		/* max value of an unsigned char */
#define	UINT_MAX	4294967295	/* max value of an unsigned int */
#define ULONG_MAX	4294967295	/* max value of an unsigned long */
#define USHRT_MAX	65535		/* max value of an unsigned short */


#endif _LIMITS_H
