#ifndef _PP_H
#define _PP_H

/*****************************************************************************
File:     pp.h
Contents: Include file for using libpp.a
System:   BBNACI Programming Environments.
Created:  October 13, 1990
Author:   Bob Thomas

$Header: /nfs/europa/usr/BF2USRSRC/include/RCS/pp.h,v 1.6 91/04/01 08:55:37 bthomas Exp $

Remarks:

*******************************************************************************/


/*******************************************************************************
 *
 * Definitions for time out handlers.
 *
 *******************************************************************************/

typedef int (* pp_timeout_handler_t)();


/*******************************************************************************
 *
 * Definitions for spinning mutual exclusion locks
 *
 *******************************************************************************/

typedef unsigned long pp_lock_t;

#if defined(c_plusplus) || defined(__STDC__)
void _pp_delay (int);			/* Used in pp_lock macro */
# endif

/*
 * The double macro definitions allow a function of the same name to exist.
 */

# define pp_lock_init(p) __pp_lock_init(p)
# define __pp_lock_init(p)  {  * (p) = 0; }

# define pp_lock(p) __pp_lock(p)
# define __pp_lock(p) \
    while (atomior32 (p, 1) != 0) _pp_delay(10)

# define pp_unlock(p) __pp_unlock(p)
# define __pp_unlock(p)  {  * (p) = 0; }

/*******************************************************************************
 *
 * Definitions for spinning readers / writers locks
 *
 *******************************************************************************/

/*
 * Macros for implementing exclusive/non-exclusive locks
 * that allow 1 exclusive accessor or multiple non-exclusive accessors.
 *
 * A 32 bit long word is used for the lock as follows:
 *
 * bits 31-24:	lock byte = _pp_LOCKTAG when lock word is being updated
 *			    0 otherwise.
 * bits 23-16:	exclusive lock = _PP_EXCLTAG when locked for exclusive access
 *				 0 otherwise.
 * bits 15-0:	count = # of non-exclusive lockers.
 *
 * The basic idea is that the lock operations first lock the lock by setting
 * the lock byte to LOCKTAG, then they perform the operation in question, and
 * then they clear the lock byte.
 */

# define _PP_LOCKTAG		0xff000000
# define _PP_EXCLTAG		0x00ff0000
# define _PP_EXCLTAGMASK	0x00ff0000

typedef unsigned long pp_rwlock_t;

/*
 * The indirect macro definitions allow a function of the same name to exist.
 */

# define pp_rwlock_init(L) __pp_rwlock_init(L)
# define __pp_rwlock_init(L)  {  * (L) = 0; }

# define pp_w_lock(L) __pp_w_lock(L)
# define __pp_w_lock(L) \
{   pp_rwlock_t v; \
    while (1) \
    {	while (_PP_LOCKTAG == (v = (pp_rwlock_t) xmemul (L, _PP_LOCKTAG))) \
	    _pp_delay (10); \
	if (0 == v) \
	{ \
	   * (L) = _PP_EXCLTAG; \
	    break; \
	} \
	else \
	{ \
	    * (L) = v; \
	} \
    } \
}

# define pp_w_unlock(L) __pp_w_unlock(L)
# define __pp_w_unlock(L) \
{   pp_rwlock_t v ; \
    while (_PP_LOCKTAG == (pp_rwlock_t) xmemul (L, _PP_LOCKTAG)) \
	_pp_delay (10); \
    * (L) = 0; \
}

# define pp_r_lock(L) __pp_r_lock(L)
# define __pp_r_lock(L) \
{ \
    while (1) \
    {	pp_rwlock_t v; \
	while (_PP_LOCKTAG == (v = (pp_rwlock_t) xmemul (L, _PP_LOCKTAG))) \
	    _pp_delay (10); \
	if (v & _PP_EXCLTAGMASK) \
	{ \
	    * (L) = v; \
	} \
	else \
	{ \
	    * (L) = v + 1; \
	    break; \
	} \
    } \
}

# define pp_r_unlock(L) __pp_r_unlock(L)
# define __pp_r_unlock(L) \
{   pp_rwlock_t v; \
    while (_PP_LOCKTAG == (v = (pp_rwlock_t) xmemul (L, _PP_LOCKTAG))) \
	_pp_delay (10); \
    * (L) = v - 1; \
}


/*******************************************************************************
 *
 * Definitions for clusters / processors / processes
 *
 *******************************************************************************/

# include <sys/cluster.h>

extern int pp_my_process_id;
extern int pp_num_processes;
extern int pp_is_parent;

#if defined(c_plusplus) || defined(__STDC__)
int pp_num_processors ();		/* Ret # processors in this cluster */
int pp_num_processors_in_cluster (cluster_id_t cluster);
                                        /* Ret # processors in spec'd cluster */
int pp_get_cluster_node_id ();		/* Ret cluster node # of this processor */
int pp_get_system_node_id ();		/* Ret system node # of this processor */
int pp_create_processes ();		/* Create process on each processor */
int pp_create_num_processes (int num);	/* Create spec'd # of processes */
int pp_cluster_node_to_physical_node (int node);
int pp_cluster_node_to_system_node (int node);
int pp_system_node_to_physical_node (int node);
int pp_system_node_node_to_cluster_node (int node);
int pp_physical_node_to_system_node (int node);
int pp_physical_node_to_cluster_node (int node);
void pp_print_node_tables ();
#else /* defined(c_plusplus) || defined (__STDC__) */
int pp_num_processors ();
int pp_num_processors_in_cluster ();
int pp_get_cluster_node_id ();
int pp_get_system_node_id ();
int pp_create_processes ();
int pp_create_num_processes ();
int pp_cluster_node_to_physical_node ();
int pp_cluster_node_to_system_node ();
int pp_system_node_to_physical_node ();
int pp_system_node_node_to_cluster_node ();
int pp_physical_node_to_system_node ();
int pp_physical_node_to_cluster_node ();
void pp_print_node_tables ();
#endif /* defined(c_plusplus) || defined (__STDC__) */


/*******************************************************************************
 *
 * Definitions for queued locks
 *
 *******************************************************************************/

/****************************************************************
Defines
****************************************************************/

#define PPHOLDSHORT  1
#define PPHOLDLONG   2

/****************************************************************
pp_q_lock_t - the global lock type.  The pp_q_lock_t is currently only
a tail pointer which points to the last process on the queue.
Participants xmem bid pointers into the pp_q_lock_t.
****************************************************************/

typedef struct qzbid qzbid;
typedef qzbid * pp_q_lock_t;

/****************************************************************
Operations on queued locks.
****************************************************************/

#if defined(c_plusplus) || defined(__STDC__)
void pp_q_init (char * name);		/* Setup queuing locks, etc. */
void pp_q_lock_init (pp_q_lock_t * p);	/* Initialize a queueing lock */
int pp_q_lock (pp_q_lock_t * qzptr, int duration);
                                        /* Lock the lock */
void pp_q_lock_convert (pp_q_lock_t * qzptr);
                                        /* Convert others from spin to sleep */
void pp_q_unlock (pp_q_lock_t * qzptr);	/* Unlock */
void pp_q_lock_settimeout (int seconds);/* Adjust timeout for queued locks */
pp_timeout_handler_t
    pp_q_lock_set_timeout_handler (pp_timeout_handler_t new_handler);
                                        /* Set routine for handling timeouts. */
int pp_q_lock_timedout(pp_q_lock_t * p);
#else /* defined(c_plusplus) || defined (__STDC__) */
void pp_q_init ();
void pp_q_lock_init ();
int pp_q_lock ();
void pp_q_lock_convert ();
void pp_q_unlock ();
void pp_q_lock_settimeout ();
pp_timeout_handler_t pp_q_lock_set_timeout_handler ();
int pp_q_lock_timedout ();
#endif /* defined(c_plusplus) || defined (__STDC__) */

/****************************************************************
Macros on queued locks
****************************************************************/

#define pp_q_shortlock(qzptr)	(pp_q_lock((qzptr), PPHOLDSHORT))
#define pp_q_longlock(qzptr)	(pp_q_lock((qzptr), PPHOLDLONG))

# include <queued_lock_impl.h>


/*******************************************************************************
 *
 * Definitions for queued readers / writers locks
 *
 *******************************************************************************/

typedef struct
{
    unsigned	state;		/* State of lock = RW_FREE, RW_EXCL, reader count */
# define RW_FREE	0x10000
# define RW_EXCL	0x20000
    pp_q_lock_t	mutex_lock;	/* Protects access to rest of structure */
    qzbid *	bid_q_tail;	/* Queue of procs waiting on lock with head of queue
				/* the bid currently holding the lock.  NULL */
                                /* lock is free means */
    qzbid *	bid_rq;		/* Only 1 queued reader is on bid_q_tail, */
                                /* The rest are here */
    unsigned	reader_qd;	/* Is a reader on bid_q_tail? */
    qzbid	reader_bid [2];	/* Used to represent readers when read lock */
                                /* attempt queued or read lock held.  Have 2 */
                                /* to implement writer priority. */
    int		r_bidding_idx;	/* The reader_bid to use for bidding for r-lock */
                                /* -1 means need to set r_bidding_idx. */
    int		r_locked_idx;	/* The reader_bid represented the r-locked lock */
                                /* -1 means lock not r-locked. */
} pp_q_rwlock_t;

/****************************************************************
Operations on queued readers /writers locks
****************************************************************/

#if defined(c_plusplus) || defined(__STDC__)
void pp_q_rwlock_init (pp_q_rwlock_t * p);
                                /* Initialize a q_rw_lock_t */
int pp_q_w_lock (pp_q_rwlock_t * lock, int duration);
                                /* Lock the lock for writing */
void pp_q_w_unlock (pp_q_rwlock_t * lock);
                                /* Unlock lock locked for writing */
int pp_q_r_lock (pp_q_rwlock_t * lock, int duration);
                                /* Lock the lock for reading */
void pp_q_r_unlock (pp_q_rwlock_t * lock);
                                /* Unlock lock locked for reading */
void pp_q_rwlock_settimeout (int seconds);
                                /* Adjust timeout for queued reader/writer locks */
pp_timeout_handler_t
    pp_q_rwlock_set_timeout_handler (pp_timeout_handler_t new_handler);
                                        /* Set routine for handling timeouts. */
int pp_q_rwlock_timedout (pp_q_rwlock_t * lock);
#else /* defined(c_plusplus) || defined (__STDC__) */
void pp_q_rwlock_init ();
int pp_q_w_lock ();
void pp_q_w_unlock ();
int pp_q_r_lock ();
void pp_q_r_unlock ();
void pp_q_rwlock_settimeout ();
pp_timeout_handler_t pp_q_rwlock_set_timeout_handler ();
int pp_q_rwlock_timedout ();
#endif /* defined(c_plusplus) || defined (__STDC__) */



/*******************************************************************************
 *
 * Definitions for events
 *
 *******************************************************************************/

# define _EVENTS_MAX_PROCS_WAITING_	128

/****************************************************************
pp_event_t - the fundamental event data structure.
****************************************************************/

typedef struct _event_bid_struct_ _event_bid_t;
struct _event_bid_struct_
{
    int			state;
    _event_bid_t *	next;
};

typedef struct
{
    int			mutex_lock;
    _event_bid_t *	q_head;		/* Head of queue of waiting processes */
    _event_bid_t *	q_tail;		/* Tail of queue ... */
    int			n_posted;	/* >0 means excess posts; < 0 means */
                                        /* -n_posted processes waiting on queue */
} pp_event_t;

/****************************************************************
eventstat - this enum describes the diffeent reasons why a
wait was terminated.
****************************************************************/

typedef enum
{
    EVENT_POSTED,
    EVENT_SIGNALED,
    EVENT_TIMEOUT,
    EVENT_NOSPACE,
} pp_eventstat_t;

#define EVENT_RESET    0x0
#define EVENT_ASLEEP   0x1
#define EVENT_WAKEUP   0x2

/****************************************************************
Operations on events
****************************************************************/

#if defined(__STDC__) || defined(c_plusplus)
void pp_event_init (pp_event_t * evptr);	/* Initialize the event */
pp_eventstat_t pp_event_wait (pp_event_t * evptr, int timeout);
                                        /* Wait for the event */
void pp_event_post (pp_event_t * evptr);/* Post the event */
#else
void pp_event_init ();
pp_eventstat_t pp_event_wait ();
void pp_event_post ();
#endif



/*******************************************************************************
 *
 * Definitions for monitors
 *
 *******************************************************************************/

typedef struct
{
    pp_q_lock_t	mutex_lock;	/* Standard lock.  Processes entering monitor */
                                /* queue up here.  Process "in" monitor is at */
                                /* end of this chain. */
    qzbid *	q_head;		/* Head of wait queue for mon_wait. */
    qzbid *	q_tail;		/* Tail of ... */
    int		q_len;		/* Length of ... */
} pp_monitor_t;

/****************************************************************
Operations on monitors
****************************************************************/

#if defined(c_plusplus) || defined(__STDC__)
void pp_mon_init (pp_monitor_t * p);	/* Initialize the event */
int pp_mon_enter (pp_monitor_t * p, int duration);
                                        /* Enter the monitor */
void pp_mon_exit (pp_monitor_t * p);	/* Exit the monitor */
int pp_mon_wait (pp_monitor_t * p);	/* Wait for a signal */
void pp_mon_signal (pp_monitor_t * p);	/* Signal on the monitor */
int pp_mon_qlength (pp_monitor_t * p);	/* Return # processes waiting for signal */
void pp_mon_settimeout (int seconds);
pp_timeout_handler_t
    pp_mon_set_timeout_handler (pp_timeout_handler_t new_handler);
                                        /* Set routine for handling timeouts. */
int pp_mon_timedout (pp_monitor_t * p);
#else /* defined(c_plusplus) || defined (__STDC__) */
void pp_mon_init ();
int pp_mon_enter ();
void pp_mon_exit ();
int pp_mon_wait ();
void pp_mon_signal ();
int pp_mon_qlength ();
void pp_mon_settimeout ();
pp_timeout_handler_t pp_mon_set_timeout_handler ();
int pp_mon_timedout ();
#endif /* defined(c_plusplus) || defined (__STDC__) */

/*******************************************************************************
 *
 * Definitions for barriers
 *
 *******************************************************************************/

#if defined(c_plusplus) || defined(__STDC__)
void pp_barrier_init (int nprocs, int * nodemap);
                                        /* Initialize barriers */
void pp_team_barrier (int tdesc, int tindex, int tsize, int iproc, int phase);
                                        /* Enter "full" barrier. */
void pp_barrier_init_simple_barrier (int my_id);
                                        /* Initialize the simple barrier */
void pp_barrier ();			/* Enter the "simple" barrier. */
#else /* defined(c_plusplus) || defined (__STDC__) */
void pp_barrier_init ();
void pp_team_barrier ();
void pp_barrier_init_simple_barrier ();
void pp_barrier ();
#endif /* defined(c_plusplus) || defined (__STDC__) */

/* ----- end of file marker ----- */
#endif _PP_H
