#ifndef _QUEUED_LOCK_IMPL_H
#define _QUEUED_LOCK_IMPL_H
/*****************************************************************************
File:     queued_lock_impl
Contents: This file describes structures used to allocate memory to the
          queued lock routines.
System:   BBNACI Programming Environments.
Created:  26-Jan-1990
Author:   ksedgwic

Remarks:

**************
Modified a little to prepare for inclusion in shared memory library (libparsm).
    - Name of file changed from qzblock.h to queued_lock_impl.h
    - Names changed from qzlockit to pp_q_lock, etc.
Bob Thomas:  9/27/90
**************

Copyright 1990 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

/* $Header: /nfs/europa/usr/BF2USRSRC/include/RCS/queued_lock_impl.h,v 1.2 91/02/05 08:21:22 bthomas Exp $ */


typedef struct qzblock    qzblock;
typedef struct qznode     qznode;
typedef struct qzbidstack qzbidstack;

/****************************************************************
Defines
****************************************************************/

#define MAXNODES       512	/* maximum number of nodes in system */

#define QZPROCSPERNODE 100	/* 100 processes per node */
#define QZLOCKSPERPROC 16	/* 16 locks per process at a time */

#define QZSPIN   1
#define QZSLEEP  2

#define QZASLEEP   0x1
#define QZPOSTED   0x2

#define QZNPTIMEOUT  1000000	/* 1 sec in usec */
#define QZSPINFACTOR 100	/* check time and mode every 100 polls */


/****************************************************************
qzblock - the master control block for the queued lock memory pool.  The
first process to perform a qzattach function creates the qzblock.
Subseqzuent attaches allocate memory from the local qznode structure.
****************************************************************/

struct qzblock
{
    int      interlock;		/* for synchronization */
    unsigned size;		/* size of qzblock + qznodes */
    qznode    *node[MAXNODES];	/* pointers to the nodes */

};


/****************************************************************
qzbid - a per process local structure.  Qbids are used to link
the participants together.  Any time a participant needs to spin 
wait it will spin on its bid which is local memory.
****************************************************************/

struct qzbid
{
    pp_q_lock_t *target;	/* which lock this bid is working on */
    qzbid  *next;		/* points to next in line */
    qzbid  *prev;		/* points to the qzbid ahead of us */
    short  duration;		/* PPHOLDLONG or PPHOLDSHORT */
    short  mode;		/* NULL, QZSPIN, QZSLEEP */
    int    interlock;		/* used for wakeups */
    qzbid  *n;			/* points to this processes next qzbid */
    qzbid  *p;			/* points to this processes prev qzbid */

};


/****************************************************************
qzbidstack - the per process local memory block.  The stack handles
successive locks held concurrently.  QZLOCKSPERPROC is the number
of locks that a process may hold at one time.
****************************************************************/

struct qzbidstack
{
    /* IT IS IMPORTANT THAT -1 NOT BE A VALID VALUE FOR ANY
     * OF THE qzbidstack's OR qzbid's FIELDS.
     */
    int   magic;		/* magic number is -1 */

    int   pid;			/* process id */
    int   clunid;		/* cluster node id */

    qzbid  bid[QZLOCKSPERPROC];	/* the qzbids */

    pp_q_lock_t interlock;	/* the lock used by q_wait/q_post */
    int    mode;		/* the mode word for q_wait/q_post */

};


/****************************************************************
qznode - a per node memory block.
****************************************************************/

struct qznode
{
    int   available;		/* number of qzbidstacks available */
    int   nextindex;		/* index of next avail qzbidstack */

    qzbidstack stack[QZPROCSPERNODE]; /* the qzbidstacks themselves */

};

/****************************************************************
An atomic operation
****************************************************************/

#define atomxmem(ap, v)		(xmemi((ap),(v)))

/* ----- end of file marker ----- */
#endif _QUEUED_LOCK_IMPL_H
