/* $Header: sha.h,v 1.2 89/04/07 17:38:57 kchin Exp $ */

#ifndef _SHA_H
#define _SHA_H

/*****************************************************************************
* File: sha.h - Definitions for the Interphase 4210 Synchronous              *
* SCSI host adapter.                                                         *
*                                                                            *
* Started 12/5/88 by drb                                                     *
*                                                                            *
* [And now, a few words from the legal department.]                          *
*                                                                            *
* Copyright 1988 by BBN Advanced Computers Inc.                              *
* ALL RIGHTS RESERVED                                                        *
*                                                                            *
* No part of this material may be reproduced, stored in a retrieval          *
* system, transmitted or used in any form or by any means, electronic,       *
* mechanical, photocopying, recording, or otherwise, without the prior       *
* written permission of, or express license from BBN Advanced Computers Inc. *
*                                                                            *
* RESTRICTED RIGHTS LEGEND                                                   *
*                                                                            *
* Use, duplication, or disclosure by the Government is subject to            *
* restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to      *
* Technical Data and Computer Software clause at DFARS 52.227-7013.          *
*                                                                            *
* BBN Advanced Computers Inc.                                                *
* 10 Fawcett Street                                                          *
* Cambridge, Ma. 02238                                                       *
* 617-873-6000                                                               *
*****************************************************************************/


/* Register addresses, WORD offsets from the controller base address */

#define MASTER_STATUS(a) (*(((short *) (a))+0))
#define MASTER_STATUS_ADDR(a) (unsigned short *)(a)
#define MASTER_CTRL(a) (*(((short *) (a))+1))
#define MASTER_CTRL_ADDR(a) (((unsigned short *) (a))+1)

/* Interrupt when Queue Element Available register */
#define IQAR(a) (*(((short *) (a))+2))


/* Command queue "next" register, NOT USED BY 4210 (or me!) */
#define QHP(a) (*(((short *) (a))+3))

/* Master Command entry */
#define MCE_ADDR(a) ((CQE *)(((short *) (a))+8))

/* These assume a 10 slot command-queue, max IOPB length of 64 bytes      */
/* Interphase mentions this configuration a lot, so I'll try it...        */

#define NCQE 10             /* Number of command queue entries  */
#define MAX_IOPB_SIZE 64    /* maximum size of an IOPB          */

/* Base of the Command Queue in dual-ported memory */
#define CQE_BASE_ADDR(a) ((CQE *)(((char *) (a))+0x1C))

/* Base of the "Host usable space" or IOPB area */
#define HUS_BASE_ADDR(a) ((IOPB *)(((char *) (a))+0x94))

/* Address of the Command Response Block */
#define CRB_ADDR(a) ((CRB *)(((char *) (a))+0x73C))


/* Definitions for the Master Status register */

#define MASTER_STATUS_FAIL      0x0     /* badly broken             */
#define MASTER_STATUS_RESETTING 0x1     /* Reset in progress        */
#define MASTER_STATUS_INDIAG    0x3     /* running diags            */
#define MASTER_STATUS_OK        0x2     /* ok                       */
#define MASTER_STATUS_QFC       0x4     /* queue flush complete     */

/* Definitions for the Master Control Register */

#define MASTER_CTRL_SQM         0x0001  /* Start Queue-mode         */
#define MASTER_CTRL_FLQR        0x0004  /* Flush Q and report(!)    */
#define MASTER_CTRL_FLQ         0x0800  /* Flush Queue              */
#define MASTER_CTRL_RESET       0x1000  /* Reset board (wait 100us) */
#define MASTER_CTRL_SYSFAIL     0x2000  /* Sysfail Enable           */

typedef struct
    {
	unsigned short  qecr;               /* queue entry ctl reg      */
#ifdef board_understood_longword_refs
    unsigned short  res1:4;             /* Reserved                 */      
    unsigned short  iopb_type:4;        /* iopb type, always 0      */      
    unsigned short  res2:5;             /* Reserved                 */      
    unsigned short  hipri:1;            /* Hi priority              */      
    unsigned short  abort_ack:1;        /* Stop aborting IOPB's     */      
    unsigned short  go:1;               /* Go / busy bit            */      
#endif
    unsigned short iopb_addr;           /* IOPB offset from ctrlr base */   
    unsigned short command_tag[2];      /* arbitrary command tag    */      
    unsigned char iopb_length;          /* iopb length              */      
    unsigned char work_queue;           /* work queue number        */      
    unsigned short res3;                /* Reserved                 */      
    } CQE;

#define CQE_GO 0x1

/* WARNING: Use these constants rather than sizeof(CQE).  The
   88K compiler feels free to pad the structure to a multiple of
   four bytes.                                                      */
#define CQE_NWORDS 6
#define CQE_NBYTES 12

/* offset of iopb in short trout space */
#define IOPB_OFFSET(a) (((unsigned)(a))&0x7ff)
#define CMDMAX 32

/* Controller command packet. */

typedef struct
    {
    unsigned short command;             /*0: command                */
    unsigned short options;             /*1: options                */
    unsigned char passback_status;      /*2: error codes            */
    unsigned char error_code;
    unsigned short res3;                /*3: reserved               */  
    unsigned char norm_vector;          /*4: vectors                */
    unsigned char error_vector;
    unsigned short interrupt_level;     /*5: VME interrupt level    */
    unsigned short res4;                /*6: reserved               */
	unsigned short mtam;                /*7: memory type, etc.      */
#ifdef board_understood_longword_refs
    unsigned short res5:4;
    unsigned short transfer_type:2;        /*7: 0=>norm 1=>block 2=>no incr */
    unsigned short memory_type:2;          /*   1=>16 bit xfer, 2=>32 bit */
    unsigned short address_modifier:8;     /* increment value during xfer?*/
#endif
    unsigned short buffer_address[2];   /*8,9: buffer address       */
    unsigned short max_transfer[2];         /*A,B: maximal transfer     */
    unsigned short res6;                /*C: reserved               */
    unsigned short res7;                /*D: reserved               */
    unsigned short res8;                /*E: reserved               */
	unsigned short scsi_addr;           /*F: target device address  */
#ifdef board_understood_longword_refs
    unsigned short extended_address:8;  /*F: target device address  */
    unsigned short extend_address:1;       /* use extended address     */
    unsigned short bus_number:1;           /* which of the two busses  */
    unsigned short lun:3;                  /* target LUN, mostly 0!    */
    unsigned short scsi_address:3;         /* device ID                */
#endif
    unsigned char scsi_command [CMDMAX];/* scsi command block       */
    } IOPB;

#define IOPB_NWORDS 32
#define IOPB_NBYTES 64

/* IOPB command codes */

#define IOPB_COMMAND_CONFIGURE  0x41
#define IOPB_COMMAND_WORKQINIT  0x42
#define IOPB_COMMAND_PASSTHRU   0x20

/* IOPB memory type codes */
#define IOPB_MEMTYPE_DUALPORT   (0x3<<8)
#define IOPB_MEMTYPE_16BIT      (0x1<<8)
#define IOPB_MEMTYPE_32BIT      (0x2<<8)

/* IOPB transfer type codes */
#define IOPB_TT_NORMAL          (0x0<<10)
#define IOPB_TT_BLOCK           (0x1<<10)

/* IOPB VME transfer type code */
#define IOPB_VME_SUPERDATA      0x3d

/* IOPB miscellany  */
#define IOPB_INTERRUPT_ENABLE   0x1
#define IOPB_DIR_READVME        0x100

/* Command response block, really a large status register           */

typedef struct
    {
	unsigned short crsw;                  /*0: command resp status word   */
#ifdef board_understood_longword_refs
    unsigned short res1:9;                    /*0: command resp status    */
    unsigned short cmd_queue_entry_avail:1;    /* 1=>queue entry avail */
    unsigned short queue_mode_started:1;   /* 1=> queue mode           */
    unsigned short abort_queue:1;          
    unsigned short exception:1;
    unsigned short error:1;
    unsigned short cmd_complete:1;
    unsigned short valid_intack:1;         /* set to 0 to dismiss int  */
#endif
    unsigned short res2;                /*1: reserved               */
    unsigned short command_tag[2];      /*2,3: command tag          */
    unsigned char iopb_length;          /*4: iopb len, queue #      */
    unsigned char work_queue;   
    unsigned short res3;                /*5: reserved               */
    IOPB     returned_iopb;             /*6-xxx: the iopb           */
    } CRB;

#define CRB_VALID_INTACK 0x0001
#define CRB_CMD_COMPLETE 0x0002
#define CRB_ERROR        0x0004
#define CRB_EXCEPTION    0x0008
#define CRB_ABORT_QUEUE  0x0010
#define CRB_QMODE_START  0x0020
#define CRB_QELM_AVAIL   0x0040

#define CRB_ACK 0

/* Controller initialization block */

typedef struct
    {
    unsigned short  cmdqlen;            /*0: # of cmd queue slots   */
    unsigned short  dmaburst;           /*1: dma burst count        */
    unsigned short  ctrlr_normvec;      /*2: controller nrml vectr  */
    unsigned short  ctrlr_errvec;       /*3: controller error vectr */
    unsigned short  primary_busid;      /*4: primary scsi bus id    */
    unsigned short  second_busid;       /*5: secondary scsi busid   */
    unsigned short  crb_offset;         /*6: crb offset             */
    unsigned short  seltimeout_msw;     /*7: select timeout MSW     */
    unsigned short  seltimeout_lsw;     /*8: select timeout LSW     */
    unsigned short  reseltimeout_msw;   /*9: reselect timeout MSW   */
    unsigned short  reseltimeout_lsw;   /*A: reselect timeout LSW   */
    unsigned short  vmetimeout_msw;     /*B: vmebus timeout MSW     */
    unsigned short  vmetimeout_lsw;     /*C: vmebus timeout LSW     */
    /* WARNING: 0xB-0xF reserved but not defined here */
    } CIB;
#define CIB_NBYTES 32

#define PRIMARY_BUSID 7
#define SECONDARY_BUSID 7
#define CRB_OFFSET 0x73c
#define SELECT_TIMEOUT 100   /* milliseconds, allegedly */
#define RESELECT_TIMEOUT 0   /* infinite                */
#define VME_TIMEOUT 100      /* x32 ms, allegedly       */

typedef struct
    {
    unsigned short command;             /*0: command                */
    unsigned short options;             /*1: options                */
    unsigned char passback_status;      /*2: error codes            */
    unsigned char error_code;
    unsigned short res3;                /*3: reserved               */  
    unsigned char norm_vector;          /*4: vectors                */
    unsigned char error_vector;
    unsigned short interrupt_level;     /*5: VME interrupt level    */
    unsigned short res4 [8];            /*6-D: reserved             */
    unsigned short workq_number;        /*E: workqueue number       */
    unsigned short workq_options;       /*F: options                */
    unsigned short workq_nslots;        /*10: number of slots       */
    unsigned short workq_priority;      /*11: priority              */
    } IWQ;

#define IWQ_NBYTES (0x11*2)
#define IWQ_ABORT_ENABLE 0x0001
#define IWQ_FORCE_INIT   0x8000
#define IWQ_WORKQ_SLOTS  20
#define IWQ_WORKQ_PRIORITY 2
/* Host adapter structure, 1 per board */

struct sha
    {
    unsigned short     *base_addr;      /* host adapter base address    */
    int                 flags;          /* what else?                   */
    int                 nextcmd;        /* next command queue element   */
    };

typedef struct sha SHA;

#define MAXSHAS     4   /* Max number of host adapters */

#define SHA_PRESENT     0x00000001      /* says the adapter is present */
#define SHA_EXCLUSIVE   0x00000002      /* says in exclusive operation */
#define SHA_RESET       0x00000004      /* says hard reset in progress */
#define SHA_READY       0x00000008      /* says ready for action       */
#define SHA_WORKQ_INIT  0x00000010      /* says setting up workq       */
#define SHA_ENTER_QMODE 0x00000020      /* says entering queue mode    */
#define SHA_CQE_AVAIL   0x00000040      /* says need a CQE             */

/* Errors understood in detail by the bus layer */

#define PORT_NOT_INSTALLED    0x12
#define SCSI_XFERCOUNT_WRONG  0x34

/* random definitions */

#define INTVEC  0xFE                /*$$$ use vector FE            */
#define INTLVL  6
#define INTERRUPT_WHEN_HALFEMPTY (0xC000 | (INTLVL<<8) | INTVEC)
#define SHA_SELECT_TIMEOUT 0x30

/* Butterfly-2 specific definitions */

#ifdef BFLY2
#define B2VME_SLAVE_MAPPER_OFFSET 0x400000
#endif

#endif _SHA_H
