/* $Header: sscsi.h,v 1.3 89/09/12 16:00:14 rgoguen Exp $ */

#ifndef _SSCSI_H
#define _SSCSI_H
#ifndef ASSEMBLER

/*****************************************************************************
File:     sscsi.h
Contents: Definitions for VME Synchronous SCSI Bus-layer interface
System:   MACH 1000

Copyright 1988 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

#define SSCSI_MAXCMD	32

struct sscsi_request
{
    int reqflags; /* Request flags 			*/
    struct buf     *bp;		/* Buffer, may be NULL 		*/
    unsigned        virtaddr;	/* Virt address, may be NULL*/
    unsigned        physaddr;	/* Phys address, may be NULL*/
    unsigned        physaddr2;	/* page boundary case       */
    unsigned        copybuffer;	/* copy buffer structure ptr*/
    int		    lognode; /* Logical node w/ device   */
    short	    busid; /* bus number on that node  */
    short	    deviceid; /* device number on that bus*/
    int             (*done_routine)(); /* Completion routine		*/
    int		    statflags; /* Status flags, eg DONE    */
    int		    cmdlen;	/* Length of command        */
    unsigned char   command [SSCSI_MAXCMD]; /* command                  */
    int             transfer_count; /* number of bytes xferred  */
    short           device_retval; /* SCSI status byte         */
    short           bus_retval; /* VME / host adapt status  */
    char sc ;                   /* sense code from a device */
    char asc ;                  /* additional sense code */
};

typedef struct sscsi_request *sscsi_request_t;
typedef struct sscsi_request SSCSI_REQUEST;

#define MAXSSCSI_REQUEST  12

/* Definitions for the reqflags member */

#define RF_INPUT		    0x00000001
#define RF_OUTPUT		    0x00000002
#define RF_HIPRI		    0x00000004
#define RF_DEVRESET         0x00000008
#define RF_BUSRESET         0x00000010
#define RF_COPYBUF          0x00000020
#define RF_COUNT_IMPRECISE  0x00000040
#define RF_TIMEOUT_EXPECTED 0x00000080
#define RF_ASYNC            0x00000100

/* Definitions for the statflags member 			*/
/* All accesses require that user hold the lock   	*/

#define SF_DONE			0x00000001
#define SF_WAKEUP		0x00000002
#define SF_ERROR        0x00000004
/* Generic stuff for a user to send SCSI commands directly to the device. */

#define SCSI_IOC _IOWR(s, 0, SCSIRAWCMD)

struct sdioc
{
    int        flags;
    char      *cmd;
    int        cmdlen;
    char      *buffer;
    int        xfercount;
    short	   bus_retval;
    short      device_retval;
    char sc,asc ;
} ;

#define SCSI_IOC_NODATA    0   /* no data xfer phase       */
#define SCSI_IOC_READ      1   /* xfer device to processor */
#define SCSI_IOC_WRITE     2   /* xfer processor to device */
#define SCSI_IOC_FUNCTION_MASK 0x3

#define SCSI_IOC_IMPRECISE 4   /* xfer count imprecise     */

typedef struct scsi_inquirydata SCSI_INQUIRY ;
typedef struct sdioc SCSIRAWCMD ;
#endif ASSEMBLER

#define DEVICES_PER_BUS 7
#define BUSSES_PER_ADAPTER 2
#define ADAPTERS_PER_NODE   4

#endif _SSCSI_H
