/*****************************************************************************
* File: smtdriver.h - Tandberg Cartridge Tape definitions                    *
*****************************************************************************/

/* drive mode setup templates.  Please refer to the tapedrive manual		*/

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res0:5;
	unsigned char res1;
	unsigned char res2;
	unsigned char paramlen;
	unsigned res3:1;
	unsigned smp:1;
	unsigned res4:4;
	unsigned flag:1;
	unsigned link:1;
	} MODE1;

#define MODE1_NBYTES 6

typedef struct
	{
	unsigned char res0;
	unsigned char res1;
	unsigned res2:3;
	unsigned bufmode:1;
	unsigned speed:4;
	unsigned char blkdesclen;
	unsigned char density;
	unsigned char res3;
	unsigned char res4;
	unsigned char res5;
	unsigned char res6;
	unsigned char res7;
	unsigned char blksizmsb;
	unsigned char blksizlsb;
	unsigned char vendpagecode;
	unsigned char writethresh;
	unsigned char readthresh;
	unsigned char bufsiz;
	unsigned char fscountmsb;
	unsigned char fscountlsb;
	unsigned char busthresh;
	unsigned char copythresh;
	unsigned char nsthresh;
	unsigned char csthresh;
	unsigned char loadfcn;
	unsigned char powupdelay;
	} MODE2;

#define MODE2_NBYTES 24

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:4;
	unsigned immed:1;
	unsigned char res2;
	unsigned char res3;
	unsigned res4:6;
	unsigned retension:1;
	unsigned load:1;
	unsigned eot:1;
	unsigned res5:5;
	unsigned flag:1;
	unsigned link:1;
	} LOADCMD;

#define LOADCMD_NBYTES 6

typedef struct 
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:4;
	unsigned immed:1;
	unsigned char nmarksmsb;
	unsigned char nmarksisb;
	unsigned char nmarkslsb;
	unsigned res2:6;
	unsigned flag:1;
	unsigned link:1;
	} FILEMARKCMD;

#define FILEMARKCMD_NBYTES 6

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:3;
	unsigned code:2;
	unsigned char countmsb;
	unsigned char countisb;
	unsigned char countlsb;	
	unsigned res2:6;
	unsigned flag:1;
	unsigned link:1;
	} SPACECMD;

#define SPACECMD_NBYTES 6

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:4;
	unsigned setme:1;
	unsigned char res2;
	unsigned char res3;
	unsigned char res4;
	unsigned res5:6;
	unsigned flag:1;
	unsigned link:1;
	} ERASECMD;			

#define ERASECMD_NBYTES 6

#define READ_COMMAND	0x08
#define WRITE_COMMAND   0x0A

#define SPACE_BLOCKS	0x00
#define SPACE_FILES		0x01
#define SPACE_EOT		0x03

#define SENSE_EOF       0x80
#define SENSE_EOT		0x40


/* Unit descriptor block */
struct smt
    {
    int     ident;
    int     flags;
	simple_lock_data_t lock;
    };

typedef struct smt SMT;

#define TDC_SCSI_IDENT 0x12345679

/* tape driver state definitions	*/

#define SMT_OPEN		0x01	/* bit says device is open				*/
#define SMT_NO_REWIND	0x02	/* bit says close doesn't rewind tape	*/
#define SMT_LOADED		0x04	/* bit says tape is loaded				*/
#define SMT_EOF			0x08	/* bit says tape at EOF					*/
#define SMT_DRIVE_THROOTE 0x10  /* bit says drive throttled             */

/* Minor device number mappings:
   low-order 3 bits: SCSI bus ID
         next 1 bit: no-rewind bit
         next 1 bit: Bus Number
        high 3 bits: Adapter number (hardly need 7, I hope!)
       
*/

#define dev_to_device(d)((d)&0x07)
#define no_rewind(d) (((d)>>3)&0x01)
#define dev_to_bus(d) (((d)>>4)&0x01)
#define dev_to_adapter(d)(((d)>>5)&0x07)

#define SENSE_BYTE_COUNT 32

#define SCSI_COMMAND_INQUIRY 0x12
#define SCSI_COMMAND_READ  0x08
#define SCSI_COMMAND_WRITE 0x0A
#define SCSI_COMMAND_SENSE 0x03

#define SCSI_CHECK 2   /* returned once after door opened */

#define BLOCK_SIZE_IN_BYTES 512
