# ifndef us_h
# define us_h yes

/*
 * Header file to be included with all Uniform System programs.
 */

/*
 * Uniform System programs should be compiled with one of the following defined:
 *
 *	FOR_CHRYSALIS	Multiprocessor version for Chrysalis OS on BPN2
 *			and BF_PLUS hardware.
 *
 *	FOR_UNIPROC_UNIX
 *			Single processor version for Berkeley Unix OS.
 *
 *	BFLY1		Multiprocessor version for Butterfly Mach OS on
 *			Butterfly 1 (BF_PLUS) hardware (symbol defined
 *			by GP1000 cc).
 *
 *	BFLY2		Multiprocessor version for Butterfly Mach OS on
 *			Butterfly 2 hardware (symbol defined by Butterfly2 cc).
 */

# ifdef BFLY2
# ifndef PSOS
# define FOR_MACH
# endif PSOS
# endif BFLY2

/*# ifndef HAVE_ATOM_OPS
# define HAVE_ATOM_OPS
# endif*/

# ifdef butterfly
/*
 * Butterfly specific stuff.
 */

extern int UsProc_Node;

# ifdef FOR_CHRYSALIS
# include <public.h>
# endif

# ifdef FOR_UNIPROC_UNIX
extern int Proc_Node;
extern unsigned rtc;
# endif

# ifdef BFLY1
# ifdef FOR_MACH
extern int Proc_Node;
extern int King;
#define rtc (getrtc())
#define Atomic_add_long(loc,val) (atomadd32(loc,val))
# else !FOR_MACH
# define Atomic_add_long(loc,val) \
    Atomic_add((unsigned short *) (loc), \
	       ((unsigned short) Atomic_add(((unsigned short *) (loc))+1,val)+val)>>16)
# endif !FOR_MACH
# endif BFLY1

# ifdef BFLY2
extern int Proc_Node;
extern int King;
# ifdef PSOS
# define btransfer(a,b,c) memcpy(a,b,c)
# else
#define rtc (getrtc())
# endif
#define Atomic_add_long(loc,val) (atomadd32(loc,val))
# endif BFLY2

# else !butterfly
/*
 * Stuff specific to front end Uniform System emulator.
 */

# define Proc_Node  5
# define UsProc_Node 0
# define rtc GetRtc ()
# define Atomic_add_long(loc,val)  *loc = *loc + val
# define Where_Is(A) Proc_Node
# define TRUE 1
# define FALSE 0
# define NULL 0
# define SECOND 1
# define Sleep sleep
# endif butterfly

# define OnNode(A) (Where_Is(A)  ==  Proc_Node)

# ifdef BFLY1
# define LOCK(p,n) \
    while (Atomic_ior((short *) p, 1)  !=  0) LockWait(n)

# define UNLOCK(p) \
    {	register int us_qzp = 0; \
	*((short *) p) = us_qzp; \
    }

# define UsLock(p,n) \
    while (Atomic_ior((short *) p, 1)  !=  0) LockWait(n)

# define UsUnLock(p) \
    {	register int us_qzp = 0; \
	*((short *) p) = us_qzp; \
    }

# define UsUnlock(p) \
    {	register int us_qzp = 0; \
	*((short *) p) = us_qzp; \
    }
# endif BFLY1

# ifdef BFLY2
# ifdef HAVE_ATOM_OPS
# define LOCK(p,n) \
    while (atomior32((unsigned long *) p, 1)  !=  0) LockWait(n)

# define UNLOCK(p) \
    {	register int us_qzp = 0; \
	*((unsigned long *) p) = us_qzp; \
    }

# define UsLock(p,n) \
    while (atomior32((unsigned long *) p, 1)  !=  0) LockWait(n)

# define UsUnLock(p) \
    {	register int us_qzp = 0; \
	*((unsigned long *) p) = us_qzp; \
    }

# define UsUnlock(p) \
    {	register int us_qzp = 0; \
	*((unsigned long *) p) = us_qzp; \
    }
# else
# define MUTEXTAG	0xff000000
char xmemc ();

# define LOCK(L,n) \
    while (MUTEXTAG == xmemul ((unsigned long *) (L), MUTEXTAG)) \
	LockWait (n);

# define UNLOCK(L) \
    * (L) = 0;

# define UsLock(L,n) \
    while (MUTEXTAG == xmemul ((unsigned long *) (L), MUTEXTAG)) \
	LockWait (n);

# define UsUnLock(L) \
   * (L) = 0;

# define UsUnlock(L) \
   * (L) = 0;
# endif HAVE_ATOM_OPS
# endif BFLY2

# define BEGIN_SHARED_DECL \
typedef struct \
    {

# define END_SHARED_DECL \
    } _share_struct_; \
_share_struct_ * _share_ptr_

# define SHARED _share_ptr_->

# define MakeSharedVariables \
    _share_ptr_ = (_share_struct_ *) Allocate (sizeof(_share_struct_)); \
    Share (&_share_ptr_);

/*
 * Generator descriptor data structure.
 * Take care to ensure that for BF2 us_lock and started are long word aligned.
 */

# define GENERATOR 4

typedef struct UsGenDescStruct
{   int		id;			/* Generator id */
    struct UsGenDescStruct * prev_ptr;	/* Hash table chain ptr */
    struct UsGenDescStruct * next_ptr;	/*         "            */
    struct UsGenDescStruct * td_next_hint;
# ifdef BFLY1
    short	us_lock;		/* Non-exclusive lock. */
    short	in_hash_table;		/* Flag, structure in active gen table. */
# endif BFLY1
# ifdef BFLY2
    unsigned long us_lock;		/* Non-exclusive lock. */
    unsigned long in_hash_table;	/* Flag, structure in active gen table. */
# endif BFLY2
    short *	completion_location;	/* Location to write to signal all */
					/* processors finished */
					/* MAKE INT for BFly Plus */
    char *	currentShare;
    int		end;
    short	td_index;		/* if -1, means structure not scattered */
					/* if >=0, scattered and is index into */
					/* td_array: defines scatter mapping */
    short	retcode;
# ifdef BFLY1
    short	started;		/* # procs currently working on this gen */
    short	endlock;
# endif BFLY1
# ifdef BFLY2
    int		started;		/* # procs currently working on this gen */
    unsigned long endlock;
# endif BFLY2
# define genEXHAUSTED -1
    int		(* init)();
    int		(* gen)();
    int		(* final)();
    int		arg;
    int		(* call)();
    int		range;
    int		range2;
    int		abortable;
    short	max_procs_to_use;
    short	language;
# define C_CALLING 0
# define FORTRAN_CALLING 1
    union
    {	long	Long;
	short Short;
    }		index;
    union
    {	unsigned long Long;
	unsigned short Short;
    }		index2;
# ifdef BFLY1
    short	lock;
# endif
# ifdef BFLY2
    unsigned long lock;
# endif
} UsGenDesc;

typedef UsGenDesc UsOneTask;

# define configProcs			1
# define configMaxSars			2
# define configSuppressInitMsgs		3
# define configTotalSars		4
# define configMaxMemObjs		5
# define configMemObjsFree		6
# define configStopOnShareFail		7
# define configWarningOnShareFail 	8
# define configAllocAcross64K		9
# define configTimeTestViaReinit 	10
# define configObjsRetRoot		11
# define configObjsRetChild		12
# define configSharedMemoryBaseAddress 	13
# define configUsSignals		14
# define configUsCachePtcl		15
# define configUsUnCachedSize 		16
# define configUsUncachedSize 		configUsUnCachedSize
# define configUsCopyBackSize 		17
# define configUsWriteThruSize 		18
# define configUsILSize 		19
# define configUsILWriteThruSize 	20
# define configUsILCopyBackSize 	21
# define configUsUseAllNodesInPublicCluster	22

# define configUsCacheCopyBack	1
# define configUsCacheWriteThru	2

# define UsUncached		0
# define UsUnCached   		UsUncached
# define UsCacheCopyBack	1
# define UsCacheWriteThru	2
# define UsInterleavedUncached	3
# define UsInterleavedUnCached	UsInterleavedUncached
# define UsInterleavedCopyBack	4
# define UsInterleavedWriteThru	5

#define UsSMFreePtrs		1
#define UsSMFreePtrsAndData	2

extern double UsWaitFactor;
extern double UsWaitGetFactor();

extern UsGenDesc * ActivateGen ();
extern char * UsAllocAndReportC ();
extern char * UsAllocOnUsProc ();
extern char * UsAllocOnUsProcC ();
extern char * UsAllocOnPhysProc ();
extern char * UsAlloc ();
extern char * UsAllocC ();
extern char * UsAllocLocal ();
extern char * * UsAllocScatterMatrix ();
extern char * * UsAllocScatterMatrixC ();

extern UsGenDesc * AsyncGenOnI ();
extern UsGenDesc * AsyncGenOnILimited ();
extern UsGenDesc * AsyncGenOnIAbortable ();
extern UsGenDesc * AsyncGenOnIFull ();
extern UsGenDesc * AsyncGenOnA ();
extern UsGenDesc * AsyncGenOnALimited ();
extern UsGenDesc * AsyncGenOnAAbortable ();
extern UsGenDesc * AsyncGenOnAFull ();
extern UsGenDesc * AsyncGenOnHA ();
extern UsGenDesc * AsyncGenOnHALimited ();
extern UsGenDesc * AsyncGenOnHAAbortable ();
extern UsGenDesc * AsyncGenOnHAFull ();

extern int TimeTestPrint ();

# undef FOR_MACH
# endif us_h

# ifdef BFLY2
extern char * us_alloc ();
extern char * us_alloc_local ();
extern char * us_alloc_on_us_proc ();
extern char * us_alloc_on_phys_proc ();
extern char * us_alloc_on_us_proc_c ();
extern char * us_alloc_and_report_c ();
extern char * us_alloc_c ();
extern char * * us_alloc_scatter_matrix ();
extern char * * us_alloc_scatter_matrix_c ();
extern char * us_alloc_pages_on_us_proc ();
# endif BFLY2
# ifdef BFLY1
# define us_alloc(type,bytes) UsAlloc(bytes)

# define us_alloc_local(type,bytes) UsAllocLocal(bytes)

# define us_alloc_on_us_proc(type,usproc,nbytes)  UsAllocOnUsProc(usproc,nbytes)

# define us_alloc_on_phys_proc(type, physproc,nbytes) \
    UsAllocOnPhysProc(physproc,nbytes)

# define us_alloc_on_us_proc_c(type,usproc,nbytes,class) \
    UsAllocOnUsProcC(usproc,nbytes,class)

# define us_alloc_and_report_c(type,usproc,wherep,nbytes,class) \
    UsAllocAndReportC(usproc,wherep,nbytes,class)

# define us_alloc_c(type,nbytes,class) \
    UsAllocC(nbytes,class)

# define us_alloc_scatter_matrix(type,rows,cols,nbytes) \
    UsAllocScatterMatrix(rows,cols,nbytes)

# define us_alloc_scatter_matrix_c(type,rows,cols,nbytes,class) \
    UsAllocScatterMatrixC(rows,cols,nbytes,class)
# endif BFLY1

