/* $Header: locore.doc.h,v 1.3 89/02/25 01:41:50 gibson Exp $ */
/*
 * Description of locore.
 */

/*
 * Brief map of locore.s:
 *	All locore.s is divided into four parts, appearing in this order.
 *
 *	1. exception handling
 *	2. bootstrap code
 *	3. relocatable code templates
 *	4. utilities
 *
 * 1. Exception handling is described in the 68000 manual.  Software
 *	gets into the act through the vector table.  This table, built
 *	in scb.s, points to routines (SCBVEC's) in locore.s. locore.s
 *	lists the SCBVEC's in the order in which they appear in the
 *	vector table.
 *   We identify 4 types of exceptions:
 *	a. Traps:
 *		These are synchronous (unlike interrupts) and are 
 *	handled by "trapcommon".  They get there through the TRAP(trapnum)
 *	macro.
 *	b. Syscalls:
 *		This is a specific trap, TRAPT (TRAPcc), used for system
 *	calls.  SCBVEC(syscall) does the processing.
 *      c. Interrupts:
 *		These SCBVEC's typically save the "scratch registers"
 *	(a0,a1,d0,d1) and call the device-specific routines.  (Other registers
 *	do not need to be saved because the compiler preserves
 *	them across procedure calls.)
 *	d. Interrupt level 1 (ilevel_1):
 *		This is an important special case of (c).  It is
 *	used for the clock stuff/callout queue, the network, and kdb.
 *	In the latter two cases, an interrupt handler running at the
 *	(high) priority of the device provokes a software interrupt
 *	so that a long, complicated job driven by an asynchronous event
 *	(e.g. checksumming and demultiplexing newly arrived packets)
 *	doesn't lock out devices.
 *	e. Asynchronous system traps, used, e.g. for profiling, are a
 *	special case.  They are polled for on return to user mode from
 *	traps, system calls, and level 1 interrupts (but not device
 *	interrupts) and handled then.
 *
 * 2. Bootstrap code
 *	The system starts running at _start and carries out various
 *	initialization tasks either directly or through C subroutine
 *	calls.  It runs on the interrupt stack until near the end.
 *	It terminates by returning to user space location 0, to
 *	run the code it has set up.
 *
 * 3. Relocatable code templates:
 *	a. init, mach_init:
 *		These are copied into location 0 to become the first
 *	real user processes.  Roughly, their job is to execv /etc/init
 *	so we finally get to run real, compiler-generated, non-kernel C code.
 *	b. sigcode:
 *		This is copied into every process's u_area at process
 *	creation time for handling signals.  (More explanation needed.)
 *
 * 4. Utilities:
 *	Random routines written in assembly, probably inherently machine-
 *	dependent, and probably called throughout the kernel.
 *	For example, 1) the copyin/copyout routines to shift data between
 *	user process address space and the kernel, and
 *	2) save_context/load_context
 */
