/* $Header: /nfs/columbia/u1/BFMACH.local/mk.2.9.6.gibson/bfly2/RCS/mmu.h,v 1.36 91/02/15 14:53:46 gibson Exp $ */

#ifndef _MMU_
#define	_MMU_


#ifdef KERNEL
#include	"../butterfly/vm_param.h"
#else KERNEL
#include	<butterfly/vm_param.h>
#endif KERNEL



#define	CMMU_IDR		0x000
#define	CMMU_SCMR		0x004
#define	CMMU_SSR		0x008
#define	CMMU_SADDR		0x00C
#define	CMMU_SCTR		0x104

#define	CMMU_LSR		0x108
#define	CMMU_LADR		0x10C

#define	CMMU_SAPR		0x200
#define	CMMU_UAPR		0x204

#define	CMMU_BWP0		0x400
#define	CMMU_BWP1		0x404
#define	CMMU_BWP2		0x408
#define	CMMU_BWP3		0x40C
#define	CMMU_BWP4		0x410
#define	CMMU_BWP5		0x414
#define	CMMU_BWP6		0x418
#define	CMMU_BWP7		0x41C

#define	CMMU_CDP0		0x800
#define	CMMU_CDP1		0x804
#define	CMMU_CDP2		0x808
#define	CMMU_CDP3		0x80C

#define	CMMU_CTP0		0x840
#define	CMMU_CTP1		0x844
#define	CMMU_CTP2		0x848
#define	CMMU_CTP3		0x84C

#define	CMMU_CSSP		0x880
#define	CMMU_CSSPG		0x884

	/* Initialize all 256  lines a) set is invalid, b) lru=1, c) kill=0 */
#define	CMMU_INIT_CSSP	0x3F0FF000

/* Id's for various MMU's */
#define	MAX_CMMU_ID		0xFF
#define	D_CMMU_ID		0x7E
#define	C_CMMU_ID		0x7F
#define	C_CMMU_ID2		0x7D

#ifndef ASSEMBLER
/* Corresponding addresses for MMU's */
#define	DATA_MMU		CMMU_ADDR(D_CMMU_ID)	/* 0xFFF7E000 */
#define	CODE_MMU		CMMU_ADDR(C_CMMU_ID)	/* 0xFFF7F000 */
#define	CODE_MMU2		CMMU_ADDR(C_CMMU_ID2)	/* 0xFFF7D000 */

/* vaddr for CMMU register space and macro for calculating addr of
 * CMMU with given id
 */
#define	LOCAL_IO		0xFFF00000		/* vaddrs for CMMU's */
#define	CMMU_ADDR(id)	(LOCAL_IO + ((id) << 12)) 	/* vaddr for CMMU id */

#define	MMU_REG(mmu,reg)	(*((unsigned *)((mmu) + (reg))))

#define	MMU_IDR(mmu)		MMU_REG(mmu, CMMU_IDR)
#define	MMU_SCMR(mmu)		MMU_REG(mmu,CMMU_SCMR)
#define	MMU_SSR(mmu)		MMU_REG(mmu, CMMU_SSR)
#define	MMU_SADDR(mmu)		MMU_REG(mmu, CMMU_SADDR)
#define	MMU_SCTR(mmu)		MMU_REG(mmu, CMMU_SCTR)
#define	MMU_LSR(mmu)		MMU_REG(mmu, CMMU_LSR)
#define	MMU_LADR(mmu)		MMU_REG(mmu, CMMU_LADR)
#define	MMU_SAPR(mmu)		MMU_REG(mmu, CMMU_SAPR)
#define	MMU_UAPR(mmu)		MMU_REG(mmu, CMMU_UAPR)
#define	MMU_BATC(mmu, batc)	MMU_REG(mmu, CMMU_BWP0 + (4*batc))
#define	MMU_CSSP(mmu)		MMU_REG(mmu, CMMU_CSSP)
#define	MMU_CSSPG(mmu)		MMU_REG(mmu, CMMU_CSSPG)

typedef struct cmmu_lsr {
	unsigned int
		lsr_reserved1: 13,
		lsr_faultcode: 3,
		lsr_reserved2: 16;
} cmmu_lsr;

#define	CMMU_BUSERR	0x03
#define	CMMU_SEGFLT	0x04
#define	CMMU_PGFLT	0x05
#define	CMMU_SUPVIOL	0x06
#define	CMMU_WRVIOL	0x07
 
typedef struct cmmu_idr {
	unsigned int
		idr_id: 8,
		idr_type: 3,
		idr_version: 5,
		idr_reserved: 16;
} cmmu_idr;

/* Constituent parts of cmmu id register */
#define	CMMU_IDR_ID_SHIFT	24
#define	CMMU_IDR_TYPE_SHIFT	21
#define	CMMU_IDR_VERSION_SHIFT	16

#define	CMMU_IDR_ID_MASK	0xFF000000
#define	CMMU_IDR_TYPE_MASK	0x00E00000
#define	CMMU_IDR_VERSION_MASK	0x001F0000

#define	CMMU_IDR_ID(mmu_idr)	\
  ((((unsigned)(mmu_idr)) & CMMU_IDR_ID_MASK) >> CMMU_IDR_ID_SHIFT)
#define	CMMU_IDR_TYPE(mmu_idr)	\
  ((((unsigned)(mmu_idr)) & CMMU_IDR_TYPE_MASK) >> CMMU_IDR_TYPE_SHIFT)
#define	CMMU_IDR_VERSION(mmu_idr)	\
  ((((unsigned)(mmu_idr)) & CMMU_IDR_VERSION_MASK) >> CMMU_IDR_VERSION_SHIFT)

#define	MC88200_TYPE		0x5
#define	MC88204_TYPE		0x6

#define	CMMU_IDR_ISVALID(mmu_idr)		\
  ((CMMU_IDR_TYPE(mmu_idr) == MC88200_TYPE) ||	\
   (CMMU_IDR_TYPE(mmu_idr) == MC88204_TYPE))

#define	CMMU_IS_88204(mmu)	(CMMU_IDR_TYPE(MMU_IDR(mmu)) == MC88204_TYPE)

#define	CMMU_VERSION		0x3		/* current version (unused) */

typedef struct cmmu_scmr {
	unsigned int
		scmr_reserved: 26,
		scmr_cmd: 4,
		scmr_arg: 2;
} cmmu_scmr;


/*****************************************************************************
* CMMU commands 
******************************************************************************/

#define	CMMU_NOOP		0x00

/* CMMU data cache commands */
#define	CMMU_INVAL_DCACHE		0x14
#define	CMMU_COPYBACK_DCACHE		0x18
#define	CMMU_COPYBACK_AND_INVAL_DCACHE	0x1C

#ifdef KERNEL
/* For user cache flushing operations */
#define	CMMU_CMD_FLUSH_BIT		0x10
#define	CMMU_CMD_INVALIDATE_BIT		0x04
#define	CMMU_CMD_COPYBACK_BIT		0x08
#endif KERNEL

/* CMMU virtual-to-physical commands */
#define	CMMU_PROBE_UADDR	0x20
#define	CMMU_PROBE_SADDR	0x24

/* CMMU address translation cache commands */
#define	CMMU_INVAL_UPATC	0x30
#define	CMMU_INVAL_SPATC	0x34

/* CMMU commands: Granularity */
#define	CMMU_C_LINE		0x00
#define	CMMU_C_PAGE		0x01
#define	CMMU_C_SEG		0x02
#define	CMMU_C_ALL		0x03

#ifdef KERNEL
#define	CMMU_LINE_SIZE		0x10		/* 16 bytes */
#define	CMMU_PAGE_SIZE		MMU_PGBYTES	/* 4 K bytes */
#define	CMMU_SEG_SIZE		0x400000	/* 4 M bytes */
#define	CMMU_BATC_SIZE		(512*1024)	/* 512 K bytes */
#define	batc_offset(addr)	((addr) & (CMMU_BATC_SIZE - 1))
#endif KERNEL

typedef struct cmmu_ssr {
	unsigned int
		ssr_reserved1: 16,
		/*
		ssr_ce: 1,
		ssr_be: 1,
		ssr_reserved2: 4,
		*/
		ssr_probe_res: 16;
		/*
		ssr_wt: 1,
		ssr_sp: 1,
		ssr_g: 1,
		ssr_ci: 1,
		ssr_r: 1,
		ssr_m: 1,
		ssr_u: 1,
		ssr_wp: 1,
		ssr_bh: 1,
		ssr_v: 1;
		*/
} cmmu_ssr;

#define	CMMU_SSR_CE	0x8000
#define	CMMU_SSR_BE	0x4000
#define	CMMU_SSR_WT	0x0200
#define	CMMU_SSR_SP	0x0100
#define	CMMU_SSR_G	0x0080
#define	CMMU_SSR_CI	0x0040
#define	CMMU_SSR_R	0x0020
#define	CMMU_SSR_M	0x0010
#define	CMMU_SSR_U	0x0008
#define	CMMU_SSR_WP	0x0004
#define	CMMU_SSR_BH	0x0002
#define	CMMU_SSR_V	0x0001

/* MMU bits that are important when doing a probe of a user address */
#define	CMMU_USRPROBEBITS	\
	       	       (CMMU_SSR_CE|CMMU_SSR_BE|CMMU_SSR_SP|CMMU_SSR_G| \
			CMMU_SSR_WP|CMMU_SSR_BH|CMMU_SSR_V)

typedef	struct cmmu_sctr {
	unsigned int
		sctr_reserved1: 16,
		sctr_status: 16;
		/*
		sctr_pe: 1,
		sctr_se: 1,
		sctr_pr: 1,
		sctr_reserved2: 13;
		*/
} cmmu_sctr;


#define	CMMU_SCTR_PE	0x8000
#define	CMMU_SCTR_SE	0x4000
#define	CMMU_SCTR_PR	0x2000

typedef struct cmmu_cssp {
	unsigned int
		/*
		cssp_state: 32;
		*/
		cssp_reserved1: 2,
		cssp_L5: 1,
		cssp_L4: 1,
		cssp_L3: 1,
		cssp_L2: 1,
		cssp_L1: 1,
		cssp_L0: 1,
		cssp_D3: 1,
		cssp_D2: 1,
		cssp_D1: 1,
		cssp_D0: 1,
		cssp_vv3: 2,
		cssp_vv2: 2,
		cssp_vv1: 2,
		cssp_vv0: 2,
		cssp_reserved2: 12;
} cmmu_cssp;


struct cmmu_area_pointer {
	unsigned int
		apr_ptr: 20,
#define	apr_descr	type
		type: 12;
		/*
		apr_wt: 1,
		apr_reserved1: 3,
		apr_g: 1,
		apr_ci: 1,
		apr_reserved2: 5,
		apr_te: 1;
		*/
};

#define	CMMU_RP_WT	0x0200
#define	CMMU_RP_G	0x0080
#define	CMMU_RP_CI	0x0040
#define	CMMU_RP_TE	0x0001

#define	CMMU_STD_RP_BITS	CMMU_RP_TE

struct cmmu_table_descriptor {
	unsigned int
		ste_phys: 20,
		ste_status: 12;
		/*
		ste_reserved1: 2,
		ste_wt: 1,
		ste_sp: 1,
		ste_g: 1,
		ste_ci: 1,
		ste_reserved2: 3,
		ste_wp: 1,
		ste_reserved3: 1,
		ste_v: 1;
		*/
};

#define	CMMU_SEG_WT	0x0200
#define	CMMU_SEG_SP	0x0100
#define	CMMU_SEG_G	0x0080
#define	CMMU_SEG_CI	0x0040
#define	CMMU_SEG_WP	0x0004
#define	CMMU_SEG_V	0x0001

#define	CMMU_STE_NONCACHABLE_BITS	(CMMU_SEG_WT|CMMU_SEG_CI|CMMU_SEG_V)
#define	CMMU_STE_CACHABLE_BITS		(CMMU_SEG_WT|CMMU_SEG_V)
#define	CMMU_STE_COPYBACK_CACHABLE_BITS	CMMU_SEG_V


struct cmmu_page_descriptor {
	union {
		struct {
			unsigned int
		      		_pte_pfn: 20,
		      		_pte_status: 12;
		} pte_bits;
		struct pte_fields {
			unsigned int
		      		_pte_pfn: 20,
		      		_pte_reserved1: 2,
		      		_pte_wt: 1,
		      		_pte_sp: 1,
		      		_pte_g: 1,
		      		_pte_ci: 1,
		      		_pte_reserved2: 1,
		      		_pte_m: 1,
		      		_pte_u: 1,
		      		_pte_wp: 1,
		      		_pte_wired: 1,
		      		_pte_v: 1;
		} pte_fields;
	} _pte;
};

#define	pte_pfn		_pte.pte_bits._pte_pfn
#define	pte_status	_pte.pte_bits._pte_status
#define	pte_wt		_pte.pte_fields._pte_wt
#define	pte_sp		_pte.pte_fields._pte_sp
#define	pte_wired	_pte.pte_fields._pte_wired
#define	pte_g		_pte.pte_fields._pte_g
#define	pte_ci		_pte.pte_fields._pte_ci
#define	pte_m		_pte.pte_fields._pte_m
#define	pte_u		_pte.pte_fields._pte_u
#define	pte_wp		_pte.pte_fields._pte_wp
#define	pte_v		_pte.pte_fields._pte_v

#define	CMMU_PTE_WT	0x0200
#define	CMMU_PTE_S	0x0100
#define	CMMU_PTE_G	0x0080
#define	CMMU_PTE_CI	0x0040
#define	CMMU_PTE_M	0x0010
#define	CMMU_PTE_U	0x0008
#define	CMMU_PTE_WP	0x0004
#define	CMMU_PTE_RW	0x0000		/* mask for allowing writes */
#define	CMMU_PTE_V	0x0001

#define	CMMU_PTE_NONCACHABLE_BITS	(CMMU_PTE_WT|CMMU_PTE_CI|CMMU_PTE_V)
#define	CMMU_PTE_CACHABLE_BITS		(CMMU_PTE_WT|CMMU_PTE_V)
#define	CMMU_PTE_COPYBACK_CACHABLE_BITS	CMMU_PTE_V

	/* The bits in the pte that hold caching information */
#define	CMMU_PTE_CACHING_BITS		(CMMU_PTE_CI|CMMU_PTE_WT)

#define	CMMU_PTE_INVALID 0x0000		/* pte is not valid */

/* Values for table entry fields */

  /* pte->wprot */
#define CMMU_WP		1		/* write protected (kernel and user) */
#define CMMU_RW		0		/* writeable */

#define	CMMU_VALID	1		/* entry is valid */

typedef struct cmmu_batc {
	unsigned int
		batc_logaddr: 13,
		batc_physaddr: 13,
		batc_descr: 6;
		/*
		batc_s: 1,
		batc_wt: 1,
		batc_g: 1,
		batc_ci: 1,
		batc_wp: 1;
		*/
} cmmu_batc;

#define	CMMU_BATC_S	0x0020
#define	CMMU_BATC_WT	0x0010
#define	CMMU_BATC_G	0x0008
#define	CMMU_BATC_CI	0x0004
#define	CMMU_BATC_WP	0x0002
#define	CMMU_BATC_V	0x0001



typedef struct cmmu_area_pointer root_ptr_t;         /* area register entry */
typedef struct cmmu_table_descriptor st_entry_t;       /* seg table entry */
typedef struct cmmu_page_descriptor mmu_pt_entry_t;    /* page table entry */
               /* seg table entry for init world */

typedef struct cmmu_table_descriptor init_st_entry_t;

#endif ASSEMBLER

#define	PT_PTR	0	/* offset in ste of ste->pt_ptr */
#define	PGTAB_NULL	0xFFFFF000

#ifndef ASSEMBLER

/*****************************************************************************
* Export to pmap layer:
*	Definitions begin with MMU, not PMMU.
*	Exported by bfly[12]/mmu.h.
******************************************************************************/

/* Type of segment table.  (Appears in root pointer register.)
 * 
 * Allow caching, copyback as well as writethrough, and turn it off
 * area by area or in tables themselves.
 */
#define	MMU_SEGTAB_TYPE		CMMU_STD_RP_BITS

#define	MMU_SEGTABSIZE	MMU_PGBYTES

/* Type of page table pages.  (Appears in segment table entries.) */
#define	MMU_PGTAB_TYPE			CMMU_STE_NONCACHABLE_BITS
#define	MMU_PGTAB_CACHABLE_TYPE		CMMU_STE_CACHABLE_BITS
#define	MMU_PGTAB_CB_CACHABLE_TYPE	CMMU_STE_COPYBACK_CACHABLE_BITS

#define	MMU_PGTABSIZE	MMU_PGBYTES

/* Types of pages.  (Appears in page table entries) */
#define	MMU_PG_TYPE		CMMU_PTE_NONCACHABLE_BITS
#define	MMU_PG_CACHABLE_TYPE	CMMU_PTE_CACHABLE_BITS
#define	MMU_PG_CB_CACHABLE_TYPE	CMMU_PTE_COPYBACK_CACHABLE_BITS

#define	MMU_PG_INVALID		CMMU_PTE_INVALID

/* More pte bits */
#define	MMU_PTE_RW	CMMU_PTE_RW
#define	MMU_PTE_WP	CMMU_PTE_WP

#define	MMU_UNMAPPED_VADDR	0xFFFFFFFF

#define	MMU_PTE_ILLEGAL_PROT	(-1)
#endif ASSEMBLER

/*****************************************************************************
* Definitions describing different parts of a virtual address
******************************************************************************/
#define	MMU_PTABALIGN	4096	 /* page tables must be this aligned */

#define	MMU_SEGBITS	10		/* bits in virt address for segment */
#define	MMU_PGTABBITS	10		/* bits in virt address for page no */
#define MMU_PGBITS	12		/* bits in virt address for page off */


#define	MMU_PGSHIFT	MMU_PGBITS		/* shift between pages/bytes */
#define	MMU_PGBYTES	4096
#ifndef ASSEMBLER
#define	MMU_PGMASK	(MMU_PGBYTES-1) 	/* offset in page */

#define	MMU_SEGS	(1 << MMU_SEGBITS)	/* segments in address space */
#define MMU_SEGSHIFT  	(32-MMU_SEGBITS)	/* convert segs to bytes */
#define	MMU_SEGBYTES	(1 << MMU_SEGSHIFT)	/* bytes per segment */
#define	MMU_SEGMASK	(MMU_SEGBYTES-1) 	/* offset in segment */

#define	MMU_PGTABPAGES	(1 << MMU_PGTABBITS)	/* pages in a segment */
#define	MMU_PGTABMASK	\
  (MMU_SEGMASK & ~MMU_PGMASK)  	/* page offset within a segment */

#endif ASSEMBLER

#ifndef ASSEMBLER
#include	"../machine/paddr.h"

/*
 * Manipulate mmu_pt_entry_t's (mapping hardware pages).
 */
#define	mmupte_mapin(pte, paddr, ptype)				\
  (*((unsigned long *)(pte)) = (((unsigned long)(paddr)) | (ptype)))
#define	mmupte_invalidate(pte)	(*(unsigned long *)(pte) = 0)
#define	mmupte_setwritable(pte)  (*((unsigned long *) (pte)) &= ~CMMU_PTE_WP, \
				  *((unsigned long *) (pte)) |= CMMU_PTE_CI)
#define	mmupte_getphys(pte) \
  paddrnonbypassed(*((unsigned long *) (pte)) & ~VM_PAGEMASK)

#define	mmupte_writeprot(pte)		((pte)->pte_wp = CMMU_WP,	\
					 (pte)->pte_ci=0)
#define	mmupte_setwired(pte,wired)	((pte)->pte_wired = (wired))
#define	mmupte_wire(pte)		((pte)->pte_wired = 1)
#define	mmupte_setmodified(pte)		((pte)->pte_m = 1)
#define	mmupte_setclean(pte)		((pte)->pte_m = 0)
#define	mmupte_clearreferenced(pte)	((pte)->pte_u = 0)
#define	mmupte_addbits(pte, prot)	(*((unsigned long *) (pte)) |= (prot))
#define	mmupte_setcaching(pte, caching)	(*((unsigned long *)(pte)) =	\
					 (*((unsigned long *)(pte)) &	\
					  ~CMMU_PTE_CACHING_BITS) | caching)
#define	mmupte_set_noncached(pte)   (*((unsigned long *)(pte)) |= CMMU_PTE_CI)
/*#define	mmupte_setbits(pte, bits)	(*((unsigned long *) (pte)) = (bits))*/

#define	mmupte_iscached(pte)		(!(pte)->pte_ci)
#define	mmupte_iswritethru(pte)		((pte)->pte_wt)
#define	mmupte_isglobal(pte)		((pte)->pte_g)
#define	mmupte_isreferenced(pte)	((pte)->pte_u)
#define	mmupte_ismodified(pte)		((pte)->pte_m)
#define mmupte_iswired(pte)		((pte)->pte_wired)
#define	mmupte_iswriteprot(pte)		((pte)->pte_wp == CMMU_WP)
#define	mmupte_iswritable(pte)		((pte)->pte_wp == CMMU_RW)
#define	mmupte_isvalid(pte)		((pte)->pte_v == CMMU_VALID)

/*****************************************************************************
* Macros for manipulating segment table entries.
*   If half our address space is used for bypassed, we make
*	all segment table entries bypassed (i.e. they point at
*	page table pages using a bypassed address).
*   If not, the hardware make the page table walks bypassed for us.
******************************************************************************/

#define	ste_getphys(ste)	\
	(paddrnonbypassed(*((unsigned long *) (ste)) & ~MMU_PGMASK))

#define	ste_setphys(ste, paddr)	\
   (*((unsigned long *) (ste)) = \
    paddrsetbypassed(((unsigned)(paddr)) | (ste)->ste_status))
#define	ste_mapin(ste, paddr, stype)	\
   (*((unsigned *) (ste)) = paddrsetbypassed(((unsigned)(paddr)) | (stype)))

#define	ste_invalidate(ste)	(*((unsigned long *) (ste)) = 0)

/*****************************************************************************
* Macros for manipulating (copies of) area pointer register.
*	The area pointer, like the segment table entries, map the next
*	level of the page table bypassed, but that is done in
*	load_area_pointers().
******************************************************************************/

#define	rootptr_getphys(rptr)	 \
  paddrnonbypassed(*((unsigned long *)(rptr)) & ~MMU_PGMASK)
#define	rootptr_setphys(rptr,paddr) (*((unsigned long *)(rptr)) |= (paddr))
#define	rootptr_setup_template(rptr,paddr)	\
	(*((unsigned *)(rptr)) = (((unsigned)(paddr)) | CMMU_STD_RP_BITS))
#define	rootptr_setuncachable(rptr) (*((unsigned long *)(rptr)) |= CMMU_RP_CI)
#define	rootptr_setcachable(rptr) (*((unsigned long *)(rptr)) &= ~CMMU_RP_CI)

#endif	ASSEMBLER

#ifndef ASSEMBLER
#ifdef KERNEL
struct mmu_info {
	unsigned	physaddr;
	unsigned	mmu_bits;
	unsigned	physnode;
	unsigned	mmu_spare;
};
#endif KERNEL
#endif ASSEMBLER

/*****************************************************************************
* "INIT" world:
*
* These values represent the temporary state of the MMU as set up by
* init_map.c, before we enable the MMU in its final form in
* pmap_bootstrap and friends.
******************************************************************************/

#define	INIT_PGSHIFT		13
#define	INIT_MMU_PGBYTES	(1 << INIT_PGSHIFT)
#define	INIT_PGOFSET		(INIT_MMU_PGBYTES-1)
#define	INIT_SEGS		1024
#define	INIT_PTPAGES		512
	  /* map remote memory at 0xFCxxxxxx before pmap_bootstrap() */
#define	INIT_REMOTE_SPACE	0xFC

#ifndef ASSEMBLER

	/* We get to set up init world's mapping tables here */
	/* right below physical 4M bytes */
#define	init_seg_table		((init_st_entry_t *) 0xFE3F0000)

#define	init_ste_mapin(ste, paddr)	ste_mapin(ste, paddr)

/* Map a physical address through the given pte in the pre-pmap_bootstrap
 * world.
 */
#define	init_ste_setphys(ste, paddr)	\
		 (*((unsigned long *) (ste)) = \
		 initpaddrsetbypassed(((unsigned)(paddr)) | (ste)->ste_status))
#define	init_ste_invalidate(ste)	(*(unsigned long *)(ste) = 0)
#define	init_ste_setvalid(ste)		((ste)->ste_status |= CMMU_SEG_V)
#endif ASSEMBLER

	/*
	 *	Random defines that probably belong elsewhere.
	 */

  /* number of bytes of ste's required to map address space */
#define	SEGTAB_SIZE	VM_PAGESIZE
  /* number of bytes of pte's required to map a segment */
#define	PGTAB_SIZE	VM_PAGESIZE
  /* number of bytes of pte's required to map 16M. */
#define	SPACETAB_SIZE	(MMU_PGBYTES * BBN_SCLSIZE)
/* #define	SPACETAB_SIZE	((SPACEBYTES/VM_PAGESIZE) * sizeof(pt_entry_t))*/

/* The 13th bit (A12) distinguishes which MMU to use, with 0 */
/* directed to the original MMU and 1 directed to the new MMU. */
/* [A12 toggles on 4K (MMU page) boundaries.  This is what	*/
/* Motorola suggests for multiple MMU's in Section 7 	*/
/* (Applications Information), Section 1 (Multiple MC88200 */
/* Devices) Subsection 1 (Instruction Pbus Connections).] */
#define	CODE_MMU_DECISION_BIT	0x1000
#endif _MMU_

