/* $Header: /nfs/columbia/u1/BFMACH.local/mk.2.9.6/bflyc/RCS/pmap_c.h,v 1.50 91/03/01 17:13:45 tappan Exp $ */
/*****************************************************************************
File:     pmap_c.h
Contents: Physical Map Implementation Definitions
System:   BBNACI Butterfly OS kernel, virtual memory subsystem.
Created:  Aug 16, 1989
Author:   drb + jcg + rew

Remarks:

[And now, a few words from the legal department.]

Copyright 1989 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

#ifndef	_PMAP_MACHINE_
#define	_PMAP_MACHINE_	1

#ifndef	ASSEMBLER

#ifdef	KERNEL
#include "../h/queue.h"
#include "../h/vm.h"
#include "../h/lock.h"
#include "../butterfly/vm_param.h"
#include "../butterfly/param.h"
#include "../vm/vm_statistics.h"
#include "bbn_tlb.h"
#ifdef BFLY2
#include "../butterfly/b2vme.h"
#endif BFLY2

#else	KERNEL
#include <sys/queue.h>
#include <sys/vm.h>
#include <sys/lock.h>
#include <butterfly/vm_param.h>
#include <butterfly/param.h>
#include <vm/vm_statistics.h>
#endif	KERNEL

#endif ASSEMBLER

#ifdef KERNEL
#include "../butterfly/mmu.h"
#else	KERNEL
#include <butterfly/mmu.h>
#endif	KERNEL

#ifndef ASSEMBLER

/*****************************************************************************
 * PT_ENTRY_T and ST_ENTRY_T STUFF:
 * These definitions relate to the actual hardware, and thus begin with
 * the prefix MMU.  Other definitions appear in vm_param.h
 *
 * We divide our 4 Gbyte virtual address space (shared between user and
 * kernel) into 256 "spaces".  On the GP-1000, each space represents
 * one segment, mapping 2048 pages of 8K bytes (i.e. address has
 * 8 bytes of segment, 11 bytes of page table, and 13 bytes of page).
 *
******************************************************************************/

#define	PT_ENTRY_NULL	((pt_entry_t *) 0)
#define ST_ENTRY_NULL  	((st_entry_t *) 0)

/*****************************************************************************
* pte manipulation:
******************************************************************************/
#ifdef BFLY1
#define	mmupte_1(pte)	(pte)
#endif BFLY1
#ifdef BFLY2
#define	mmupte_1(pte)	(&(pte)->mmupte1)
#define	mmupte_2(pte)	(&(pte)->mmupte2)
#endif BFLY2

#define	pte_getphys(pte)	mmupte_getphys(mmupte_1(pte))
#define	pte_pnode(pte)		paddr2pnode(mmupte_getphys(mmupte_1(pte)))
#define	pte_iswriteprot(pte)	mmupte_iswriteprot(mmupte_1(pte))
#define	pte_iswritable(pte)	mmupte_iswritable(mmupte_1(pte))
#define	pte_iscached(pte)	mmupte_iscached(mmupte_1(pte))
#define	pte_iswritethru(pte)	mmupte_iswritethru(mmupte_1(pte))
#define	pte_isglobal(pte)	mmupte_isglobal(mmupte_1(pte))
#define	pte_isvalid(pte)	mmupte_isvalid(mmupte_1(pte))

#if BBN_CLSIZE == 1

#ifdef BFLY2
typedef struct system_pte {
	mmu_pt_entry_t	mmupte1;
} pt_entry_t;
#endif BFLY2
	  
#define	pte_mapin(pte,paddr,ptype) mmupte_mapin(mmupte_1(pte), paddr, ptype)
#define	pte_invalidate(pte)	mmupte_invalidate(mmupte_1(pte))

#define	pte_setwritable(pte)	mmupte_setwritable(mmupte_1(pte))
#define	pte_writeprot(pte)	mmupte_writeprot(mmupte_1(pte))
#define	pte_setmodified(pte)	mmupte_setmodified(mmupte_1(pte))
#define	pte_setclean(pte)	mmupte_setclean(mmupte_1(pte))
#define	pte_addbits(pte, prot)	mmupte_addbits(mmupte_1(pte), prot)
#define	pte_setcaching(pte, caching)	\
  				mmupte_setcaching(mmupte_1(pte), caching)
#define	pte_set_noncached(pte)  mmupte_set_noncached(mmupte_1(pte))
/*#define	pte_setbits(pte, bits)	mmupte_setbits(mmupte_1(pte), bits)*/

#define	pte_isreferenced(pte)	mmupte_isreferenced(mmupte_1(pte))
#define	pte_clearreferenced(pte) mmupte_clearreferenced(mmupte_1(pte))
#define	pte_ismodified(pte)	mmupte_ismodified(mmupte_1(pte))

#endif BBN_CLSIZE

#if BBN_CLSIZE == 2
typedef struct system_pte {
	mmu_pt_entry_t	mmupte1;
	mmu_pt_entry_t	mmupte2;
} pt_entry_t;

#define	pte_mapin(pte,paddr,ptype) (mmupte_mapin(mmupte_1(pte),paddr,ptype),\
		       mmupte_mapin(mmupte_2(pte),(paddr)+MMU_PGBYTES,ptype))
#define	pte_invalidate(pte)	(mmupte_invalidate(mmupte_1(pte))	, \
				 mmupte_invalidate(mmupte_2(pte)))
#define	pte_setwritable(pte)	(mmupte_setwritable(mmupte_1(pte))	, \
				 mmupte_setwritable(mmupte_2(pte)))
#define	pte_writeprot(pte)	(mmupte_writeprot(mmupte_1(pte))	, \
				 mmupte_writeprot(mmupte_2(pte)))
#define	pte_setwired(pte,wired)	(mmupte_setwired(mmupte_1(pte), wired), \
				 mmupte_setwired(mmupte_2(pte), wired))
#define	pte_wire(pte)		(mmupte_wire(mmupte_1(pte))		, \
				 mmupte_wire(mmupte_2(pte)))
#define	pte_setmodified(pte)	(mmupte_setmodified(mmupte_1(pte))	, \
				 mmupte_setmodified(mmupte_2(pte)))
#define	pte_setclean(pte)	(mmupte_setclean(mmupte_1(pte))	, \
				 mmupte_setclean(mmupte_2(pte)))
#define	pte_addbits(pte, prot)	(mmupte_addbits(mmupte_1(pte), prot)	, \
				 mmupte_addbits(mmupte_2(pte), prot))
#define	pte_setcaching(pte, caching)	\
  				(mmupte_setcaching(mmupte_1(pte), caching),\
				 mmupte_setcaching(mmupte_2(pte), caching))
#define	pte_set_noncached(pte)	(mmupte_set_noncached(mmupte_1(pte))	, \
			 	 mmupte_set_noncached(mmupte_2(pte)))
#define	pte_isreferenced(pte)	(mmupte_isreferenced(mmupte_1(pte))	|| \
				 mmupte_isreferenced(mmupte_2(pte)))
#define	pte_clearreferenced(pte)(mmupte_clearreferenced(mmupte_1(pte)), \
				 mmupte_clearreferenced(mmupte_2(pte)))
#define	pte_ismodified(pte)	(mmupte_ismodified(mmupte_1(pte))	|| \
				 mmupte_ismodified(mmupte_2(pte)))
#endif BBN_CLSIZE

/*****************************************************************************
* Macros for rounding, truncating, and converting between segments, pages,
* and bytes.
******************************************************************************/

/* general macros for rounding and truncating */
#define	pmap_round(x,y)	(((x) + pmap_mask(y)) & ~pmap_mask(y))
#define	pmap_trunc(x,y)	((x) & ~pmap_mask(y))
#define	pmap_mask(y)	((y) - 1)

/*
 * Hardware page conversion macros
 */

/*
 *	Convert bytes to pages and convert pages to bytes.
 *	No rounding is used.
 */
#define	mmu_btop(x)		(((unsigned)(x)) >> MMU_PGSHIFT)
#define	mmu_ptob(x)		((caddr_t)(((unsigned)(x)) << MMU_PGSHIFT))

/*
 *	Round off or truncate to the nearest hardware page.  These will work
 *	for either addresses or counts.  (i.e. 1 byte rounds to 1 page
 *	bytes.
 */

#define mmu_round_page(x)	((((unsigned)(x)) + MMU_PGMASK) & \
					~MMU_PGMASK)
#define mmu_trunc_page(x)	(((unsigned)(x)) & ~MMU_PGMASK)
#define mmu_pgoffset(x)		(((unsigned)(x)) & MMU_PGMASK)

/*
 * Segment conversion macros
 */
#define	btoseg(addr)		(((unsigned) (addr)) >> MMU_SEGSHIFT)


/* "space" conversion macros.  There are 16 Megabytes in a space, which is
 * a standard Mach unit of kernel virtual space
 */
#define spaceoffset(addr)	(((unsigned) (addr)) & (SPACE_SIZE-1))
#define	round_space(x)		pmap_round(x,SPACE_SIZE)
#define	trunc_space(x)		pmap_trunc(x,SPACE_SIZE)

/* offset in FE space (physical offset on node) */
#define	FEoffset(addr)		spaceoffset((addr))

#endif ASSEMBLER

#ifndef ASSEMBLER
#define	VECPG_POFFSET	0x00014000

/* Each node gets two pages of virtual address for itself (one each from
 * phys_map_vaddr1 and phys_map_vaddr2) and two corresponding pte's
 * for itself, so it can make its two pages of virtual address space
 * point to whatever physical address it wants.
 */
extern vm_offset_t	phys_map_vaddr1, phys_map_vaddr2;
#if 0
#ifdef BFLY1
pt_entry_t	*phys_map_pte1, *phys_map_pte2;
#endif BFLY1
#endif 0
#ifdef BFLY2
extern mmu_pt_entry_t	*phys_map_pte1, *phys_map_pte2;
#endif BFLY2
#ifdef BFLY2
/* $$$ Mokus in BFLY1 pcb.c */
	/* pte of user area for each node's kernel */
extern pt_entry_t	*u_area_pte[][UPAGES];
#endif BFLY2

#define my_phys_map_vaddr1()	phys_map_vaddr1 + (my_cpu_number*MMU_PGBYTES)
#define my_phys_map_vaddr2()	phys_map_vaddr2 + (my_cpu_number*MMU_PGBYTES)
#define	my_phys_map_pte1()	(&phys_map_pte1[my_cpu_number])
#define	my_phys_map_pte2()	(&phys_map_pte2[my_cpu_number])

#endif ASSEMBLER

#ifdef BFLY1
/* offset of the table descriptor used for mapping the text segment
 * of the kernel. ( (8*0xC0) + 4)
 */
#define	KTEXT_SEG_OFFSET 1540

/* 
 * offset of the table descriptor used for mapping the u area 
 * relative to the beginning of the root page. ( (8*0xFB) + 4)
 */
#define U_AREA_OFFSET	2012

/* 
 * offset of the table descriptor for FE region relative to 
 * the beginning of the root page. ( (8*0xFE) + 4)
 */
#define FE_SEG_OFFSET	2036
#endif BFLY1

#ifdef BFLY2
#ifdef KERNEL_GENASSYM

#define PTES_PER_PAGE		(VM_PAGESIZE / sizeof(pt_entry_t))

/* offset of the physical address portion of the table descriptor used
 * for mapping the given space.  Offset is relative to root page.
 * Macro is also used by gas assembler code.
 */
#define	SEGTAB_SPACEOFFSET(seg)	((seg) * (sizeof(st_entry_t)*BBN_SCLSIZE))

/* offset of the table descriptor used for mapping the kernel text segment. */
#define	KTEXT_SEG_OFFSET 	SEGTAB_SPACEOFFSET(KTEXT_SPACE)

/* offset of the table descriptor used for mapping the u area/kstack  */
#define UAREA_SEG_OFFSET	(SEGTAB_SPACEOFFSET(U_AREA_SPACE+1)-sizeof(st_entry_t))

/* offset of the the table descriptor for seg mapping local physical memory. */
#define FE_SEG_OFFSET		SEGTAB_SPACEOFFSET(LOC_PHYS_SPACE)

#endif KERNEL_GENASSYM
#endif BFLY2


/*****************************************************************************
* software stuctures and definitions to support pmap layer
******************************************************************************/

#ifdef KERNEL
#ifndef ASSEMBLER

struct pmap {
	st_entry_t     *virt_mmu_table; /* virtaddr of mmu tables */
	vm_addr_t       phys_mmu_table; /* physaddr of mmu tables */
	pt_entry_t    **virt_seg_table; /* virtaddr of software seg tbl */
	simple_lock_data_t pm_lock;	 /* lock on map */
	short   	home_node;	 /* home node for map */
	short   	current_node;	 /* node pmap currently active upon */
	vm_queue_data_t pmapq;	         /* queue of pmaps on this node */
	short           flags;		 /* flags: see below */
	short		spl;		 /* saved priority level from spl */
	int             pagecnt;	 /* physical pages used in table */
	int 		refcnt;		/* map and revpte references */
};

#define PMAP_SCAN_DISABLE 0x0001 /* flag says pmap scanning disabled */
#define PMAP_DEALLOCATED  0x0002 /* pmap no longer associated with a map */

#define	PMAP_NULL	((pmap_t) 0)

#define	pmap_lock_init(pmap)	simple_lock_init(&(pmap)->pm_lock)
/* See pmap_lock() for a comment regarding the kernel pmap and spl */
extern pmap_t kernel_pmap;
#ifdef notdef
#define	pmap_unlock(pmap)	{ \
				  simple_unlock(&(pmap)->pm_lock); \
				  if ((pmap) == kernel_pmap) \
				      splx((pmap)->spl); \
				 }
#else
/* XXX */
#define pmap_unlock(pmap)	{ \
				if ((pmap) != kernel_pmap) \
					simple_unlock(&(pmap)->pm_lock);\
			      } 
#endif


/* Increment or decrement pmap reference count */
void pmap_free();
#define PMAP_INCR(pmap) Atomic_add32(&(pmap)->refcnt, 1)

#define PMAP_DECR(pmap) if (Atomic_add32(&(pmap)->refcnt, -1) == 1) \
				pmap_free(pmap)

#endif	ASSEMBLER


#ifndef ASSEMBLER
struct bootinfo {
	root_ptr_t	boot_root;		/* copy of mmu register */
#ifdef BFLY1
	struct pmmu_tcr	boot_tcr;		/* copy of 68851 tc register */
#endif BFLY1
	int		(*boot_pc)();		/* pc to jump to */
};

/* base address of the start command block transferred to
 * remote nodes.  Pick an address not overwritten by kernel text,
 * so that we COULD someday download the kernel into its own low memory.
 */
#define	BOOTINFO_BASE	(*((struct bootinfo *) _BOOTINFO_BASE))
#endif ASSEMBLER

/* 
 * Routine to map an address into the local scratch pte, and invalidate
 * in the right order
 * This may be overkill...
 */
#ifdef BFLY2
#define SCRATCH_MAPIN(pa, type) { \
      mtpr(TBIS, my_scratch_va); \
      pte_mapin(my_scratch_pte, pa, type); \
      mmu_flushs(my_scratch_va); \
      }
#endif
#ifdef BFLY1
#define SCRATCH_MAPIN(pa, type) { \
      pte_mapin(my_scratch_pte, pa, type); \
      pflushs(my_scratch_va); \
      }
#endif
#endif KERNEL

#endif	_PMAP_MACHINE_


