/* $Header: /nfs/columbia/u1/BFMACH.local/mk.gibson/bflyc/RCS/signali_c.h,v 1.2 91/01/03 18:11:12 gibson Exp $ */

/*****************************************************************************
* INTERNAL DATA STRUCTURES AND DEFINITIONS ASSOCIATED WITH USER SIGNAL
*	HANDLING.
*
* Three players are involved, the kernel, the signal trampoline code, and
* the user-defined handler.  These data structures are used to communicate
* amongst them.  The comments attempt to specify the communicators and
* the direction of communication.
* struct sighhandler_stack (below) gives the complete layout of the
* data structures when in the handler.
******************************************************************************/

#ifdef BFLY1
#ifdef KERNEL
#include	"../machine/fpu.h"
#else KERNEL
#include	<machine/fpu.h>
#endif KERNEL
#endif BFLY1

#ifdef KERNEL
#include	"bbn_sig_debug.h"
#endif KERNEL

/*****************************************************************************
* sigtramp_args is pushed on user stack by kernel (after the kernel pushes the
*	scp), and contains arguments passed from the kernel to the signal
*	trampoline code so it can call the handler, and later call sigreturn().
*
*	sta_s[xnf]ip fields are not really args to sigtramp, but are
*	for informational purposes only.
******************************************************************************/
struct sigtramp_args {
	int	(*sta_handler)();  /* handler */
	int	sta_signum;	   /* argument to handler */
	int	sta_code;	   /* argument to handler */
	struct sigcontext *sta_scp;/* arg to hndlr and sigreturn */
};

/*****************************************************************************
* struct sigtramp_regs is an implicit structure representing the state
*	saved on the stack by the signal trampoline code before it calls
*	the user handler (see sighandler_stack, below).
*
*	This state is restore after the handler returns and before a sigreturn.
*	The handler could thus change the value of these registers in
*	the post-sigreturn context by modifying the appropriate values on
*	the stack.
******************************************************************************/
#ifdef BFLY1
struct sigtramp_regs {
	struct _fpreg	str_fp0;	/* fp0 (if scp fpstate non-null) */
	struct _fpreg	str_fp1;	/* fp1 (if scp fpstate non-null) */
	unsigned	str_fpcr;	/* fp ctl/stat (if fpstate non-null) */
	unsigned	str_fpsr;
	unsigned	str_fpiar;
	unsigned	str_d0;	/* saved dynamic regs */
	unsigned	str_d1;
	unsigned	str_a0;
	unsigned	str_a1;
};
#endif BFLY1

#ifdef BFLY2
struct sigtramp_regs {
	unsigned	str_fpcr;	/* fp ctl reg */
	unsigned	str_fpsr;	/* fp status reg */
	unsigned	str_r0;	/* ignored */
	unsigned	str_r1;	/* saved dynamic regs*/
	unsigned	str_pad1;	/* r2 in scp (keep st.d alignment) */
	unsigned	str_r3;	/* saved dynamic regs*/
	unsigned	str_r4;	/* saved dynamic regs*/
	unsigned	str_r5;	/* saved dynamic regs*/
	unsigned	str_r6;	/* saved dynamic regs*/
	unsigned	str_r7;	/* saved dynamic regs*/
	unsigned	str_r8;	/* saved dynamic regs*/
	unsigned	str_pad2;	/* r9 in scp (keep st.d alignment) */
	unsigned	str_r10;	/* saved dynamic regs*/
	unsigned	str_r11;	/* saved dynamic regs*/
	unsigned	str_r12;	/* saved dynamic regs*/
	unsigned	str_r13;	/* saved dynamic regs*/
};
#endif BFLY2

/*****************************************************************************
* struct sighandler_stack is an implicit structure representing all
*	state saved by kernel [handler and scp] and the signal
*	trampoline code (_sigcode in locore.s) [sigtramp_regs].
*	(It does not include linkage associated with the handler
*	routine itself.)
*
*	A signal handler may obtain a pointer to this struct using
*	scp2sighandler_stack(scp) (below) and the scp passed as
*	an argument to a handler.
*
*	NB: This is a kernel-internal data structure, not part of the
*		nX-exported interface and subject to change without
*		notice.  Use at your own risk.
******************************************************************************/
struct sighandler_stack {
	struct sigtramp_regs	shdlr_regs;	/* handler -> trampoline */
	struct sigtramp_args 	shdlr_args;	/* kernel -> trampoline */
	struct sigcontext	shdlr_scntxt;	/* kernel <-> handler */
};

#define	scp2sighandler_stack(scp)	\
  ((struct sighandler_stack *) (((unsigned)scp) + \
				sizeof(struct sigcontext) - \
				sizeof(struct sighandler_stack)))


/*****************************************************************************
* Abbreviations for conveniently accessing machine-specific part of
*	sigcontext (defined in signal.h).
******************************************************************************/
#define	_scm_sigsynch		_sc_machine.scm_sigsynch

#ifdef BFLY1
#define	_scm_fpstate_valid	_sc_machine.scm_fpstate_valid
#define	_scm_fpstate		_sc_machine.scm_fpstate
#endif BFLY1

#ifdef BFLY2
#define	_scm_r2		_sc_machine.scm_r2
#define	_scm_r9		_sc_machine.scm_r9
#define	_scm_sxip	_sc_machine.scm_sxip
#define	_scm_snip	_sc_machine.scm_snip
#define	_scm_sfip	_sc_machine.scm_sfip
#define	_scm_datapipe	_sc_machine.scm_datapipe
#endif BFLY2


/*****************************************************************************
* Internal signal-handling definitions
******************************************************************************/
#ifdef KERNEL
/* potential values for scp->_scm_sigsynch */
#define	SYNCHTYPE_NONE		0	/* not a synchronous signal */
#define	SYNCHTYPE_UNIX		1	/* Unix semantics */
#define	SYNCHTYPE_SYNCH		2	/* special handling */
#define	SYNCHTYPE_SYNCH_NOISY	3	/* special handling, but message */

/* default for per-machine hardwired overrides for process-specific
 *SIGSEGV handling
 */
#define	HSYNCHTYPE_DEFAULT	SYNCHTYPE_SYNCH	/* for now */

/* per-signal u.u_sigsynch_state flags (only supported for SIGSEGV now) */
#define	SYNCHSIG_NONE	0x0000 /* no synchsig in progress */
#define	SYNCHSIG_PEND	0x0001 /* synchsig psignalled, not yet delivered */
#define	SYNCHSIG_INPROG	0x0002 /* synchsig psignalled, not yet sigreturned */

/* signal debugging/tracing package */

#if BBN_SIG_DEBUG
#define	SDBG(arg)	(arg)
#else BBN_SIG_DEBUG
#define	SDBG(arg)
#endif BBN_SIG_DEBUG

#endif KERNEL

