/* $Header: /nfs/europa/u1/BFMACH.local/mk.gibson.TMP/bflyc/RCS/vm_param_c.h,v 1.15 90/08/21 11:04:18 gibson Exp $ */
/*
 ****************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 ****************************************************************
 */
/*
 *	File:	butterfly/vm_param.h
 *	Author:	Avadis Tevanian, Jr.
 *
 *	Copyright (C) 1985, Avadis Tevanian, Jr.
 */
/*
 *********************************************************************
 *	Machine dependent parameters for nX virtual memory system.
 *	Most of the declarations are preceeded by MMU_ (or mmu_)
 *	which is OK because only code specific to the machine's MMU
 *	will be using them.
 *
 * Definitions that are internal to the pmap layer appear in pmap.h
 *********************************************************************
 */

#ifndef	_MACHINE_VM_PARAM_
#define	_MACHINE_VM_PARAM_

/*
 * Machine dependent constants for nX virtual memory implementation.
 */

/*****************************************************************************
* "page size" under nX
*
* 1. Page size under 4.3 involved two parameters.
*	a. NBPG was the number of bytes per hardware page.
*	b. CLSIZE was the number of hardware pages "clustered" together
*		to make a system page.  CLSIZE appears explicitly
*		throughout the 4.3 code whenever pte's are copied,
*		where we copy CLSIZE pte's to copy a system page.
* Under 4.3, a "struct pte" represented a hardware page.
*
* 2. Page size under nX
*	a. MMU_PGBYTES is the number of bytes per hardware page.
*	b. VM_PAGESIZE is the number of bytes per system page.
* Under nX, a "pt_entry_t" represents a hardware page.
*
* Under nX, the distinction between hardware and system pages
* is encapsulated in the pmap layer, with the exception of some assembler
* macros defined in locore.s.  pt_entry_t's only appear in pmap.c and kdb.
*
* Compatibility with 4.3 is maintained by setting NBPG=VM_PAGESIZE and
* CLSIZE=1.  A struct pte represents VM_PAGESIZE worth of bytes.
*
* All conversion of struct pte's into constituent parts and all manipulation
* of internal fields of pt_entry_t is done in the pmap layer.
******************************************************************************/

#ifdef BFLY1

/* number of hardware page table entries mapping a system page. */
#define	BBN_CLSIZE	1
/* number of segments mapping a 16 megabyte "space", a standard unit of BBN
 * nX storage allocation.
 */
#define	BBN_SCLSIZE	1

#endif BFLY1

#ifdef BFLY2

/* number of hardware page table entries mapping a system page. */
#define	BBN_CLSIZE	2
/* number of segments mapping a 16 megabyte "space", a standard unit of BBN
 * nX storage allocation.
 */
#define	BBN_SCLSIZE	4

#endif BFLY2

	/* should be more careful about including vmap.h.
	 * For now, just protect bootstraps from these definitions.
	 */
#if !defined(STANDALONE)
#ifdef KERNEL
#include	"../machine/vmap.h"
#else KERNEL
#include	<machine/vmap.h>
#endif KERNEL
#endif STANDALONE

#endif	_MACHINE_VM_PARAM_
