/* $Header: /nfs/columbia/u1/BFMACH.local/mk.2.9.6/butterflydev/RCS/bma.c,v 1.58 91/04/03 15:46:09 rgoguen Exp $ */

#include	"../h/param.h"
#include	"../h/types.h"
#include	"../butterfly/cpu.h"
#include	"../butterflydev/mbvar.h"
#include	"../butterflydev/mbreg.h"
#ifdef BFLY2
#include	"../butterfly/pmap.h"
#include	"../butterfly/b2vme.h"
#include	"../butterfly/mtpr.h"
#include	"reboot.h"
#include	"../machine/tcs.h"
#endif BFLY2
#include        "conf.h"
#include	"../h/kmalloc.h"

#include	"mb.h"
#include	"lock.h"
#include "../machine/bitmap.h"

/* #define BMA_DEBUG 1 */

#if 0
/*****************************************************************************
	THE MULTIPLE-MAINBUS SCHEME ON THE GP1000/GP2000 EXPLAINED:

The goal of the scheme is to be able to easily hook up devices to
many different nodes in a convenient way.  The most important device
we imagine treating this way is the disk.

NB: For historical reasons (The BFLY1 uses the multibus and the butterfly
	multibus adaptor.) we speak of the "main bus" (mbus) and
	"butterfly mainbus adaptor" (bma).

On the BFLY1:

The scheme has the following properties: Assuming the kernel was
compiled with the proper configuration, attaching a disk drive to
another node requires a multibus cage, bma, and disk controller.  The
system manager must 1) jumper the new bma's id bits to a
machine-unique ID less than 32. 2) plug the bma and disk driver into
the multibus cage.  3) plug the bma's cable into any node (NB: this
cable can be changed to a different node without harm at any time
provided the machine is not running.)  4. creat /dev/xy?? and
/dev/rxy?? entries for the disk using the encoding scheme described
below.

The basis of the scheme from a hardware point of view is that a bma
has certain bits that can be hardware jumpered to produce a "bma id".

On the BFLY2:

The scheme has the following properties: Assuming the kernel was
compiled with the proper configuration, attaching a disk drive to
another node requires a VMEbus cage and disk controller.  The
system manager must 1) choose a new mainbus id which is unique
across the machine and is less than 32. 2) plug the disk driver into
the VMEbus cage.  3) attach the VMEbus to the chosen node. 4) make
an entry in the bootconf file specifying that the chosen node has
the mainbus chosen in step 1 above.  NB: Unlike the GP1000 scheme,
the GP2000 scheme requres that you change the bootconf file entry
if you move the VMEbus from one node to another. 5) creat /dev/xy?? and
/dev/rxy?? entries for the disk using the encoding scheme described
below.

The basis of the scheme from a software point of view is that
certain drivers are "vectorized".  That means that each node has the
potential to drive a separate instance of the device.  This is
customarily achieved by having driver data structures in the
node-private data area.

Furthermore a device with a vectorized driver (e.g. Xylogics disk
controller with CSR's at 0xee40) can be declared to config to be
vectorized with the "xyc0 at mbio *" construct.  That means the
kernel, at I/O discovery time, will look on every node for such a
controller, and will enable ALL that it finds.  Thus, the disk
controller does not have to be rejumpered to some new address - a
standard controller may be used.

Finally, the major device number of a device (in the sense of a UNIX
special file), as set in the device's inode, encodes what mainbus the
device is attached to [as well as answering the traditional question
of what driver in the bdevsw or cdevsw table should be used for the
device.]
*******************************************************************************

The encoding works as follows:

      The device special file's major device number tells us that
      our disk controller is on an mainbus with a certain id.

      On the BFLY1:

      At I/O discovery time, we discovered what bma's are attached to
      what nodes.

      On the BFLY2:

      At boot time, the bootconf file indicated what nodes had a VMEbus 
      attached, and gave a unique mainbus id.

      In either case:

      With those two pieces of information, we know that
      our device is on a certain node.  We can switch to that node
      (and hence to that node's driver for our device) to operate on
	the device.

*                                                                       *
* ******** The following is the old format dev_t encoding ************* *
The encoding scheme uses the 8 bits of the major device number, and works
as follows (NB: major_routine actually implements this).

1. The top two bits (MAJOR_VEC_BITS) determine whether the device is
	normal or "vectorized" (i.e. an instantiation of it can appear on
	any node).

	a. If the top two bits are 00, the lower 8 bits are the
	traditional major device number (index into bdevsw or cdevsw
	table).  That allows 64 possible major devices of each type.
	For reasons described below, some of these may already be
	allocated.

	b. If the top two bits are anything non-zero, we have a
	vectorized device, and drop into the vectorized scheme.

2. (Vectorized Scheme):  The major device number is broken up into
	a 3-bit field and a 5-bit field.

	a. The 3-bit field determines the major device number.  Note
	that since 000 and 001 are already used for non-vectorized
	device, we are allowing up to 6 vectorized drivers of each
	(block/character) type, for a total of 12.
	
	We have allocated a set of slots in cdevsw and bdevsw at
	VEC_OFFSET for these drivers.  Thus, we add the value from the
	3-bit field to VEC_OFFSET to get the index into whichever of
	cdevsw or bdevsw is appropriate.

	b. The 5 bit field is the mainbus id for this device, allowing
	up to 32 (densely numbered) mainbuses on a GP1000.
* ********************************************************************* *

*                                                                       *
*  ******** New format dev_t encoding ********                          *
* A dev_t is a 32 bit value with bmaid, major, and minor packed as      *
*  defined in h/type.h.                                                 *
*                                                                       *
* The 'vectorized' property is defined by a bit in cdevsw[]. If         *
* cdevsw[].noswitch is set then the driver has been vectorized.         *
* ********************************************************************* *
******************************************************************************/
#endif 0


/* The BFLY1 bma hardware allows up to 256 id's.  The BFLY2 scheme allows an
 * arbitrary number.
 * Our software scheme allows only 0-31.  To save the extra bytes in the
 * array, yet still work with a BFLY1 bma that hasn't been jumpered and
 * might have a bma id > 31 on the master, we treat the master as a
 * special case using master_bmaid.
 */

short	master_bmaid;
short	bmaid_to_lognode_map[NBMAS];
/*short	lognode_to_bmaid_map[NCPUS];*/

/*****************************************************************************
* initialize_io - Called once to initialize bus mapping system (for multiple
*	multibus stuff).  If config declared a specific mapping between
*	a multibus (e.g. mb0) and a bma jumpered id, fill in that
*	multibus so that we know a bma has already been allocated to it.
*	Also, initialize bmaid/lognode maps and do some consistency checking.
******************************************************************************/
initialize_io()
{
	struct mb_adptr *ma;
	int		i;

	/* initialize mapping arrays */
	for (i = 0; i < NMB; i++)
	  mb_hd[i].mh_bmaid = BMAID_NULL;	/* multibus to bmaid */
	for (i = 0; i < NBMAS; i++)
	  bmaid_to_lognode_map[i] = NULL_NODE;	/* bmaid to logical node */

	master_bmaid = BMAID_NULL;
	/*for (i = 0; i < NCPUS; i++)
	  lognode_to_bmaid_map[i] = BMAID_NULL;*/ /* logical node to bmaid */
	  
	if (NMB >= QUES || NMB >= ANY)
	  panic("initialize_bma_info: too many multibuses");

	/* Record which multibuses have been preallocated to specific bma's.
	 * via config and kernel configuration file (ioconf.c).
	 */
	for (ma = mbainit; ma->ma_bmaid != -1; ma++) {
		if (ma->ma_mbnum == QUES || ma->ma_bmaid == QUES)
		  continue;
		if (ma->ma_mbnum > NMB)
		  panic("initialize_bma_info: multibus number too large");
		if (ma->ma_bmaid > NBMAS)
		  panic("initialize_bma_info: bmaid too large");
		mb_hd[ma->ma_mbnum].mh_bmaid = ma->ma_bmaid;
	}
}

/*****************************************************************************
* bma_init - Find the right entry in mb_hd for mbus attached to this
*	node.  Fill that entry in by marking it alive and pointing it at
*	the right mb_adptr struct from ioconf.c.
*    Return 0 if bma not configured.  1 on success.
******************************************************************************/

/*
 * The bma_config_lock is used to synchronize access around serial
 * parts of the I/O code. Currently this is finding an 'md' slot
 * and when probing '?' devices.
 */
simple_lock_data_t bma_config_lock;

#ifdef BFLY1
bma_init()
#endif BFLY1
#ifdef BFLY2
bma_init(bma)
	int	bma;		/* On BFLY2, passed in from bootconf file */
#endif BFLY2
{
	struct mb_hd	*mh;
#ifdef BFLY1
	int		bma;		/* holds id from bma misc register */
#endif BFLY1
	int		mb_index;	/* number of this multibus */
#ifdef BFLY2
	char		nodestr[6];
#endif BFLY2
	if (my_cpu_number == master_cpu) {
		simple_lock_init(&bma_config_lock);
		peek_init();
	}

	simple_lock(&bma_config_lock);

#ifdef BFLY1
#ifdef VME130
	bma = 0 ;
#else VME130
	if((bma = peek(MISC)) == -1)
	  panic("bma_init: expected bma not found\n");

	/* convert to jumpered id */
	bma = misc_Jumpers(bma);
#endif VME130
	
	if (bma >= NBMAS && my_cpu_number != master_cpu)
	  panic("bma_init: illegal bmaid");
	if (bma < NBMAS && bmaid_to_lognode_map[bma] != NULL_NODE)
	  panic("bma_init: Can't have two nodes with same bma id\n");
#endif BFLY1
	/*
	 * Search for mb structure already matching our bma id.
	 */
	for (mb_index = 0; mb_index < NMB; mb_index++) {
		mh = &mb_hd[mb_index];
		if (mh->mh_bmaid != bma)
		  continue;
		/* Duplicate allocation */
		if (mh->mh_alive) {
			printf("WARNING: attempt to reallocate iobus %d\n",
			       bma);
			goto UNCONFIGURED;
		}
		goto mh_found;
	}
	
	/*
	 * Search for first free mb structure.
	 */
	for (mb_index = 0; mb_index < NMB; mb_index++) {
		mh = &mb_hd[mb_index];
		/* can't be alive already */
		if (mh->mh_alive)
		  continue;

		/* if we have a bma adaptor picked out, this entry
		 * can't already have one.
		 */
		if (mh->mh_bmaid != BMAID_NULL)
		  continue;

		/* This one's okay!! */
		goto mh_found;
	}
	/* Couldn't find an mb */
	if (my_cpu_number == master_cpu)
	  panic("bma_init: unable to find slot for multibus");
	else
	  goto UNCONFIGURED;
	
mh_found:
	/* At this point, we have an mb_index.  Now fill in all the fields.
	 */

	if (my_cpu_number == master_cpu)
	  master_bmaid = bma;

	mh = &mb_hd[mb_index];

	if (mh->mh_alive)
	  panic("bma_init: multibus already allocated");

	mh->mh_alive = 1;

	simple_unlock(&bma_config_lock);	/* Done with one serialized section */

	mh->mh_node = phys_cpu_number();
	mh->mh_bmaid = bma;

	if (bma < NBMAS)
	  bmaid_to_lognode_map[bma] = my_cpu_number;
	/*lognode_to_bmaid_map[my_cpu_number] = bma;*/

#ifdef BFLY1
	printf("mb%d is iobus %d at processor node 0x%x\n",
	       mb_index, bma, mh->mh_node);
#endif BFLY1
#ifdef BFLY2
	printf("mb%d is iobus %d on processor node %s\n",
	       mb_index, bma, tcs_node_str(mh->mh_node, nodestr));
#endif BFLY2

	bma_register_init();		/* set up bma mappings */
	mbinitmaps(mh);			/* set up memory maps */
		/* look for devices in multibus I/O and memory space */
	mbioinit(mb_index);
#ifdef BFLY1
	mbmeminit(mb_index);
#endif BFLY1
	return(1);

UNCONFIGURED:
	simple_unlock(&bma_config_lock);/* Done with one serialized section */
	printf("unconfigured multibus at iobus %d\n", bma);
	return(0);
}      

/*
 * The convert_old_dev_t() routine is used to convert an old format
 * dev_t (as it may be stored in the inode) to a new format.
 */
/* vectorized devices start at 20 ... since lowest from above is 2 ... */
#define VEC_OFFSET	18
/* different ways of splitting up 8-bit major device number */
#define	MAJOR_VEC_BITS	0xC0
#define	MAJOR_BMA_BITS	0x1F
#define MAJOR_VDEV_BITS 0xE0
#define ALWAYS_PARALLEL 26	/* devices that exist on each node */
#define ALWAYS_PARALLEL_END 33

dev_t convert_old_dev_t(x)
     int x;
{
  int major_device = (x >> 8) & 0xFF;
  int minor_device = (x & 0xFF);
  int bma = MASTER_IOBUS;	/* default to the master */

  /* See if it's the new style format. */
  if (major(x) > 0)
      return(x) ;

  if (major_device & MAJOR_VEC_BITS) {
    bma = (major_device & MAJOR_BMA_BITS);
    major_device = ((major_device & MAJOR_VDEV_BITS) >> 5) + VEC_OFFSET;
  } 

  if (major_device >= ALWAYS_PARALLEL && major_device <= ALWAYS_PARALLEL_END)
    bma = ALLNODE_IOBUS;

  return(makedev(major_device, minor_device, bma));
}

/*
 * convert_new_dev_t - convert a new format dev_t to old format, if possible.
 * This is used for backward compatability, so we do not store new format
 * dev_t's on disk unless necessary.
 */
convert_new_dev_t(dev)
{
  int maj = major(dev);
  int min = minor(dev);
  int bma = iobusid(dev);
  int retval = -1;

  if (min < 256) {

    if (maj >= ALWAYS_PARALLEL && maj <= ALWAYS_PARALLEL_END)
      retval = (maj<<8) | min;
    else if (maj >= VEC_OFFSET+2 && maj < VEC_OFFSET+8 && bma < 32)
      retval = ((((maj-VEC_OFFSET)<<5)|bma)<<8)|min;
    else if ((bma == (MASTER_IOBUS&DEV_T_IOBUSMASK)))
      retval = (maj<<8)|min;
  }

  return(retval);
}


/*****************************************************************************
* dev_to_bmaid - return the bmaid associated with a device
******************************************************************************/
dev_to_bmaid(dev)
     dev_t	dev;
{
  register int bma = iobusid(dev);
  register int major_device = major(dev);

  if (bma == (MASTER_IOBUS&DEV_T_IOBUSMASK))
    bma = master_bmaid;

  if (bma == (ALLNODE_IOBUS&DEV_T_IOBUSMASK) || 
      bma == (NFS_IOBUS&DEV_T_IOBUSMASK))
    bma = BMAID_NULL;

  return(bma);
}

/*****************************************************************************
* dev_to_log_node - return the logical node associated with a device, or
* 	NULL_NODE if no such node exists.
******************************************************************************/
dev_to_log_node(dev)
     dev_t	dev;
{
	register int bma = iobusid(dev);
	register int major_device = major(dev);

	if (bma == (MASTER_IOBUS&DEV_T_IOBUSMASK))
		return(MASTER_CPU);

	if (bma == (ALLNODE_IOBUS&DEV_T_IOBUSMASK) || 
	    bma == (NFS_IOBUS&DEV_T_IOBUSMASK))
		return(my_cpu_number);

	if (bma >= NBMAS)
		panic("dev_to_log_node: bmaid too large\n");

	return(bmaid_to_lognode_map[bma]);
}
 

/*****************************************************************************
* bmaid_to_log_node - return the logical node associated with a bmaid, or
* 	NULL_NODE if no such node exists.
******************************************************************************/
bmaid_to_log_node(bma)
	register unsigned int	bma;
{
	/* If this is a single-bma system, bma may not have been jumpered
	 * to a valid (0-31) bmaid.  Make sure this still works.
	 */
	if (bma >= NBMAS)
	  if (bma == (MASTER_IOBUS&DEV_T_IOBUSMASK))
	    return(master_cpu);
	  else if (bma == master_bmaid)
	    return(master_cpu);
	  else if (bma == (ALLNODE_IOBUS&DEV_T_IOBUSMASK))
	    return(my_cpu_number);
	else
	    panic("bmaid_to_log_node: bmaid too large");

	if (bma == BMAID_NULL)
	  return(NULL_NODE);

	return(bmaid_to_lognode_map[bma]);
}

/*****************************************************************************
* log_node_to_bmaid() - return the bmaid associated with a logical node,
*     or BMAID_NULL if no such node or bmaid exists.
******************************************************************************/
log_node_to_bmaid(lnode)
	register int	lnode;
{
  register int i;

  if (lnode == MASTER_CPU)
    return(master_bmaid);

  for(i = 0; i < NBMAS; ++i)
    if (lnode == bmaid_to_lognode_map[i]) 
      return(i);

  /* here iff no mapping exists */
  return(BMAID_NULL);
}


#ifdef BFLY1
/*****************************************************************************
* bma_register_init: set up various bma registers that map between
*	multibus and processor, determine levels that devices
*	interrupt on, etc.
* For now, assume that USD has set everything up as we want it.
******************************************************************************/
bma_register_init()
{
#ifndef VME130
	register int i;
	unsigned char *vector_ram;
	struct vector_ram {
		unsigned char top;
		unsigned char bot;
	}*vector_table;

	/*
	 * set up the vector ram in the BMA first.
	 * The priority and the interrupt mapping is outlined as follows:
	 *
	 *		bma servicing	bma intr   68020 intr
         *	device	  priority	  level	     level	vector
	 *	tty	     1		   4	       3	 0x14
	 *	disk	     2		   3	       3	 0x17
	 *	tape	     3		   2	       3	 0x16
	 *	scsi	     4		   1	       3	 0x46
	 *	ether	     5		   5	       3	 0x15
	 */

	vector_ram = (unsigned char *) 0xfff7f000;

	for(i=0; i < 256; i++) {
		if(i & 0x10) {		/* tty interrupt level 4 */
			*(vector_ram + i) = 0xc;	/* CPU level 3 */
			continue;			/* serviced first */
		}
		if(i & 0x8) {		/* disk intterupt level 3 */
			*(vector_ram + i) = 0xb;	/* CPU level 3 */
			continue;
		}
		if(i & 0x4) {		/* tape interrupt level 2 */
			*(vector_ram + i) = 0xa;	/* CPU level 3 */
			continue;
		}
		if(i & 0x2) {
			*(vector_ram + i) = 0x9;	/* CPU level 3 */
			continue;
		}
		if(i & 0x20) {		/* ethernet interrupt level 5 */
			*(vector_ram + i) = 0xd;	/* CPU level 3 */
			continue;			/* serviced last */
		}
	}

	/* now set up the vector table */

	vector_table = (struct vector_ram *) 0xfe00005e;

	/* see scb.s for vector values (vector * 4 = VEC number) */
	(vector_table + 1)->bot = 0x49;		/* scsi vector (scsi0) */
	(vector_table + 2)->bot = 0x46;		/* tape vector (tape0) */
	(vector_table + 3)->bot = 0x47;		/* disk vector (disk0) */
	(vector_table + 4)->bot = 0x44;		/* tty vector  (tty0) */
	(vector_table + 5)->bot = 0x45;		/* ethernet vector (ether0) */
#endif VME130
}
#endif BFLY1

#ifdef BFLY2
/*****************************************************************************
* bma_register_init: set up various registers that map between
*	mbus and processor, set up m configuration register
******************************************************************************/
bma_register_init()
{
	int i;
	struct tcs_node_descriptor node;
	extern char my_node_eeprom_signature;

#ifdef BMA_DEBUG
	printf("bma_register_init: routine entry\n");
#endif BMA_DEBUG

	/* Reset the VMEbus for 400ms (200ms minimum), then intialize VMEbus
	 * Configuration Register and VMEbus timer registers. The adaptor
	 * card only broadcasts standard addresses, otherwise we'd use
	 * VME_EXTENDED here.
	 */
	VME_CONFIG_REG = VME_RESET;
	DELAY(400);
	VME_CONFIG_REG = (VME_NO_RESET | VME_RELEASE_ON_REQUEST |
			  VME_STANDARD | VME_SLAVE_WINDOW);


	tcs_parse_node(my_node_eeprom_signature, &node);

	/* Wait for all the VMEbus controllers to finish self-testing.
	 */
	i = 0;
	printf("Waiting for VMEbus diagnostics to complete...\n");
	while (VME_CONFIG_REG & VME_SYSFAIL) {	/* high true */
		DELAY(1000);
		/* If it takes more than a 3 minute, panic. */
		if (++i > (3 * 60))
			panic("VMEbus SYSFAIL signal asserted too long");
	}

	/* Initialize THIS node's per-node chains of VME master & slave
	 * mapper RAM entries as well as the static mapper allocations.
	 */
	vme_map_alloc_init (my_cpu_number);

	/*
	 * Initialize THIS node's per-node VME vector dispatch/argument 
	 * table.
	 */
	vme_vector_init(my_cpu_number);

#ifdef BMA_DEBUG
	printf("bma_register_init: VMEbus Conf Reg : 0x%x\n", VME_CONFIG_REG);
#endif BMA_DEBUG

	VME_MBUS_TIMER_REG = 0;

	/*
	 * The system bus timer register controls how long the system will
	 * allow a VME bus master to hold the bus while waiting for a slave
	 * to respond. The time == 4x(0xffff-N) microseconds. This timeout
	 * must be less than 1/2 the switch timeout to avoid switch timeout errors
	 * on remote accesses. Switch timeout is about 10ms.
	 * On older boards this counter was only 8 bits. 
	 * **** Above comment is wrong as to time: switch timeout
	 *     is more like 255us. Both these timers need to be shorter,
	 *     But we should watch out for the case where an I/O bus contains
	 *     a slow device and will not have remote references done to it.
	 * *****
	 */
	if (node.node_16bit_sbus_timer)
		VME_SBUS_TIMER_REG = 0xfc00;
	else
		VME_SBUS_TIMER_REG = 0;
	VME_ARBITER_TIMER_REG = 0;
	INT_ENABLE_MASK_REG |= ALL_VME_INTS;	/* all but IPI for now */

#ifdef BMA_DEBUG
	printf("bma_register_init: VMEbus timer registers initialized\n");
	printf("bma_register_init: routine exit\n");
#endif BMA_DEBUG
}

extern pt_entry_t *vme_slot_ptes;      	/* PTE's covering vme master rams */
simple_lock_data_t vme_slot_lock;	/* vme I/O slot multiprocessor lock */
/*
 * vme_slots defines the maximum number of vme pages that can be
 * mapped over the machine. This is set to 1024 which will span
 * entire FC segment allocated to VME devices.
 */
int vme_slots = 1024;

bitmap_data_t vme_slot_bitmap;	/* bitmap of available vme slot addresses */

/*
 * Allocate the next available range in VME address space and
 * use those pages to map in the specified physical address range, and
 * and return the virtual address which provides that mapping.
 *
 * This assumes that the physical address space is spanned by a contiguous
 * set of VME map registers.
 *
 * Return NULL if page is out of range.
 */
vm_offset_t
vme_master_mapin (physaddr, slotp, range)
	vm_offset_t physaddr;
	int *slotp;
	int  range;
{
	long offset;
	int page;

	offset = physaddr & (VM_PAGESIZE - 1);
	physaddr &= ~(VM_PAGESIZE - 1);
	
	if ((page = vme_slot_alloc(range)) == -1)
		return(NULL);

	*slotp = page;		/* return the first page index */

#define slot_to_vaddr(a, b) \
	(VEC_VME_MASTER_MAP + (VM_PAGESIZE * (a)) + (b))

#ifdef BMA_DEBUG
	printf("vme_master_mapin: slot=0x%x, range=0x%x\n",
		       page, range);
#endif BMA_DEBUG

	while(range > 0) {
#ifdef BMA_DEBUG
		printf("vme_master_mapin: va=0x%x, pa=0x%x\n",
		       slot_to_vaddr(page, 0), physaddr);
#endif BMA_DEBUG
		pte_mapin(&vme_slot_ptes[page], physaddr, MMU_PG_TYPE);

		mtpr(TBIS, slot_to_vaddr(page, 0));

		++page;
		physaddr += VM_PAGESIZE;
		range -= VM_PAGESIZE;
	}

	return (slot_to_vaddr(*slotp, offset));
}

/*
 * vme_slot_alloc - allocate a range of contiguous vme slots
 * returns the initial page index.
 */
int vme_slot_alloc(range)
	int  range;
{
	int s = splhigh();
	bitnum_t page;

	range = vm_round_page(range)/VM_PAGESIZE; /* number of pages we need */

	simple_lock(&vme_slot_lock); /* Grab the bitmap */

	/* Find a contiguous region */
	page = findbitrange(&vme_slot_bitmap, range);
	
	if (page != NOBIT)
		/* Mark those pages as being in use */
		clearbitrange(&vme_slot_bitmap, page, range);

	simple_unlock(&vme_slot_lock);

	splx(s);

	return((int)page);
}

/*
 * vme_slot_free(page, range) - free a range of vme slots starting at
 * a given address.
 */
vme_slot_free(page, range)
	int page;		/* vme slot number */
	int  range;
{
	int s = splhigh();

	/* number of pages to free */
	range = vm_round_page(range)/VM_PAGESIZE;

	simple_lock(&vme_slot_lock);

	setbitrange(&vme_slot_bitmap, (bitnum_t)page, range);
	simple_unlock(&vme_slot_lock);

	/* We must also invalidate out local mapping for the address range */
	while(range-- > 0) {
		mtpr(TBIS, slot_to_vaddr(page, 0));
		++page;
	}

	splx(s);
}


/* vectorized variables describing each node's VME slave mapper */
/* RAM allocation pool.                                         */

struct vme_slave_map_entry {
	unsigned nextidx;
};

/*  map of free of master entries */
extern bitmap_data_t my_vme_master_map_bitmap;
extern int	my_vme_slave_map_entries;  /* head of list of slave entries */
extern char     my_slave_map_needed;      /* per-node flag indicating a
					   * process needs slave_map registers.
					   */
extern simple_lock_data_t my_vme_map_alloc_lock; /* master/slave synch lock */

/*****************************************************************************
* vme_map_alloc_init - initialize the per-node chains of VME master          *
* and slave mapper RAM entries for the indicated node.  (In our world,       *
* this is always the current node, but retain the generality of using        *
* vvrmtptr.)                                                                 *
*****************************************************************************/
vme_map_alloc_init (lognode)
	int lognode;
{
	vm_offset_t bfly2_paddr, vme_addr, mmap_entry_check;
	vm_offset_t vme_master_map_alloc();
	unsigned long *slave_map_registers;
	unsigned int slave_map_null_entry;
	int i, *ip;
	u_int his_mb_phys_start;
	extern u_int mb_phys_start;
	vm_offset_t p;

	if (lognode != my_cpu_number) 
		panic("vme_map_alloc_init: not local node");

	/* Initialize the vme_map_alloc lock for lognode */
	simple_lock_init (vvrmtptr(&my_vme_map_alloc_lock, lognode));

	/* MASTER MAPPER: Map Butterfly References out onto VME BUS
	 *                (e.g. for DEVICE REGISTERS).
	 * The Master Mapper cannot be loaded directly, but uses a special
	 * hardware hack as indicated below.
	 *
	 * Initialize the free list by filling the entry with a
	 * pointer to the next free master mapper ram.
	 */

	his_mb_phys_start = *(u_int *)
		vvrmtptr(&mb_phys_start, lognode);

	if (his_mb_phys_start == VME_WINDOW_NONE)
		panic("vme_map_alloc_init: no vme on node %d\n", lognode);

	if (lognode == MASTER_CPU) {
		/* Allocate a bitmap for use when allocating 
		 * vme addresses
		 */
		create_bitmap(&vme_slot_bitmap, vme_slots, 1);

		simple_lock_init(&vme_slot_lock);
	}
		
	/*
	 * Initialize the bitmap of master mapper ram entries
	 */
	create_bitmap(&my_vme_master_map_bitmap, NUM_MMAP_ENTRIES, 1);

	/* Init the map simple lock */
	simple_lock_init(vvrmtptr(&my_vme_map_alloc_lock, lognode));

	/* Map 1 page for the IACK block */
	/* This had better allocate page 0 */
	(void) vme_master_map_alloc(0, my_cpu_number, 0, MMAP_ENTRY_SIZE);

#ifdef BMA_DEBUG
	printf("vme_map_alloc_init: Master Map Initialized\n");
#endif BMA_DEBUG

	/* SLAVE MAPPER: Map VME references into Butterfly physical memory
	 *               (e.g. for DMA by devices).
	 * The slave map registers can be loaded directly, unlike the master
	 * map registers.
	 *
	 * First, set all the registers to null entries.  Then go back and
	 * fix up the ones in the window in which we are interested.
	 *
	 * Make a template that maps a VME reference to an invalid
	 * butterfly2 physical address with paths disabled and fill
	 * each slave mapper ram with this null entry.
	 */
	slave_map_null_entry = paddr2smram(INVALID_PHYS_ADDR) |
			       SLAVE_MAP_TEMPLATE;
	for (i = 0, slave_map_registers = VME_SLAVE_MAP;
	     i < NUM_SMAP_ENTRIES; i++)
		slave_map_registers[i] = slave_map_null_entry;

	/* Now statically map starting at VME_SLAVE_START, the first 4M
	 * of the local node's physical memory so that old device drivers
	 * which do not use the slave mapper allocation routines will
	 * continue to work.
	 */
	slave_map_registers = VME_SLAVE_MAP +
			      (VME_SLAVE_START / SMAP_ENTRY_SIZE);
	for (i = 0, bfly2_paddr = 0;
	     i < VME_SLAVE_SIZE / SMAP_ENTRY_SIZE;
	     i++, bfly2_paddr += SMAP_ENTRY_SIZE)
		slave_map_registers[i] = paddr2smram(bfly2_paddr) |
					 SLAVE_MAP_TEMPLATE;

	/* Gound the free list head (should already be statically set) */
	*((int *) vvrmtptr(&my_vme_slave_map_entries, lognode)) =
		VME_MAP_CHAIN_END;

	/* Add first 512K worth of registers in window
	 * to this node's free list.
	 */
	for (vme_addr = VME_SLAVE_START;
	     vme_addr < VME_SLAVE_START + (512 * 1024);
	     vme_addr += SMAP_ENTRY_SIZE)
	{
		/* Might want to DMA to vectorized page */
		if (vme_addr == (VME_SLAVE_START + VECPG_POFFSET))
		  continue;
		vme_slave_map_free(vme_addr, lognode);
	}

#ifdef BMA_DEBUG
	printf("vme_map_alloc_init: Slave Map initialized\n");
#endif BMA_DEBUG
}

/*****************************************************************************
* vme_master_map_alloc - allocate VME master mapper RAM entries on the       *
* indicated node which points to the supplied VME-bus address and uses       *
* the indicated address modifier.  Returns the physical address to use       *
* and which may be also be handed to vme_master_map_free when.               *
* If the address modifier specified is NULL, then set the IACK bit.          *
* Allocates enough contiguous entries to cover the indicated range.          *
* Note that this routine relies on the free list being initially in ascending*
* order.
*****************************************************************************/

vm_offset_t
vme_master_map_alloc (vmeaddr, lognode, addrmod, range)
	vm_offset_t vmeaddr;
	int lognode;
	int addrmod;
        int  range;
{
	simple_lock_t his_vme_map_alloc_lock;
	bitmap_t his_vme_master_map_bitmap;
	bitnum_t free_map_addr;
	u_int his_mb_phys_start;
	vm_offset_t physaddr, first_physaddr;
	int s;
	
	if (lognode != my_cpu_number)
		panic("vme_master_map_alloc: not my node");

	/* Find the map allocation lock */
	his_vme_map_alloc_lock = (simple_lock_t) 
		vvrmtptr(&my_vme_map_alloc_lock, lognode);

	/* Find the start of the bitmap of free pages */
	his_vme_master_map_bitmap =
		(bitmap_t) vvrmtptr(&my_vme_master_map_bitmap, lognode);

	/* Find the start of his mb address space */
	his_mb_phys_start = *(u_int *)vvrmtptr(&mb_phys_start, lognode);

	if (his_mb_phys_start == VME_WINDOW_NONE)
		panic("vme_master_map_alloc: no vme on node %d\n", lognode);

	/* Convert address range to number of entries needed */
	range =	vm_round_page(range)/MMAP_ENTRY_SIZE;

	if (range == 0)
		panic("vme_master_map_alloc: empty address range");

	s = splhigh();
	simple_lock(his_vme_map_alloc_lock);

	free_map_addr = findbitrange(his_vme_master_map_bitmap,
				     range);

	if (free_map_addr == NOBIT) {
		simple_unlock(his_vme_map_alloc_lock);
		splx(s);
		printf("vme_master_map_alloc: out of master mapper ram");
		return(-1);
	}

	clearbitrange(his_vme_master_map_bitmap,
		      free_map_addr,
		      range);

#ifdef BMA_DEBUG
	printf("vme_master_map_alloc: got regs %d, npages=%d\n",
	       free_map_addr, range);
#endif BMA_DEBUG

	/* Fill in the master map registers */
	free_map_addr = free_map_addr*MMAP_ENTRY_SIZE + his_mb_phys_start;
	for(first_physaddr = 0;
	    range-- > 0; 
	    free_map_addr += MMAP_ENTRY_SIZE,
	    vmeaddr += MMAP_ENTRY_SIZE) {
		unsigned offset;
		/* Fill in the newly allocated mapper entry with the vmeaddr 
		 * and AM
		 */
		IAR = free_map_addr;
		IDR = VME_ADDR_FIELD(vmeaddr) | (addrmod << MMAP_AM_SHIFT) |
			(addrmod ? NO_IACK : 0);
		dummy_use_value(WRITE_MMAP_RAM);

		/* Create the physical address */
		offset = free_map_addr + (vmeaddr & (MMAP_ENTRY_SIZE - 1));
		physaddr = paddrcreate(log_to_phys_map[lognode], offset);

		/* Make the address bypassed */
		physaddr = paddrsetbypassed(physaddr);
#ifdef BMA_DEBUG
		printf("vme_master_map_alloc: IAR=0x%x, vme=0x%x, phys=0x%x\n",
		       free_map_addr, vmeaddr, physaddr);
#endif BMA_DEBUG
		if (first_physaddr == 0)
			first_physaddr = physaddr;
	}

	simple_unlock(his_vme_map_alloc_lock);
	splx(s);

	/* Return the address of the first allocated mapper plus offset */
	return (first_physaddr);
}

/*
 * Specify a new vmebus mapping for the range of master
 * mapping rams located at physaddr on lognode.
 */
vme_master_map_realloc (physaddr, vmeaddr, lognode, range)
	vm_offset_t physaddr;
	vm_offset_t vmeaddr;
	int lognode;
        int  range;
{
	simple_lock_t his_vme_map_alloc_lock;
	int addrmod;
	int tmpidr;
	int s;
	
	if (lognode != my_cpu_number)
		panic("vme_master_map_realloc: not my node");

	/* Find the map allocation lock */
	his_vme_map_alloc_lock = (simple_lock_t) 
		vvrmtptr(&my_vme_map_alloc_lock, lognode);

	/* Convert address range to number of entries needed */
	range =	vm_round_page(range)/MMAP_ENTRY_SIZE;

	if (range == 0)
		panic("vme_master_map_realloc: empty address range");

	s = splhigh();
	simple_lock(his_vme_map_alloc_lock);

	physaddr = paddr2bankoffset(vm_trunc_page(physaddr));

	/* Fill in the master map registers */
	while (range-- > 0) {
		/* Fill in the newly allocated mapper entry with the vmeaddr 
		 * and AM
		 */
		IAR = physaddr;
		dummy_use_value(READ_MMAP_RAM);
		tmpidr = IDR;
		addrmod = (tmpidr >> MMAP_AM_SHIFT) & 0x3f;

		IAR = physaddr;
		IDR = VME_ADDR_FIELD(vmeaddr) | (addrmod << MMAP_AM_SHIFT) |
			(addrmod ? NO_IACK : 0);
		dummy_use_value(WRITE_MMAP_RAM);

		vmeaddr += MMAP_ENTRY_SIZE;
		physaddr += MMAP_ENTRY_SIZE;
	}
	simple_unlock(his_vme_map_alloc_lock);
	splx(s);
}

/*****************************************************************************
* vme_master_map_free - returns the mapper entry corresponding to the        *
* indicated physical address to the allocation pool on lognode.              *
*****************************************************************************/

vme_master_map_free (physaddr, lognode, range)
	vm_offset_t physaddr;
	int lognode;
    	int  range;
{
	simple_lock_t his_vme_map_alloc_lock;
	bitmap_t his_vme_master_map_bitmap;
	vm_offset_t free_map_addr;
	int s;
	int first_free_reg;
	u_int his_mb_phys_start;

#ifdef BMA_DEBUG
	printf("vme_master_map_free (0x%x, %d, %d)\n", physaddr, lognode, 
	       range);
#endif BMA_DEBUG

	/* Find the map allocation lock */
	his_vme_map_alloc_lock = (simple_lock_t) 
		vvrmtptr(&my_vme_map_alloc_lock, lognode);

	/* and the free register bitmap */
	his_vme_master_map_bitmap = 
		(bitmap_t) vvrmtptr(&my_vme_master_map_bitmap, lognode);

	his_mb_phys_start = *(u_int *)
		vvrmtptr(&mb_phys_start, lognode);

	if (his_mb_phys_start == VME_WINDOW_NONE)
		panic("vme_master_map_free: no vme on node &d\n",
		      lognode);

	/* Convert range size to number of registers */
	range = vm_round_page(range)/MMAP_ENTRY_SIZE;

	s = splhigh();
	simple_lock(his_vme_map_alloc_lock);

	/* Convert the physical address back to a register offset */
	first_free_reg =
	  vm_trunc_page(paddr2memoffset(physaddr) - his_mb_phys_start)
		  				/MMAP_ENTRY_SIZE;
	
#ifdef BMA_DEBUG
	printf("vme_master_map_free: setting bits %d - %d+%d\n",
	       first_free_reg, first_free_reg, range);
#endif BMA_DEBUG

	setbitrange(his_vme_master_map_bitmap,
		    first_free_reg,
		    range);
	
	simple_unlock(his_vme_map_alloc_lock);
	splx(s);
}

/*****************************************************************************
* vme_slave_map_alloc - allocate a VME slave mapper RAM entry on the         *
* indicated node which points to the supplied physical address.  Returns the *
* VME-bus address to use, which must be handed to vme_slave_map_free.        *
* Amateur neurosurgery will almost certainly spell disaster.                 *
*****************************************************************************/
vme_slave_map_alloc (physaddr, lognode)
	vm_offset_t physaddr;
	int lognode;
{
	simple_lock_t   his_vme_map_alloc_lock;
	int s;
	unsigned        vme_address;

	his_vme_map_alloc_lock = (simple_lock_t) 
		vvrmtptr (&my_vme_map_alloc_lock, lognode);

	s = splhigh();
	simple_lock (his_vme_map_alloc_lock);

	if ((vme_address = vme_slave_map_alloc_internal(physaddr,lognode)) ==
	    VME_SLAVE_MAP_ALLOC_EMPTY)
		panic_printf ("vme_slave_map_alloc",
			      "vme_slave_map_alloc: node %d\n", lognode);

	simple_unlock (his_vme_map_alloc_lock);
	splx(s);

	return(vme_address);
}

/* vme_multi_slave_map_alloc -
 *    This routine allows allocating more than one vme_slave_map register.
 * The flag variable is set to one of three types of allocation:
 *
 * VME_SLAVE_MAP_ALLOC_WAIT_P:
 *     Allocate count slave_map registers. If there aren't enough slave_map
 * registers to satisfy the request, poll until there are enough free registers
 * to satisfy the request.
 *
 * VME_SLAVE_MAP_ALLOC_WAIT_I:
 *     Allocate count slave_map registers. If there aren't enough slave_map
 * registers to satisfy the request, goto sleep and wait for an I/O interrupt
 * that frees registers and try again.
 *
 * VME_SLAVE_MAP_ALLOC_NOWAIT:
 *    If there aren't enough slave_map registers to satisfy this request,
 * return VME_SLAVE_MAP_ALLOC_EMPTY.
 *
 *  IN: physaddrs - This is a pointer to an array of physical addresses.
 *  IN: lognode   - This is the logical node where the slave+_map register are
 *                  to be allocated from.
 *  IN: count     - This is the number of slave_map register needed.
 *  IN: flag      _ This describes the allocation method.
 * OUT: vmeaddrs  _ This is an array of vme addresses corresponding to the
 *                  count physaddrs.
 */
int vme_slave_map_needed_i,vme_slave_map_needed_p;

vme_multi_slave_map_alloc (physaddrs, vmeaddrs, lognode, count, flag)
	vm_offset_t *physaddrs;
	unsigned long *vmeaddrs;
	int lognode;
	int flag;
{
	simple_lock_t   his_vme_map_alloc_lock;
	char           *his_slave_map_needed;
	int s;
	unsigned        vme_address;
	int i;


	his_vme_map_alloc_lock = (simple_lock_t) 
		vvrmtptr (&my_vme_map_alloc_lock, lognode);

	s = splhigh();
 tryagain:
	simple_lock (his_vme_map_alloc_lock);
	for (i = 0 ; i < count; i++) {
		vmeaddrs[i] = 
			vme_slave_map_alloc_internal(physaddrs[i],lognode);

		if (vmeaddrs[i] == VME_SLAVE_MAP_ALLOC_EMPTY) {
			int j;

			for (j = 0; j < i; j++)
				vme_slave_map_free(vmeaddrs[j],lognode);

			/* Check to see if allocation mode is interruptable. */
			if (flag & VME_SLAVE_MAP_ALLOC_WAIT_I) {
				/* Get the slave_map_needed on lognode. */
				his_slave_map_needed = (char *)
					vvrmtptr(&my_slave_map_needed,lognode);
				/* Mark it as needing slave_map registers. */
				++vme_slave_map_needed_i;
				*his_slave_map_needed = TRUE;
				/* Go to sleep; wait for some to be freed. */
				sleep_and_unlock(his_slave_map_needed, PRIBIO,
						 his_vme_map_alloc_lock);
				/* Try it again. */
				goto tryagain;
			}
			/* Check to see if allocation mode is pollable. */
			if (flag & VME_SLAVE_MAP_ALLOC_WAIT_P) {
				/* Unlock the slave_map lock. */
				simple_unlock(his_vme_map_alloc_lock);
				/* Lower the priority to allow interrupts. */
				splx(s);
				++vme_slave_map_needed_p;
				/* Wait a while. */
				DELAY(100);
				/* Block out interrupts. */
				s = splhigh();
				/* Try it again. */
				goto tryagain;
			}
			/* Check to see if allocation mode is NOWAIT. */
			if (flag & VME_SLAVE_MAP_ALLOC_NOWAIT) {
				/* Unlock the slave_map lock. */
				simple_unlock(his_vme_map_alloc_lock);
				/* Lower the priority to allow interrupts. */
				splx(s);
				return(VME_SLAVE_MAP_ALLOC_EMPTY);
			} else /* IF all else fails .. panic. */ {
				panic_printf ("vme_slave_map_alloc",
					      "vme_slave_map_alloc: node %d\n",
					      lognode);
			}
		}
	}

	simple_unlock (his_vme_map_alloc_lock);
	splx(s);

	return(VME_SLAVE_MAP_ALLOC_OK);
}

vme_slave_map_alloc_internal(physaddr, lognode)
	vm_offset_t physaddr;
	int lognode;
{
	int             *his_vme_slave_map_entries;
	struct vme_slave_map_entry    *slave_map_registers;
	int             entry;
	unsigned        vme_address;

	/* first make sure we are not trying to set up a smram to point
	   across the switch.
	 */
	if (paddr2pnode(physaddr) != log_to_phys_map[lognode]) {
		printf("vme_slave_map_alloc_internal: non-local mapping");
		dump_full_stack();
	}

	his_vme_slave_map_entries =
	  (int *) vvrmtptr (&my_vme_slave_map_entries, lognode);

	/* build pointer in vectorized slave mapper RAM space to this */
	/* node's mapper RAM entries.  Adjust it to point to this node's */
	/* window of entries we are interested in. */

	slave_map_registers = VEC_VME_SLAVE_MAP +
	  (lognode*NUM_SMAP_ENTRIES) + (VME_SLAVE_START/SMAP_ENTRY_SIZE);
		
	/* allocate first entry on chain */
	entry = *his_vme_slave_map_entries;

	if ((entry & VME_MAP_CHAIN_MASK) == VME_MAP_CHAIN_END)
		return(VME_SLAVE_MAP_ALLOC_EMPTY);

	/* advance head of list to next entry */
	*his_vme_slave_map_entries =
	  slave_map_registers [entry].nextidx & VME_MAP_CHAIN_MASK;

	/* The appropriate vme bus address (ON LOGNODE) is the start of the */
	/* slave VME window plus 8K times the number of mapper RAM entries  */
	/* from the base of the window.                                     */

	vme_address = VME_SLAVE_START + 
		SMAP_ENTRY_SIZE * entry;
	slave_map_registers [entry].nextidx =
	  paddr2smram(physaddr) | SLAVE_MAP_TEMPLATE;
	/* DEBUGGING */
#ifdef DEBUG
	printf("vme_slave_map_alloc: returning 0x%x for physaddr 0x%x\n",
		   vme_address, physaddr);
 	printf("\t(Entry: 0x%x (0x%x) vs ", entry, *((unsigned *)entry));
	entry = &slave_map_registers[physoffset(physaddr)/SMAP_ENTRY_SIZE];
 	printf("\tEntry: 0x%x (0x%x) )\n", entry, *((unsigned *)entry));
#endif DEBUG
	/* Return offset in vme space corresponding to target physical addr */
	return ((int) vme_address + (physaddr & (SMAP_ENTRY_SIZE-1)));
}

 
/*****************************************************************************
* paddr2smram()
*	Convert a physical address into an entry for a VME slave mapper
*	ram register that maps that address.
******************************************************************************/

	/* different fields in slave mapper ram entry */
#define	SMAP_HIGHBAY_MASK	0x80000000		/* high bay (1 bit) */
#define	SMAP_LOWBAY_MASK	0x00000003		/* high bay (1 bit) */
#define	SMAP_NODE_MASK		0x7E000000		/* node bits (6 bits)*/
#define	SMAP_OFFSET_MASK	0x01FFE000	/* offset on node (12 bits) */
		/* entry magic bits or offset in entry (13 bits) */
#define	SMAP_ENTRY_OFFSET	(SMAP_ENTRY_SIZE-1)  

#define	SMAP_NODE_SHIFT	25			        /* loc of node bits */
#define	SMAP_BAY_BITS(bay)	( ((bay)  >> 1) | (((bay) & 1) << 31) )
#define	smram2bay(smram)					\
	((( ((unsigned)(smram)) & SMAP_HIGHBAY_MASK) >> 31) |	\
	 ((((unsigned)(smram)) & SMAP_LOWBAY_MASK) << 1))

paddr2smram(physaddr)
	vm_offset_t physaddr;
{
	unsigned result;
	unsigned node;
	short bay;
	
	/* Mask off low bits */
	physaddr &= MASK_AD31to13;
	result = 
	  paddr2bank(physaddr) << 23		|
	  paddr2bankoffset(physaddr);

	/* If not local address, add in node and bay bits */
	if (node = paddr2pnode(physaddr))
	{
	    bay = (node & BAY_MASK) >> BAY_SHIFT;
	    result |= (SMAP_BAY_BITS(bay) 
		       | ((node & (SLOT_MASK|MIDPLANE_MASK)) <<
			  SMAP_NODE_SHIFT));
        }

	return(result);
}

/*****************************************************************************
* paddr2smram()
*	Given an entry in a VME slave mapper ram register, return the
*	physical address that it maps.
******************************************************************************/
smram2paddr(smram)
unsigned	smram;
{
	unsigned	node;
	unsigned	offset;
	
	node = (smram2bay(smram) << BAY_SHIFT) +
		((smram >> SMAP_NODE_SHIFT) & (SLOT_MASK|MIDPLANE_MASK));
	offset = smram & SMAP_OFFSET_MASK;
	
	return(paddrcreate(node, offset));
}
/*****************************************************************************
vme_slave_map_map - map a new physical address into a slave map register
*****************************************************************************/
vme_slave_map_map(vme_address, lognode, physaddr)
	vm_offset_t vme_address, physaddr;
	int lognode;
{
	struct vme_slave_map_entry *slave_map_registers;
	int                         entry;

	vme_address &= ~(SMAP_ENTRY_SIZE-1);

	slave_map_registers = VEC_VME_SLAVE_MAP +
	  (lognode*NUM_SMAP_ENTRIES) + (VME_SLAVE_START/SMAP_ENTRY_SIZE);

	entry = (vme_address - VME_SLAVE_START) / SMAP_ENTRY_SIZE;

	slave_map_registers [entry].nextidx =
	  paddr2smram(physaddr) | SLAVE_MAP_TEMPLATE;
}

/*****************************************************************************
* vme_slave_map_free - returns the mapper entry corresponding to the         *
* indicated vme_address to the allocation pool on lognode.  Disaster awaits  *
* if the vme_address or the lognode parameters are broken.                   *
*****************************************************************************/

vme_slave_map_free (vme_address, lognode)
	vm_offset_t vme_address;
	int lognode;
{
	simple_lock_t               his_vme_map_alloc_lock;
	int                        *his_vme_slave_map_entries;
	char                       *his_slave_map_needed;
	struct vme_slave_map_entry *slave_map_registers;
	int                         entry;
	int                         s;

	/* build pointer in vectorized slave mapper RAM space to this 
	 * node's mapper RAM entries.  Adjust it to point to this node's 
	 * window of entries we are interested in.
	 */
	vme_address &= ~(SMAP_ENTRY_SIZE-1);
	
	slave_map_registers = VEC_VME_SLAVE_MAP +
	  (lognode*NUM_SMAP_ENTRIES) + (VME_SLAVE_START/SMAP_ENTRY_SIZE);
		
	/* Compute the map entry virtual address from the supplied VME 
	 * address
	 */
	entry = (vme_address - VME_SLAVE_START) / SMAP_ENTRY_SIZE;

	/* XXX - debugging */
	if (entry >= VME_MAP_CHAIN_MASK)
		panic_printf("vme_slave_map_free",
			      "bad vme_address = 0x%x, node=%d, entry=%d\n",
			      vme_address, lognode, entry);
	if (slave_map_registers[entry].nextidx <= VME_MAP_CHAIN_MASK)
		panic_printf("vme_slave_map_free",
			     "entry already free, vme_address=0x%x, node=%d, old value 0x%x\n",
			     vme_address, lognode,
			     slave_map_registers[entry].nextidx);
		      
	his_vme_map_alloc_lock = (simple_lock_t) 
		vvrmtptr (&my_vme_map_alloc_lock, lognode);

	his_vme_slave_map_entries =
		(int *) vvrmtptr (&my_vme_slave_map_entries, lognode);

	s = splhigh();
	simple_lock (his_vme_map_alloc_lock);

	/* Linklist insertion of this map entry at head of list */
	slave_map_registers [entry].nextidx = *his_vme_slave_map_entries;

	/* Head now points at this entry */
	*his_vme_slave_map_entries = entry;

	/* Get the slave_map_needed on lognode. */
	his_slave_map_needed = (char *)	vvrmtptr(&my_slave_map_needed,lognode);
	if (*his_slave_map_needed == TRUE) {
		*his_slave_map_needed = FALSE;
		wakeup(his_slave_map_needed);
	}
	simple_unlock (his_vme_map_alloc_lock);
	splx (s);
}

vmeaddr2paddr (vme_address, lognode)
	vm_offset_t vme_address;
	int lognode;
{
	struct vme_slave_map_entry *slave_map_registers;
	int                         entry;
	int                         s;
	unsigned		    offset;

	/*printf("vmeaddr2paddr: address: 0x%x, lognode: 0x%x\n",
	  vme_address, lognode);*/
	/* build pointer in vectorized slave mapper RAM space to this */
	/* node's mapper RAM entries.  Adjust it to point to this node's */
	/* window of entries we are interested in. */

	slave_map_registers = VEC_VME_SLAVE_MAP +
	  (lognode*NUM_SMAP_ENTRIES) + (VME_SLAVE_START/SMAP_ENTRY_SIZE);
		
	/* Compute map entry index from supplied VME address */
	offset = vme_address & (SMAP_ENTRY_SIZE-1);
	vme_address &= ~(SMAP_ENTRY_SIZE-1);
	entry = (vme_address - VME_SLAVE_START) / SMAP_ENTRY_SIZE;

	return(smram2paddr(slave_map_registers[entry].nextidx) + offset);
}


/*
 * Interface to the 'vme' device. 
 *
 * This is mostly just used as a hook for the vm_mapmem system call to
 * map vme memory.
 */
#include "errno.h"

vmeopen(dev, flag)
	dev_t dev;
	int flag;
{
	/* Make sure that the IOBUS exists */

	if (bmaid_to_log_node(iobusid(dev)) == NULL_NODE)
		return(ENXIO);
	return(0);
}

vmeclose(dev, flag)
	dev_t dev;
	int flag;
{
	return(0);		/* No cleanup needed */
}
#endif BFLY2

/*
 * Emacs C format control
 *
 * Local Variables:
 * c-indent-level: 8
 * c-continued-statement-offset: 8
 * c-brace-offset: -8
 * c-argdecl-indent: 8
 * c-label-offset: -8
 * End:
 */
