#ifdef	BFLY2
char *dr_vstr = "$Header: /nfs/columbia/u1/BFMACH.local/mk/butterflydev/RCS/dr.c,v 1.11 91/01/14 14:32:29 tappan Exp $";

#include "dr.h"
#if	NDR > 0
/*
 * File: dr.c
 * Definitions for the Ikon 10089 buffered VMEbus DR11-W emulator.
 *
 * September 2, 1989 Thomas W. Uban
 *
 * Copyright 1989 by BBN Advanced Computers Inc.
 * ALL RIGHTS RESERVED
 *
 * No part of this material may be reproduced, stored in a retrieval
 * system, transmitted or used in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise, without the prior
 * written permission of, or express license from BBN Advanced
 * Computers Inc.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights
 * to Technical Data and Computer Software clause at DFARS 52.227-7013.
 *
 * BBN Advanced Computers Inc.
 * 10 Fawcett Street
 * Cambridge, Ma. 02238
 * 617-873-6000
 */

/*
 * The DR11-W is a high-speed parallel DMA interface often used
 * for interprocessor communication links. The DR11-W supports
 * a 16 bit data path plus control and handshake signal paths.
 */

#include "param.h"
#include "systm.h"
#include "ioctl.h"
#include "errno.h"
#include "lock.h"
#include "buf.h"
#include "../butterfly/ioconf.h"
#include "../butterfly/cpu.h"
#include "../butterfly/vm_param.h"
#include "../butterfly/b2vme.h"
#include "../butterfly/mmu.h"
#include "../butterflydev/drreg.h"
#include "../butterflydev/mbvar.h"
#include "../butterflydev/mbreg.h"

int drprobe(), drattach(), drintr();
int dropen(), drclose(), drstrategy(), drread(), drwrite(), drioctl();
u_long vme_slave_map_alloc();
char *rcs_version(), *dr_status();

/*
 * Device configuration structures and device specific datum
 * declared and initialized in the kernel vectorized page.
 * (see mk/machine/locore_vec.s)
 */
#define VECTORIZED extern
VECTORIZED struct mb_ctlr *drctlr[];
VECTORIZED struct mb_device *drdevice[];
VECTORIZED struct dr_softc *dr_softc[];
VECTORIZED int dr_chan[];
struct mb_driver drdriver = {
    drprobe,
    0,
    drattach,
    0,
    0,
    drintr,
    sizeof(struct drdevice),
    "dr",
    drdevice,
    "dr",
    drctlr,
    MDR_DMA,
    0
    };

extern int ndr;	/* text space NDR variable */

/*
 * Probe the device on the local node to see if it
 * exists. If it does, allocate memory on the local
 * node of the device for per device information,
 * DMA copy buffers, etc.
 *
 * Must only be called from the local node.
 */
drprobe (addr, unit)
    caddr_t addr;
    int unit;
{
    struct dr_softc *xs;
    caddr_t cp;
    int lnode;
    int size;

    if (unit >= ndr)
	return(0);
    
    /*
     * Poke at the device register to see if the board is there.
     */
    if (peekc(addr) == -1)
	return(0);

    /*
     * Check to be sure that the interrupt handler is installed.
     */
    if (VecTable[DR_VEC] != drintr) {
	printf("%s%u: drprobe: interrupt handler not in VecTable\n",
	       drdriver.mi_dname, unit);
	return(0);
    }

    /*
     * Make sure there is room for the copy buffer.
     */
    if (NBPG < DR_MINPHYS) {
	printf("%s%u: drprobe: no memory for raw buffer\n",
	       drdriver.mi_dname, unit);
	return(0);
    }

    /*
     * Allocate a local page for the per device structure, etc.
     */
    size = vm_round_page(sizeof(struct dr_softc));
    
    if (size > NBPG) {
	printf("%s%u: drprobe: device structure too big (%d)\n",
	       drdriver.mi_dname, unit, size);
	return(0);
    }
    
    /*
     * The probe routine gets called on the node in question.
     */
    lnode = my_cpu_number;
    cp = (caddr_t)vm_kmem_alloc_on_node(size, lnode);
    if (cp == 0) {
	printf("%s%u: drprobe: no memory for private structures\n",
	       drdriver.mi_dname, unit);
	return(0);
    }
    xs = dr_softc[unit] = (struct dr_softc *)cp;
    bzero((caddr_t)xs, sizeof(struct dr_softc));
    cp += ((sizeof(struct dr_softc) + sizeof(long)) &
	   ~(sizeof(long)-1));
    xs->xs_rawbp = (struct buf *)cp;
    cp += sizeof(struct buf);
    xs->xs_rawbuf = cp;
    xs->xs_rawbuf_busaddr =
	vme_slave_map_alloc(svtop(xs->xs_rawbuf), lnode);
    simple_lock_init(&xs->xs_lock);
    
    /*
     * Record the logical node number and mark the device as alive.
     */
    xs->xs_lnode = lnode;
    xs->xs_flags |= DR_ALIVE;
    
    return(sizeof(struct drdevice));
}

/*
 * The device exists, so reset the board and enable interrupts.
 *
 * Must only be called from the local node.
 */
drattach (md)
    struct mb_device *md;
{
    int unit = md->md_unit;
    struct dr_softc *xs = dr_softc[md->md_unit];
    int seconds;
    int s;
    
    if (unit >= ndr || xs == (struct dr_softc *)0)
	return(-1);
    
    xs->xs_intlevel = vme2syspri(md->md_intpri);

    s = spldr();
    simple_lock(&xs->xs_lock);
    
    /*
     * Store away a pointer to the device information.
     */
    xs->xs_md = md;
    
    /*
     * Save away the device address in our local information.
     */
    xs->xs_dr = (struct drdevice *)md->md_addr;

    /*
     * Reset the device and ensure that it goes ready.
     */
    xs->xs_dr->dr_pulse = PUL_MCLR;
    seconds = 0;
    while ((xs->xs_dr->dr_csr & STS_REDY) == 0) {
	DELAY(DR_ONE_SECOND);
	if (++seconds > DR_WAIT_SECONDS) {
	    simple_unlock(&xs->xs_lock);
	    splx(s);
	    printf("%s%u: board failed to reset\n",
		   drdriver.mi_dname, unit);
	    return(-1);
	}
    }

    /*
     * Record the device's existence on the console.
     */
    printf("%s%u: [%s] Ikon 10089 buffered VMEbus DR11-W emulator\n",
	   drdriver.mi_dname, unit, rcs_version(dr_vstr));
    
    /*
     * Disable device interrupts,
     * set the address modifier
     * and interrupt vector.
     */
    xs->xs_dr->dr_pulse = PUL_RMSK;
    xs->xs_dr->dr_mod = DR_MOD;
    xs->xs_dr->dr_vec = DR_VEC;
    xs->xs_dmacnt = 0;

    simple_unlock(&xs->xs_lock);
    splx(s);
    
    return(0);
}

/*
 * DR11-W device interrupt. Scan the possible units for interrupt(s).
 */
#include "bbn_ihandler_stack_trace.h"
drintr (ef)
    int *ef;			/* exception frame on bfly2 */
{
    int unit;
    int *chanp;
    struct dr_softc *xs;
    struct mb_device *md;
    u_short status;
    
    for (unit = 0; unit < ndr; unit++) {
	xs = dr_softc[unit];
	if (xs == 0 || (md = xs->xs_md) == 0 || md->md_addr == 0)
	    continue;

	/*
	 * Find the vectorized page address of the wait channel.
	 */
	chanp = (int *)vvrmtptr(&dr_chan[unit], my_cpu_number);
	simple_lock(&xs->xs_lock);
	*chanp = 0;			/* return error status */
	status = xs->xs_dr->dr_csr;
	if (status & STS_BERR) {	/* DMA bus error */
	    printf("%s%u: bus error!\n", drdriver.mi_dname, unit);
	    xs->xs_dr->dr_pulse = PUL_RDMA;
	    *chanp = EIO;
	}
	if (status & STS_PERR) {	/* DMA parity error */
	    printf("%s%u: parity error!\n", drdriver.mi_dname, unit);
	    xs->xs_dr->dr_pulse = PUL_RPER;
		*chanp = EIO;
	}
	if (status & STS_ATTF) {	/* attention signal */
	    xs->xs_dr->dr_pulse = PUL_RATN;
	    if ((xs->xs_flags & DR_WAITING) == 0)
		*chanp = EIO;
	}
	if (status & STS_DMAF)		/* DMA End-Of-Range */
	    xs->xs_dr->dr_pulse = PUL_RDMA;

	if ((status & STS_IENB) == 0)
	    xs->xs_dr->dr_pulse = PUL_SMSK;

	/*
	 * Clear process waiting flag, clear write
	 * pending signal, and wake waiting process.
	 */
	xs->xs_flags &= ~DR_WAITING;
	xs->xs_dr->dr_csr = (xs->xs_dr->dr_csr & CTL_BITS & ~CTL_FCN3);
	wakeup((caddr_t)chanp);
	simple_unlock(&xs->xs_lock);
    }
#if BBN_IHANDLER_STACK_TRACE
    dummy_use_value(ef);	/* so compiler preserves the argument */
#endif
}

/*
 * Open the DR11-W device for use. Only allow one process to
 * have the device open at one time. If the device is already
 * open, then return EBUSY. Reset the device on successful open.
 */
dropen (dev, flag)
    dev_t dev;
    int flag;
{
    struct dr_softc *xs;
    int error;
    int lnode;
    int unit;
    int s;

    /*
     * Find the logical processor attached to the device's
     * iobus as well as the unit on that iobus.
     */
    error = 0;
    lnode = dev_to_log_node(dev);
    unit = minor(dev);

    /*
     * Verify that the device in question exists.
     */
    if (lnode == NULL_NODE || unit >= ndr)
	return(ENXIO);

    /*
     * Locate the device specific data structure for specified
     * unit and doublecheck the health of the device.
     */
    xs = *((struct dr_softc **)vvrmtptr(&dr_softc[unit], lnode));
    if (xs == (struct dr_softc *) NULL ||
	(xs->xs_flags & DR_ALIVE) == 0)
	return(ENXIO);

    /*
     * Allow only one process to control the device at one time.
     */
    s = spldr();
    simple_lock(&xs->xs_lock);
    if (xs->xs_flags & DR_OPEN)
	error = EBUSY;
    else
	xs->xs_flags |= DR_OPEN;

    /*
     * Reset the device and enable interrupts.
     */
    xs->xs_dr->dr_pulse = PUL_MCLR;
    xs->xs_dr->dr_pulse = PUL_SMSK;
    xs->xs_dmacnt = 0;
    simple_unlock(&xs->xs_lock);
    splx(s);

    return(error);
}

/*
 * Close the DR11-W device.
 */
drclose (dev, flag)
    dev_t dev;
    int flag;
{
    struct dr_softc *xs;
    int lnode;
    int unit;
    int s;

    /*
     * Find the logical processor attached to the device's
     * iobus as well as the unit on that iobus.
     */
    lnode = dev_to_log_node(dev);
    unit = minor(dev);

    /*
     * Verify that the device in question exists.
     */
    if (lnode == NULL_NODE || unit >= ndr)
	return(ENXIO);

    /*
     * Locate the device specific data structure for specified
     * unit and doublecheck the health of the device.
     */
    xs = *((struct dr_softc **)vvrmtptr(&dr_softc[unit], lnode));
    if (xs == (struct dr_softc *) NULL ||
	(xs->xs_flags & DR_ALIVE) == 0)
	return(ENXIO);

    /*
     * Reset the device, enable interrupts, and let
     * the next process opening the device succeed.
     */
    s = spldr();
    simple_lock(&xs->xs_lock);
    xs->xs_dr->dr_pulse = PUL_MCLR;	/* reset */
    xs->xs_dr->dr_pulse = PUL_RMSK;	/* disable interrupts */
    xs->xs_dmacnt = 0;
    xs->xs_flags &= ~DR_OPEN;
    simple_unlock(&xs->xs_lock);
    splx(s);

    return(0);
}

/*
 * DR11-W strategy routine. This device only supports character
 * special access, but performs read and write operations via
 * physio, thus the existence of this routine.
 */
drstrategy (bp)
    struct buf *bp;
{
    struct dr_softc *xs;
    u_long busaddr;
    u_long page0;
    u_long page1;
    char *cp;
    int error;
    int lnode;
    int unit;
    int len;
    int s;

    /*
     * Find the logical processor attached to the device's
     * iobus as well as the unit on that iobus.
     */
    lnode = dev_to_log_node(bp->b_dev);
    unit = minor(bp->b_dev);

    /*
     * Verify that the device in question exists.
     */
    if (lnode == NULL_NODE || unit >= ndr) {
	dr_bp_error(bp, ENXIO);
	iodone(bp);
	return;
    }

    /*
     * Locate the device specific data structure for specified
     * unit and doublecheck the health of the device.
     */
    xs = *((struct dr_softc **)vvrmtptr(&dr_softc[unit], lnode));
    if (xs == (struct dr_softc *) NULL ||
	(xs->xs_flags & DR_ALIVE) == 0) {
	dr_bp_error(bp, ENXIO);
	iodone(bp);
	return;
    }

    /*
     * Make sure the request specifies a valid size.
     */
    if (bp->b_bcount < 0) {
	dr_bp_error(bp, EIO);
	iodone(bp);
	return;
    }

    /*
     * While there is data to read/write and no
     * errors have occurred, move the data.
     */
    error = 0;
    cp = bp->b_un.b_addr;
    while (bp->b_bcount > 0 && error == 0) {
	len = min(bp->b_bcount, DR_MINPHYS);

	/*
	 * If the user buffer is local to the processor attached
	 * to the DR11-W's iobus and the buffer does not cross
	 * a physical page boundary, then don't use a copy buffer.
	 */
	page0 = uvtop(cp);	/* system virtual to physical */
	page1 = uvtop(cp + len);
	if (page0 == page1 &&
	    log_node_map[paddr2pnode(page0)] == xs->xs_lnode) {

	    /*
	     * Allocate a slave mapper RAM entry to map the
	     * VMEbus address to the physical page address.
	     */
	    busaddr = vme_slave_map_alloc(page0, xs->xs_lnode);

	    /*
	     * Setup and initiate the DMA operation.
	     */
	    error = dr_dma_rdwr(xs, busaddr, len,
				(bp->b_flags & B_READ) ?
				B_READ : B_WRITE);

	    /*
	     * Free the slave mapper RAM entry.
	     */
	    vme_slave_map_free(busaddr, xs->xs_lnode);
	} else {
	    /*
	     * Use the static slave mapper RAM entry, setup
	     * and initiate the DMA operation using the copy
	     * buffer.
	     */
	    busaddr = xs->xs_rawbuf_busaddr;
	    if (bp->b_flags & B_READ) {
		error = dr_dma_rdwr(xs, busaddr, len, B_READ);
		if (!error)
		    bcopy(xs->xs_rawbuf, cp, len);
	    } else {
		bcopy(cp, xs->xs_rawbuf, len);
		error = dr_dma_rdwr(xs, busaddr, len, B_WRITE);
	    }
	}

	/*
	 * If no error occurred, then update the
	 * count and bump the buffer pointer.
	 */
	if (!error) {
	    bp->b_bcount -= len;
	    cp += len;
	}
    }

    /*
     * If no error occurred, then we're done.
     */
    if (!error) {
	iodone(bp);
	return;
    }

    /*
     * An error occurred! Return the error code and residual.
     */
    bp->b_resid = bp->b_bcount;
    dr_bp_error(bp, error);
    iodone(bp);
    return;
}

/*
 * Read from the DR11-W raw device.
 */
drread (dev, uio)
    dev_t dev;
    struct uio *uio;
{
    return(dr_raw_rdwr(dev, uio, B_READ));
}

/*
 * Write to the DR11-W raw device.
 */
drwrite (dev, uio)
    dev_t dev;
    struct uio *uio;
{
    return(dr_raw_rdwr(dev, uio, B_WRITE));
}

/*
 * Ioctl support for /dev/dr? DR11-W device operations.
 */
drioctl (dev, cmd, data, flag)
    dev_t dev;
    int cmd;
    caddr_t data;
    int flag;
{
    struct dr_softc *xs;
    int error;
    int lnode;
    int unit;
    int s;

    /*
     * Find the logical processor attached to the device's
     * iobus as well as the unit on that iobus.
     */
    lnode = dev_to_log_node(dev);
    unit = minor(dev);

    /*
     * Verify that the device in question exists.
     */
    if (lnode == NULL_NODE || unit >= ndr)
	return(ENXIO);

    /*
     * Locate the device specific data structure for specified
     * unit and doublecheck the health of the device.
     */
    xs = *((struct dr_softc **)vvrmtptr(&dr_softc[unit], lnode));
    if (xs == (struct dr_softc *) NULL ||
	(xs->xs_flags & DR_ALIVE) == 0)
	return(ENXIO);

    /*
     * Perform the operation specified.
     */
    error = 0;
    switch (cmd) {
	case DR_IOCTL_RESET:	/* reset the device */
	    s = spldr();
	    simple_lock(&xs->xs_lock);
	    xs->xs_dr->dr_pulse = PUL_MCLR;	/* reset */
	    xs->xs_dr->dr_pulse = PUL_SMSK;	/* enable interrupts */
	    xs->xs_flags &= ~DR_WAIT;		/* clear write wait */
	    xs->xs_flags &= ~DR_WAITING;	/* clear waiting */
	    xs->xs_dmacnt = 0;
	    simple_unlock(&xs->xs_lock);
	    splx(s);
	    break;

	case DR_IOCTL_WAIT:	/* set wait for read on write mode */
	    s = spldr();
	    simple_lock(&xs->xs_lock);
	    xs->xs_flags |= DR_WAIT;
	    simple_unlock(&xs->xs_lock);
	    splx(s);
	    break;

	case DR_IOCTL_NOWAIT:	/* clear wait for read on write mode */
	    s = spldr();
	    simple_lock(&xs->xs_lock);
	    xs->xs_flags &= ~DR_WAIT;
	    simple_unlock(&xs->xs_lock);
	    splx(s);
	    break;

	default:		/* invalid ioctl command */
	    error = EINVAL;
	    break;
    }
    return(error);
}

/*
 * DR11-W minphys routine. Used with kernel physio to limit
 * maximum size of any one piece of a transfer to DR_MINPHYS
 * or less. This ensures that the copy buffer does overflow.
 */
drminphys (bp)
    struct buf *bp;
{
    if (bp->b_bcount > DR_MINPHYS)
        bp->b_bcount = DR_MINPHYS;
}

/*
 * DR11-W raw read/write routine. Common raw read/write routine
 * which packages the user's request with a raw buffer header and
 * calls the kernel physio routine.
 */
dr_raw_rdwr (dev, uio, rw)
    dev_t dev;
    struct uio *uio;
    int rw;
{
    struct dr_softc *xs;
    struct buf *rbp;
    int error;
    int lnode;
    int unit;

    /*
     * Find the logical processor attached to the device's
     * iobus as well as the unit on that iobus.
     */
    lnode = dev_to_log_node(dev);
    unit = minor(dev);

    /*
     * Verify that the device in question exists.
     */
    if (lnode == NULL_NODE || unit >= ndr)
	return(ENXIO);

    /*
     * Locate the device specific data structure for specified
     * unit and doublecheck the health of the device.
     */
    xs = *((struct dr_softc **)vvrmtptr(&dr_softc[unit], lnode));
    if (xs == (struct dr_softc *) NULL ||
	(xs->xs_flags & DR_ALIVE) == 0)
	return(ENXIO);

    /*
     * Set up a raw buffer header pointer
     * thru which the I/O will be performed.
     */
    rbp = (struct buf *)xs->xs_rawbp;
    bzero((caddr_t)rbp, sizeof(struct buf));
    xs->xs_dmacnt = 0;

    /*
     * Perform the raw I/O.
     */
    error = physio(drstrategy, rbp, dev, rw, drminphys, uio);

    return(error);
}

/*
 * Set error values in buffer header on error.
 */
dr_bp_error (bp, error)
    struct buf *bp;
{
    int s;

    bp->b_resid = bp->b_bcount;
    bp->b_error = error;
    bp->b_flags |= B_ERROR;
}

/*
 * Perform a DMA operation to read data from the DR11-W
 * into the raw data buffer, or write data contained in
 * the raw data buffer out to the DR11-W.
 */
dr_dma_rdwr (xs, busaddr, len, rw)
    struct dr_softc *xs;
    u_long busaddr;
    int len;
    int rw;
{
    int *chanp;
    int error;
    int s;

    /*
     * Find the vectorized page address of the wait channel.
     */
    chanp = (int *)vvrmtptr(&dr_chan[xs->xs_md->md_unit],
			    xs->xs_lnode);

    /*
     * Acquire the multiprocessor lock.
     */
    s = spldr();
    simple_lock(&xs->xs_lock);

    /*
     * Wait for the DMA currently in progress to complete.
     */
    while ((xs->xs_dr->dr_csr & STS_REDY) == 0) {
	/*
	 * If the user interrupted this sleep, then return EINTR.
	 */
	if (sleep_and_unlock((caddr_t)chanp, DR_PRIO | PCATCH,
			     &xs->xs_lock)) {
	    splx(s);
	    return(EINTR);
	}
	simple_lock(&xs->xs_lock);
    }

    /*
     * In the case of a write, if WAIT mode is set, or this is a
     * portion of a multiblock DMA other than the first block, then
     * sleep waiting for the other side to hang a read. Use the
     * FCN3/STTA signals to indicate that a write is pending.
     *
     * If WAIT mode is not set and no read is pending, return ENOENT.
     */
    if (rw == B_WRITE) {
	if (xs->xs_dmacnt == 0 && (xs->xs_flags & DR_WAIT) == 0 &&
	    (xs->xs_dr->dr_csr & STS_STTA) == 0) {
	    simple_unlock(&xs->xs_lock);
	    splx(s);
	    return(ENOENT);
	} else if (xs->xs_dmacnt || xs->xs_flags & DR_WAIT) {
	    while ((xs->xs_dr->dr_csr & STS_STTA) == 0) {
		xs->xs_flags |= DR_WAITING;
		xs->xs_dr->dr_csr =
		    (xs->xs_dr->dr_csr & CTL_BITS) | CTL_FCN3;
		if (sleep_and_unlock((caddr_t)chanp, DR_PRIO | PCATCH,
		    &xs->xs_lock)) {
		    /*
		     * The user interrupted this sleep.
		     * Cleanup and return EINTR.
		     */
		    simple_lock(&xs->xs_lock);
		    xs->xs_flags &= ~DR_WAITING;
		    xs->xs_dr->dr_csr =
			(xs->xs_dr->dr_csr & CTL_BITS & ~CTL_FCN3);
		    simple_unlock(&xs->xs_lock);
		    splx(s);
		    return(EINTR);
		}
		simple_lock(&xs->xs_lock);
	    }
	}
    }

    /*
     * Load up the DMA registers.
     *
     * Remember that the address refers to 16 bit values,
     * and thus must be divided by two before it is loaded.
     *
     * The DR11-W is jumpered to restrict the range to 16 bits.
     * The range is the number of 16 bit vales to be transferred
     * minus one.
     */
    xs->xs_dr->dr_dma_addrl = (u_short)((busaddr >> 1) & 0xffff);
    xs->xs_dr->dr_dma_addrh = (u_short)((busaddr >> 17)& 0xffff);
    xs->xs_dr->dr_dma_rangel = (u_short)(((len + 1) / 2) - 1);
    xs->xs_dr->dr_dma_rangeh = (u_short)0;

    /*
     * Use the FCN3/STTA signals to indicate that a read is pending.
     */
    if (rw == B_READ) {
	xs->xs_dr->dr_csr = (xs->xs_dr->dr_csr & CTL_BITS) | CTL_FCN1;
	xs->xs_dr->dr_pulse = PUL_GO;
	xs->xs_dr->dr_csr = (xs->xs_dr->dr_csr & CTL_BITS) | CTL_FCN3;

	/*
	 * The reader checks the STTA flag to determine if there
	 * is a writer waiting for a reader. If there is a writer
	 * waiting, then the reader causes an attention interrupt.
	 */
	if (xs->xs_dr->dr_csr & STS_STTA)
	    xs->xs_dr->dr_pulse = PUL_FCN2;
    } else {
	xs->xs_dr->dr_csr = (xs->xs_dr->dr_csr & CTL_BITS & ~CTL_FCN1);
	xs->xs_dr->dr_pulse = PUL_GO;
	xs->xs_dr->dr_pulse = PUL_CYCL;	/* force DMA cycle on write */
	xs->xs_dmacnt++;
    }

    /*
     * Sleep waiting for the DMA to complete.
     */
    if (sleep_and_unlock((caddr_t)chanp, DR_PRIO | PCATCH,
			 &xs->xs_lock)) {
	/*
	 * We took a user interrupt in the middle of
	 * a DMA operation! Terminate the DMA, send
	 * an attention to the remote end, and return
	 * EINTR to the interrupting process.
	 */
	simple_lock(&xs->xs_lock);
	xs->xs_dr->dr_pulse = PUL_TERM;
	xs->xs_dr->dr_pulse = PUL_FCN2;
	simple_unlock(&xs->xs_lock);
	splx(s);
	return(EINTR);
    }

    /*
     * The interrupt handler passes an error code back
     * in the channel word. Pass the code (if any) to
     * the user.
     */
    error = *chanp;
    splx(s);

    return(error);
}

/*
 * For debugging purposes, return a pointer to a string
 * filled with mnemonic representations of the bits in
 * the specified status word.
 */
char *
dr_status (status)
    u_short status;
{
    static char str[128];

    str[0] = '\0';
    if (status & STS_DMAF) strcat(str, "DMAF, ");
    if (status & STS_ATTF) strcat(str, "ATTF, ");
    if (status & STS_ATTN) strcat(str, "ATTN, ");
    if (status & STS_PERR) strcat(str, "PERR, ");
    if (status & STS_STTA) strcat(str, "STTA, ");
    if (status & STS_STTB) strcat(str, "STTB, ");
    if (status & STS_STTC) strcat(str, "STTC, ");
    if (status & STS_REDY) strcat(str, "REDY, ");
    if (status & STS_IENB) strcat(str, "IENB, ");
    if (status & STS_BERR) strcat(str, "BERR, ");
    if (status & STS_SDIR) strcat(str, "SDIR, ");
    if (status & STS_FCN3) strcat(str, "FCN3, ");
    if (status & STS_FCN2) strcat(str, "FCN2, ");
    if (status & STS_FCN1) strcat(str, "FCN1, ");
    if (status & STS_DFLG) strcat(str, "DFLG, ");
    if (str[0] == '\0') strcat(str, "----, ");

    return(str);
}
#endif	NDR > 0
#endif	BFLY2
