/* $Header: /nfs/europa/u1/BFMACH.local/mk/butterflydev/RCS/icc_kern_mach.h,v 1.6 90/12/19 15:09:42 tdonahue Exp $ */

#ifndef	ICC_KERN_MACH_H
#define ICC_KERN_MACH_H

#ifdef KERNEL
#include "../h/ioctl.h"
#else KERNEL
#include <sys/ioctl.h>
#endif KERNEL


/*
 * ICCM Device ioctl() Control Blocks
 */
#ifdef notyet
typedef struct
{
  unsigned long status;		/* Returned status     */
  unsigned long pnn;		/* Target PNN          */
  unsigned long tid;		/* pSOS task ID        */
  unsigned long event;		/* pSOS wakeup event   */
} IIOWAKEUPPB;

#endif 

/*
 * ICCM Device ioctl() Parameter Block
 */
typedef struct kern_icc_pb_s 
{
  unsigned long	cmd;		/* Function code   */
  unsigned long	rc;		/* Returned status */
  unsigned long	pnn;		/* Local (output) or remote (input) PNN */
  unsigned long	ringbuf;	/* Physical offset of local ring buffer */
  unsigned long	reserved[4];
} KERN_ICC_PB;
    

/*
 *  /dev/icc ioctl() Function Codes
 */
#define KERN_ICC_INIT		1
#define KERN_ICC_INTERRUPT      2
#define KERN_ICC_RINGBUF_PUT    3

/*
 * /dev/icc ioctl() Operations
 * 
 * Functon Codes 0-3 are not used for backward compatibility.
 */
#define IIOCINIT         (_IOWR(I, KERN_ICC_INIT, KERN_ICC_PB))
#define IIOCRINGBUFPUT   (_IOWR(I, KERN_ICC_RINGBUF_PUT, KERN_ICC_PB))
#define IIOCINTERRUPT    (_IOWR(I, KERN_ICC_INTERRUPT, KERN_ICC_PB))
#define IIOCWAKEUP       (_IOWR(I, 4, KERN_ICC_PB))     
#define IIOCGETEBPARAMS  (_IOWR(I, 5, KERN_ICC_PB))
#define IIOCUNMAPRB      (_IOWR(I, 6, KERN_ICC_PB))



#ifdef KERNEL
/* 
 * The following definitions must be kept in synch with pSOS header files 
 */

/* 
 * Synchronize with /usr/rt2/include/interrupts.h 
 */
#define ICC_INTR_BIT   (16)	/* Multiplexed IPI interrupt bit for ICC */

typedef struct ipi_bitvector_s 
{
  long	lock;
  long	vector;
} IPI_BITVECTOR;


/*
 * Synchronize with /usr/rt2/include/sys/memory_map.h 
 */
#define IPI_VECTOR_PO	   (0x0000C000)	   /* IPI bit vectors */
#define IPI_VECTOR_SIZE	   (0x1000)


/*
 * ICCM Ring Buffer and Interprocessor Bit Vector Lock Definitions
 */ 
#define SUPV_SPACE 0
#define USER_SPACE 1

/*
 * The following values yield a lock timeout value of about 8 milliseconds
 * on a 20 MHz FPV with 2 I CMMUs.
 */
#define RINGBUF_LOCK_LIMIT (64)
#define RINGBUF_LOCK_DELAY  (8)

#define IPI_LOCK_LIMIT     (64)
#define IPI_LOCK_DELAY      (8)


/*
 * Error Codes for use by /dev/icc Driver
 */
#define ICC_ERROR_BASE                  0xF300
#define ICC_OK	0

typedef enum icc_rc_e 
{
  ICC_EBADICC = ICC_ERROR_BASE,
  ICC_EINACTIVE,
  ICC_EMEMORY,
  ICC_EMAPPING,
  ICC_EBUFSIZE,
  ICC_ENOBUF,
  ICC_ENOTALIGNED,
  ICC_EBADBUF,
  ICC_EBADADDR,
  ICC_EBADOFFSET,
  ICC_EQFULL,
  ICC_EQEMPTY,
  ICC_EWAKEUP,
  ICC_EMAXPENDING,
  ICC_ETIMEOUT,
  ICC_ELOCK,
  ICC_ERINGBUF,
  ICC_EATTACHED,
  ICC_EOSERROR,
  ICC_EBADVERSION,
} ICC_RC;

#define ICC_ERROR_MAX                 (ICC_EBADVERSION)


#endif	KERNEL
#endif	ICC_KERN_MACH_H
