#ifndef IF_EGREG_H
#define IF_EGREG_H
/*
 * Register and structure definitions for accessing the 
 * Interphase Eagle Ethernet controller.
 *
 * Dan Tappan 2/21/90
 */
/* Copyright 1990 by BBN Advanced Computers Inc.  
 * ALL RIGHTS RESERVED 
 *
 * No part of this material may be reproduced, stored in a retrieval
 * system, transmitted or used in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise, without the prior
 * written permission of, or express license from BBN Advanced Computers Inc. 
 * 
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
 * Technical Data and Computer Software clause at DFARS 52.227-7013. 
 *
 * BBN Advanced Computers Inc. 
 * 10 Fawcett Street * Cambridge, Ma. 02238 
 * 617-873-6000 
 */

/*
 * Master control/status block.
 */
typedef struct {
    u_short msr;		/* status register */
    u_short res1;		/* reserved */
    u_short mcr;		/* control register */
    u_short res2;		/* reserved */
    u_long qheadptr;		/* Queue head pointer (available to host) */
    u_long res3;		/* reserved */
}  eagle_mcsb;
/* Bits in the MSR */
#define MSR_NOTAVAIL	0x0001	/* controller not available */
#define MSR_OK	    	0x0002	/* board OK */

/* Bits in the MCR */
#define MCR_STARTQ   	0x0001	/* Start queue mode */
#define MCR_RESET	0x1000	/* Reset board */
#define MCR_SYSFAIL	0x2000	/* SYSFAIL enable */

/* Format of a command queue entry */
typedef struct {
    u_short cr;			/* control register */
    u_short iopb;		/* iopb address */
    u_long tag;			/* command tag */
    u_char res1;		/* reserved */
    u_char workq;		/* work queue number */
#define WORKQ_MISC	0x00	/* miscellaneouis work queue */
#define WORKQ_DMA	0x01	/* DMA work queue */
#define WORKQ_RCV	0x02	/* Receive work queue */
#define WORKQ_XMT	0x03	/* Transmit work queue */
    u_short res2;		/* reserved */
} eagle_cqe;

/* Bits in the cqe control register */
#define CQE_GO		0x0001	/* GO/BUSY */

/* The master cqe does not use the work queue field */

/* Defined commands */
#define EAGLE_DIAG	0x0040	/* Perform extended diagnostics */
#define EAGLE_INIT	0x0041	/* Initialize controller */
#define EAGLE_INITWQ	0x0042	/* Initialize work queue */
#define EAGLE_SADDR	0x0045	/* Change default node address */
#define EAGLE_SFILTER	0x0046	/* Change logical address filter */
#define EAGLE_DL	0x0047	/* Download software */
#define EAGLE_XCT	0x0048	/*  Execute downloaded sofgtware */
#define EAGLE_XMT	0x0050	/* Transmit */
#define EAGLE_RCV	0x0060	/* Receive */
#define EAGLE_DMA	0x0070	/* Misc. DMA command */

/*
 * Format of an iopb
 */
typedef struct {
    u_short	cmd;		/* command code */
    u_short	options;	/* command options */
#define OPT_INTR	0x0001	/* Interrupt on command completion */
#define OPT_GATHER	0x0002	/* Gather enable (xmt/receive) */
#define OPT_DMA		0x0004	/* DMA enable (xmt/receive) */
#define OPT_CATHDR	0x0200	/* Concat header (xmt/rcv, not imp.) */
/* Other commands, not currently used,  have additional options defined.
 */
    u_short	status;		/* return status */
    u_char	lev_norm;	/* Normal completion level */
    u_char	vec_norm;	/* Normal completion vector */
    u_char	lev_error;	/* Error completion level */
    u_char	vec_error;	/* Error completion level */
    u_short	dma_options;	/*  VME transfer options */
#define DMA_AMOD 0x003d		/* Address modifier in low byte */
#define DMA_MEM32 0x0200	/* 32 bit VME memory  */
#define DMA_MEM16 0x0100	/* 16 bit VME memory */
#define DMA_NORM  0x0000	/* Normal transfer */
#define DMA_BLK	  0x0400	/* Block mode transfer */
#define DMA_BLK2  0x0800	/* Block mode w/o incrementing addresses */
#define DMA_READ  0x0000	/* Direction bit 0 -> eagle */
#define DMA_WRITE 0x1000	/* Direction bit 1 -> host */
    u_long  dma_addr;		/*  VME transfer address */
    u_long  dma_size;		/* VME DMA transfer size */
    u_short res;		/* reserved */
    u_short type;		/* packet type/length (rcv) */
    u_char src[6];		/* packet source (rcv) */
    u_short ngather;		/* gather count (xmt, notused) */
    u_short lance1;		/* lance xmit descriptor 1 */
    u_short lance2;		/* lance xmit descriptor 2 */
} eagle_dma_iopb;

typedef struct {
    u_short	cmd;		/* command code */
    u_short	options;	/* command options */
    u_short	status;		/* return status */
    u_char	lev_norm;	/* Normal completion level */
    u_char	vec_norm;	/* Normal completion vector */
    u_char	lev_error;	/* Error completion level */
    u_char	vec_error;	/* Error completion level */
    u_short res;		/* reserved */
    u_long  initb;		/* Offset to controller init block */
} eagle_init_iopb;

typedef eagle_dma_iopb eagle_iopb; /* standard type of iopb */

/*
 * format of response block.
 */
typedef struct {
    u_short status;		/* status word */
    u_short res1;		/* reserved */
    u_long tag;			/* command tag */
    u_char res2;		/* reserved */
    u_char workq;		/* work queue number */
    u_short res3;		/* reserved */
    u_long fill;		/* alignment fill */
    eagle_iopb iopb;		/* returned iopb */
} eagle_crb;
#define CRB_VALID	0x0001	/* response valid/clear interrupt */
#define CRB_COMPLETE	0x0002	/* command complete */
#define CRB_ERROR	0x0004	/* Error condition */
#define CRB_EXCEPTION	0x0008	/* Exception condition */
#define CRB_QMODE	0x0020	/* Queue mode started */

/*
 * Format of controller initialization block
 */
typedef struct {
    u_char res1;		/* reserved */
    u_char ncqe;		/* number of command queue entries */
    u_short net_opts;		/* network options */
#define NETOPT_ETHERNET	0x0000	/* Ethernet */
#define NETOPT_802_3	0x0001	/* 802.3 */
#define NETOPT_NORCV	0x0100	/* Disable receive */
#define NETOPT_NOCRC	0x0800	/* Disable xmit CRC */
#define NETOPT_COL	0x1000	/* Force collision */
#define NETOPT_NORETRY	0x2000	/* disable retry */
#define NETOPT_PMODE	0x8000	/* promiscuous receive mode */
    u_short res2;		/* reserved */
    u_char eaddr[6];		/* Physical node address */
    u_char filter[8];		/* logical address filter */
    u_char res3[10];		/* reserved */
    u_char lev_cmd;		/* VME interrupt level for cmd completion */
    u_char vec_cmd;		/* VME vector for cmd completion */
    u_char lev_error;		/* VME interrupt level for random errors */
    u_char vec_error;		/* VME interrupt vector for random errors */
    u_short dma_burst;		/* DMA burst count (0 == full message) */
#define DMA_BURST 8		/* DMA burst size */
    u_char res4[8];		/* reserved */
} eagle_cib;

/*
 * format of configuration status block.
 */
typedef struct {
    char res1;			/* reserved */
    char pcode[3];		/* product code */
    char res2;			/* reserved */
    char pvers[1];		/* product version */
    char res3;			/* reserved */
    char firmrevlev[3];		/* firmware rev level */
    char firmrevdate[8];	/* firmware revision date */
    char res4[12];		/* reserved */
    u_short bsize;		/* buffer size in 2K chunks */
    /* offset 32 from start so long aligned */
    u_long hostspace;		/* offset to host usable space */
    u_long rcvpool;		/* recieve pool offset */
    u_long xmtpool;		/* transmit pool offset */
    u_long avail;		/* available buffer memory offset */
    u_char eaddr[6];		/* board physical address */
    u_char filter[8];		/* current logical address filter */
    u_char res5[10];		/* unused */
} eagle_csb;

/*
 * format of controller statistics block
 */
typedef struct {
    char x;			/* to be defined */
} eagle_cstatb;


/*
 * offsets into the communication block in short I/O space.
 */
/* Master control/status block */
#define EAGLE_MCSB(eg) ((eagle_mcsb *)(eg->eg_base+0x0000))

/* Master command entry */
#define EAGLE_MCQE(eg) ((eagle_cqe *)(eg->eg_base+0x0010))

/* Other command queue entries */
#define EAGLE_CQE(eg, i) ((eagle_cqe *)(eg->eg_base+0x001c+(i*12)))

/* The start of the host usable space depends on the number of CQE */
/*
 * Maximum number of transmits and receives outstanding
 */
#define EAGLE_NRECEIVE 16
#define EAGLE_MAXTRANSMIT 8
#define EAGLE_NCQE 32
#if EAGLE_NRECEIVE+EAGLE_MAXTRANSMIT >= EAGLE_NCQE
ERROR Too many CQE's
#endif

/* Eagle host space is divided into a number of IOPBs, each corresponding
 * to a CQE.
 */
#define EAGLE_IOPB(eg, i) \
    ((eagle_iopb *)((eg)->eg_base+0x01c+(EAGLE_NCQE*12)+(i*sizeof(eagle_iopb))))

/* Address of command response block */
#define EAGLE_CRB(eg) ((eagle_crb *)((eg)->eg_base+0x0730))

/* Address of configuration status block */
#define EAGLE_CSB(eg) ((eagle_csb *)((eg)->eg_base+0x0764))

/* Address of controller statistics block */
#define EAGLE_CSTATB(eg) ((eagle_cstatb *)((eg)->eg_base+0x07ac))

/* The initialization block is stored at the end of the host usable space */
#define EAGLE_CIB(eg) \
    ((eagle_cib *)((eg)->eg_base+0x730-sizeof(eagle_cib)))

/* Find the offset of a structure into board's comm region */
#define OFFSET_OF(eg, s) ((u_long)(s) & 0x7ff)

/* Format of an ifdma structure which records the DMA usage for a transfer
 */
struct ifdma {
    u_long busaddr;		/* Address of allocated slave map ram */
    struct mbuf *mbuf;		/* mbuf which holds the data */
    struct ifdma *next;		/* Free list pointer */
#define IFDMA_INUSE(ifdma) ((ifdma)->next == (struct ifdma *)-1)
#define IFDMA_SETINUSE(ifdma) ((ifdma)->next = (struct ifdma *)-1)
};

/*
 * Ethernet software status per interface.
 *
 * Each interface is referenced by a network interface structure, eg_if,
 * which the routing code uses to locate the interface. This structure
 * contains the output queue for the interface, its address, ...
 */
struct	eg_softc {
    u_short	eg_nreceive;	/* # receives currently pending */
    u_short	eg_ntransmit;	/* # of transmits pending */
    short	eg_flags;	/* flag information */
    short	eg_nextcqe;	/* next free CQE */
    struct	arpcom eg_ac;	/* Ethernet common part */
    struct	ifdma eg_ifdma[EAGLE_NCQE]; /* DMA resources */
    struct	ifdma *eg_free_ifdma; /* Pointer to ifdma free list */
    u_short	eg_intcount;	/* count of interrupts, for watchdog */
    u_short	eg_intwatch;	/* Last count */
    u_short	eg_deathwatch;	/* countdown to controller hung */
    struct	egdevice *eg_ex; /* vectorized egdevice pointer */
    simple_lock_data_t eg_lock;	/* multiprocessor lock */
    caddr_t	eg_base;	/* base address of eagle in I/O space  */
};

/*
 * eg_softc structure element equivalent definitions
 */
#define	eg_if	eg_ac.ac_if		/* network-visible interface */
#define	eg_addr	eg_ac.ac_enaddr		/* hardware Ethernet address */

/* Flags bits */
#define EG_ALIVE	0x0001	/* Interface is alive */
#define EG_RUNNING	0x0002	/* Interface is running */


/* Interrupt level used by all commands */
#define EG_LEVEL	3	/* VME level 3 */

/* mutual exclusion macros */
#define EG_LOCK(eg) simple_lock(&eg->eg_lock)
#define EG_UNLOCK(eg) simple_unlock(&eg->eg_lock)

#define EGDEATHINTERVALS 2	/* watchdog ticks before interface hung */

#endif IF_EGREG_H
