/* $Header: /nfs/europa/u1/BFMACH.local/mk/butterflydev/RCS/if_exp.h,v 1.3 90/04/06 09:22:33 tappan Exp $ */
/*
 *	Copyright (c) 1984 by Communication Machinery Corporation
 *
 *	This file contains material which is proprietary to
 *	Communication Machinery Corporation (CMC) and which
 *	may not be divulged without the written permission
 *	of CMC.
 *
 *	ENP-10 Ram Definition
 *
 *	3/15/85 Jon Phares
 *	Update 7/10/85 S. Holmgren
 *	ENP-10 update 7/21/85 J. Mullen
 *	ENP-20 update 8/11/85 J. Mullen
 *	ENP-100 (CNP) update 1/5/88 J. Mullen
 *	CMC-135 (JD) update 2/27/89 J. Mullen
 * 
 * Dan Tappan (BBNACI) 3/6/90 - conversion to nX.
 */

#define K		*1024

#define JDSIZE		(256 K)		/* VME bus space allocated to jd */
#define MINPKTSIZE	60		/* minimum ethernet packet size */

#define RAM_WINDOW	(254 K)		/* `real' RAM */
#define IOACCESS_WINDOW (2 K)		/* control window */

#define	UINT	unsigned

/* Top 2K space for JD is for control functions */

struct iow100 {
	char	hst2jd_interrupt;	/* R or W interrupts JD */
	char	i_pad1[1 K -1];
	char	hst2jd_reset;		/* R or W resets JD */
	char	i_pad2[1 K -1];
};

#define ETHADDR		struct ether_addr

struct ethlist
{
	UINT	e_listsize;		/* active addr entries */
	ETHADDR	e_baseaddr;		/* addr lance is working with */
	ETHADDR e_addrs[16];		/* possible addresses */
};
#define	ETHLIST	struct ethlist

struct jdstat
{
	UINT e_xmit_successful;		/* Successful transmissions */
	UINT e_mult_retry;		/* multiple retries on xmit */
	UINT e_one_retry;		/* single retries */
	UINT e_fail_retry;		/* too many retries */
	UINT e_deferrals;		/* xmit delayed due to active medium */
	UINT e_xmit_buff_err;		/* xmit chaining failed, can't happen */
	UINT e_silo_underrun;		/* transmit data fetch failed */
	UINT e_late_coll;		/* collision after xmit */
	UINT e_lost_carrier;
	UINT e_babble;			/* xmit length > 1518 */
	UINT e_no_heartbeat;		/* transceiver mismatch, not an error */
	UINT e_xmit_mem_err;
	UINT e_rcv_successful;		/* good receptions */
	UINT e_rcv_missed;		/* no recv buff available */
	UINT e_crc_err;			/* checksum failed */
	UINT e_frame_err;		/* crc error && data len != 0 mod 8 */
	UINT e_rcv_buff_err;		/* rcv chaining failed, can't happen */
	UINT e_silo_overrun;		/* receive data store failed */
	UINT e_rcv_mem_err;
};
#define	JDSTAT	struct jdstat

struct ring
{
	short	r_rdidx;
	short	r_wrtidx;
	short	r_size;
	short	r_pad;
	UINT	r_slot[1];
};
#define	RING	struct ring

#define r_threshold r_pad

struct ring32
{
	short	r_rdidx;
	short	r_wrtidx;
	short	r_size;
	short	r_pad;			/* to make VAXen happy */
	UINT	r_slot[32];
};
#define	RING32	struct ring32

struct ring256
{
	short	r_rdidx;
	short	r_wrtidx;
	short	r_size;
	short	r_pad;			/* to make VAXen happy */
	UINT	r_slot[256];
};
#define	RING256	struct ring256

/*
 * 	JD Ram data layout
 *
 *	If you don't put it here - it isn't there
 *
 */
struct jddevice {
	union {
		char	all_ram[RAM_WINDOW];
		struct {
			char	       t_pad[0x1000];
			unsigned short t_go;
			unsigned short t_csr;
			UINT t_pstart;
		} t;
		struct {
			char	pad7[ 0x1100 ];	/* starts 0x1100 - 0x2000 */
			short	e_jdstate;		
			short	e_jdmode;		
			UINT	e_jdbase;		
			short	e_jdrun;
			short	e_intvector;

			RING256	h_tojd;		
			RING256	h_hostfree;		
			RING256	e_tohost;		
			RING256	e_jdfree;		
			RING256 e_rcvdma;
			RING256 h_rcv_d;

			JDSTAT	e_stat;
			ETHLIST	e_netaddr;		
		} iface;
	} jd_u;
	struct iow100	jd_iow;
};

#define	JDDEVICE	struct jddevice

/*
 * jd csr bit defs
 */
#define	JDCSR_ONLINE	0x2		/* entered application firmware */
#define	JDCSR_RDY	0x4		/* reset complete */
#define	JDCSR_OVPROM	0x8		/* don't transfer to prom on reset */
#define	JDCSR_IE	0x40		/* interrupt enable for bus debug */
#define	JDCSR_ERR	0x8000		/* jd detected error */

#define	jd_ram		jd_u.all_ram
#define	jd_go		jd_u.t.t_go
#define	jd_csr		jd_u.t.t_csr
#define	jd_prog_start	jd_u.t.t_pstart
#define jd_state	jd_u.iface.e_jdstate
#define jd_mode		jd_u.iface.e_jdmode
#define jd_base		jd_u.iface.e_jdbase
#define jd_jdrun	jd_u.iface.e_jdrun
#define jd_intvector	jd_u.iface.e_intvector
#define jd_tojd		jd_u.iface.h_tojd
#define jd_hostfree	jd_u.iface.h_hostfree
#define jd_tohost	jd_u.iface.e_tohost
#define jd_jdfree	jd_u.iface.e_jdfree
#define jd_rcvdma	jd_u.iface.e_rcvdma
#define jd_rcv_d	jd_u.iface.h_rcv_d
#define jd_stat		jd_u.iface.e_stat
#define jd_addr		jd_u.iface.e_netaddr

#define JDVAL		0xff	/* poke in jd_iow.hst2jd_interrupt */
#define RESETVAL	0x00	/* poke in jd_iow.jd2hst_clear_intr */

#define INTR_JD(addr)		addr->jd_iow.hst2jd_interrupt = JDVAL
#define RESET_JD(addr)		addr->jd_iow.hst2jd_reset = JDVAL
#define GO_JD(addr,xfer)	{addr->jd_prog_start = xfer; addr->jd_go = 0x8080;}
#define START_JD(addr)		addr->jd_go = 0x8080
#define TRAN_JD(addr,xfer)	addr->jd_prog_start = xfer

/*
 * state bits
 */
#define S_JDRESET	01		/* jd is in reset state */
#define S_JDRUN		02		/* jd is in run state */

/*
 * mode bits
 */
#define E_SWAP16		0x1		/* swap two octets within 16 */
#define E_SWAP32		0x2		/* swap 16s within 32 */
#define E_SWAPRD		0x4		/* swap on read */
#define E_SWAPWRT		0x8		/* swap on write */
#define E_DMA			0x10		/* jd does xmit data moving */
#define E_XMIT_DMA E_DMA
#define E_ARP			0x20		/* Reserved for Future Use */
#define E_DONTINT		0x40		/* do not interrupt host */
#define E_NOTERM		0x80		/* don't use serial port */
#define E_RCV_DMA		0x100		/* jd moves rcv data */
#define E_DO_TRAILERS		0x200		/* Reserved for Future Use */
#define E_UNDO_TRAILERS		0x400		/* de-encapusulate trailers */
#define E_MULTICAST		0x800		/* accept all multicast */
#define E_PROMISCUOUS		0x1000		/* accept everything */
#define E_RBUF_SHWD		0x2000		/* receive buffers are aligned
						   on an odd short boundary */
#define E_EXAM_LIST		0x8000	/* jd should examine addr list */

/*
 * 	Download ioctl definitions
 */

#define mkioctl(type,value) (0x20000000|('type'<<8)|value)

#define JDIOGO		mkioctl( S,1 )		/* start the jd */
#define JDIORESET	mkioctl( S,2 )		/* reset the jd */

/*
 *	The bus description structure 
 */

struct bus_descr {
	struct	bus_descr *bd_link;
	UINT	*bd_addr;		/* Physical bus address */
	UINT	bd_len;			/* length in bytes */
	unsigned short bd_mode;		/* For VME, address modifier */
	unsigned short bd_pad;
};
#define BUS_D struct bus_descr

/*
 * 	The JD Data Buffer Structure
 */
struct bcb
{
	struct bcb *b_link;
	short	b_stat;
	short	b_len;
	UINT	*b_addr;
	short	b_msglen;
	short	b_reserved;
	/* From here on is reserved for the host's use */
	struct mbuf *b_mbuf;
	struct bus_descr *b_bus_descr;
	UINT	b_pad1;
	UINT	*b_raddr;	/* Received address */
};
#define	BCB	struct bcb

/*
 * BCB b_stat field defines
 */
#define	BCB_DONE	0x8000		/* buffer complete */
#define	BCB_ERR		0x4000		/* error summary */
#define	BCB_FRAME	0x2000		/* framing error */
#define	BCB_OFLO	0x1000		/* silo overflow */
#define	BCB_CRC		0x0800		/* crc error */
#define	BCB_RXBUF	0x0400		/* rx buffer err */
#define	BCB_STP		0x0200		/* start of packet */
#define	BCB_ENP		0x0100		/* end of packet */
#define	BCB_MEMERR	0x0002		/* memory error */
#define	BCB_MISSED	0x0001		/* missed packet */
