/* $Header: /nfs/europa/u1/BFMACH.local/mk/butterflydev/RCS/plxreg.h,v 1.3 90/03/05 09:01:15 tappan Exp $ */

#ifndef	_PLXREG_H
#define	_PLXREG_H

/*
 * File: plxreg.h
 * Definitions for the Parallax 1280 series graphics controller.
 *
 * August 23, 1989 Thomas W. Uban
 *
 * Copyright 1989 by BBN Advanced Computers Inc.
 * ALL RIGHTS RESERVED
 *
 * No part of this material may be reproduced, stored in a retrieval
 * system, transmitted or used in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise, without the prior
 * written permission of, or express license from BBN Advanced Computers Inc.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
 * Technical Data and Computer Software clause at DFARS 52.227-7013.
 *
 * BBN Advanced Computers Inc.
 * 10 Fawcett Street
 * Cambridge, Ma. 02238
 * 617-873-6000
 */

#ifdef	KERNEL
/*
 * The Parallax 1280 series VME board lives in the short I/O space
 * and has the following device registers...
 */
struct plxdevice {
    u_short plx_csr;			/* control/status register */
    u_short plx_cdr;			/* control/data register */
};

/*
 * Control/status register definitions. (RO - read only, RW - read write)
 */
#define	CSR_ERRRST	0x8000		/* error/reset (RW) */
#define	CSR_USR		0x4000		/* user/kernel mode (RW) */

#define	CSR_ERC_SHIFT	8		/* error code shift */
#define	CSR_ERC_MASK	0x3f00		/* error code mask */
#define	CSR_ERC_NM	0x0000		/* non-existent memory */
#define	CSR_ERC_II	0x0100		/* illegal instruction */
#define	CSR_ERC_PVO	0x0200		/* polygon vertex overflow */
#define	CSR_ERC_FSO	0x0300		/* fill stack overflow */
#define	CSR_ERC_PV	0x0400		/* parameter violation */
#define	CSR_ERC_ME	0x0500		/* mode error */
#define	CSR_ERC_EI	0x0600		/* EIS instruction error */
#define	CSR_ERC_RES	0x0700		/* reserved */
#define	CSR_ERC_RA	0x0800		/* RAM allocation error */
#define	CSR_ERC_MSL	0x0900		/* microcode store load error */
#define	CSR_ERC_F	0x0a00		/* flash error */
#define	CSR_ERC_RSWPS	0x0b00		/* restore stream without prev save */

#define	CSR_RDY		0x0080		/* CDR is ready (RO) */
#define	CSR_IE		0x0040		/* interrupt enable (RW) */
#define	CSR_IP		0x0020		/* interrupt pending (RO) */
#define	CSR_DE		0x0010		/* not used, RT-Bus only */
#define	CSR_RES		0x0008		/* reserved */
#define	CSR_WAC		0x0004		/* CDR waiting for bus addr/wc (RO) */
#define	CSR_WC		0x0002		/* CDR waiting for command (RO) */
#define	CSR_DAV		0x0001		/* data available (RO) */

/*
 * Parallax software status per interface.
 */
struct	plx_softc {
    short	xs_flags;		/* flag information */
    int		xs_lnode;		/* logical node where device lives */
    u_long	xs_busaddr;		/* bus address of plx_softc */
    u_short	xs_cmdstate;		/* command state */
    u_short	xs_piobuf;		/* virtual address of PIO buffer */
    u_long	xs_piobuf_busaddr;	/* bus address of PIO buffer */
    u_short	xs_cmdlen;		/* number of commands in the packet */
    u_short	xs_cmdbuf_size;		/* size of command buffer */
    caddr_t	xs_cmdbuf;		/* virtual address of command buffer */
    u_long	xs_cmdbuf_busaddr;	/* bus address of command buffer */
    struct	mb_device *xs_md;	/* pointer to device information */
    struct	plxdevice *xs_plx;	/* vectorized plxdevice pointer */
    int		xs_intlevel;		/* device interrupt level */
    simple_lock_data_t xs_lock;		/* multiprocessor lock */
};

/*
 * xs_flags definitions.
 */
#define	PLX_ALIVE	0x0001		/* device is alive */
#define	PLX_INUSE	0x0002		/* someone is using the device */
#define	PLX_WANTED	0x0004		/* someone wants to use the device */
#define	PLX_FATAL	0x0008		/* a fatal device error occurred */

/*
 * xs_cmdstate definitions.
 */
#define	PLX_STATE_ZERO	0		/* load low address of user command */
#define	PLX_STATE_ONE	1		/* load high address of user command */
#define	PLX_STATE_TWO	2		/* load word count of user command */
#define	PLX_STATE_THREE	3		/* user DMA done or load PIO low addr */
#define	PLX_STATE_FOUR	4		/* load high address of PIO command */
#define	PLX_STATE_FIVE	5		/* load word count of PIO command */
#define	PLX_STATE_SIX	6		/* DMA of PIO command complete */

/*
 * Device delay definitions.
 */
#define	PLX_SHORT_DELAY		10	/* short delay 10 millisecond */
#define	PLX_ONE_SECOND		1000	/* DELAY one second value */
#define	PLX_WAIT_SECONDS	4	/* wait up to four seconds */

/*
 * General definitions.
 */
#define	splplx()	setpl(xs->xs_intlevel)
#define	BUS_ADDR(x)	physoffset(x)
#define	PLX_I_PRIO	(PZERO + 6)	/* interruptable sleep priority */
#define	PLX_U_PRIO	(PZERO - 6)	/* uninterruptable sleep priority */

/*
 * Definitions for commands used internally by the driver.
 */
#define	PLX_CMD_VEC16L		0x003f	/* set interrupt vector */
#define	PLX_CMD_ADDMOD		0x013f	/* set VME address modifier */
#define	PLX_CMD_SHOW		0x002c	/* show configuration info */
#define	PLX_CMD_JUMPB		0x0329	/* enter buffered DMA mode */
#define	PLX_CMD_PIO		0x0028	/* enter programmed I/O mode */
#define	PLX_CMD_NOP		0x0000	/* nop */

#endif	KERNEL

/*
 * All user interactions with the Parallax are through the
 * ioctl interface. There is one ioctl (PLXCMD) which takes
 * a plx_ioctl structure as its argument. The command which
 * the user places into the plx_ioctl structure is then
 * passed to the device by the driver. Users are restricted
 * to user mode commands.
 */
struct plx_ioctl {
    u_char flags;	/* status flags */
    u_char error;	/* error return value, if error bit set in flags */
    u_short bufsize;	/* size of buffer */
    caddr_t buf;	/* argument/return data buffer pointer */
};

/*
 * Ioctl packet flags definitions.
 */
#define	PLX_IOCTL_ERROR		0x01	/* an error occured, check error */

/*
 * Ioctl error return codes.
 */
#define	PLX_IOCTL_ERR_NM	0	/* non-existent memory */
#define	PLX_IOCTL_ERR_II	1	/* illegal instruction */
#define	PLX_IOCTL_ERR_PVO	2	/* polygon vertex overflow */
#define	PLX_IOCTL_ERR_FSO	3	/* fill stack overflow */
#define	PLX_IOCTL_ERR_PV	4	/* parameter violation */
#define	PLX_IOCTL_ERR_ME	5	/* mode error */
#define	PLX_IOCTL_ERR_EI	6	/* EIS instruction error */
#define	PLX_IOCTL_ERR_RES	7	/* reserved */
#define	PLX_IOCTL_ERR_RA	8	/* RAM allocation error */
#define	PLX_IOCTL_ERR_MSL	9	/* microcode store load error */
#define	PLX_IOCTL_ERR_F		10	/* flash error */
#define	PLX_IOCTL_ERR_RSWPS	11	/* restore stream without prev save */
#define	PLX_IOCTL_ERR_INTERNAL	12	/* parallax internal error */

#endif	_PLXREG_H
