/* $Header: /nfs/columbia/u1/BFMACH.local/mk.2.9.6/butterflydev/RCS/ssmtdriver.h,v 1.11 91/03/08 14:29:28 rgoguen Exp $ */
/*****************************************************************************
* File: smtdriver.h - SCSI magnetic tape drive definitions		     *
*****************************************************************************/

#ifdef KERNEL

/* drive mode setup templates.  Please refer to the tapedrive manual */

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res0:5;
	unsigned char res1;
	unsigned char res2;
	unsigned char paramlen;
	unsigned res3:1;
	unsigned smp:1;
	unsigned res4:4;
	unsigned flag:1;
	unsigned link:1;
	} TA_MODE1;

#define TA_MODE1_NBYTES 6

typedef struct
	{
	unsigned char res0;
	unsigned char res1;
	unsigned res2:1;
	unsigned bufmode:3;
	unsigned speed:4;
	unsigned char blkdesclen;
	unsigned char density;
	unsigned char res3;
	unsigned char res4;
	unsigned char res5;
	unsigned char res6;
	unsigned char res7;
	unsigned char blksizmsb;
	unsigned char blksizlsb;
	unsigned char vendpagecode;
	unsigned char writethresh;
	unsigned char readthresh;
	unsigned char bufsiz;
	unsigned char fscountmsb;
	unsigned char fscountlsb;
	unsigned char busthresh;
	unsigned char copythresh;
	unsigned char nsthresh;
	unsigned char csthresh;
	unsigned char loadfcn;
	unsigned char powupdelay;
	} TA_MODE2;

#define TA_MODE2_NBYTES 24

typedef struct
	{
	unsigned char res0;
	unsigned char res1;
	unsigned res2:1;
	unsigned bufmode:3;
	unsigned speed:4;
	unsigned char blkdesclen;
	unsigned char density;
	unsigned char num_blocks[3];
	unsigned char res3 ;
	unsigned char bsize[3] ;
	unsigned char vendorbits ;
	unsigned res4 ;
	} EX_MODE2;


#define EX_MODE2_NBYTES 14

typedef struct
	{
	unsigned char res0;
	unsigned char res1;
	unsigned res2:1;
	unsigned bufmode:3;
	unsigned speed:4;
	unsigned char blkdesclen;
	unsigned char density;
	unsigned char num_blocks[3];
	unsigned char res3 ;
	unsigned char bsize[3] ;
	} ARCHIVE_MODE2;


#define ARCHIVE_MODE2_NBYTES 12

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:4;
	unsigned immed:1;
	unsigned char res2;
	unsigned char res3;
	unsigned res4:6;
	unsigned retension:1;
	unsigned load:1;
	unsigned eot:1;
	unsigned res5:5;
	unsigned flag:1;
	unsigned link:1;
	} LOADCMD;

#define LOADCMD_NBYTES 6

typedef struct 
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:4;
	unsigned immed:1;
	unsigned char nmarksmsb;
	unsigned char nmarksisb;
	unsigned char nmarkslsb;
	unsigned res2:6;
	unsigned flag:1;
	unsigned link:1;
	} FILEMARKCMD;

#define FILEMARKCMD_NBYTES 6

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:3;
	unsigned code:2;
	unsigned char countmsb;
	unsigned char countisb;
	unsigned char countlsb;	
	unsigned res2:6;
	unsigned flag:1;
	unsigned link:1;
	} SPACECMD;

#define SPACECMD_NBYTES 6

typedef struct
	{
	unsigned char command;
	unsigned lun:3;
	unsigned res1:4;
	unsigned setme:1;
	unsigned char res2;
	unsigned char res3;
	unsigned char res4;
	unsigned res5:6;
	unsigned flag:1;
	unsigned link:1;
	} ERASECMD;			

#define ERASECMD_NBYTES 6
#define REWIND_NBYTES 6

#define READ_COMMAND	0x08
#define WRITE_COMMAND   0x0A

#define SPACE_BLOCKS	0x00
#define SPACE_FILES	0x01
#define SPACE_EOT	0x03

#define SENSE_EOF       0x80
#define SENSE_EOT	0x40
#define SENSE_ILI       0x20


/* Unit descriptor block */
struct smt
    {
    int ident ;
    int flags ;
    int types ;
    int in_progress ;
    int low_water_mark ;
    simple_lock_data_t lock ;
    int block_size ;       /* block_size in bytes */
    struct st_ops *s_ops ; /* SCSI Tape operations */
    };

typedef struct smt SMT;

struct st_ops {
	int	(*setmode)();	/* set the modes of a tape drive */
	int flags;
	} ;

/* st_ops flags. */
#define USE_DEFAULT_MODE 1

#define CAN_OVERIDE_DEFAULT(thissmt)\
    (!((thissmt)->s_ops->flags & USE_DEFAULT_MODE))

#define SETMODE(THISSMT,M1,M2,C,T,D)\
    (*(*(THISSMT))->s_ops->setmode)(M1,M2,C,T,D)

/* maximum transfer size is 64K if the tranfer is greater than 8k
 * scatter/gather is used.
 */

#define MAXIMUM_TRANSFER_SIZE 65536

/* tape driver state (flags) definitions	*/

#define SMT_OPEN		0x01	/* bit says device is open */
#define SMT_NO_REWIND		0x02	/* bit says no rewind on close */
#define SMT_LOADED		0x04	/* bit says tape is loaded */
#define SMT_EOF			0x08	/* bit says tape at EOF */
#define SMT_DRIVE_THROTTLE	0x10	/* bit says drive throttled */
#define SMT_THROTTLE_LOWATER	3	/* # of WQE's left before restart */

/* tape device characteristics (types) definitions	*/

#define SMT_TYPES_AUTOLOAD	0x01	/* bit says tape loads automatically */
#define SMT_TYPES_BACKSPACE	0x02	/* bit says device can backspace */ 
#define SMT_TYPES_BIGTRANSFER   0x04    /* bit say device does 64k transfer.*/

/* Minor device number mappings:
   bit 0 1 2 : SCSI bus ID ( Support SCSI id 0 - 6)
   bit 3     : no-rewind bit (0 -> Rewind upon close, 1 -> Norewind upon close)
   bit 4     : Bus Number.   (0 -> SCSI bus 0, 1 -> SCSI bus 1)
   bit 5     : Adaptor bit.  (0 -> Adpt 0, 1 -> Adpt 1. Currently only 0)
   bit 6 7   : Density bit.  (0 -> high density, 1 -> low density)
   bit 8 9   : blksize.      (0 -> Variable 1 -> 512b fixed 10 -> 1024b fixed,
                              11 -> 1536b fixed)
   bit 10    : reserved
   bit 11    : reserved
*/

#define dev_to_device(d)((d)&0x07)
#define no_rewind(d) (((d)>>3)&0x01)
#define dev_to_bus(d) (((d)>>4)&0x01)
#define dev_to_adapter(d)(((d)>>5)&0x01)
#define density_is_low(d)(((d)>>6)&0x01)
#define denisity_is_high(d)((((d)>>6)&0x01) == 0)
#define dev_to_fixblksize(d) ((((d)>>8)&0x03)*512)
#define dev_is_fixed(d)(((d)>>8)&0x03)
#define dev_is_variable(d)((((d)>>8)&0x03) == 0)

#define SENSE_BYTE_COUNT		40
#define SCSI_CHECK			2 /* returned once after door opened */
#define TA_BLOCK_SIZE_IN_BYTES		512
#define VARIABLE_BLOCK_SIZE_IN_BYTES	1

#define IS_VARIABLE(t) ((t)->block_size == VARIABLE_BLOCK_SIZE_IN_BYTES)
#define IS_FIXED(t)  ((t)->block_size != VARIABLE_BLOCK_SIZE_IN_BYTES)

#define TRANSFERTYPE(t) (IS_VARIABLE((t)) ? 0 : 1)

#define TAPE_ERROR 1
#endif KERNEL

#define SCSI_COMMAND_READ  0x08
#define SCSI_COMMAND_WRITE 0x0A
#define SCSI_COMMAND_SENSE 0x03
#define SCSI_COMMAND_REWIND 0x01

/* smtstat mt_dsreg codes */
#define	SMT_DS_BOT		0x0001
#define	SMT_DS_EOT		0x0002
#define	SMT_DS_WP		0x0004
#define	SMT_DS_EOM		0x0008
#define	SMT_DS_EOF		0x0010
#define	SMT_DS_ONL		0x0020
#define	SMT_DS_HS		0x0040
#define	SMT_DS_RDY		0x0080

#define SMT_DS_BITS       \
"\20\10RDY\7HS\6ONL\5EOF\4EOM\3WP\2EOT\1BOT"

/* sense key codes */
#define	SMT_NO_SPECIFIC_INFO	0x00
#define	SMT_RESERVED_1		0x01
#define	SMT_NOT_READY		0x02
#define	SMT_MEDIA_ERROR		0x03
#define	SMT_HARDWARE_ERROR	0x04
#define	SMT_ILLEGAL_REQUEST	0x05
#define	SMT_UNIT_ATTENTION	0x06
#define	SMT_WRITE_PROTECTED	0x07
#define	SMT_BLANK_CHECK		0x08
#define	SMT_RESERVED_2		0x09
#define	SMT_COPY_ABORTED	0x0A
#define	SMT_ABORTED_COMMAND	0x0B
#define	SMT_RESERVED_3		0x0C
#define	SMT_VOLUME_OVERFLOW	0x0D
#define	SMT_MISCOMPARE		0x0E
#define	SMT_RESERVED_4		0x0F

#define SMT_ER_BITS       \
"\20\17MISCOMPARE\16VOLUME_OVERFLOW\15RESERVED_3\14ABORTED_COMMAND\
\13COPY_ABORTED\12RESERVED_2\11BLANK_CHECK\10WRITE_PROTECTED\
\7UNIT_ATTENTION\6ILLEGAL_COMMAND\5HARDWARE_ERROR\4MEDIA_ERROR\
\3NOT_READY\2RESERVED_1\1NO_SPECIFIC_INFO"

