#!/bin/csh -f

######################################################################
# HISTORY
#  5-Dec-86  Avadis Tevanian (avie) at Carnegie-Mellon University
#	Don't run a make depend since we don't do this anymore.
#
# 21-Oct-86  Michael Young (mwyoung) at Carnegie-Mellon University
#	Added "-loud", "-all" flags.
#
# 18-Aug-86  Mike Accetta (mja) at Carnegie-Mellon University
#	Added -make switch and changed meaning of -config;  upgraded to
#	allow multiple attributes per configuration and to define
#	configurations in terms of these attributes within MASTER.
#
# 14-Apr-83  Mike Accetta (mja) at Carnegie-Mellon University
#	Added -config switch to only run config without make.
#
######################################################################

set prog=$0
set prog=$prog:t

unset domake
unset doconfig
unset loud
unset justpretend
unset list

while ($#argv >= 1)
    if ("$argv[1]" =~ -*) then
        switch ("$argv[1]")
	case "-c":
	case "-config":
	    set doconfig
	    breaksw
	case "-m":
	case "-make":
	    set domake
	    breaksw
	case "-v":
	case '-verbose':
	case "-loud":
	    set loud
	    breaksw
	case "-a":
	case "-all":
	    set list=(` sed -n -e '/^#/s/.*[ 	]\([A-Z][A-Z]*\)[ 	]*=.*/\1/p' < MASTER `)
	    breaksw
	case '-n':
	case '-f':
	case '-test':
	    set justpretend
	    breaksw
	default:
	    echo "${prog}: ${argv[1]}: unknown switch"
	    exit 1
	    breaksw
	endsw
	shift
    else
	break
    endif
end

if ($?list == 0) set list=($argv)
if ($#list == 0) set argv=(GENERIC)

foreach SYS ($list)
    set conf=(`sed <MASTER -n -e "/^[^#]/d" -e "s;.*[ 	]${SYS}[ 	]*=[ 	]*\[\(.*\)\].*;\1;p"`)
    if ($#conf <= 0) then
	echo "${prog}: ${SYS}: no such configuration in MASTER"
	exit 1
    endif
    if ($?loud) echo "[ generating $SYS configuration file from MASTER ]"

    if ($?justpretend == 0) then
	set keep=()
	foreach attr ($conf)
	    set keep=(-e '/[<,]'"$attr"'[,>]/b keep' $keep:q)
	end
	set unkeep=()
	foreach attr ($conf)
	    set unkeep=(-e '/['\!',]'"$attr"'[,>]/d' $unkeep:q)
	end
	set omit=\
	(\
		-e '/<[^'\!'].*>$/{' \
		$keep:q \
		-e 'd' \
		-e ': keep' \
		-e '}' \
		-e '/<'\!'.*>$/{' \
		$unkeep:q \
		-e '}' \
	)
	<MASTER sed $omit:q \
	    -e '/^$/d' \
	    -e '/^#/d' \
	    -e 's;[ 	]*#.*$;;' \
	>$SYS
	if ($?doconfig) then
	    if (! -d ../$SYS) mkdir ../$SYS
	    echo "[ configuring $SYS ]"
	    config $SYS
	endif
	if ($?domake) then
	    echo "[ making $SYS ]"
	    (cd ../$SYS; make)
	endif
    endif
end
