/* $Header: /nfs/columbia/u1/BFMACH.local/mk.2.9.6/h/RCS/buf.h,v 1.28 91/03/14 10:39:02 tappan Exp $	*/
#include "../h/lock.h"

/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)buf.h	7.1 (Berkeley) 6/4/86
 *	@(#)buf.h	2.1 88/05/18 4.0NFSSRC SMI;	from UCB 7.1 6/4/86
 */
#ifndef __SYS_BUF_H
#define __SYS_BUF_H yes

/*
 * Dynamically allocated buffer cache:
 *
 * Physical memory pages can refer to cached disk pages. If this is the case
 * then a number of fields in the page structure (see vm/vmi.h) which link
 * the page onto 
 *	- a hash queue off of vnode/disk offset
 *	- a list of all pages cached for a given device vnode
 *	- a list of all pages cached for a given file vnode
 *
 * Notes:
 * - The 'struct buf' for a given page exists only while I/O is active
 *   for the page.
 * - The 'busy' bit is on a page while I/O is active.
 * - All buffer pages are allocated to a memory object. If the page has
 *   only been used for programmed I/O then it is assigned to a special
 *   object which depends on which hash list the buffer is on. If
 *   the page has been used for memory mapped I/O then it is part
 *   of the appropriate memory object.
 * - Virtual memory for buffer pages is only assigned when the page is
 *   active.
 * - The object vm_lock or page busy bit must be held while manipulating
 *   a buffer page. The buffer hash list lock must be held while searching
 *   the hash list or adding or removing a page from the list. The vm lock
 *   must be held before the acquiring the buffer list lock (the buffer
 *   list lock is a simple lock).
 *   The same rules apply when manipulating the vnode lists for a page
 *   (substitute vnode simple lock).
 * - Buffer header flags should be manipulated using atomic operations.
 */
#ifdef KERNEL
#include "../h/vm.h"
#else
#include <sys/vm.h>
#endif

/*
 * The 'struct buf' is the canonical buffer structure used by disk drivers
 * etc.
 * These structures are allocated dynamically and only exist while the
 * page is being accessed through the buffer cache (or I/O is in progress)
 */
struct buf
{
	long	b_flags;		/* too much goes here to describe */
	struct	buf *av_forw, *av_back;	/* position on free list if not BUSY */
#define	b_actf	av_forw			/* alternate names for driver queue */
#define	b_actl	av_back			/*    head - isn't history wonderful */
	struct  buf *b_multbp ;         /* points to the original bp. */
	int     b_multcount ;           /* # of outstanding multiple bp's. */
	long	b_bcount;		/* transfer count */
#define	b_active b_bcount		/* driver queue head: drive active */
	dev_t	b_dev;			/* major+minor device name */
	union {
	    caddr_t b_addr;		/* low order core address */
	    int	*b_words;		/* words for clearing */
	    struct fs *b_fs;		/* superblocks */
	    struct csum *b_cs;		/* superblock summary information */
	    struct cg *b_cg;		/* cylinder group block */
	    struct dinode *b_dino;	/* ilist */
	    daddr_t *b_daddr;		/* indirect block */
	} b_un;
	vm_page_t b_pp;
	daddr_t	b_blkno;		/* block # on device */
	long	b_resid;		/* words not transferred after error */
#define	b_errcnt b_resid		/* while i/o in progress: # retries */
	struct  proc *b_proc;		/* proc doing physical or swap I/O */
	struct	vnode *b_vp;		/* vnode associated with block */
#define b_pfcent b_spare0		/* center page when swapping cluster */
	int	b_spare0;		/* Internal driver use if not above */
	int	b_spare1;		/* Word for internal driver use */
	int	b_spare2;		/* Word for internal driver use */
	int	b_spare3;		/* Word for internal driver use */
	int	b_spare4;		/* Word for internal driver use */
	int	b_spare5;		/* Word for internal driver use */
	vm_addr_t	b_va;		/* Saved copy of assigned va */
	short	b_error;
	short   b_pagelist;		/* original page list for underlying 
					 * page.
					 */
	char	b_busy;		/* set if buffer is in use */
	char	b_wanted;	/* set if buffer wanted */
	short	b_pad;		/* so cross compilation works */
/* The following are used only if buffer tracing is enabled */
	vm_queue_data_t b_traceq;
	caddr_t	b_traceaddr;
};

#ifdef	KERNEL
#define	BUFHSZ	64
#define RND	(VM_PAGESIZE/DEV_BSIZE)

/* These routines are really part of the ufs */
struct	buf *alloc();
struct	buf *realloccg();

/*
 * Structure used to pass additional information to the buffer cache
 */
struct bio {
	unsigned      bio_flags;
#define BIO_READ	0x0001	/* This is a read operation */
#define BIO_LOCAL	0x0002	/* Allocate page locally rather than I/O node*/
#define BIO_ASYNC	0x0004	/* operation will be asynchronous */
#define BIO_ZEROED	0x0008	/* Prefer a zeroed page */
#define BIO_CACHE	0x0010	/* map the buffer cached */
#define BIO_READONLY	0x0020	/* page will not be written */
	vm_page_t     bio_page;	/* Pre-allocated page  */
};

/*
 * Interface to the buffer cache - sys/vfs_bio.c
 */

/* Flags to 'buffer invalidate' */
#define BINVAL_DIRTYOK 	0x01	/* ignore dirty buffers */
#define BINVAL_ALL	0x02	/* wait for all buffers */

/* Backward compatability */
#define getblk(vp, blkno, size) Getblk(vp, blkno, size, (struct bio *)0)
#define bread(vp, blkno, size) Bread(vp, blkno, size, (struct bio *)0)
#define breada(vp, blkno, size, rablkno, rabsize) \
    Breada(vp, blkno, size, rablkno, rabsize, (struct bio *)0)

#define bflush(vp) blkflush(vp, 0, VM_PAGESIZE)
#define brelse(bp) Brelse(bp, 0)

#define incore(vp, blkno) Incore(vp, blkno, VM_PAGESIZE)

#define biowait(bp) Biowait(bp, PRIBIO)
#define binval(vp) Binval(vp, 0)
#define binvalfree(vp) { struct vnode *tmpvp = (vp); \
			 bflush(vp); \
			 Binval(vp, BINVAL_DIRTYOK); \
		       }
#ifdef __STDC__
struct buf *Getblk(struct vnode *, daddr_t, int, struct bio *);
struct buf *getzbuf(void);		/* Get a constant, zero, page */
struct	buf *Bread(struct vnode *, daddr_t, int, struct bio *);
struct	buf *Breada(struct vnode *, daddr_t, int, daddr_t, int, struct bio *);
int Incore(struct vnode *, daddr_t, int);

struct buf *geteblk(int);

struct buf *getblk_overlap(struct vnode *, daddr_t, int, struct buf *);

void blkflush(struct vnode *, daddr_t, int);
void Binval(struct vnode *, int);
void bflushall(void);

int Brelse(struct buf *, int);

void buf_rempage(vm_page_t);
void buf_privatize(vm_page_t);

struct	vnode *bdevvp(dev_t);	/* specfs/spec_subr.c */

unsigned minphys();		/* sys/vm_swp.c */

caddr_t bp_getpaddr(struct buf *, caddr_t);
#else
struct buf *Getblk();
struct buf *Bread();
struct buf *Breada();
int Incore();
struct buf *getblk_overlap();
struct buf *getzbuf();
struct buf *geteblk();

void buf_rempage();
void buf_privatize();

void blkflush();
void Binval();
void bflushall();
int Brelse();

struct	vnode *bdevvp();	/* specfs/spec_subr.c */

unsigned minphys();		/* sys/vm_swp.c */

caddr_t bp_getpaddr();

#endif

extern struct buf *brelse_list;	/* List of buffers to be brelsed at 
				 * process level
				 */
#endif

/*
 * These flags are kept in b_flags.
 */
#define	B_WRITE		0x00000000	/* non-read pseudo-flag */
#define	B_READ		0x00000001	/* read when I/O occurs */
#define	B_DONE		0x00000002	/* transaction finished */
#define	B_ERROR		0x00000004	/* transaction aborted */
#define	B_PHYS		0x00000008	/* physical IO (to raw device) */
#define	B_ASYNC		0x00000010	/* don't wait for I/O completion */
#define	B_TAPE		0x00000020	/* this is a magtape (no bdwrite) */
#define	B_CACHE		0x00000040	/* Cache data in this buffer */
#define	B_NOCACHE	0x00000080	/* don't cache block when released */
#define	B_RAWBUF	0x00000100	/* xy: Special iopb in b_count */
#define	B_NOECC		0x00000200	/* xy: No ECC correction */
#define	B_IOCTL		0x00000400	/* xy: Special request from xyioctl */
#define	B_MAPIO		0x00000800	/* xy: Special io for mapped sectors */
#define	B_PHYSMEM	0x00001000	/* b_addr is physical address */
#define B_STRIPE        0x00002000      /* buf is part of a stripe request. */
#define B_MULTIO        0x00004000      /* buf is part of a multio request. */
#define	B_USRSPACE	0x00008000	/* b_addr is user addr */
#ifdef BFLY2
#define B_INTRLVD	0x00010000	/* b_addr is an interleaved address */
#endif BFLY2
#define B_ZERO		0x00020000	/* Special 'always zero' buffer */
#define B_KEEPPAGE	0x00040000	/* Buffer page was passed down */
#define B_WAKEUP	0x00080000	/* Wakeup needed on iodone */
#define B_AGE		0x00100000	/* Buffer data may not be useful */
#define B_BIO		0x00200000	/* Set on all real buffers */
#define B_ZEROED	0x00400000	/* buffer page was from zeroed list */
#define B_GETEBLK	0x00800000	/* Buffer allocated via geteblk */
#define B_READONLY	0x01000000	/* read-only data */
#define B_BACKPOCKET	0x02000000	/* emergency page */

/* Bits relevant at the driver level */
#define B_DRIVERBITS \
     (B_READ	|\
      B_WRITE	|\
      B_PHYS	|\
      B_ASYNC	|\
      B_RAWBUF	|\
      B_NOECC	|\
      B_IOCTL	|\
      B_MAPIO	|\
      B_PHYSMEM	|\
      B_STRIPE	|\
      B_MULTIO	|\
      B_USRSPACE|\
      B_INTRLVD)

#define	iodone	biodone
#define	iowait	biowait

#endif __SYS_BUF_H
