#ifdef 	KERNEL
#include "../h/types.h"
#else KERNEL
#include <sys/types.h>
#endif KERNEL

#define PART_CNT 	0x8            /* Number of Partitions (max 8) 		*/

/* Butterfly Disk Format structures */
#define NPARTITIONS 8
#define NUM_BACKUP_LABELS 3
#define TRKS_PER_BACKUP 2

/* STRUCTURE USED TO DEFINE LOGICAL PARTITIONS ON THE DISK */
typedef struct smd_part {
  unsigned int    offset;      /* sector offset of start of partition */
  unsigned int    size;        /* number of sectors in the partition */
} SMD_PART;

/* Structure used for a stripe group label entry. Each label that is
 * part of a stripe group will have to following structure filled in.
 */
#define MAX_SGROUP_ID 32
#define MAX_SGROUP_SIZE 32

/* VERSION 0000 */
struct sgroup_label_entry_v0000 {
    char group_id ;                    /* Index into the sgroup_id_table. */
    char group_size;                   /* Number of disks in this sgroup. */
    char ldisk;                        /* Logical disk number in sgroup. */
    struct {
	unsigned long hi;
	unsigned long lo;
    } unique_id ;                      /* Unique id for each disk in sgroup */
    long stripe_size[NPARTITIONS];     /* a per partition stripe size */
} ;

/* VERSION 0001 */
struct sgroup_label_entry_v0001 {
    struct {
	char group_id;                 /* Index into the sgroup_id_table. */
	char group_size;               /* Number of disks in this sgroup. */
	char ldisk;                    /* Logical disk number in sgroup. */
	long stripe_size;              /* Partition stripe size */
    } part[NPARTITIONS];               /* Up to one group per partition. */
    struct {
	unsigned long hi;
	unsigned long lo;
    } unique_id ;                      /* Unique id for each disk in sgroup */
} ;

/* VERSION 0002 */
struct sgroup_label_entry_v0002 {
    struct {
	char group_id;                 /* Index into the sgroup_id_table. */
	char group_size;               /* Number of disks in this sgroup. */
	char ldisk;                    /* Logical disk number in sgroup. */
	long stripe_size;              /* Partition stripe size */
	struct {
	    unsigned long hi;
	    unsigned long lo;
	} unique_id ;                  /* Unique id for each disk in sgroup */
    } part[NPARTITIONS];               /* Up to one group per partition. */
} ;

union sgroup_label_entry {
    struct sgroup_label_entry_v0000 v0000;
    struct sgroup_label_entry_v0001 v0001;
    struct sgroup_label_entry_v0002 v0002;
} ;

#define SD_MAGIC 0x8161
#define LABEL_MAGIC_NUM 0
/* STRUCTURE USED TO HOLD THE CURRENT DRIVE PARAMETER SETTINGS */
typedef struct smd_label {
  unsigned short lbl_chksum;	/* checksum of disk label */
  unsigned short lbl_magic;		/* label version id */
  unsigned short lbl_nheads;	/* number of read/write heads */
  unsigned short lbl_ncyls;		/* number of cylinders */
  unsigned short lbl_nsects;	/* number of sectors per track (less spares) */
  unsigned short lbl_sectsize;	/* number of bytes per sector (overhead included) */
  unsigned short lbl_sectspares;/* number of spare sectors for slipping */
  unsigned short lbl_trkspares;	/* number of spare tracks for slipping */
  unsigned short lbl_altcyls;	/* number of alternate cylinders */
  unsigned short lbl_interleave;/* interleave factor */
  unsigned short lbl_label;		/* sector offset of this data structure */
  unsigned short lbl_defect;	/* sector offset of defect list */
  unsigned short lbl_badsect;	/* sector offset of bad sector list */
  unsigned short lbl_sectmap;	/* sector offset of unslippable sector map */
  char		 lbl_pad1[128 - (14 * sizeof(unsigned short))];
  SMD_PART   lbl_part[NPARTITIONS];/* partitions information */
  char		 lbl_pad2[128 - (NPARTITIONS * sizeof(struct smd_part))];
  char		 lbl_name[64];	/* disk id string */

/* Entries to support Disk Striping. */

  char stripe_magic[8];   /* set to "sg<version>" if disk is part of sgroup. */
  union sgroup_label_entry sg_label ; /* layout of this logical disk. */
  char lbl_pad3[256 - ((64*sizeof(char)) + (8*sizeof(char)) +
		sizeof(union sgroup_label_entry))];
} SMD_LABEL;

/*
 * Arbitrarily picked 8k - gives us over 600 map entries.
 * Since we have around 600 alt sectors to map to, seems like a reasonable
 * number.  Make sure that this number is always a multiple of 512 for
 * disk driver.
 */
#define MAX_SECTMAP_SIZE	8192
/* Structure used to hold one mapped sector list element */
typedef struct smd_sectmap {
  unsigned short badcyl;	/* cylinder of bad sector */
  unsigned short badhead;	/* read/write head of bad sector */
  unsigned short badsector;	/* logical sector number of bad sector */
  unsigned short cyl;		/* cylinder of good */
  unsigned short head;		/* read/write head of good sector */
  unsigned short sector;	/* logical sector number of good sector */
} SMD_SECTMAP;

/* Structure used to hold a whole mapped sector list */
typedef struct smd_sectmap_tab {
  unsigned short checksum;	/* checksum of sector map table */
  unsigned short nummapped;	/* number of mapped sectors in the table */
  SMD_SECTMAP    mapsects[1];	/* array of mapped sector entries */
} SMD_SECTMAP_TAB;

/* Sun disk label definitions, meant to facilitate dual-porting */

#define	DKL_MAGIC	0xDABE	/* magic number */
#define	NDKMAP	8		/* # of logical partitions */

/*
 * Format of a Sun SMD disk label.
 * Resides in cylinder 0, sector 0 on each head which is the first
 * head of a physical partition (e.g., heads 0 and 2 for a CDC Lark).
 * dkl_ppart gives the physical partition number (currently only 0 or 1).
 * dkl_bhead must match the head on which the label is found;
 * otherwise the label was probably overwritten by another.
 *
 * sizeof(struct dk_label) should be 512 (sector size)
 */

struct dk_label {
    char	dkl_asciilabel[128];	/* for compatibility */
char	dkl_pad[512-(128+8*8+12*2)];
unsigned short	dkl_apc;	/* alternates per cylinder */
unsigned short	dkl_gap1;	/* size of gap 1 */
unsigned short	dkl_gap2;	/* size of gap 2 */
unsigned short	dkl_intrlv;	/* interleave factor */
unsigned short	dkl_ncyl;	/* # of data cylinders */
unsigned short	dkl_acyl;	/* # of alternate cylinders */
unsigned short	dkl_nhead;	/* # of heads in this partition */
unsigned short	dkl_nsect;	/* # of 512 byte sectors per track */
unsigned short	dkl_bhead;	/* identifies proper label location */
unsigned short	dkl_ppart;	/* physical partition # */
/* */
struct dk_map {			/* logical partitions */
    daddr_t	dkl_cylno;	/* starting cylinder */
    daddr_t dkl_nblk;	/* number of blocks */
} dkl_map[NDKMAP];
unsigned short	dkl_magic;	/* identifies this label format */
unsigned short	dkl_cksum;	/* xor checksum of sector */
};

/* CONTROLLER TYPES */
#define c421                0
#define c431                1
#define c432                2
#define c450                3
#define c450E               4
#define c451                5
#define c751                6


/* Flag contents */
#define NORMAL_ECC 0
#define NO_ECC 1

/* Completion codes returned by xylogics commands */
#define SUCCESS				0x0
#define INTERRUPT_PENDING		0x1
#define BUSY_CONFLICT			0x3
#define OPERATION_TIMEOUT		0x4
#define HEADER_NOT_FOUND		0x5
#define HARD_ECC_ERROR			0x6
#define ILLEGAL_CYLINDER_ADDRESS	0x7
#define ILLEGAL_SECTOR_ADDRESS		0xA
#define LAST_SECTOR_TOO_SMALL		0xD
#define SLAVE_ACK_ERROR			0xE
#define CYL_AND_HEAD_HEADER_ERROR	0x12
#define AUTO_SEEK_RETRY_SUCCESS		0x13
#define WRITE_PROTECT_ERROR		0x14
#define UNIMPLEMENTED_COMMAND		0x15
#define DRIVE_NOT_READY			0x16
#define SECTOR_COUNT_ZERO		0x17
#define DRIVE_FAULTED			0x18
#define ILLEGAL_SECTOR_SIZE		0x19
#define SELF_TEST_A			0x1A
#define SELF_TEST_B			0x1B
#define SELF_TEST_C			0x1C
#define SOFT_ECC_ERROR			0x1E
#define SOFT_ECC_RECOVERED		0x1F
#define ILLEGAL_HEAD			0x20
#define DISK_SEQUENCER_ERROR		0x21
#define SEEK_ERROR			0x25

#define NO_SUCH_ADDRESS			0x26

#define COMP_CODE(x) ((x) & 0xff)
#define COMP_SECT(x) (((x) & 0xff00) >> 8)
#define SOFT_ERROR(x) ((((x) & 0xff) == AUTO_SEEK_RETRY_SUCCESS)\
		       || (((x) & 0xff) == SOFT_ECC_RECOVERED))

/* A completion code returned by read_trk_pattern when the xylogics
   controller found no problem, but the data didn't match the pattern. */
#define PATTERN_ERROR 0x26
