#if	CMU
/* 
 **********************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 **********************************************************************
 * HISTORY
 * 26-Mar-86  Mike Accetta (mja) at Carnegie-Mellon University
 *	CS_BUGFIX: Fixed KERNEL_FILE symbol to include all
 *	definitions.
 *
 * 29-Jan-86  Mike Accetta (mja) at Carnegie-Mellon University
 *	CS_COMPAT:  Added FDIROK definition.
 *
 * 25-Jan-86  Avadis Tevanian (avie) at Carnegie-Mellon University
 *	Upgraded to 4.3.
 *
 * 25-Nov-85  Mike Accetta (mja) at Carnegie-Mellon University
 *	CS_BUGFIX: Added new KERNEL_FILE symbol to enable struct
 *	file definitions when desired outside the kernel.
 *
 * 20-Jul-85  Mike Accetta (mja) at Carnegie-Mellon University
 *	CS_RFS:  Added DTYPE_RFSINO definition.
 *	[V1(1)]
 *
 * 13-May-85  Mike Accetta (mja) at Carnegie-Mellon University
 *	Upgraded from 4.1BSD.  Carried over change below.
 *	CS_COMPAT: retained pipe definitions for now,
 *	retained exclusive use definitions for now.
 *	[V1(1)]
 *
 **********************************************************************
 */
 
#ifdef	KERNEL
#include "cs_bugfix.h"
#include "cs_rfs.h"
#else	KERNEL
#include <sys/features.h>
#include <sys/lock.h>
#endif	KERNEL
#endif	CMU
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)file.h	7.1 (Berkeley) 6/4/86
 *	@(#)file.h	2.1 88/05/18 4.0NFSSRC SMI;	from UCB 7.1 06/04/86
 */

#ifdef KERNEL
#include "../h/fcntl.h"
#else KERNEL
#include <sys/fcntl.h>
#endif KERNEL

#if	CS_BUGFIX
#ifdef	KERNEL_FILE
#define	KERNEL
#endif	KERNEL_FILE
#endif	CS_BUGFIX

#ifdef KERNEL
/*
 * File struct queue next and prev pointers.
 */
struct	fileq {
	struct file *fq_next;
	struct file *fq_prev;
};

/*
 * Descriptor table entry.
 * One for each kernel object.
 */
struct	file {
	struct fileq f_queue;	/* file structure queue pointers */
	int	f_flag;		/* see below */
	short	f_type;		/* descriptor type */
	short	f_count;	/* reference count */
	short	f_msgcount;	/* references from message queue */
	struct	fileops {
		int	(*fo_rw)();
		int	(*fo_ioctl)();
		int	(*fo_select)();
		int	(*fo_close)();
	} *f_ops;
	simple_lock_data_t	f_lock;
	caddr_t	f_data;		/* ptr to specific struct (vnode/socket) */
	off_t	f_offset;
	struct ucred *f_cred;	/* credentials of user who opened file */
};

extern	struct fileq file;
extern	int nfile;
struct	file *getf();
struct	file *falloc();
#endif KERNEL

/*
 * flags- also for fcntl call.
 */
#define	FOPEN		(-1)
#define	FREAD		00001		/* descriptor read/receive'able */
#define	FWRITE		00002		/* descriptor write/send'able */
#define	FMARK		00020		/* mark during gc() */
#define	FDEFER		00040		/* defer for next gc pass */
#define	FSHLOCK		00200		/* shared lock present */
#define	FEXLOCK		00400		/* exclusive lock present */

/* bits to save after open */
#define	FMASK		020113		/* FSYNC|FASYNC|FAPPEND|FWRITE|FREAD */
#define	FCNTLCANT	(FREAD|FWRITE|FMARK|FDEFER|FSHLOCK|FEXLOCK)

/*
 * User definitions.
 */

/*
 * Flock call.
 */
#define	LOCK_SH		1	/* shared lock */
#define	LOCK_EX		2	/* exclusive lock */
#define	LOCK_NB		4	/* don't block when locking */
#define	LOCK_UN		8	/* unlock */

/*
 * Lseek call.
 */
#define	L_SET		0	/* absolute offset */
#define	L_INCR		1	/* relative to current offset */
#define	L_XTND		2	/* relative to end of file */

#ifdef KERNEL
#define	GETF(fp, fd) { \
	if ((unsigned)(fd) >= NOFILE || ((fp) = u.u_ofile[fd]) == NULL) { \
		u.u_error = EBADF; \
		return; \
	} \
}
#define	DTYPE_VNODE	1	/* file */
#define	DTYPE_SOCKET	2	/* communications endpoint */
#if	CS_RFS
#define	DTYPE_RFSINO	3	/* remote file system inode */
#define	DTYPE_RFSCTL	4	/* remote file system control */
#endif	CS_RFS
#endif
#if	CS_BUGFIX
#ifdef	KERNEL_FILE
#undef	KERNEL
#endif	KERNEL_FILE
#endif	CS_BUGFIX
