/*
 ****************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 ****************************************************************
 */
/*
 *	File:	h/mach_param.h
 *	Author:	Avadis Tevanian, Jr., Michael Wayne Young
 *
 *	Copyright (C) 1986, Avadis Tevanian, Jr., Michael Wayne Young
 *
 *	Mach system sizing parameters
 *
 * HISTORY
 * 18-Nov-86  Avadis Tevanian (avie) at Carnegie-Mellon University
 *	Increased PORT_MAX to 2048.  I really hate doing this because
 *	there are some algorithms that are linear related for PORT_MAX.
 *	They should of course be eliminated and are just left over from
 *	the Accent implementation.
 *
 * 21-Sep-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Added PORT_BACKLOG_MAX.
 *
 * 21-Sep-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Changed number of registered ports, to make the task structure
 *	power-of-two sized.  Don't ask.
 *
 * 31-Aug-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Added TASK_PORT_REGISTER_MAX.
 *
 * 10-Aug-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Added port parameters.
 *
 *  8-Aug-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Added number of ports definition.
 *
 * 24-Jul-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Increase TASK parameters.
 *
 * 29-Jan-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Created
 *
 */

#ifndef	_MACH_PARAM_
#define	_MACH_PARAM_

#define	THREAD_MAX	512		/* Max number of threads */
#define	THREAD_CHUNK	128		/* Allocation chunk */

#define	TASK_MAX	512		/* Max number of tasks */
#define	TASK_CHUNK	128		/* Allocation chunk */

#define	PORT_MAX	2048		/* Number of ports, system-wide */
#define	PORT_BACKLOG_DEFAULT	5
#define	PORT_BACKLOG_MAX	16

#define	TASK_PORT_REGISTER_MAX	4	/* Number of "registered" ports */
			
#endif	_MACH_PARAM_
