/*CMU:	$Header: /nfs/europa/u1/BFMACH.local/mk/h/RCS/machine.h,v 1.20 91/01/11 19:43:42 lkaplan Exp $	*/
/*
 ****************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 ****************************************************************
 */
/*
 *	File:	h/machine.h
 *	Author:	Avadis Tevanian, Jr.
 *
 *	Copyright (C) 1986, Avadis Tevanian, Jr.
 *
 *	Machine independent machine abstraction.
 *
 ************************************************************************
 * HISTORY
 * 30-Oct-86  Avadis Tevanian (avie) at Carnegie-Mellon University
 *	Add cpu_idle array.
 *
 * 24-Sep-86  Michael Young (mwyoung) at Carnegie-Mellon University
 *	Changed to directly import boolean declaration.
 *
 * 16-May-86  Avadis Tevanian (avie) at Carnegie-Mellon University
 *	Created.
 *
 ************************************************************************
 */

#ifndef	_MACHINE_H_
#define	_MACHINE_H_

#ifdef	KERNEL
#include "cpus.h"
#include "../vm/vm_param.h"
#include "../h/boolean.h"
#else	KERNEL
#include <sys/features.h>
#include <vm/vm_param.h>
#include <sys/boolean.h>
#endif	KERNEL

/*
 *	For each host, there is a maximum possible number of
 *	cpus that may be available in the system.  This is the
 *	compile-time constant NCPUS, which is defined in cpus.h.
 *
 *	In addition, there is a machine_slot specifier for each
 *	possible cpu in the system.
 */

struct machine_info {
	int		max_cpus;	/* max number of cpus compiled */
	int		avail_cpus;	/* number actually available */
	vm_size_t	memory_size;	/* size of memory in bytes */
};

typedef struct machine_info	*machine_info_t;

typedef int	cpu_type_t;
typedef int	cpu_subtype_t;

struct machine_slot {
	boolean_t	is_cpu;		/* is there a cpu in this slot? */
	int		phys_cpu_number;
	cpu_type_t	cpu_type;	/* type of cpu */
	cpu_subtype_t	cpu_subtype;	/* subtype of cpu */
	boolean_t	running;	/* is cpu running */
};

typedef struct machine_slot	*machine_slot_t;

#ifdef	KERNEL
struct machine_info		machine_info;
struct machine_slot		machine_slot[NCPUS];

extern char should_exit;
extern char cpu_idle;
vm_offset_t	interrupt_stack[NCPUS];
#endif	KERNEL

/*
 *	Machine types known by all.
 */

#define	CPU_TYPE_VAX		((cpu_type_t) 1)
#define	CPU_TYPE_ROMP		((cpu_type_t) 2)
#define	CPU_TYPE_MC68020	((cpu_type_t) 3)
#define CPU_TYPE_NS32032	((cpu_type_t) 4)
#define	CPU_TYPE_BFLYPLUS	((cpu_type_t) 5)
#ifdef BFLY2
#define CPU_TYPE_BFLYTWO	((cpu_type_t) 6)
#endif BFLY2

/*
 *	Machine subtypes (these are defined here, instead of in a machine
 *	dependent directory, so that any program can get all definitions
 *	regardless of where is it compiled).
 */

/*
 *	VAX subtypes (these do *not* necessary conform to the actual cpu
 *	ID assigned by DEC available via the SID register).
 */

#define CPU_SUBTYPE_VAX780	((cpu_subtype_t) 1)
#define CPU_SUBTYPE_VAX785	((cpu_subtype_t) 2)
#define CPU_SUBTYPE_VAX750	((cpu_subtype_t) 3)
#define CPU_SUBTYPE_VAX730	((cpu_subtype_t) 4)
#define CPU_SUBTYPE_UVAXI	((cpu_subtype_t) 5)
#define CPU_SUBTYPE_UVAXII	((cpu_subtype_t) 6)
#define CPU_SUBTYPE_VAX8200	((cpu_subtype_t) 7)
#define CPU_SUBTYPE_VAX8500	((cpu_subtype_t) 8)
#define CPU_SUBTYPE_VAX8600	((cpu_subtype_t) 9)
#define CPU_SUBTYPE_VAX8650	((cpu_subtype_t) 10)
#define CPU_SUBTYPE_VAX8800	((cpu_subtype_t) 11)

/*
 *	ROMP subtypes.
 */

#define CPU_SUBTYPE_RT_PC	((cpu_subtype_t) 1)

/*
 *	68020 subtypes.
 */

#define CPU_SUBTYPE_SUN3	((cpu_subtype_t) 1)

/*
 *	32032 subtypes.
 */

#define CPU_SUBTYPE_MMAX	((cpu_subtype_t) 1)

#ifdef butterfly

/*
 *	Subtypes in the butterfly consist of orthogonal bits in a 32-bit mask.
 *	These defines are not lowest order to allow their use in secboot.c.
 *	The bits correspond to the following.
 */

#define NODE_TYPE_CPU		((cpu_subtype_t) 0x10)
#define NODE_TYPE_VME		((cpu_subtype_t) 0x20)

#define	NODE_TYPE_CPU_VME_MASK	(NODE_TYPE_CPU|NODE_TYPE_VME)

/*
 *	Butterfly plus subtypes.
 */

#define	CPU_SUBTYPE_BFLYPLUS	((cpu_subtype_t) 1)	/* unsused */

#ifdef BFLY2

/*
 *	Butterfly two subtypes.
 */

#define	CPU_SUBTYPE_BFLYTWO	((cpu_subtype_t) 1)	/* unused */

/* b2vmes have both processors and vmes */

#define	NODE_TYPE_B2VME		((cpu_subtype_t) NODE_TYPE_CPU|NODE_TYPE_VME)

/* NOTE: The following node type bits must not overlap the cluster type
   bits defined in cluster.h */

#define	NODE_TYPE_ICACHE1	((cpu_subtype_t) 0x80000000)
#define	NODE_TYPE_ICACHE2	((cpu_subtype_t) 0x40000000)
#define	NODE_TYPE_DCACHE1	((cpu_subtype_t) 0x20000000)
#define	NODE_TYPE_DCACHE2	((cpu_subtype_t) 0x10000000)
#define	NODE_TYPE_16MHZ		((cpu_subtype_t) 0x08000000)
#define	NODE_TYPE_20MHZ		((cpu_subtype_t) 0x04000000)
#define	NODE_TYPE_4MB		((cpu_subtype_t) 0x01000000)
#define	NODE_TYPE_16MB		((cpu_subtype_t) 0x00400000)
#define	NODE_TYPE_ICACHE4	((cpu_subtype_t) 0x00040000)
#define	NODE_TYPE_DCACHE4	((cpu_subtype_t) 0x00020000)

#define	NODE_TYPE_SPEED_MASK	(NODE_TYPE_16MHZ|NODE_TYPE_20MHZ)
#define	NODE_TYPE_SIZE_MASK	(NODE_TYPE_4MB|NODE_TYPE_16MB)
#define	NODE_TYPE_ICACHE_MASK	(NODE_TYPE_ICACHE1|NODE_TYPE_ICACHE2 \
				 |NODE_TYPE_ICACHE4)
#define	NODE_TYPE_DCACHE_MASK	(NODE_TYPE_DCACHE1|NODE_TYPE_DCACHE2 \
				 |NODE_TYPE_DCACHE4)

#endif BFLY2

#define	NODE_TYPE_PUBLIC	((cpu_subtype_t) 0x00010000)

#define NODE_TYPE_BPNE		NODE_TYPE_CPU
#define NODE_TYPE_BVME		NODE_TYPE_VME

#endif butterfly

#endif	_MACHINE_H_

