/* $Header: mbuf.h,v 1.18 90/02/02 06:36:29 tappan Exp $ */
/*
 * Copyright (c) 1982, 1986, 1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of California at Berkeley. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 *
 *	@(#)mbuf.h	7.10 (Berkeley) 2/8/88
 *	@(#)mbuf.h	2.1 88/05/18 4.0NFSSRC SMI;	from UCB 7.1 6/4/86
 */

#ifndef _mbuf_h

#define _mbuf_h yes

#include "kmalloc.h"

/*
 * Constants related to memory allocator.
 */
#define	MSIZE		256	/* size of an mbuf */
#define	MCLBYTES	2048	/* Use 2K cluster sizes */
#define	MCLSHIFT	11	/* Log2(2K) */
#define	MCLOFSET	(MCLBYTES-1) /* mask */

#define	MMINOFF		12			/* mbuf header length */
#define	MTAIL		4
#define	MMAXOFF		(MSIZE-MTAIL)		/* offset where data ends */
#define	MLEN		(MSIZE-MMINOFF-MTAIL)	/* mbuf data length */

/*
 * Macros for type conversion
 */

/* address in mbuf to mbuf head */
#define	dtom(x)		((struct mbuf *)((int)x & ~(MSIZE-1)))

/* mbuf head, to typed data */
#define	mtod(x,t)	((t)((int)(x) + (x)->m_off))

/*
 * Standard 4.3 supports two kinds of mbufs: normal mbufs, with data
 * residing in the mbuf itself; and cluster mbufs with data stored
 * elsewhere (at a higher address than the corresponding mbuf).
 * The data area for cluster mbufs is allocated as a pool and is
 * managed by the mbuf system itself.  We add another kind of cluster
 * mbuf, whose data area is not part of the mbuf system.  Instead,
 * a routine wishing to use this kind of mbuf must provide a buffer
 * for the storage associated with the mbuf and a pointer to a routine
 * to free the buffer when the mbuf is freed.  The fields in mun_cl
 * below record the necessary bookkeeping information.
 *
 * Under nX all mbufs are allocated using the KMALLOC allocator.
 * Because the 4.3 cluster reference count scheme does not work if 
 * mbufs are not contiguous virtually, we have added a new field
 * 'm_refcnt' to the cluster mbufs. This can be 0 if for loaned mbufs
 * if reference counts are not used.
 */
struct mbuf {
	struct	mbuf *m_next;		/* next buffer in chain */
	u_long	m_off;			/* offset of data */
	short	m_len;			/* amount of data in this mbuf */
	short	m_type;			/* mbuf type (0 == free) */
	union {
		u_char	mun_dat[MLEN];	/* data storage */
		struct {
			short	mun_cltype;	/* "cluster" type */
			int	(*mun_clfun)();
			int	mun_clarg;
			int	(*mun_clswp)();
			/* Use a short for the reference count so we
			 * can use Atomic_add16 to manipulate it:
			 * more efficent than Atomic_add32 on the
			 * bfly1
			 */
			short	*mun_refcnt; 
			u_long mun_clmask; /* mask to start of buffer */
		} mun_cl;
	} m_un;
	struct	mbuf *m_act;		/* link in higher-level mbuf list */
};
#define	m_dat	m_un.mun_dat
#define	m_cltype m_un.mun_cl.mun_cltype
#define	m_clfun	m_un.mun_cl.mun_clfun
#define	m_clarg	m_un.mun_cl.mun_clarg
#define	m_clswp	m_un.mun_cl.mun_clswp
#define m_refcnt m_un.mun_cl.mun_refcnt
#define m_clmask m_un.mun_cl.mun_clmask

/* mbuf types */
#define	MT_FREE		0	/* should be on free list */
#define	MT_DATA		1	/* dynamic (data) allocation */
#define	MT_HEADER	2	/* packet header */
#define	MT_SOCKET	3	/* socket structure */
#define	MT_PCB		4	/* protocol control block */
#define	MT_RTABLE	5	/* routing tables */
#define	MT_HTABLE	6	/* IMP host tables */
#define	MT_ATABLE	7	/* address resolution tables */
#define	MT_SONAME	8	/* socket name */
#define	MT_ZOMBIE	9	/* zombie proc status */
#define	MT_SOOPTS	10	/* socket options */
#define	MT_FTABLE	11	/* fragment reassembly header */
#define	MT_RIGHTS	12	/* access rights */
#define	MT_IFADDR	13	/* interface address */

/*
 * Values for m_cltype: applicable only for cluster mbufs
 */
#define	MCL_STATIC	1	/* data in mbuf cluster pool */
#define	MCL_LOANED	2	/* data allocated elsewhere and "loaned"

/* flags to m_get */
#define	M_DONTWAIT	0
#define	M_WAIT		1

/* length to m_copy to copy all */
#define	M_COPYALL	1000000000

/*
 * m_pullup will pull up additional length if convenient;
 * should be enough to hold headers of second-level and higher protocols. 
 */
#define	MPULL_EXTRA	32

/*
 * Macros for mbuf manipulation.
 *
 */

/*
 * MGET_ON_NODE - allocate an mbuf on a particular node
 */
#define MGET_ON_NODE(m, i, t, n) { \
    KMALLOC_ON_NODE(m, struct mbuf *, MSIZE, \
		    i == M_DONTWAIT ? KMALLOC_NOWAIT : KMALLOC_WAIT, n, \
		    KMALLOC_MAKETYPE(KM_MT_MBUF, t)); \
    if (m) { \
        (m)->m_off = MMINOFF; \
        (m)->m_type = (t); \
        (m)->m_next = 0; \
	(m)->m_act = 0; \
        Atomic_add16(&mbstat.m_mtypes[(t)], 1); \
    }\
    }

/*
 * MGET(var mbuf, canwait, type)
 */
#define	MGET(m, i, t) \
    MGET_ON_NODE(m, i, t, my_cpu_number)			    


/*
 * Mbuf page cluster macros.
 * MCLALLOC allocates mbuf page clusters.
 * Note that it works only with a count of 1 at the moment.
 * MCLGET adds such clusters to a normal mbuf.
 * m->m_len is set to MCLBYTES upon success, and to MLEN on failure.
 */
#define	MCLALLOC(m, i) \
    KMALLOC(m, struct mbuf *, MCLBYTES, 0, KM_MT_MCL)

#define MCLALLOC_ON_NODE(m, i, n) \
    KMALLOC_ON_NODE(m, struct mbuf *, MCLBYTES, 0, n, KM_MT_MCL)

#define	M_HASCL(m)	((m)->m_off >= MSIZE)
#define	MTOCL(m)	((struct mbuf *)(mtod((m), int) & m->m_clmask))

#define	MCLGET_ON_NODE(m, node) \
	{ struct mbuf *p; \
	  MCLALLOC_ON_NODE(p, 1, node); \
	  if (p) { \
		(m)->m_off = (int)p - (int)(m); \
		(m)->m_len = MCLBYTES; \
		(m)->m_cltype = MCL_STATIC; \
		(m)->m_refcnt = 0; \
		(m)->m_clmask = ~(MCLBYTES-1); \
	  } else \
		(m)->m_len = MLEN; \
	}

#define MCLGET(m) MCLGET_ON_NODE(m, my_cpu_number)

/*
 * The new reference count scheme requires that the reference 
 * count be maintained from the pointer mbuf, not from the cluster
 * mbuf.
 * Therefore MCLFREE should not be used unless one is certain
 * that the reference count is 0.
 */
/*
 * MCLFREE must be called at splimp with the mbuf_lock held.
 */
#define	MCLFREE(m) { \
	printf("MCLFREE called\n"); \
	KFREE(m, KM_MT_MCL); \
	}

#define	MFREE(m, n) { \
	int mtype = (m)->m_type; \
	if (M_HASCL(m) && \
            ((m)->m_refcnt == 0 || \
	    Atomic_add16((m)->m_refcnt, -1) == 1)) \
		switch (m->m_cltype) { \
		case MCL_STATIC: \
		      if ((m)->m_refcnt != 0) \
			  mref_free((m)->m_refcnt);\
		      (n) = MTOCL(m); \
		      KFREE(n, KM_MT_MCL);\
		      break; \
		case MCL_LOANED: \
		      (*m->m_clfun)(m->m_clarg); \
		      break; \
		default: \
		       panic("MFREE"); \
		} \
	Atomic_add16(&mbstat.m_mtypes[mtype], -1); \
        (n) = (m)->m_next; \
	KFREE(m, KMALLOC_MAKETYPE(KM_MT_MBUF, mtype)); \
}
/*
 * Mbuf statistics.
 */
struct mbstat {
	u_long	m_mbufs;	/* mbufs obtained from page pool */
	u_long	m_clusters;	/* clusters obtained from page pool */
	u_long	m_spare;	/* spare field */
	u_long	m_clfree;	/* free clusters */
	u_long	m_drops;	/* times failed to find space */
	u_long	m_wait;		/* times waited for space */
	u_long	m_drain;	/* times drained protocols for space */
	u_short	m_mtypes[256];	/* type specific mbuf allocations */
};

#ifdef	KERNEL
struct	mbstat mbstat;
struct	mbuf *m_get(),*m_getclr(),*m_free(),*m_more(),*m_copy(),*m_pullup();
short *mref_alloc();
caddr_t	m_clalloc();
struct	mbuf *mclgetx();
#endif
#endif _mbuf_h

