/* $Header: node.h,v 1.19 90/02/26 10:06:21 gibson Exp $ */
/*****************************************************************************
* File: node.h - definitions for the node pseudo-device driver.              *
*****************************************************************************/

#ifndef ASSEMBLER
/* note: requires ioctl.h */

typedef struct
	{
	int		     	nodenum;	/* node number in question			*/
	unsigned long	memsize;	/* also: number of bytes of ucode	*/
	unsigned long   crp_hi;		/* CPU root pointer (start cmd)		*/
								/* NULL=>leave PMMU alone			*/
	unsigned long	crp_lo;		/* more of same						*/	
	unsigned long	tc;			/* translation control reg (start)	*/
	unsigned long	pc;			/* pc for start						*/
	} NIOCMD;

typedef struct
	{
	int	nio_nodenum;			/* node to do op to */
	int	nio_space;		       /* which part of address space*/
	char	*nio_addr;			/* ff space address */
	union
		{
		unsigned long	_nio_value;	/* OUT */
		unsigned short	_nio_svalue;	/* OUT */
		unsigned char	_nio_cvalue;	/* OUT */
		} _nio_arg;
	unsigned long	nio_size;		/* length (1,2, or 4) */
	} NIORWFFADDR;
#define	nio_value	_nio_arg._nio_value
#define	nio_svalue	_nio_arg._nio_svalue
#define	nio_cvalue	_nio_arg._nio_cvalue

typedef struct
{
	int	physnode;
	unsigned int	type;
} NIOPROBE;

#define	DB_MEM		0
#define	DB_SS0		1
#define	DB_IO		2
#define	DB_MB_MEM	3
#define	DB_MB_SWAP_MEM	4
#define	DB_MB_IO	5
#define	DB_MB_SWAP_IO	6

/*****************************************************************************
* NIOCNODEP. Node Existence Predicate. Uses remote block transfer to         *
* decide if the node in question exists. Looks for the USD variable          *
* s_pnn on the remote node. If we find the appropriate value, the node       *
* exists.  Returns the node type.                                            *
*****************************************************************************/

#define NIOCNODEP	_IOWR(n, 0, NIOPROBE)

/*****************************************************************************
* NIOCNODEMEM. Node Memory sizing predicate.  Tries to block transfer        *
* data from the node in question, handles the expected PNC errors.           *
*****************************************************************************/

#define NIOCNODEMEM _IOWR(n, 1, NIOCMD)

/*****************************************************************************
* NIOCSTART.  Starts the indicated node at the supplied address.  Returns    *
* EINVAL if the node isn't in the USD active state.                          *
*****************************************************************************/

#define NIOCSTART	_IOWR(n, 2, NIOCMD)

/*****************************************************************************
* NIOCSTOP.  Returns the indicated node to the USD active state.  At         *
* best a suggestion, since this operation requires cooperation from the      *
* current operating system.  Returns EINVAL if the node is already in        *
* the USD active state.                                                      *
*****************************************************************************/

#define NIOCSTOP 	_IOWR(n, 3, NIOCMD)

/*****************************************************************************
* NIOCMACHPARM. Mach bootstrap operation, sets up the NIOCMD PMMU            *
* registers and pc from the appropriate kernel variables.  Meant to save     *
* a little sweat in the short term.                                          *
*****************************************************************************/

#define NIOCMACHPARM	_IOWR(n, 4, NIOCMD)
#define CURRENT_NODE	(-1)

/*****************************************************************************
* NIOCVTOP. Returns the result of vtop() on the supplied address.  Operates  *
* on the PC member of the NIOCMD structure.                                  *
*****************************************************************************/

#define NIOCVTOP		_IOWR(n, 5, NIOCMD)

/*****************************************************************************
* NIOCGIST. Returns the virtual address in the supplied node's               *
* gist_pointer as well as the physical and virtual addresses of the          *
* node's gist_buffer.  NB: the supplied node in this case is a LOGICAL node. *
* Also returns the supplied logical node's physical node in nodenum field.   *
* This is a hack to make GIST available at low cost			     *
*****************************************************************************/

#define NIOCGIST		_IOWR(n, 6, NIOCMD)
/*****************************************************************************
 * NIOCGPHYSNODE. Maps logical to physical node.  Supplied		     *
 * logical node, in nodenum field, is replaced by physical node.	     *
*****************************************************************************/
#define NIOCGPHYSNODE		NIOCGIST

/*****************************************************************************
* NIOCGFFADDR. Returns the contents of the FF space address on the specified
* node.
*****************************************************************************/

#define	NIOCGFFADDR		_IOWR(n, 7, NIOCMD)

/*****************************************************************************
* NIOCSFFADDR. Sets the contents of the FF space address on the specified
* node.
*****************************************************************************/

#define	NIOCSFFADDR		_IOWR(n, 8, NIOCMD)

/*****************************************************************************
* NIOCENBGIST - Turns GIST event capture on or off for logical
* node NODENUM. NODENUM=-1 => all nodes. memsize =0 =>disable, =1 =>enable
*****************************************************************************/

#define NIOCENBGIST    _IOWR(n, 9, NIOCMD)

/*****************************************************************************
* NIOCRSTGIST - Resets the GIST buffer pointer for logical node NODENUM
* NODENUM=-1 => all nodes.
*****************************************************************************/

#define NIOCRSTGIST    _IOWR(n, 10, NIOCMD)

/*****************************************************************************
* NIOCRESET.  Returns the indicated node to the USD active state for BFLY1.  *
* or uboot ready state after reseting and clearing the node for BFLY2.	     *
* Returns EINVAL or EACCESS if the user does not have the necessary          *
* permissions on the node. 						     *
*****************************************************************************/

#define NIOCRESET 	_IOW(n, 11, NIOCMD)

#ifdef BFLY2
/*****************************************************************************
* NIOCSOFTRESET.  returns the indicated node uboot ready state without	     *
* reseting and clearing the node.  Only on BFLY2			     *
* Returns EINVAL or EACCESS if the user does not have the necessary          *
* permissions on the node. 						     *
*****************************************************************************/

#define NIOCSOFTRESET 	_IOW(n, 15, NIOCMD)
#endif BFLY2

/*****************************************************************************
* NIOCOWNGIST. Attempts to obtain ownership of the "gist resource,"          *
* returns 0 if all OK, EBUSY if not granted                                  *
*****************************************************************************/

#define NIOCOWNGIST  _IOWR(n, 12, NIOCMD)

/*****************************************************************************
* NIOCDISOWNGIST. Disowns the gist resource                                  *
* Return EINVAL if caller didn't own it, and np->tc != -1                    *
*****************************************************************************/

#define NIOCDISOWNGIST  _IOWR(n, 13, NIOCMD)

/* base address of the start command block transferred to
   remote nodes.  For no particularly compelling reason,
   we use the Mach primary bootstrap loadpoint.				*/

/*****************************************************************************
* NIOCGISTTRIG. Sets the GIST trigger position to the supplied offset from   *
* the trigger event. Takes the argument modulo the buffer size.  For it      *
* all to work the supplied offset (in bytes) MUST exactly hit an event       *
* position.  Place the offset into np->nodenum; -1 => no triggering.         *
*****************************************************************************/

#define NIOCGISTTRIG    _IOWR(n, 14, NIOCMD)

#ifdef BFLY2
/*****************************************************************************
* NIOCSLOTTING.  Returns an int that is non-zero when slotting is required   *
* to run the current configuration.  returns zero if slotting is not required*
*****************************************************************************/

#define NIOCSLOTTING	_IOR(n, 16, int)
#endif BFLY2
 
#endif ASSEMBLER

