/*****************************************************************************

   This header file is designed to allow the kernel and user programs to share
   header files properly as far as Posix is concerned.  The kernel
   option "BBN_KERNEL_POSIX" must not overlap the user's means for getting
   Posix compliant versions of include files, since <sys/features.h> may
   define BBN_KERNEL_POSIX, but this must not change the user's Posix
   compliant application's compile environment.  The user program should define
   _POSIX_SOURCE in order obtain the proper definitions from include files.
   If the user program is compiled with the "-posix" switch, however, 
   _BBN_POSIX_SUPPORT will be defined by the compiler driver for the user, 
   and the user will obtain the proper definitions from include files even if
   _POSIX_SOURCE was not defined.
   
   Any kernel header which is shared with user code AND contains Posix
   definitions will include this file in order to define _BBN_POSIX_SUPPORT
   as long as it is not already defined.  Later in the kernel header, the
   Posix definitions are controlled by an #ifdef _BBN_POSIX_SUPPORT.

   Therefore, if a user does #define _POSIX_SOURCE, and subsequently includes
   some such kernel header, _BBN_POSIX_SUPPORT will be defined, and the
   Posix definitions will appear.

******************************************************************************/
   
#ifndef _BBN_POSIX_SUPPORT
#   ifdef _POSIX_SOURCE
#      define _BBN_POSIX_SUPPORT
#   else _POSIX_SOURCE
#      ifdef KERNEL
#         include "bbn_kernel_posix.h"
#         if BBN_KERNEL_POSIX  
#            define _BBN_POSIX_SUPPORT
#         endif BBN_KERNEL_POSIX
#      endif KERNEL
#   endif _POSIX_SOURCE
#endif _BBN_POSIX_SUPPORT
