/*CMU:	$Header: /nfs/columbia/u1/BFMACH.local/mk/h/RCS/proc.h,v 1.42 91/01/28 09:32:00 tappan Exp $	*/
#if	CMU
/* 
 **********************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 **********************************************************************
 */
 
#ifdef	KERNEL
#include "cs_generic.h"
#include "cs_security.h"
#include "../h/time.h"
#include "../h/posix.h"
#include "../h/callout.h"
#else	KERNEL
#include <sys/features.h>
#include <sys/time.h>
#include <sys/posix.h>
#include <sys/callout.h>
#endif	KERNEL
#endif	CMU
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)proc.h	7.1 (Berkeley) 6/4/86
 *	@(#)proc.h	2.2 88/05/24 4.0NFSSRC SMI;	from UCB 7.1 6/4/86
 */
#ifndef _PROC_
#define	_PROC_

#if	CS_GENERIC
#ifdef	KERNEL
#include "../h/time.h"
#include "../h/types.h"
#include "../h/sched.h"
#else	KERNEL
#include <sys/time.h>
#include <sys/types.h>
#include <sys/sched.h>
#endif	KERNEL
#endif	CS_GENERIC

#ifdef	KERNEL
#include "../h/lock.h"
#else	KERNEL
#include <sys/lock.h>
#endif	KERNEL

/*
 * One structure allocated per active
 * process. It contains all data needed
 * about the process while the
 * process may be swapped out.
 * Other per process data (user.h)
 * is swapped with the process.
 */
struct proc {

	/*
	 * linked lists
	 */
	queue_chain_t p_rqlink;	/* current run queue links -- MUST BE FIRST */
	struct	proc *p_link;	/* linked list of sleeping processes */
	struct	proc *p_nxt;	/* linked list of allocated proc slots, */
	struct	proc **p_prev;	/*       or zombies, or free proc's */
	struct  proc *p_curnext;/* linked list of local procs for schedcpu */
	struct	proc *p_pptr;	/* pointer to process structure of parent */
	struct	proc *p_cptr;	/* pointer to youngest living child */
	struct	proc *p_osptr;	/* pointer to older sibling processes */
	struct	proc *p_ysptr;	/* pointer to younger siblings */

	/*
	 * waiting and sleeping
	 */
	caddr_t p_wchan;	/* event process is awaiting */
	struct	rusage2 *p_ru;	/* mbuf holding exit information for wait1 */
	unsigned long p_wcaller1;  /* sleep callers (BBN_SLEEP_DISORDER) */
	unsigned long p_wcaller2;  
	unsigned long p_wcaller3;
	long usw_handle;	/* user_sleep_and_unlock & new timeouts */

	/*
	 * general status/info
	 */
	int	p_stat;		
	int	p_flag;
	uid_t	p_uid;		/* user id, used to direct tty signals */
	short	p_pgrp;		/* name of process group leader */
	short	p_pid;		/* unique process id */
	short	p_idhash;	/* #ifdef OLDPFIND hashed based on p_pid */
	short	p_ppid;		/* process id of parent */
	u_short	p_xstat;	/* Exit status for wait */

	/*
	 * locking
	 */
	simple_lock_data_t p_lock; /* for signals and other stuff */
	int	p_lock_owner;	   /* used for debugging */
	int	p_lock_spl;	   /* used to save spl while p_lock is held */
	short	p_lock_splcount;   /* used only by proc_lock, proc_unlock */
	short	p_num_locks;	   /* # of simple locks (debugging only) */

	/*
	 * scheduling
	 */
	short	p_curnode;	/* node doing schedcpu */
	short	p_lastrun;	/* processor thread last active on */
	int	p_home_pnn;	/* home node (logical) */
	run_queue_t p_runq;	/* run queue p is on */

	char	p_usrpri;	/* user-priority based on p_cpu and p_nice */
	char	p_pri;		/* scheduling priority, negative is high */
				/* this is either usrpri (if running)
				 * or a sleep priority */
	char	p_cpu;		/* cpu usage for scheduling */
	char	p_time;		/* resident time for scheduling */
	char	p_nice;		/* nice for cpu usage */
	char	p_slptime;	/* time since last block */
	short	p_cpticks;	/* ticks of cpu time */
	float	p_pctcpu;	/* %cpu for this process during p_time */

	/*
	 * clusters
	 */
	int	p_home_cluster_id;  /* home cluster */
	int	p_clusters_created; /* # "created" actually */

	/*
	 * signals
	 */
	char	p_cursig;
	char	p_sigpad[3];
	int	p_sig;		/* signals pending to this process */
	int	p_sigmask;	/* current signal mask */
	int	p_sigignore;	/* signals being ignored */
	int	p_sigcatch;	/* signals being caught by user */

	/*
	 * sizes
	 */
	size_t	p_tsize;	/* size of text (clicks) */
	size_t	p_dsize;	/* size of data space (clicks) */
	size_t	p_ssize;	/* copy of stack size (clicks) */
	size_t 	p_rssize; 	/* current resident set size in clicks */
	size_t	p_maxrss;	/* copy of u.u_limit[MAXRSS] */
	size_t	p_swrss;	/* resident set size before last swap */

	/* 
	 * memory, pcb, vm
	 */
	vm_offset_t	p_pcb_addr;	/* pcb physical address */
	struct pcb	*p_pcb;		/* (kernel virtual) hardware pcb */
	vm_offset_t	p_kernel_stack;	/* Where kernel stack was allocated */
	vm_offset_t	p_recover;	/* page fault recovery (copyin/out) */
	vm_map_t	p_map;		/* map */

	/*
	 * ptrace 
	 */
	struct	proc *p_tracer;	/* pointer to attach'ing process */
	struct	proc *p_alist;	/* pointer to first attach'ed process */
	struct	proc *p_anext;	/* linked list of attached processes */

	/*
	 * miscellaneous
	 */
	int     p_async_count; /* async requests outstanding (see multio.c) */
	struct	itimerval p_realtimer;

	unsigned int p_uflagaddr; /* address of user nopreempt flag addr */
	unsigned int p_utimeaddr; /* address of user nopreempt timeout addr */

	/*
	 * POSIX specific state information
	 */
	u_short		p_posixflags,
			p_sessid;
	struct proc	*p_sessforw,
			*p_sessback;
	struct tty	*p_ttyp;
	uid_t		p_suid,
			p_ruid;
	gid_t		p_sgid;
	short		p_posixpad;

	/*
	 * Process tracing signal mask.
	 * Signals that are set in this mask are not intercepted by
	 * a debugging/tracing process, but get handled directly by the
	 * process itself.  This can be an important performance boost
	 * for programs that are standardly run under a debugger and
	 * handle frequent floating point or segment violation signals.
	 * A BBNACI OEM prospect runs production under a debugger with
	 * frequent SIGSEGV trap recovery, and motivated this addition.
	 * This field should move up with the signal fields later, I didn't
	 * want to change the shape or size of the proc struct right now.
	 * -Robert Wells, BBNACI, 8-Jan-1990.
	 */

	int     p_signotrace;	/* current ptrace signal mask */

	/*
	 * Special cell used to keep track of when the process is
	 * doing file system activity which makes it
	 * inappropriate to use the vnode pager while allocating
	 * a page.
	 */
	int	p_fsactive;

	/*
	 * Saved 'ast' state for the current process. This state
	 * includes:
	 *	- process has a signal pending that hasn't been noticed 
	 *	  yet.
	 *	- a higher priority process is ready to run
	 *	- process has a profiling tick pending.
	 */
	int	p_ast;

	/*
	 * Callout handle for realtimer callout */
	callout_handle_t	p_realtimerhandle;

	/* 
	 * NOTE:  The size of the proc structure must divide evenly
	 * into the size of a page.
	 */
#ifdef BFLY1
	int	bigpad[53];
#endif BFLY1
#ifdef BFLY2
	int	bigpad[54];
#endif BFLY2
};

/*
 * note:	local_nproc * ncpu == nproc
 *		local_nproc * sizeof(struct proc) == N * pagesize
 *		   where N is an integer
 */
#define PROC_SIZE_REQUIRED	512
int	nproc;		/* total number of proc structures */
int	local_nproc;	/* proc structures in local mem for each node */
/* pointer to beginning of local proc memory for node */
#define local_proc(node) (proc + (node)*local_nproc)


#define	PIDHSZ		64
#define	PIDHASH(pid)	((pid) & (PIDHSZ - 1))

#define MAXPID		30000	 /* actually can go over a little see newproc */

/*
   This is the pid/pgrp reference count hash table.  It is used for pid
   generation and various forms of pfind calls.
 */

struct pidcount
{
    short	     pid;
    unsigned short   count;
    struct pidcount *next;
    struct proc	    *procp;
};

struct pidcounthead
{
    simple_lock_data_t lock;
    struct pidcount   *next;
};

#ifdef KERNEL

#ifdef OLDPFIND
short	pidhash[PIDHSZ];
struct	proc *oldpfind();
#endif OLDPFIND

typedef struct proc *proc_t;
struct pidcounthead pidcounthash[PIDHSZ];
struct pidcount *pidfind();
struct	proc *pfind();
struct	proc *proc, *procNPROC;	/* the proc table itself */
struct	proc *zombproc, *allproc; /* lists of procs in various states */

extern proc_t active_procs[];	
extern proc_t my_active_proc;
#define	current_proc()	my_active_proc
#define PROC_NULL	((proc_t) 0)

/* each node has a free proc list with the proc structures in its local mem */
extern struct proc* freeproclist[];
/* always use alloc_proc and free_proc to manage freeproclist */
struct proc* alloc_proc();

/*
 * Notes on schedcpu and the curproc list
 *
 * Each node does schedcpu for procs in its own curproc list.
 * Each proc should be in my_curproc_list on the node p->p_curnode.
 * The list is linked by p->p_curnext.
 * The list is protected by the local schedlock.
 * The functions add_curproc and remove_curproc manage the list.
 */
extern proc_t my_curproc_list;
proc_t *curproc_ptr();		/* takes node as argument, does vvrmtptr */
#define P_NOCUR	 (0xff)		/* used to invalidate p_curnode */

#endif KERNEL

/* p_stat codes */
#define	SSLEEP	1		/* awaiting an event */
#define	SWAIT	2		/* (abandoned state) */
#define	SRUN	3		/* running */
#define	SIDL	4		/* intermediate state in process creation */
#define	SZOMB	5		/* intermediate state in process termination */
#define	SSTOP	6		/* process being traced */

/* flag codes */
#define	SLOAD	 0x00000001	/* in core */
#define	SSYS	 0x00000002	/* swapper or pager process */
#define SPCATCH	 0x00000004	/* don't do issig() in PCATCH */
#define SONPROC	 0x00000008	/* Process is on processor */
#define	STRC	 0x00000010	/* process is being traced */
#define	SWTED	 0x00000020	/* another tracing flag */
#define STRCEXIT 0x00000040	/* process has been ptrace killed */
#define	SPAGE	 0x00000080	/* process is handling a page fault */
/*#define SKEEP	 0x00000100	   flag to prevent swap during fork  UNUSED */
#define	SOMASK	 0x00000200	/* restore old mask after taking signal */
#define	SWEXIT	 0x00000400	/* working on exiting */
#define	SPHYSIO	 0x00000800	/* doing physical i/o (bio.c) */
#define	SVFORK	 0x00001000	/* process resulted from vfork() */
/*#define SVFDONE 0x00002000	   another vfork flag		     UNUSED */
/*#define SNOVM	 0x00004000	   no vm, parent in a vfork()        UNUSED */
#define	SPAGV	 0x00008000	/* init data space on demand, from vnode */
#define	SSEQL	 0x00010000	/* user warned of sequential vm behavior */
#define	SUANOM	 0x00020000	/* user warned of random vm behavior */
#define	STIMO	 0x00040000	/* timing out during sleep */
#define	SACTIVE	 0x00080000	/* process is executing */
#define	SOUSIG	 0x00100000	/* using old signal mechanism */
#define	SOWEUPC	 0x00200000	/* owe process an addupc() call at next ast */
#define	SSEL	 0x00400000	/* selecting; wakeup/waiting danger */
#define	SLOGIN	 0x00800000	/* a login process (legit child of init) */
#define SSTEP	 0x01000000	/* single stepping 1 instruction (BFLY2 only) */
#define	SXONLY	 0x02000000	/* process image read protected (CS_SECURITY) */
#define SIDLE	 0x04000000	/* is an idle process */
#define	SLKDONE	 0x08000000	/* record-locking has been done */
#define	SWAITEV	 0x10000000	/* child proc exited or stopped */
#define	SISSIG	 0x20000000	/* process is in issig */
/*#define SVMFAULT 0x40000000	   process is handling a page fault  UNUSED */
#define SLLNLSIG 0x80000000	/* handled usr12 signals for job control */

#ifdef _BBN_POSIX_SUPPORT
/* Values for p_posixflags */

#define	SISPOSIX	1	/* process is a POSIX application */
#define	SEXECDONE	2	/* exec has been done */
#define	SNOCLDSTOP	4	/* don't deliver SIGCHLD for stopped children */
#define	SORPHANPG	8	/* process group is orphanned */
#define	STTYREAD	16	/* process is asleep reading tty */

/* Process type macros */
#define	POSIX_PROC(p)	((p)->p_posixflags & SISPOSIX)
#define	MACH_PROC(p)	(((p)->p_posixflags & SISPOSIX) == 0)

#endif _BBN_POSIX_SUPPORT

#define LLNL_PROC(procp)	(Atomic_fetch32(&((procp)->p_flag)) & SLLNLSIG)

#endif	_PROC_
