/* $Header: /nfs/columbia/u1/BFMACH.local/mk.gibson/h/RCS/user.h,v 1.20 91/01/17 13:38:24 gibson Exp $ */

/* 
 **********************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 **********************************************************************
 */
 
#ifdef	KERNEL
#include "cs_bugfix.h"
#include "cs_generic.h"
#include "cs_rpause.h"
#include "cs_rfs.h"
#include "cputypes.h"
#else	KERNEL
#include <sys/features.h>
#endif	KERNEL
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)user.h	7.1 (Berkeley) 6/4/86
 *	@(#)user.h	2.1 88/05/18 4.0NFSSRC SMI;	from UCB 7.1 6/4/86
 */

#if	CS_GENERIC
#ifdef	KERNEL
#include "../h/types.h"
#else	KERNEL
#include <sys/types.h>
#endif	KERNEL

#endif	CS_GENERIC
#ifdef KERNEL
#include "../machine/pcb.h"
#include "dmap.h"
#include "time.h"
#include "resource.h"
#include "ucred.h"
#else
#include <machine/pcb.h>
#include <sys/dmap.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/ucred.h>
#endif

/*
 * Per process structure containing data that
 * isn't needed in core when the process is swapped out.
 */
 
#define	MAXCOMLEN	16		/* <= MAXNAMLEN, >= sizeof(ac_comm) */
 
struct	user {
	struct	pcb u_pcb;
	struct	proc *u_procp;		/* pointer to proc structure */
	int	*u_ar0;			/* address of users saved R0 */
	char	u_comm[MAXCOMLEN + 1];
	char	pad[4 - ((MAXCOMLEN+1) & 3)];	/* for 88K/68K alignment compat*/

/* syscall parameters, results and catches */
	int	u_arg[8];		/* arguments to current system call */
	int	*u_ap;			/* pointer to arglist */
	label_t	u_qsave;		/* for non-local gotos on interrupts */
	union {				/* syscall return values */
		struct	{
			int	R_val1;
			int	R_val2;
		} u_rv;
#define	r_val1	u_rv.R_val1
#define	r_val2	u_rv.R_val2
		off_t	r_off;
		time_t	r_time;
	} u_r;
	char	u_error;		/* return error code */
	char	u_eosys;		/* special action on end of syscall */

/* 1.1 - processes and protection */
#if defined(BFLY2) && defined(KERNEL_GENASSYM)
	short	pad2;			/* for xcompilation/genassym.c */
#endif
	struct ucred *u_cred;
#define	u_uid		u_cred->cr_uid
#define	u_gid		u_cred->cr_gid
#define	u_groups	u_cred->cr_groups
#define	u_ruid		u_cred->cr_ruid
#define	u_rgid		u_cred->cr_rgid
#define	u_auid		u_cred->cr_auid

/* 1.2 - memory management */
	size_t	u_tsize;		/* text size (clicks) */
	size_t	u_dsize;		/* data size (clicks) */
	size_t	u_ssize;		/* stack size (clicks) */
	struct	dmap u_dmap;		/* disk map for data segment */
	struct	dmap u_smap;		/* disk map for stack segment */
	struct	dmap u_cdmap, u_csmap;	/* shadows of u_dmap, u_smap, for
					   use of parent during fork */
	label_t u_ssave;		/* label variable for swapping */
	size_t	u_odsize, u_ossize;	/* for (clumsy) expansion swaps */
	time_t	u_outime;		/* user time at last sample */

/* 1.3 - signal management */
#if	CS_BUGFIX
	int	(*u_signal[NSIG+1])();	/* disposition of signals */
	int	u_sigmask[NSIG+1];	/* signals to be blocked */
#else	CS_BUGFIX
	int	(*u_signal[NSIG])();	/* disposition of signals */
	int	u_sigmask[NSIG];	/* signals to be blocked */
#endif	CS_BUGFIX
	int	u_sigonstack;		/* signals to take on sigstack */
	int	u_sigintr;		/* signals that interrupt syscalls */
	int	u_oldmask;		/* saved mask from before sigpause */
	int	u_code;			/* default code to trap */
#ifdef BFLY2
	int	u_fpecode;		/* on TC, can get multiple excepts */
	int	u_segvcode;		/* on 1 trap. (see sendsig()) */
#endif BFLY2
	int	u_sigsynch;		/* per-process synch signal handling */
	unsigned short u_sigsynch_default; /* process SEGV-handling semantics*/
	unsigned short u_sigsynch_state; /* current state of SEGV structures */
#ifdef BFLY1
	int	u_berr_frame[23];	/* one (largest) 68K exception frame */
#endif BFLY1
#ifdef BFLY2
	int	u_datapipe[9];		/* saved pending data accesses */
#endif BFLY2
	struct	sigstack u_sigstack;	/* sp & on stack state variable */
#define	u_onstack	u_sigstack.ss_onstack
#define	u_sigsp		u_sigstack.ss_sp


/* 1.4 - descriptor management */
	struct	file *u_ofile[NOFILE];	/* file structures for open files */
	char	u_pofile[NOFILE];	/* per-process flags of open files */
	int	u_lastfile;		/* high-water mark of u_ofile */
#define	UF_EXCLOSE 	0x1		/* auto-close on exec */
#define	UF_MAPPED 	0x2		/* mapped from device */
#define	UF_FDLOCK 	0x4		/* file desc locked (SysV style) */
	struct	vnode *u_cdir;		/* current directory */
	struct	vnode *u_rdir;		/* root directory of current process */
	struct	tty *u_ttyp;		/* controlling tty pointer */
	dev_t	u_ttyd;			/* controlling tty dev */
	short	u_cmask;		/* mask for file creation */

/* 1.5 - timing and statistics */
	struct	rusage2 u_ru;		/* stats for this proc */
	struct	rusage2 u_cru;		/* sum of stats for reaped children */
	struct	itimerval u_timer[3];
	int	u_XXX[3];
	struct	timeval u_start;
	short	u_acflag;

	struct uprof {			/* profile arguments */
		short	*pr_base;	/* buffer base */
		unsigned pr_size;	/* buffer size */
		unsigned pr_off;	/* pc offset */
		unsigned pr_scale;	/* pc scaling */
	} u_prof;
#if	CS_GENERIC
	short	u_aid;			/* account id */
#endif	CS_GENERIC
#if	CS_RPAUSE
	struct fs *u_rpsfs;		/* resource pause file system */
	char	u_rpswhich;		/* resource pause operation selection */
	u_char	u_rpause;		/* resource pause flags: */
#define	URPS_AGAIN	01		/* - no child processes available */
#define	URPS_NOMEM	02		/* - no memory available */
#define	URPS_NFILE	04		/* - file table overflow */
#define	URPS_NOSPC	010		/* - no space on device */
#endif	CS_RPAUSE
#if	CS_GENERIC
	char	u_modes;		/* process modes: */
#define	UMODE_P_GID	01		/* - 4.2 parent GID on inode create */
#define	UMODE_NOFOLLOW	02		/* - don't follow symbolic links */
#define	UMODE_ALLOWOLD	010		/* - TEMP: allow 4.1 feature use */
#define	UMODE_DIRTRACE	020		/* - TEMP: trace old directory reads */
#define	UMODE_NEWDIR	040		/* - TEMP: mkdir() new format */
#define	UMODE_SETGROUPS	0100		/* - TEMP: setgroups() done */
#define	UMODE_COMPATRACE 0200		/* - TEMP: trace 4.1/4.2 mode */
#endif	CS_GENERIC
#if	CS_RFS
	char	u_rfs;			/* remote syscall state bits: */
#define	URFS_CDIR	01		/* - current directory is remote */
#define	URFS_RDIR	02		/* - root directory is remote */
	short	u_rfscode;		/* remote file system call number */
	short	u_rfsncnt;		/* remote file system namei() call count */
#endif	CS_RFS

/* 1.6 - resource controls */
	struct	rlimit u_rlimit[RLIM_NLIMITS];

/* 1.7 - miscellaneous machine-specific control */

	int	u_systrace;		/* per-process tracing */
#ifdef	BFLY2
	int	u_fix_misaligned_flag;	/* per-process misalignment handling */
	int	u_signal_all_fp_faults;	/* per-process fp-fault handling */
	int     u_oem_tabadr;	/* OEM_FEATURE_1: process entry table addr */
#endif	BFLY2

	int	u_fslookups;	/* count of recent lookup operations */
				/* used to avoid inode/dnlc cache thrashing */
	int	u_pnode;		/* home node (physical) */
	int	u_stack[1];		/* limit of stack growth */
};

/*
 * Check whether process has been doing more file system lookup operations
 * than anything else recently. This is used to detect processes, such as 
 * 'find', which are likely to cause cache thrashing.
 */
#define TOOMANYLOOKUPS() (u.u_fslookups > 25)

/* u_eosys values */
#define	JUSTRETURN	1
#define	RESTARTSYS	2
#define NORMALRETURN	3

/* u_error codes */
#ifdef KERNEL
#include "errno.h"
#else
#include <errno.h>
#endif

#ifdef KERNEL
/* u_systrace values */
#define	SYSTRACE_SYSCALL	0x00000001
#define	SYSTRACE_SIGNAL		0x00000002
#endif KERNEL

#ifdef KERNEL
extern	struct user u;
extern	struct user u_user;	/* virtual copy of u, mapped into user space */
#endif KERNEL
