/* $Header: /nfs/columbia/u1/BFMACH.local/mk/h/RCS/vm.h,v 1.24 91/01/14 14:26:06 tappan Exp $ */

/*****************************************************************************
File:     vm.h
Contents: Public include file for new vm subsystem.
System:   BBNACI Butterfly OS kernel, virtual memory subsystem.
Created:  15-Mar-1989
Author:   drb and rwells

Remarks:  

[And now, a few words from the legal department.]

Copyright 1989 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

#ifndef _VM_H_
#define _VM_H_

#ifdef	KERNEL
#include "../machine/vmap.h"
#else KERNEL
#include <machine/vmap.h>
#include <sys/vmmac.h>
#include <sys/vmmeter.h>
#include <sys/vmsystm.h>
#endif	KERNEL

/* ----- Simple typedefs, for clarity ------------------------------------- */

typedef struct   pmap*            pmap_t; /* Encapsulated pmap handle */
#ifdef BFLY1
/* $$$Fix */
typedef struct pmmu_short_page_descriptor* pt_entry_ptr_t;
#endif BFLY1
#ifdef BFLY2
typedef struct   pt_entry*        pt_entry_ptr_t;
#endif BFLY2
typedef unsigned long             vm_addr_t;
typedef unsigned long             vm_address_t; /* Deprecated */
typedef struct vm_entry_data_t*   vm_entry_t; /* Public handle for entry */
typedef          int              vm_errkern_t; /* kern_return_t alias */
typedef          int              vm_errno_t;   /* <errno.h> codes */
typedef          int              vm_inherit_t;
typedef struct vm_map_data_t*     vm_map_t; /* Public handle for map */
typedef unsigned long             vm_offset_t;
typedef struct vm_page_data_t*    vm_page_t; /* Public handle for page frame */
typedef          int              vm_prot_t;
typedef struct vm_queue_data_t*   vm_queue_t;
typedef unsigned long             vm_size_t;
typedef          int              vm_task_t; /* MACH task identifier */
typedef struct vm_zone_data_t*    vm_zone_t;
typedef struct   vnode*           vnode_t; /* Encapsulated vnode handle */

/* ----- System-wide vm-related definitions ------------------------------- */

#define	VM_MIN_ADDRESS		((vm_offset_t) USER_MIN_VADDR)
#define VM_MIN_ANYWHERE_ADDRESS ((vm_offset_t) 0x10000000)
#define	VM_MAX_ADDRESS		((vm_offset_t) USER_MAX_VADDR)
#define VM_MIN_KERNEL_ADDRESS	((vm_offset_t) KERNEL_TEXT_VADDR)
#define VM_MAX_KERNEL_ADDRESS	((vm_offset_t) KERNEL_MAX_VADDR)
#define VM_PAGESIZE            	8192
#define VM_PAGEMASK            	(VM_PAGESIZE-1)
#define VM_PAGESHIFT           	13
#define VM_CHUNK_BASE   	((vm_offset_t) CHUNK_POOL_VADDR)

#define same_page(va1, va2) \
    (((u_long)(va1) & ~VM_PAGEMASK) == ((u_long)(va2) & ~VM_PAGEMASK))

#ifdef BFLY1
#define VM_SEGSIZE             0x1000000  /* segment size 16mb */
#define VM_SEGMASK             (VM_SEGSIZE - 1)
#define VM_NODESHIFT           24
#endif
#ifdef BFLY2
#define VM_SEGSIZE             0x800000  /* segment size 8mb */
#define VM_SEGMASK             (VM_SEGSIZE - 1)
#define VM_NODESHIFT           23
#endif


#define vm_isaligned(va)               (!(((vm_addr_t)(va))&(VM_PAGEMASK)))
#define vm_trunc_page(va)   ((vm_addr_t)(((vm_addr_t)(va))&~(VM_PAGEMASK)))
#define vm_round_page(va)   vm_trunc_page(((vm_addr_t)(va))+(VM_PAGEMASK))
#define	vm_page_offset(va)  (((vm_addr_t)(va)) & VM_PAGEMASK)

#define vm_atop(pa)         (((unsigned)(pa)) >> VM_PAGESHIFT)
#define vm_ptoa(pframe)     ((vm_addr_t)((pframe) << VM_PAGESHIFT))

/* ----- Protection definitions ------------------------------------------- */

#define	VM_PROT_NONE	     0x00
#define VM_PROT_READ	     0x01 /* read permission */
#define VM_PROT_WRITE	     0x02 /* write permission */
#define VM_PROT_EXECUTE	     0x04 /* execute permission */
#define VM_PROT_DEFAULT     (VM_PROT_READ|VM_PROT_WRITE|VM_PROT_EXECUTE)
#define VM_PROT_ALL         (VM_PROT_READ|VM_PROT_WRITE|VM_PROT_EXECUTE)
#define VM_PROT_MASK         VM_PROT_ALL

#ifdef BFLY2
/*****************************************************************************
* MMU CACHING INFORMATION:
*	1. User-settable caching attributes on a range of virtual addresses.
*	2. User-maintainable cache consistency via user callable cache flush.
******************************************************************************/

/*****************************************************************************
* Definitions for setting caching attributes ("setupflags" argument to
*	vm_cache_setup.
******************************************************************************/
typedef int	vm_cache_t;

#define	VM_CACHE_NONE		((vm_cache_t) 0x00)

#define	VM_CACHE_INHIBIT	((vm_cache_t) 0x01) /* even private */
#define	VM_CACHE_ENABLE		((vm_cache_t) 0x02) /* even shared  */
#define	VM_CACHE_WRITE_THRU	((vm_cache_t) 0x04) /* cache policy */
#define	VM_CACHE_COPY_BACK	((vm_cache_t) 0x08) /* cache policy */

#define	VM_CACHE_DEFAULT	VM_CACHE_NONE	/* let system decide */

#ifdef KERNEL
	/* mask for paramater checking */
#define	VM_CACHE_MASK		((vm_cache_t) 0x0F)

/*
 *	Indicator to pmap layer (pmap_protect) that this is a caching
 *	rather than protection-related change.  Used by vm_cache_setup().
 */
#define	VM_CACHE_KLUGE_FLAG	(1 << 20)
#endif KERNEL
#endif BFLY2

#ifdef KERNEL
#ifdef BFLY1
#define	vm_prot_package(prot,caching)	(prot)
#define	vm_prot_unpackage_prot(prot)	(prot)
#define	vm_prot_unpackage_caching(prot)	(VM_CACHE_NULL)
#endif BFLY1
#ifdef BFLY2
#define	vm_prot_package(prot,caching)	(((caching) << 16) | (prot))
#define	vm_prot_unpackage_prot(prot)	((prot) & VM_PROT_MASK)
#define	vm_prot_unpackage_caching(prot)	(((prot) >> 16) & VM_CACHE_MASK)
#define	vm_cache_package(caching)	(((caching) << 16)|VM_CACHE_KLUGE_FLAG)
#define	vm_cache_is_caching(caching)	((caching) & VM_CACHE_KLUGE_FLAG)
#endif BFLY2
#endif KERNEL

#ifdef BFLY2
/*****************************************************************************
* Definitions for flushing cache  ("flushflags" argument to vm_cache_flush).
******************************************************************************/

/* Control of cache flushing operations */
#define	VM_CACHE_LINE		0x0000
#define	VM_CACHE_PAGE		0x0100
#define	VM_CACHE_SEGMENT	0x0200
#define	VM_CACHE_ALL		0x0300
#define	VM_CACHE_GRANULARITY	0x0300	/* Mask */
#define	vm_cache_flush_granularity(flushflags)	\
  (((flushflags) & VM_CACHE_GRANULARITY) >> 8)
#define	VM_CACHE_OPERATION	0x3000	/* Mask */
#define	VM_CACHE_INVALIDATE	0x1000
#define	VM_CACHE_NOCOPYBACK	0x2000
#endif BFLY2

/* ----- Inherit definitions ---------------------------------------------- */

#define	VM_INHERIT_SHARE	0 /* share with child */
#define	VM_INHERIT_COPY		1 /* copy into child */
#define VM_INHERIT_NONE		2 /* absent from child */
#define	VM_INHERIT_DONATE_COPY	3 /* copy and delete */
#define VM_INHERIT_DEFAULT	VM_INHERIT_COPY


/* ----- vm_mapmem definitions -------------------------------------------- */

#define VM_BEST_NODE (-1)

#define VM_MAPMEM_ANYWHERE	0x0001 /* mapflags: va not specified */
#define VM_MAPMEM_READONLY      0x0002 /* mapflags: writing not allowed */
#define VM_MAPMEM_ALLOCATE	0x0004 /* mapflags: special allocate entry */
#define VM_MAPMEM_TRUNCATE	0x0008 /* mapflags: trunc on first/last map */
#define VM_MAPMEM_WIRE		0x0010 /* mapflags: wire pages down */
#define	VM_MAPMEM_INTERLEAVED   0x0020 /* mapflags: h/w interleaved pages */
#define VM_MAPMEM_SCATTERED     0x0040 /* mapflags: page interleaving */
#define VM_MAPMEM_NOBORROW      0x0080 /* mapflags: never borrow pages */
#define VM_MAPMEM_TEXT          0x0100 /* mapflags: text segment semantics */
#define VM_MAPMEM_ANON          0x0200 /* mapflags: "anonymous" semantics */
#define VM_MAPMEM_VALIDBITS     0x03ff /* mapflags: mask for valid bits */

/* ----- The following are for the kernel's use only ---------------------- */
#define VM_MAPMEM_KERNEL_ZONE   0x8000 /* mapflags: pages in a kernel zone */
#define VM_MAPMEM_KERNEL_KMEM   0x4000 /* mapflags: pages for kmem alloc */
#define VM_MAPMEM_KERNEL_ELOG   0x2000 /* mapflags: kernel elog buf pages */
#define VM_MAPMEM_KERNEL_PROC   0x1000 /* mapflags: kernel proc table pages */

/* ----- vm_transfer definitions ------------------------------------------ */

#define VM_PID_KERNEL		(-1)	/* operate on kernel's map */

/* ----- vm_sync definitions --------------------------------------------- */

#define VM_SYNC_WAIT            0x01 /* syncflags: wait for synchronous I/O */
#define VM_SYNC_INVALIDATE      0x02 /* syncflags: invalidate incore pages */
#define VM_SYNC_VALIDBITS       0x03 /* syncflags: mask for valid bits */

/* ----- kernel vm_page status definitions, mutually exclusive ------------ */

#ifdef KERNEL			/* if KERNEL is defined... */

#define VM_PAGE_FREE	     0	/* page is on free queue */
#define VM_PAGE_ZEROED	     1	/* page is on zeroed page queue	*/
#define VM_PAGE_SUSPENDED    2	/* page on suspended queue */
#define VM_PAGE_INACTIVE     3	/* page on inactive queue */
#define VM_PAGE_ACTIVE	     4	/* active queue */
#define VM_PAGE_IOERROR      5	/* we got error when tried to write page */
#define VM_PAGE_WIRED	     6	/* page is wired */
#define VM_PAGE_IDLECACHE    7 /* Inactive cached text pages */
#define VM_PAGE_KERNEL_ZONE  8	/* page in kernel zone chunk, not pagable */
#define VM_PAGE_KERNEL_PMAP  9	/* page holding PTEs in pmap, not pagable */
#define VM_PAGE_KERNEL_KMEM  10	/* vm_kmem_alloc_on_node, not pagable */
#define VM_PAGE_KERNEL_ELOG  11	/* kernel event logging page, not pagable */
#define VM_PAGE_KERNEL_PROC  12	/* kernel proc table page, not pagable */
#define VM_PAGE_KERNEL_PFREE 13	/* free list of pmap pages, can be reclaimed */
#define VM_PAGE_KERNEL_BUF   14	/* kernel buffer pool pages, not pagable */
#define VM_PAGE_KERNEL_KMALLOC 15 /* kernel memory allocator, not pagable */
#define VM_PAGE_INACTIVE_BUF 16	/* buffer pages which likely won't be needed */
#define VM_PAGE_NUMLISTS     17	/* Used to size array of lists */

#endif				/* end if KERNEL is defined. */

/* ----- page victimization scanner definitions -------------------------- */

#ifdef KERNEL

#define VM_SCAN_IDLE  0		/* scanner not running */
#define VM_SCAN_DELAY 1		/* scanner delaying between runs */
#define VM_SCAN_NOW   2		/* scanner running now */
#endif

/* ----- kernel VM queue data structure ----------------------------------- */

/* This structure is defined publicly so each pmap can contain a queue */
/* head used to string together the physical page frames used to */
/* hold the sets of PTE entries.  If pmap.h included vmi.h, it would */
/* be included by most of the kernel world, more or less.  So instead */
/* we make just enough public to allow a queue head to be defined. */

typedef struct vm_queue_data_t
{
    vm_queue_t prev;		/* Ptr to previous queue entry */
    vm_queue_t next;		/* Ptr to next queue entry */
} vm_queue_data_t;

#ifdef KERNEL			/* if KERNEL is defined... */

/* Definitions for useful macros that act like functions */

#define vm_queue_init(qp)      (((qp)->prev = (qp)),((qp)->next = (qp)))
#define vm_queue_isalone(qp)    ((qp)->next == (qp))

/* Magic helper macro to coerce pointer to queue object into */
/* pointer to enclosing object, given type of object and queue field */
/* in the object.  This macro should not be used directly, but is used */
/* to define friendlier macros for objects containing queue entries. */

#define vm_queue_coerce(qp,typ,fld) \
    ((typ) (((char *)(qp)) - ((int)&(((typ) 0)->fld))))

#if defined(__STDC__) || defined(c_plusplus)
void       vm_queue_putnext(vm_queue_t head_qp, vm_queue_t qp);
void       vm_queue_putprev(vm_queue_t head_qp, vm_queue_t qp);
vm_queue_t vm_queue_getnext(vm_queue_t head_qp);
void       vm_queue_remove(vm_queue_t qp);
int        vm_queue_panic(vm_queue_t qp);
#else
void       vm_queue_putnext(/*see above*/);
void       vm_queue_putprev(/*see above*/);
vm_queue_t vm_queue_getnext(/*see above*/);
void       vm_queue_remove(/*see above*/);
int        vm_queue_panic(/*see above*/);
#endif

#endif				/* end if KERNEL is defined */

/* ----- Kernel page allocation flags ------------------------------------ */
#ifdef KERNEL
#define VM_ALLOC_ZERO		0x0001	/* Allocate a pre-zeroed page */
#define VM_ALLOC_CANFAIL	0x0002 /* OK to return NULL */
#define VM_ALLOC_NOACTIVE	0x0004 /* Do not allocate from active list */
#define VM_ALLOC_EASY		0x0008 /* allocate only pages which are
					* clean or easy to clean */
#endif KERNEL

/* ----- Kernel interface VM functions ----------------------------------- */

#ifdef KERNEL			/* if KERNEL is defined... */

extern vm_map_t   kernel_map;	/* GLOBAL: on the master */
extern vm_map_t   my_kernel_map; /* VECTORIZED version of the above */
extern vm_size_t  mem_size;	/* GLOBAL: size of physical memory (bytes) */


#if defined(__STDC__) || defined(c_plusplus) /* if prototypes... */
void         vm_boot_chunks(vm_map_t kmap, vm_addr_t bootpages,
			    vm_addr_t chunkaddr, vm_size_t sizepernode);
void         vm_boot_kmem(vm_addr_t start, vm_addr_t end);
vm_offset_t  vm_boot_pages(vm_addr_t *start, vm_addr_t *end, vm_addr_t vaddr);
vm_addr_t    vm_chunk_alloc(vm_size_t size, int node);
vm_addr_t    vm_chunk_alloc_nowait(vm_size_t size, int node);
vm_addr_t    vm_kmem_alloc_on_node(vm_size_t size, int node);
void         vm_kmem_free(vm_addr_t start, vm_size_t size);
void         vm_list_print(int node);
vm_errno_t   vm_map_access(vm_map_t mp, vm_addr_t start, vm_size_t size,
			   vm_prot_t newprot);
vm_errno_t   vm_map_allocate(vm_map_t mp, vm_addr_t *vap, vm_size_t size,
			     int mapflags, int lognode);
vm_map_t     vm_map_create(int lognode, vm_addr_t limit_start,
			   vm_addr_t anywhere_start, vm_addr_t limit_end);
vm_errno_t   vm_map_deallocate(vm_map_t mp, vm_addr_t va, vm_size_t size);
void         vm_map_destroy(vm_map_t mp);
vm_errkern_t vm_map_errno(vm_errno_t ecode);
vm_errno_t   vm_map_fault(vm_map_t mp, vm_addr_t va, vm_prot_t type,
			  vm_entry_t wired_ep);
vm_errno_t   vm_map_file(vm_map_t mp, vm_addr_t *vap, vm_size_t size,
			 int mapflags, vnode_t vp, vm_offset_t voffset,
			 int sysnode);
vm_errno_t   vm_map_foreign(vm_map_t mp, vm_addr_t *vap, vm_size_t size,
			    int mapflags, vm_offset_t offset);
vm_map_t     vm_map_fork(vm_map_t mp, int node);
vm_errno_t   vm_map_inherit(vm_map_t mp, vm_addr_t start, vm_size_t size,
			    vm_inherit_t newinherit);
int          vm_map_iswired(vm_map_t mp, vm_addr_t start, vm_size_t size);
pmap_t       vm_map_pmap(vm_map_t mp);
void         vm_map_print(vm_map_t mp);
vm_errno_t   vm_map_protect(vm_map_t mp, vm_addr_t start, vm_size_t size,
			    int setmax, vm_prot_t newprot);
void         vm_map_set_cluster_id(vm_map_t mp, int cid);
vm_errno_t   vm_map_sync(vm_map_t mp,vm_addr_t start,vm_size_t size,int flags);
vm_addr_t    vm_map_u_area_copy(vm_map_t mp, vm_addr_t uaddr);
vm_addr_t    vm_map_u_area_create(vm_map_t mp, vm_addr_t uaddr);
vm_errno_t   vm_map_unwire(vm_map_t mp, vm_addr_t start, vm_size_t size);
void         vm_map_unwire_page(vm_page_t wired_pp);
vm_errno_t   vm_map_wire(vm_map_t mp, vm_addr_t start, vm_size_t size);
vm_page_t    vm_map_wire_page(vm_map_t mp, vm_addr_t va,
			      /*OUT*/vm_addr_t* physaddrp);
void         vm_mapping_cache_flush(vnode_t vp);
void         vm_objoff_pp_print(void);
void         vm_objoff_vnode_print(void);
vm_page_t    vm_page_alloc_on_node(int lognode, int flags);
void         vm_page_free(vm_page_t pp);
vm_addr_t    vm_page_physaddr(vm_page_t pp);
vm_addr_t    vm_pmap_page_alloc(vm_queue_t qphead, int lognode);
void         vm_pmap_page_free_all(vm_queue_t qphead);
vm_addr_t    vm_zone_alloc(vm_zone_t zp);
vm_addr_t    vm_zone_alloc_sleepflag(vm_zone_t zp, int waitflag);
void         vm_zone_expand(vm_zone_t zp);
void         vm_zone_free(vm_zone_t zp, vm_addr_t elem);
vm_zone_t    vm_zone_init(vm_size_t size, vm_size_t max, vm_size_t alloc,
			 int pageable, char *name, int logname);
void         vm_zone_preallocate(vm_zone_t zp, vm_addr_t memory,
				 vm_size_t size);
#else				/* else if not prototypes... */
void         vm_boot_chunks(/*see above*/);
void         vm_boot_kmem(/*see above*/);
vm_offset_t  vm_boot_pages(/*see above*/);
vm_addr_t    vm_chunk_alloc(/*see above*/);
vm_addr_t    vm_chunk_alloc_nowait(/*see above*/);
vm_addr_t    vm_kmem_alloc_on_node(/*see above*/);
void         vm_kmem_free(/*see above*/);
void         vm_list_print(/*see above*/);
vm_errno_t   vm_map_access(/*see above*/);
vm_errno_t   vm_map_allocate(/*see above*/);
vm_map_t     vm_map_create(/*see above*/);
void         vm_map_destroy(/*see above*/);
vm_errno_t   vm_map_deallocate(/*see above*/);
vm_errkern_t vm_map_errno(/*see above*/);
vm_errno_t   vm_map_fault(/*see above*/);
vm_errno_t   vm_map_foreign(/*see above*/);
vm_map_t     vm_map_fork(/*see above*/);
vm_errno_t   vm_map_inherit(/*see above*/);
int          vm_map_iswired(/*see above*/);
pmap_t       vm_map_pmap(/*see above*/);
void         vm_map_print(/*see above*/);
vm_errno_t   vm_map_protect(/*see above*/);
void         vm_map_set_cluster_id(/*see above*/);
vm_errno_t   vm_map_sync(/*see above*/);
vm_addr_t    vm_map_u_area_create(/*see above*/);
vm_addr_t    vm_map_u_area_copy(/*see above*/);
vm_errno_t   vm_map_unwire(/*see above*/);
void         vm_map_unwire_page(/*see above*/);
vm_errno_t   vm_map_wire(/*see above*/);
vm_page_t    vm_map_wire_page(/*see above*/);
void         vm_mapping_cache_flush(/*see above*/);
void         vm_objoff_pp_print(/*see above*/);
void         vm_objoff_vnode_print(/*see above*/);
vm_page_t    vm_page_alloc_on_node(/*see above*/);
void         vm_page_free(/*see above*/);
vm_addr_t    vm_page_physaddr(/*see above*/);
vm_addr_t    vm_pmap_page_alloc(/*see above*/);
void         vm_pmap_page_free_all(/*see above*/);
vm_addr_t    vm_zone_alloc(/*see above*/);
vm_addr_t    vm_zone_alloc_sleepflag(/*see above*/);
void         vm_zone_expand(/*see above*/);
void         vm_zone_free(/*see above*/);
vm_zone_t    vm_zone_init(/*see above*/);
void         vm_zone_preallocate(/*see above*/);
#endif				/* end if not prototypes */
#endif				/* end if KERNEL is defined */

/* ----- System call function prototypes --------------------------------- */

#ifndef KERNEL			/* if KERNEL is not defined */
#if defined(__STDC__) || defined(c_plusplus) /* if prototypes... */
				/* ANSI C and C++ function prototypes */
vm_errkern_t vm_mapmem(vm_task_t task, vm_addr_t *addressp,
		       vm_size_t size, int mapflags, int fd,
		       vm_offset_t off, int pnn);

vm_errkern_t vm_sync(vm_task_t task, vm_addr_t address,
		     vm_size_t size, int syncflags);

#else				/* else if not prototypes... */
vm_errkern_t vm_mapmem(/*see above*/);
vm_errkern_t vm_sync(/*see above*/);
#endif				/* end if not prototypes */
#endif				/* end if KERNEL is not defined */


/* ----- end of the file... ---------------------------------------------- */

#endif _VM_H_
