/* $Header: raw_cb.h,v 1.7 90/01/01 12:36:20 tappan Exp $ */
/*
 * Copyright (c) 1980, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)raw_cb.h	7.3 (Berkeley) 6/27/88
 */

/*
 * Raw protocol interface control block.  Used
 * to tie a socket to the generic raw interface.
 */
struct rawcb {
	struct	rawcb *rcb_next;	/* doubly linked list */
	struct	rawcb *rcb_prev;
	struct	socket *rcb_socket;	/* back pointer to socket */
	struct	sockaddr rcb_faddr;	/* destination address */
	struct	sockaddr rcb_laddr;	/* socket's address */
	struct	sockproto rcb_proto;	/* protocol family, protocol */
	caddr_t	rcb_pcb;		/* protocol specific stuff */
	struct	mbuf *rcb_options;	/* protocol specific options */
	struct	route rcb_route;	/* routing information */
	short	rcb_flags;
#ifdef butterfly
	/* locking */
	simple_lock_data_t	rcb_lock; /* lock on this connection */
	u_short		rcb_spl; /* spl before lock */
	u_short		rcb_deleting; /* set if block is being deleted */
#endif butterfly
};

/*
 * Since we can't interpret canonical addresses,
 * we mark an address present in the flags field.
 */
#define	RAW_LADDR	01
#define	RAW_FADDR	02
#define	RAW_DONTROUTE	04		/* no routing, default */

#ifdef KERNEL
#ifdef butterfly
/*
 * Pointer to lock/unlock functions. 
 * simple_lock_IFNP (IF Network Parallel)
 * simple_unlock_IFNP
 * These are set to point to simple_(un)lock if  the network code runs
 * in parallel otherwise they point to a nop.
 */
extern void (*simple_lock_IFNP)(), (*simple_unlock_IFNP)();

#define UNLOCKRAWCB(rcb) \
{ int s = (rcb)->rcb_spl;\
  (*simple_unlock_IFNP)(&rcb->rcb_lock); \
  splx(s);\
}

struct rawcb *sotorawcb();
struct rawcb *raw_disconnect();
struct rawcb *raw_detach();
#else
#define	sotorawcb(so)		((struct rawcb *)(so)->so_pcb)
#endif butterfly
#endif KERNEL


/*
 * Nominal space allocated to a raw socket.
 */
#define	RAWSNDQ		2048
#define	RAWRCVQ		2048

/*
 * Format of raw interface header prepended by
 * raw_input after call from protocol specific
 * input routine.
 */
struct raw_header {
	struct	sockproto raw_proto;	/* format of packet */
	struct	sockaddr raw_dst;	/* dst address for rawintr */
	struct	sockaddr raw_src;	/* src address for sbappendaddr */
};

#ifdef KERNEL
struct rawcb rawcb;			/* head of list */
#endif
