/*
 * $Header: /nfs/columbia/u1/BFMACH.local/mk/nfs/RCS/rnode.h,v 1.8 91/01/31 13:04:59 tappan Exp $
 *
 *      @(#)rnode.h	1.4 88/07/15 NFSSRC4.0 from 1.21 88/02/08 SMI
 */
/*	Copyright (C) 1988, Sun Microsystems Inc. */

#ifndef _nfs_rnode_h
#define _nfs_rnode_h

#ifdef	KERNEL
#include "../h/lock.h"
#else	KERNEL
#include <sys/features.h>
#include <sys/lock.h>
#endif	KERNEL

/*
 * Remote file information structure.
 * The rnode is the "inode" for remote files.  It contains all the
 * information necessary to handle remote file on the client side.
 */
struct rnode {
	struct rnode	*r_freef;	/* free list forward pointer */
	struct rnode	*r_freeb;	/* free list back pointer */
	struct rnode	*r_hash;	/* rnode hash chain */
	struct vnode	r_vnode;	/* vnode for remote file */
	fhandle_t	r_fh;		/* file handle */
	u_short		r_flags;	/* flags, see below */
	short		r_error;	/* async write error */
	daddr_t		r_lastr;	/* last block read (read-ahead) */
	short		r_owner;	/* proc index for locker of rnode */
	short		r_count;	/* number of rnode locks for r_owner */
	int		r_waiters;	/* # of procs waiting for RLOCK */
	struct ucred	*r_cred;	/* current credentials */
	struct ucred	*r_unlcred;	/* unlinked credentials */
	char		*r_unlname;	/* unlinked file name */
	struct vnode	*r_unldvp;	/* parent dir of unlinked file */
	struct vattr	r_attr;		/* cached vnode attributes */
	struct timeval	r_attrtime;	/* time attributes become invalid */
	simple_lock_data_t r_lock;	/* multiprocessor lock */
	int		r_async; /* outstanding async writes */
};

#define r_size	r_attr.va_size		/* file size in bytes */

/*
 * Flags
 */
#define	RLOCKED		0x01		/* rnode is in use */
#define	RATTRVALID	0x02		/* Attributes in the rnode are valid */
#define	REOF		0x04		/* EOF encountered on read */
#define	RDIRTY		0x08		/* dirty pages from write operation */
#define	RFREE		0x10		/* free rnode at last reference */
#define RASYNCWAIT	0x20		/* waiting for async writes */

/*
 * Convert between vnode and rnode
 */
#define	rtov(rp)	(&(rp)->r_vnode)
#define	vtor(vp)	((struct rnode *)((vp)->v_data))
#define	vtofh(vp)	(&(vtor(vp)->r_fh))
#define	rtofh(rp)	(&(rp)->r_fh)

/*
 * Lock and unlock rnodes.
 */
/*
 * The RLOCK/RUNLOCK macros are now routines in nfs_subr.c.
 */
#define SLRP(rp) (simple_lock(&((rp)->r_lock)))
#define SURP(rp) (simple_unlock(&((rp)->r_lock)))

/*
 * Used by nfs_lookup/makenfsnode for deadlock avoidance.
 */
#define	DONT_WAIT_FOR_RLOCK	0
#define	WAIT_FOR_RLOCK		1

#endif !_nfs_rnode_h
