/* $Header: /nfs/europa/u1/BFMACH.local/mk/sys/RCS/init_sysent.c,v 1.46 91/02/01 16:34:30 lkaplan Exp $	*/
/* 
 **********************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 **********************************************************************
 */

#include "../h/posix.h"
#include "mach_time.h"
#include "cs_syscall.h"
#include "nfsclient.h"
#include "nfsserver.h"
#include "sun_quota.h"
#include "ufs.h"
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)init_sysent.c	7.1 (Berkeley) 6/5/86
 *	@(#)init_sysent.c	2.3 88/06/13 4.0NFSSRC SMI;from UCB 7.1 6/5/86
 */

/*
 * System call switch table.
 */

#include "param.h"
#include "systm.h"

	/* serial or parallel system call */
#define syss(fn,no) {no, 0, fn}
#define sysp(fn,no) {no, 1, fn}

int	nosys();
int	nullsys();
int	errsys();

/* 1.1 processes and protection */
int	sethostid(),gethostid(),sethostname(),gethostname(),getpid();
int	setdomainname(),getdomainname();
int	fork(),rexit(),execv(),execve(),wait();
int	getuid(),setreuid(),getgid(),getgroups(),setregid(),setgroups();
int	getpgrp(),setpgrp(), sys_bbn ();

/* 1.2 memory management */
int	sbrk(),sstk();
int	getpagesize(),smmap(),mremap(),munmap(),mprotect(),madvise(),mincore();

/* 1.3 signals */
int	sigvec(),sigblock(),sigsetmask(),sigpause(),sigstack(),sigreturn();
int	kill(), killpg();
int	longjmpsigreturn();

/* 1.4 timing and statistics */
int	gettimeofday(),settimeofday();
int	getitimer(),setitimer();
int 	adjtime();

/* 1.5 descriptors */
int	getdtablesize(),dup(),dup2(),close();
int	select(),getdopt(),setdopt(),fcntl(),flock();

/* 1.6 resource controls */
int	getpriority(),setpriority(),getrusage(),getrlimit(),setrlimit();
int	user_yield(),user_nopreempt(), user_sleep_and_unlock(), user_wakeup();
int	getrusage2();
#if	SUN_QUOTA
int	oldquota(),quotactl();
#else
#define oldquota nullsys	/* for backward compatability with old login */
#endif	SUN_QUOTA

/* 1.7 system operation support */
int	umount(),unmount(),smount(),vnode_pager_ctl();
int	sync(),reboot(),sysacct();

/* 2.1 generic operations */
int	read(),write(),readv(),writev(),ioctl();

/* 2.2 file system */
int	chdir(),chroot(),wait4();
int	mkdir(),rmdir(),getdirentries(),getdents();
int	creat(),open(),mknod(),unlink(),stat(),fstat(),lstat();
int	chown(),fchown(),chmod(),fchmod(),utimes();
int	link(),symlink(),readlink(),rename();
int	lseek(),truncate(),ftruncate(),access(),fsync();
int	ustat(),statfs(),fstatfs();

/* Backward compatability for old format dev_t */
int     old_fstat();
int     old_stat();
int     old_lstat();

int     mkionode();		/* create a device special file */
/* 2.3 communications */
int	socket(),bind(),listen(),accept(),connect();
int	socketpair(),sendto(),send(),recvfrom(),recv();
int	sendmsg(),recvmsg(),shutdown(),setsockopt(),getsockopt();
int	getsockname(),getpeername(),pipe();

int	umask();		/* XXX */

/* 2.4 processes */
int	ptrace();

/* 2.5 terminals */

#if	MACH_TIME
int	getutime();
#endif	MACH_TIME

#ifdef	SUN_UNIX
/* emulation for SUN-specific system calls */
int	getdirentries(), getdomainname(), setdomainname();
#endif	SUN_UNIX

#ifdef _BBN_POSIX_SUPPORT
int	setposix(), sigpending(), sigprocmask(), waitpid(), setsid();
#endif _BBN_POSIX_SUPPORT

#ifdef COMPAT
/* emulations for backwards compatibility */
#define	compat(name,n)	{n, 0, name}
#define	compatp(name,n)	{n, 1, name}

int	owait();		/* now receive message on channel */
int	otime();		/* now use gettimeofday */
int	ostime();		/* now use settimeofday */
int	oalarm();		/* now use setitimer */
int	outime();		/* now use utimes */
int	opause();		/* now use sigpause */
int	onice();		/* now use setpriority,getpriority */
int	oftime();		/* now use gettimeofday */
int	osetpgrp();		/* ??? */
int	otimes();		/* now use getrusage */
int	ossig();		/* now use sigvec, etc */
int	ovlimit();		/* now use setrlimit,getrlimit */
int	ovtimes();		/* now use getrusage */
int	osetuid();		/* now use setreuid */
int	osetgid();		/* now use setregid */
int	ostat();		/* now use stat */
int	ofstat();		/* now use fstat */
#else
#define	compat(n, name)		{0, 1, nosys}
#define	compatp(n, name)	{0, 1, nosys}
#endif

int multread(); 
int multwrite();


/* BEGIN JUNK */
#ifdef vax
int	resuba();
#ifdef TRACE
int	vtrace();
#endif
#endif
int	profil();		/* 'cuz sys calls are interruptible */
int	vhangup();		/* should just do in exit() */
int	vfork();		/* awaiting fork w/ copy on write */
int	obreak();		/* awaiting new sbrk */
int	ovadvise();		/* awaiting new madvise */
/* END JUNK */

#if	NFSCLIENT
/* nfs */
int	async_daemon();		/* client async daemon */
#endif
#if	NFSSERVER
int	nfs_svc();		/* run nfs server */
int	nfs_getfh();		/* get file handle */
int	exportfs();		/* export file systems */
#endif

#if	CS_SYSCALL
extern int setmodes();
extern int getmodes();
extern int setaid();
extern int getaid();
extern int table();
extern int nulldev();			/* to test 4.1/4.2 kernel */

struct sysent cmusysent[] =
{
	sysp(getmodes, 0),		/* -9 = get process modes */
	sysp(setmodes, 1),		/* -8 = set process modes */
	sysp(nosys, 0),			/* -7 = old CMU IPC */
	syss(table, 5),			/* -6 = table lookup */
	sysp(nosys, 0),			/* -5 = old xstat */
	sysp(nosys, 0),			/* -4 = old xfstat */
	sysp(nulldev, 0),		/* -3 = old chacct */
	sysp(getaid, 0),		/* -2 = get account ID */
	sysp(setaid, 1),		/* -1 = set account ID */
}; 
/* 
 *  The preceding table is effectively a negative extension of the following
 *  table.  No other definitions should intervene here.  The system call
 *  handler assumes that these two tables are contiguous.
 */
#endif	CS_SYSCALL
struct sysent sysent[] = {
	sysp(nosys,0),			/*   0 = indir */
	sysp(rexit,1),			/*   1 = exit */
	sysp(fork,0),			/*   2 = fork */
	sysp(read,3),			/*   3 = read */
	sysp(write,3),			/*   4 = write */
	sysp(open,3),			/*   5 = open */
	sysp(close,1),			/*   6 = close */
	sysp(wait4,4),			/*   7 = wait4 */
	sysp(creat,2),			/*   8 = creat */
	sysp(link,2),			/*   9 = link */
	sysp(unlink,1),			/*  10 = unlink */
	sysp(execv,2),			/*  11 = execv */
	sysp(chdir,1),			/*  12 = chdir */
		compat(otime,0),	/*  13 = old time */
	sysp(mknod,3),			/*  14 = mknod */
	sysp(chmod,2),			/*  15 = chmod */
	sysp(chown,3),			/*  16 = chown; now 3 args */
	sysp(obreak,1),			/*  17 = old break */
		compatp(ostat,2),	/*  18 = old stat */
	sysp(lseek,3),			/*  19 = lseek */
	sysp(getpid,0),			/*  20 = getpid */
	sysp(nosys,0),			/*  21 = old mount */
#if	UFS
	sysp(umount,1),			/*  22 = old umount */
#else
	sysp(nosys,0),			/*  22 = old umount */
#endif
		compatp(osetuid,1),	/*  23 = old setuid */
	sysp(getuid,0),			/*  24 = getuid */
		compat(ostime,1),	/*  25 = old stime */
	syss(ptrace,5),			/*  26 = ptrace */
		compat(oalarm,1),	/*  27 = old alarm */
		compatp(ofstat,2),	/*  28 = old fstat */
		compat(opause,0),	/*  29 = opause */
		compat(outime,2),	/*  30 = old utime */
	sysp(nosys,0),			/*  31 = was stty */
	sysp(nosys,0),			/*  32 = was gtty */
	sysp(access,2),			/*  33 = access */
		compat(onice,1),	/*  34 = old nice */
		compat(oftime,1),	/*  35 = old ftime */
	sysp(sync,0),			/*  36 = sync */
	sysp(kill,2),			/*  37 = kill */
	sysp(old_stat,2),		/*  38 = old stat */
		compatp(osetpgrp,2),	/*  39 = old setpgrp */
	sysp(old_lstat,2),		/*  40 = old lstat */
	sysp(dup,2),			/*  41 = dup */
	sysp(pipe,0),			/*  42 = pipe */
		compat(otimes,1),	/*  43 = old times */
	sysp(profil,4),			/*  44 = profil */
	sysp(nosys,0),			/*  45 = nosys */
		compatp(osetgid,1),	/*  46 = old setgid */
	sysp(getgid,0),			/*  47 = getgid */
		compat(ossig,2),	/*  48 = old sig */
	sysp(nosys,0),			/*  49 = reserved for USG */
	sysp(nosys,0),			/*  50 = reserved for USG */
	syss(sysacct,1),		/*  51 = turn acct off/on */
	sysp(nosys,0),			/*  52 = old set phys addr */
	sysp(nosys,0),			/*  53 = old lock in core */
	sysp(ioctl,3),			/*  54 = ioctl */
	sysp(reboot,1),			/*  55 = reboot */
	sysp(nosys,0),			/*  56 = old mpxchan */
	sysp(symlink,2),		/*  57 = symlink */
	sysp(readlink,3),		/*  58 = readlink */
	sysp(execve,3),			/*  59 = execve */
	sysp(umask,1),			/*  60 = umask */
	sysp(chroot,1),			/*  61 = chroot */
	sysp(old_fstat,2),		/*  62 = old fstat */
	sysp(nosys,0),			/*  63 = used internally */
	sysp(getpagesize,1),		/*  64 = getpagesize */
	syss(mremap,5),			/*  65 = mremap */
	sysp(vfork,0),			/*  66 = vfork */
	sysp(read,3),			/*  67 = old vread */
	sysp(write,3),			/*  68 = old vwrite */
	sysp(sbrk,1),			/*  69 = sbrk */
	sysp(sstk,1),			/*  70 = sstk */
	syss(smmap,6),			/*  71 = mmap */
	syss(ovadvise,1),		/*  72 = old vadvise */
	syss(munmap,2),			/*  73 = munmap */
	syss(mprotect,3),		/*  74 = mprotect */
	syss(madvise,3),		/*  75 = madvise */
	sysp(vhangup,1),		/*  76 = vhangup */
		compat(ovlimit,2),	/*  77 = old vlimit */
	sysp(mincore,3),		/*  78 = mincore */
	sysp(getgroups,2),		/*  79 = getgroups */
	sysp(setgroups,2),		/*  80 = setgroups */
	sysp(getpgrp,1),		/*  81 = getpgrp */
	sysp(setpgrp,2),		/*  82 = setpgrp */
	sysp(setitimer,3),		/*  83 = setitimer */
	sysp(wait,0),			/*  84 = wait */
	sysp(nosys,0),			/*  85 = old swapon */
	sysp(getitimer,2),		/*  86 = getitimer */
	sysp(gethostname,2),		/*  87 = gethostname */
	sysp(sethostname,2),		/*  88 = sethostname */
	sysp(getdtablesize,0),		/*  89 = getdtablesize */
	sysp(dup2,2),			/*  90 = dup2 */
	sysp(getdopt,2),		/*  91 = getdopt */
	sysp(fcntl,3),			/*  92 = fcntl */
	sysp(select,5),			/*  93 = select */
	sysp(setdopt,2),		/*  94 = setdopt */
	sysp(fsync,1),			/*  95 = fsync */
	sysp(setpriority,3),		/*  96 = setpriority */
	sysp(socket,3),			/*  97 = socket */
	sysp(connect,3),		/*  98 = connect */
	sysp(accept,3),			/*  99 = accept */
	sysp(getpriority,2),		/* 100 = getpriority */
	sysp(send,4),			/* 101 = send */
	sysp(recv,4),			/* 102 = recv */
#ifdef BFLY1
	sysp(longjmpsigreturn,1),	/* 103 = "sigreturn" for longjmp */
#endif BFLY1
#ifdef BFLY2
	sysp(longjmpsigreturn,2),	/* 103 = "sigreturn" for longjmp (NB: second arg) */
#endif BFLY2
	sysp(bind,3),			/* 104 = bind */
	sysp(setsockopt,5),		/* 105 = setsockopt */
	sysp(listen,2),			/* 106 = listen */
		compat(ovtimes,2),	/* 107 = old vtimes */
	sysp(sigvec,3),			/* 108 = sigvec */
	sysp(sigblock,1),		/* 109 = sigblock */
	sysp(sigsetmask,1),		/* 110 = sigsetmask */
	sysp(sigpause,1),		/* 111 = sigpause */
	sysp(sigstack,2),		/* 112 = sigstack */
	sysp(recvmsg,3),		/* 113 = recvmsg */
	sysp(sendmsg,3),		/* 114 = sendmsg */
#ifdef	TRACE
	sysp(vtrace,2),			/* 115 = vtrace */
#else	TRACE
	sysp(nosys,0),			/* 115 = nosys */
#endif	TRACE
	sysp(gettimeofday,2),		/* 116 = gettimeofday */
	sysp(getrusage,2),		/* 117 = getrusage */
	sysp(getsockopt,5),		/* 118 = getsockopt */
#ifdef	butterfly
	sysp(getrusage2,3),		/* 119 = getrusage2 */
#else	butterfly
	sysp(nosys,0),			/* 119 = nosys */
#endif	butterfly
	sysp(readv,3),			/* 120 = readv */
	sysp(writev,3),			/* 121 = writev */
	sysp(settimeofday,2),		/* 122 = settimeofday */
	sysp(fchown,3),			/* 123 = fchown */
	sysp(fchmod,2),			/* 124 = fchmod */
	sysp(recvfrom,6),		/* 125 = recvfrom */
	sysp(setreuid,2),		/* 126 = setreuid */
	sysp(setregid,2),		/* 127 = setregid */
	sysp(rename,2),			/* 128 = rename */
	sysp(truncate,2),		/* 129 = truncate */
	sysp(ftruncate,2),		/* 130 = ftruncate */
	sysp(flock,2),			/* 131 = flock */
	sysp(nosys,0),			/* 132 = nosys */
	sysp(sendto,6),			/* 133 = sendto */
	sysp(shutdown,2),		/* 134 = shutdown */
	sysp(socketpair,5),		/* 135 = socketpair */
	sysp(mkdir,2),			/* 136 = mkdir */
	sysp(rmdir,1),			/* 137 = rmdir */
	sysp(utimes,2),			/* 138 = utimes */
	sysp(sigreturn,1),		/* 139 = sigreturn */
	syss(adjtime,2),		/* 140 = adjtime */
	sysp(getpeername,3),		/* 141 = getpeername */
	sysp(gethostid,0),		/* 142 = gethostid */
	sysp(sethostid,1),		/* 143 = sethostid */
	sysp(getrlimit,2),		/* 144 = getrlimit */
	sysp(setrlimit,2),		/* 145 = setrlimit */
	sysp(killpg,2),			/* 146 = killpg */
	sysp(nosys,0),			/* 147 = nosys */
	sysp(oldquota,0),		/* 148 = old quota */
	sysp(oldquota,0),		/* 149 = old qquota */
	sysp(getsockname,3),		/* 150 = getsockname */
	sysp(sys_bbn,4)			/* 151 = sys_bbn */
	sysp(nosys,0),			/* 152 = reserved */
	sysp(nosys,0),			/* 153 = reserved */
#if	NFSSERVER
	sysp(nosys,0),			/* 154 = old nfs_mount */
	sysp(nfs_svc,1),		/* 155 = nfs_svc */
#else
	sysp(nosys,0),			/* 154 = old nfs_mount */
	sysp(errsys,0),			/* 155 = errsys */
#endif
	sysp(getdirentries,4),		/* 156 = getdirentries */
	sysp(statfs,2),			/* 157 = statfs */
	sysp(fstatfs,2),		/* 158 = fstatfs */
	sysp(unmount,1),		/* 159 = unmount */
#if	NFSCLIENT
	sysp(async_daemon,0),		/* 160 = async_daemon */
#else
	sysp(errsys,0),			/* 160 = errsys */
#endif
#if	NFSSERVER
	sysp(nfs_getfh,2),		/* 161 = get file handle */
#else
	sysp(nosys,0),			/* 161 = nosys */
#endif
	sysp(getdomainname,2),		/* 162 = getdomainname */
	sysp(setdomainname,2),		/* 163 = setdomainname */
	sysp(nosys,0),			/* 164 = old pcfs_mount */
#if	SUN_QUOTA
	sysp(quotactl,4),		/* 165 = quotactl */
#else
	sysp(errsys,0),			/* 165 = not configured */
#endif
#if	NFSSERVER
	sysp(exportfs,2),		/* 166 = exportfs */
#else
	sysp(errsys,0),			/* 166 = not configured */
#endif
	sysp(smount,4),			/* 167 = mount */
	sysp(ustat,2),			/* 168 = ustat */
	sysp(nosys,0),			/* 169 = reserved */
	sysp(nosys,0),			/* 170 = reserved */
	sysp(nosys,0),			/* 171 = reserved */
	sysp(nosys,0),			/* 172 = reserved */
	sysp(nosys,0),			/* 173 = reserved */
	sysp(getdents,3),		/* 174 = getdents */
	sysp(nosys,0),			/* 175 = reserved */
	sysp(nosys,0),			/* 176 = reserved */
	sysp(nosys,0),			/* 177 = reserved */
	sysp(nosys,0),			/* 178 = reserved */
	sysp(multread,3),		/* 179 = multread */
 	sysp(multwrite,3),		/* 180 = multwrite*/
	sysp(user_yield,1),		/* 181 = user_yield*/
	sysp(user_nopreempt,2),		/* 182 = user_nopreempt*/
	sysp(user_sleep_and_unlock,3),	/* 183 user_sleep_and_unlock */
	sysp(user_wakeup,1),		/* 184 user_wakeup */
	sysp(nosys,0),			/* 185 = reserved */
	sysp(vnode_pager_ctl,3),	/* 186 = vnode_pager_ctl */
	sysp(stat, 2),			/* 187 - stat */
	sysp(lstat, 2),			/* 188 - lstat */
	sysp(fstat, 2),			/* 189 - fstat */
	sysp(mkionode, 6)		/* 190 - mkionode */
#ifdef _BBN_POSIX_SUPPORT
	sysp(setposix,0),		/* 191 = setposix */
	sysp(sigpending,1),		/* 192 = sigpending */
	sysp(sigprocmask,3),		/* 193 = sigprocmask */
	sysp(waitpid,3),		/* 194 = waitpid */
	sysp(setsid,0),			/* 195 = setsid */
#endif _BBN_POSIX_SUPPORT
};
int	nsysent = sizeof (sysent) / sizeof (sysent[0]);
#if	CS_SYSCALL
int	ncmusysent = sizeof (cmusysent) / sizeof (cmusysent[0]);
int	nallsysent = (sizeof (cmusysent)+sizeof(sysent)) / sizeof (sysent[0]);
#endif	CS_SYSCALL

double ccpu = 0.951;		/* exp(-1/20) for schedcpu */
