/* $Header: /nfs/columbia/u1/BFMACH.local/mk/ufs/RCS/inode.h,v 1.27 91/01/31 13:03:51 tappan Exp $	*/
/* 
 **********************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 **********************************************************************
 */
 
#ifdef	KERNEL
#include "cs_rfs.h"
#else	KERNEL
#include <sys/features.h>
#endif	KERNEL
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)inode.h	7.1 (Berkeley) 6/4/86
 *	@(#)inode.h	2.2 88/06/20 4.0NFSSRC SMI;	from UCB 7.1 6/4/86
 */

/*
 * The I node is the focus of all local file activity in UNIX.
 * There is a unique inode allocated for each active file,
 * each current directory, each mounted-on file, text file, and the root.
 * An inode is 'named' by its dev/inumber pair. (iget/iget.c)
 * Data in icommon is read in from permanent inode on volume.
 */

#if	CS_RFS
/*
 *  The following macro is used to identify the special CMU file type which is
 *  currently used to implement both old style symbolic links and remote node
 *  pointer files.
 */
#define	isesctype(ip)	\
	((ip)->i_gid == 64)
 
#endif	CS_RFS
#if	CS_RFS
/*
 *  The following macro is used to distinguish the special CMU remote node pointer.
 */
#define	isrfslnk(ip)	\
	(isesctype(ip) && (((ip)->i_mode&(IFMT|IEXEC|(IEXEC>>3)|(IEXEC>>6))) == (IFREG|IEXEC)))
 
#endif	CS_RFS
#define	NDADDR	12		/* direct addresses in inode */
#define	NIADDR	3		/* indirect addresses in inode */

/*
 * Inode locking:
 * An inode is protected by a reader writer lock. 
 *
 * The writer lock should be held for accesses which change the inode
 * data, or most inode fields. 
 *
 * The reader lock can be held for accesses which only read data and
 * inode fields, with the exception that i_lastr and i_diroff can
 * be written while holding the reader lock (these fields are 
 * only optimizations, an inconsistant value will still produce 
 * correct results accessing the file).
 *
 * The 'latch' field of the rwlock can be used as a simple lock,
 * which is used to protect the i_flag and i_ref fields. XXX: at some point
 * this should change to use atomic operations on the flags field.
 *
 * 'i_list' can be used to make a list of interesting inodes. It should
 * only be used while protected by the ufs_sync_lock. Note that because
 * inodes are garbage collected by ino_update, which operates under
 * that lock, there is no need to worry about inode storage being freed
 * while on the list.
 */

struct inode {
	struct	inode *i_chain[2];	/* must be first */
	int	i_magic;
#define I_MAGIC 0xbdf111aa
	struct	vnode i_vnode;	/* vnode associated with this inode */
	struct	vnode *i_devvp;	/* vnode for block I/O */
	rwlock_data_t	i_rwlock;
#define i_lock i_rwlock.latch
	u_short	i_flag;
	u_short	i_spl;		/* SLIP_SPL/SUIP_SPL saved spl */
	dev_t	i_dev;		/* device where inode resides */
	ino_t	i_number;	/* i number, 1-to-1 with device address */
	off_t	i_diroff;	/* offset in dir, where we found last entry */
	struct	fs *i_fs;	/* file sys associated with this inode */
	struct	dquot *i_dquot;	/* quota structure controlling this file */
	struct inode *i_list;
	union {
		daddr_t	if_lastr;	/* last read (read-ahead) */
		struct	socket *is_socket;
	} i_un;
	struct 	icommon
	{
		u_short	ic_mode;	/*  0: mode and type of file */
		short	ic_nlink;	/*  2: number of links to file */
		uid_t	ic_uid;		/*  4: owner's user id */
		gid_t	ic_gid;		/*  6: owner's group id */
		quad	ic_size;	/*  8: number of bytes in file */
		time_t	ic_atime;	/* 16: time last accessed */
		long	ic_atspare;
		time_t	ic_mtime;	/* 24: time last modified */
		long	ic_mtspare;
		time_t	ic_ctime;	/* 32: last time inode changed */
		long	ic_ctspare;
		daddr_t	ic_db[NDADDR];	/* 40: disk block addresses */
		daddr_t	ic_ib[NIADDR];	/* 88: indirect blocks */
		long	ic_flags;	/* 100: status, currently unused */
		long	ic_blocks;	/* 104: blocks actually held */
		long	ic_gen;		/* 108: generation number */
		long	ic_spare[4];	/* 112: reserved, current unused */
	} i_ic;
	time_t	i_gctime;	/* time to garbage collect this inode */
	u_short	i_wasfree;	/* set if a free inode is read from disk */
};

struct dinode {
	union {
		struct	icommon di_icom;
		char	di_size[128];
	} di_un;
};

#define	i_mode		i_ic.ic_mode
#define	i_nlink		i_ic.ic_nlink
#define	i_uid		i_ic.ic_uid
#define	i_gid		i_ic.ic_gid
/* ugh! -- must be fixed */
#ifdef vax
#define	i_size		i_ic.ic_size.val[0]
#endif vax
#define i_size		i_ic.ic_size.val[1]
#define	i_db		i_ic.ic_db
#define	i_ib		i_ic.ic_ib
#define	i_atime		i_ic.ic_atime
#define	i_mtime		i_ic.ic_mtime
#define	i_ctime		i_ic.ic_ctime
#define i_blocks	i_ic.ic_blocks
#define	i_rdev		i_ic.ic_db[0]
#define	i_gen		i_ic.ic_gen
#define	i_lastr		i_un.if_lastr
#define	i_socket	i_un.is_socket
#define	i_forw		i_chain[0]
#define	i_back		i_chain[1]
#define	i_freef		i_fr.if_freef
#define	i_freeb		i_fr.if_freeb

#define di_ic		di_un.di_icom
#define	di_mode		di_ic.ic_mode
#define	di_nlink	di_ic.ic_nlink
#define	di_uid		di_ic.ic_uid
#define	di_gid		di_ic.ic_gid
#ifdef vax
#define	di_size		di_ic.ic_size.val[0]
#endif vax
#define di_size		di_ic.ic_size.val[1]
#define	di_db		di_ic.ic_db
#define	di_ib		di_ic.ic_ib
#define	di_atime	di_ic.ic_atime
#define	di_mtime	di_ic.ic_mtime
#define	di_ctime	di_ic.ic_ctime
/* New format exploded dev_t */
/* #define di_rdev	di_ic.ic_db[0] */
#define I_RDEVNEW       0x80000000 /* New format dev_t storage */
#define di_rdevflags    di_ic.ic_db[0]
#define di_rdevmajor    di_ic.ic_db[1]
#define di_rdevminor    di_ic.ic_db[2]
#define di_rdeviobus    di_ic.ic_db[3]
#ifdef KERNEL
extern dev_t convert_old_dev_t();
/* Kludge on bits 17-30 because sometimes devices got created sign extended */
#define CONS_DEV_T(dp) (((dp)->di_rdevflags & I_RDEVNEW && \
			 ((dp)->di_rdevflags & 0x7fff000) == 0) ? \
			  makedev((dp)->di_rdevmajor, (dp)->di_rdevminor, \
				  (dp)->di_rdeviobus) : \
				    convert_old_dev_t((dp)->di_rdevflags))
#endif KERNEL
#define	di_blocks	di_ic.ic_blocks
#define	di_gen		di_ic.ic_gen

/*
 * inode hash table structures
 */
#define	INOHSZ	512
#if	((INOHSZ&(INOHSZ-1)) == 0)
#define	INOHASH(dev,ino)	(((dev)+(ino))&(INOHSZ-1))
#else
#define	INOHASH(dev,ino)	(((unsigned)((dev)+(ino)))%INOHSZ)
#endif

extern struct ihead {				/* inode LRU cache, Chris Maltby */
	union {
		struct ihead *ihun_head[2];
		struct inode *ihun_chain[2];
	} ih_un;
#define	ih_head		ih_un.ihun_head
#define	ih_chain	ih_un.ihun_chain
	simple_lock_data_t ih_lock;
	int ih_state;
} ihead[INOHSZ];



#ifdef KERNEL

extern struct vnodeops ufs_vnodeops;	/* vnode operations for ufs */

struct inode *iterate_inode_init(), *iterate_inode();

struct	inode *ialloc();
struct	inode *iget();		/* returns with reader lock held for inode */
#if	CS_RFS
struct	inode *cnamei();
#endif	CS_RFS
ino_t	dirpref();
#endif

/* flags */
#define	IUPD		0x002		/* file has been modified */
#define	IACC		0x004		/* inode access time to be updated */
#define IMOD		0x008		/* inode has been modified */
#define	ICHG		0x040		/* inode has been changed */
#define	ILWAIT		0x080		/* someone waiting on file lock */
#define IREF		0x100		/* inode has been referenced recently*/
#define IGC		0x200		/* inode eligible for GC */

/* modes */
#define	IFMT		0170000		/* type of file */
#define	IFIFO		0010000		/* named pipe (fifo) */
#define	IFCHR		0020000		/* character special */
#define	IFDIR		0040000		/* directory */
#define	IFBLK		0060000		/* block special */
#define	IFREG		0100000		/* regular */
#define	IFLNK		0120000		/* symbolic link */
#define	IFSOCK		0140000		/* socket */

#define	ISUID		04000		/* set user id on execution */
#define	ISGID		02000		/* set group id on execution */
#define	ISVTX		01000		/* save swapped text even after use */
#define	IREAD		0400		/* read, write, execute permissions */
#define	IWRITE		0200
#define	IEXEC		0100

#ifdef KERNEL
/*
 * Convert between inode pointers and vnode pointers
 */
#define VTOI(VP)	((struct inode *)(VP)->v_data)
#define ITOV(IP)	((struct vnode *)&(IP)->i_vnode)

/*
 * Convert between vnode types and inode formats
 */
extern enum vtype	iftovt_tab[];
extern int		vttoif_tab[];
#define IFTOVT(M)	(iftovt_tab[((M) & IFMT) >> 13])
#define VTTOIF(T)	(vttoif_tab[(int)(T)])

#define MAKEIMODE(T, M)	(VTTOIF(T) | (M))

#define SLIP(IP) simple_lock(&((IP)->i_lock))
#define SUIP(IP) simple_unlock(&((IP)->i_lock))

#define SLIP_SPL(IP) { \
	int s = splbio(); \
	SLIP(IP); \
	(IP)->i_spl = s; \
}
#define SUIP_SPL(IP) { \
	int s = (IP)->i_spl; \
	SUIP(IP); \
	(void) splx(s); \
}

#define	IUPDAT(ip, waitfor) { \
	if (ip->i_flag&(IUPD|IACC|ICHG|IMOD)) \
		iupdat(ip, waitfor); \
}

/*
 * Mark an inode with the current (unique) timestamp.
 */
#define IMARK(ip) { \
	if ((ip)->i_flag & IACC) { \
		(ip)->i_atime = time.tv_sec; \
	} \
	if ((ip)->i_flag & IUPD) { \
		(ip)->i_mtime = time.tv_sec; \
	} \
	if ((ip)->i_flag & ICHG) { \
		(ip)->i_ctime = time.tv_sec; \
	} \
}

#define ITIMES(ip) { \
	if ((ip)->i_flag & (IUPD|IACC|ICHG)) { \
		(ip)->i_flag |= IMOD; \
		IMARK(ip); \
		(ip)->i_flag &= ~(IACC|IUPD|ICHG); \
	} \
}

/*
 * Routines for locking the inode
 */
#define ILOCK(ip) rw_wlock(&(ip)->i_rwlock) /* exclusive lock */
#define ILOCK_R(ip) rw_rlock(&(ip)->i_rwlock) /* shared lock */
#define ILOCK_R2W(ip) \
	if (!rw_r2wlock(&(ip)->i_rwlock)) { \
		rw_runlock(&(ip)->i_rwlock); \
		rw_wlock(&(ip)->i_rwlock); \
	}
#define ILOCK_TRY(ip) rw_wlock_try(&(ip)->i_rwlock) /* try for exclusive lock*/

#define ILOCK_W2R(ip) rw_w2rlock(&(ip)->i_rwlock) /* write to read */

#define IUNLOCK(ip) rw_unlock(&(ip)->i_rwlock)	/* unlock */

#define ESAME	(-1)		/* trying to rename linked files (special) */

/*
 * Check that file is owned by current user or user is su.
 */
#define OWNER(CR, IP)	(((CR)->cr_uid == (IP)->i_uid)? 0: (suser()? 0: u.u_error))

/*
 * enums
 */
enum de_op	{ DE_CREATE, DE_LINK, DE_RENAME };	/* direnter ops */

/*
 * This overlays the fid sturcture (see vfs.h)
 */
struct ufid {
	u_short	ufid_len;
	ino_t	ufid_ino;
	long	ufid_gen;
};
#endif
