/* $Header: /nfs/columbia/u1/BFMACH.local/mk.2.9.6/vm/RCS/vmi.h,v 1.43 91/02/21 13:14:23 tappan Exp $ */

/*****************************************************************************
File:     vmi.h
Contents: PRIVATE ENCAPSULATED VM subsystem defines, read Remarks below.
System:   BBNACI Butterfly OS kernel, virtual memory subsystem.
Created:  15-Mar-1989
Author:   drb and rwells

Remarks: DON'T INCLUDE THIS FILE unless your name is "vm/vm_*.c"!
This header file is only to be included by source code within the vm
subdirectory.  Include "../h/vm.h" if you need VM definitions from
other parts of the kernel.  Please take this restriction seriously;
future maintainability depends upon it.  If you have to include this
header file, add your code to a "vm/vm_*.c" file, where everybody can
find it and use it.

[And now, a few words from the legal department.]

Copyright 1989 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

#ifndef _VMI_H_
#define _VMI_H_
#ifdef KERNEL			/* Only KERNEL definitions in this file */

/* ----- Definitions to control debugging features ----------------------- */
/* $$$ We should really be including feature header files to control */
/* $$$ this stuff, but it makes for more stuff requiring for compilation */
/* $$$ during initial standalone development.  -rwells, 24-April-89 */

#ifndef VM_TEST_PROGRAM		/* If for real, rather than test program...  */
#define VM_LOCK_DEBUG_DATA 1	/* Turns on scaffolding space in locks */
#include "vm_queue_debug.h"	/* to set up VM_QUEUE_DEBUG */
#include "vm_assert_debug.h"	/* ditto for VM_ASSERT_DEBUG */
#define VM_BBTREE_DEBUG 0       /* really verbose tracing, usually OFF */
#else				/* else if for test program... */
#define VM_LOCK_DEBUG_DATA 0	/* Turns on scaffolding space in locks */
#define VM_QUEUE_DEBUG 1	/* $$$ Should use "bbn_vm_queue_debug.h" */
#define VM_ASSERT_DEBUG 1	/* $$$ Should use "bbn_vm_assert_debug.h" */
#define VM_BBTREE_DEBUG 0       /* really verbose tracing, usually OFF */

#endif				/* end else for test program.  */

#include "../h/vm.h"		/* Include public vm subsystem definitions */
#include "../h/vm_stat.h"	/* Include public vm stat definitions */
#include "../h/lock.h"		/* For simple_lock stuff */
#include "../h/errno.h"		/* For errno error codes, used internally. */

/* ----- Machine definitions versus test program definitions -------------- */

#ifndef VM_TEST_PROGRAM		/* If this is for real, not test program... */
#include "../machine/cpu.h"     /* to get MASTER_CPU, log_node_map, etc.   */
#include "../machine/paddr.h"	/* manipulation of physical addresses */
#include "../vm/vm_pmap.h"	/* And pmap layer interface definitions */
#else				/* else if for test program... */

unsigned char log_node_map[256]; /* GLOBAL: on master */
short    mach_cpus;		/* GLOBAL: on master? */
#define  MASTER_CPU 0		/* System logical node number for master */
#define  NCPUS 3

/* This definition is just for the test program, and fits its */
/* notion of physical and virtual page addresses, and node numbers. */
/* We store the test node in the top 8 bits, with the high order bit */
/* set.  So we have to mask out the eighth bit. */

#undef paddr2pnode
#define paddr2pnode(pa)  ((((unsigned) (pa)) >> 24) & 0177)

#endif				/* end else VM_TEST_PROGRAM is defined */

/* ----- Forward declarations of types defined here ---------------------- */

typedef struct vm_chunk_data_t   *vm_chunk_t;
typedef struct vm_ihash_data_t   *vm_ihash_t;
typedef struct vm_list_data_t    *vm_list_t;
typedef struct vm_lock_data_t    *vm_lock_t;
typedef struct vm_mapping_data_t *vm_mapping_t;
typedef struct vm_object_data_t  *vm_object_t;
typedef struct vm_ovrec_data_t   *vm_ovrec_t;


/* ----- Internal typedefs, for clarity ---------------------------------- */

/* boolean_t is defined via "../h/lock.h" */

typedef unsigned short    ushort_t; /* POSIX style */

/* ---- Internal definitions --------------------------------------------- */

#define VM_LENDING_THRESHOLD   10
#define VM_PAGE_MAX_END        (0xffffe000)
#define VM_CID_NIL             (-1) /* Non-existent cluster id */

/* ----- Encapsulated interface with rest of system ----------------------- */

/* Defining this here avoids including param.h in vm_lock.c, */
/* where it would pull in the whole Mach world ad introduce conflicts */
/* between old and new vm definitions during development. */

#ifndef NULL
#define NULL 0
#endif

/* --- Functions in rest of system --- */

#if defined(__STDC__) || defined(c_plusplus)
void        blkclr(vm_addr_t va, vm_size_t);
void        bzero(vm_addr_t va, vm_size_t);
int         caller(void);
int         caller_caller(void);
int         caller_caller_caller(void);
int         cluster_noborrow(int cid);
short*      cluster_node_list(int cid, /*OUT*/ int *node_countp);
int         cluster_sysnode(int cid, int clusternode);
void        panic_printf();	/* VARARGS */
void        pmap_copy_page(vm_addr_t src_physaddr, vm_addr_t dest_physaddr);
pmap_t      pmap_create(int node, vm_size_t softsize);
void        pmap_destroy(pmap_t);
int         pmap_enter(pmap_t, vm_addr_t va, vm_addr_t pa, vm_prot_t,
		       int wirestat);
pmap_t      pmap_kernel(void);
vm_addr_t   pmap_map(vm_addr_t va, vm_addr_t start, vm_addr_t end, int prot);
void	    pmap_protect(pmap_t, vm_addr_t start, vm_addr_t end,
			 vm_prot_t newprot);
void	    pmap_remove(pmap_t, vm_addr_t start, vm_addr_t end);
void	    pmap_remove_all(vm_addr_t physaddr);
int         pmap_stat_pages(pmap_t, vm_addr_t start, vm_addr_t end,
			    int structsize, int maxcnt, int* int_psp/*OUT*/,
			    vm_addr_t* nextp/*OUT*/);
void        pmap_zero_page(vm_addr_t physaddr);
void        printf(/*VARARGS*/);
void        simple_lock_init(/*simple_lock_t slp*/); /* defed in lock.h */
void        simple_lock(/*simple_lock_t slp*/);      /* defed in lock.h */
void        simple_unlock(/*simple_lock_t slp*/);    /* defed in lock.h */
int         sleep_and_unlock(vm_addr_t chan, int pri, simple_lock_t lockp);
int         splhigh(void);
void        splx(int level);
vm_errno_t  vnode_fsync(vnode_t vp);
void        vnode_mapping_decr(vnode_t vp);
vm_errno_t  vnode_mapping_incr(vnode_t vp, int istext, int readonly,
			       int truncate);
vm_errno_t  vnode_mapstat(vnode_t vp, /*OUT*/vm_regionstat_t rsp);
vm_errno_t  vnode_page_alloc(/*OUT*/vnode_t *vpp, /*OUT*/vm_offset_t *offsetp);
vm_errno_t  vnode_page_out(vnode_t vp, vm_offset_t offset, vm_addr_t pa, 
			   int flags);
vm_errno_t  vnode_page_in(vnode_t vp, vm_offset_t offset, vm_addr_t pa,
			  int flags);
vm_errno_t  vnode_page_free(vnode_t vp, vm_offset_t offset);
#ifndef vvrmtptr
vm_addr_t   vvrmtptr(vm_addr_t vectorized_addr, int node);
#endif vvrmtptr
void        wakeup(vm_addr_t);
#else
void        bzero(/*see above*/);
int         caller(/*see above*/);
int         caller_caller(/*see above*/);
int         caller_caller_caller(/*see above*/);
int         cluster_noborrow(/*see above*/);
short*      cluster_node_list(/*see above*/);
int         cluster_sysnode(/*see above*/);
void        panic_printf(/*see above*/);
void        pmap_copy_page(/*see above*/);
pmap_t      pmap_create(/*see above*/);
void        pmap_destroy(/*see above*/);
int         pmap_enter(/*see above*/);
pmap_t      pmap_kernel(/*see above*/);
vm_addr_t   pmap_map(/*see above*/);
void	    pmap_protect(/*see above*/);
void	    pmap_remove(/*see above*/);
void	    pmap_remove_all(/*see above*/);
int         pmap_stat_pages(/*see above*/);
void        printf(/*see above*/);
void        simple_lock_init(/*see above*/);
void        simple_lock(/*see above*/);
void        simple_unlock(/*see above*/);
int         sleep_and_unlock(/*see above*/);
int         splhigh(/*see above*/);
void        splx(/*see above*/);
vm_errno_t  vnode_fsync(/*see above*/);
void        vnode_mapping_decr(/*see above*/);
vm_errno_t  vnode_mapping_incr(/*see above*/);
vm_errno_t  vnode_mapstat(/*see above*/);
vm_errno_t  vnode_page_alloc(/*see above*/);
vm_errno_t  vnode_page_out(/*see above*/);
vm_errno_t  vnode_page_in(/*see above*/);
vm_errno_t  vnode_page_free(/*see above*/);
#ifndef vvrmtptr
vm_addr_t   vvrmtptr(/*see above*/);
#endif vvrmtptr
void        wakeup(/*see above*/);
#endif

/* --- Globals in rest of system --- */

int              my_cpu_number;	/* VECTORIZED: $$$ is this the right type? */

/* ----- lock data structure ---------------------------------------------- */

typedef struct vm_lock_data_t
{
#if VM_LOCK_DEBUG_DATA > 0
    simple_lock_data_t	latch;	/* spin lock, no sleeping 	*/
#else
    unsigned long	latch;	/* allocate minimal space if not debugging */
#endif

    u_short		flags;	/* wanted, locked, etc.   	*/
    u_short		wanted;
    unsigned long     	owner;	/* owning proc pointer		*/
#if VM_LOCK_DEBUG_DATA > 0
    int			owner_cpu; /* and logical cpu number	*/
    unsigned long	caller;	/* traceback			*/
    unsigned long	c2;
    unsigned long	c3;
#endif
} vm_lock_data_t;

#define VM_LOCK_LOCKED	0x00000001

/* This macro is used in assertions */
/* $$$ maybe later make sure is really our lock if VM_LOCK_DEBUG is defed */

#define vm_lock_ismine(lp) ((lp)->flags & VM_LOCK_LOCKED)


#if defined(__STDC__) || defined(c_plusplus)
void       vm_lock_init(vm_lock_t lp); /* Initialize the sleepable lock */
void       vm_lock(vm_lock_t lp); /* Lock the sleepable lock */
int        vm_lock_nowait(vm_lock_t lp); /* Lock the sleepable lock */
void       vm_unlock(vm_lock_t lp); /* Unlock the sleepable lock */
#else
void       vm_lock_init(/*see above*/);
void       vm_lock(/*see above*/);
int        vm_lock_nowait(/*see above*/);
void       vm_unlock(/*see above*/);
#endif

/* ----- debugging support for queues ------------------------------------- */

/* The rest of the queue definitions are public, in vm.h */

#if VM_QUEUE_DEBUG > 0
#define  VM_QUEUE_DEBUG_NOTINLIST(qp) \
    (((qp)->prev != (qp) || (qp)->next != (qp)) ? vm_queue_panic(qp) : 0)
#define  VM_QUEUE_DEBUG_VALID(qp) \
    (((qp)->prev == NULL || (qp)->prev->next != (qp) || \
      (qp)->next->prev != (qp)) ? vm_queue_panic(qp) : 0)
#define  VM_QUEUE_DEBUG_INLIST(qp) \
    (((qp)->prev == NULL || (qp)->prev == (qp) || (qp)->prev->next != (qp) || \
      (qp)->next->prev != (qp)) ? vm_queue_panic(qp) : 0)
#else
#define VM_QUEUE_DEBUG_NOTINLIST(qp)
#define VM_QUEUE_DEBUG_INLIST(qp)
#define VM_QUEUE_DEBUG_VALID(qp)
#endif

/* ----- herd object definitions ------------------------------------------ */

#define VM_HERD_BORROW		 0
#define VM_HERD_NOBORROW	 1
#define VM_HERD_SCATTERED	 2
#define VM_HERD_SCATNOBORROW	 3
#define VM_HERD_INTRLV		 4
#define VM_HERD_INTRLVNOBORROW	 5
#define VM_HERD_NHERDS		 6

/* ----- map data structure ----------------------------------------------- */

typedef struct vm_map_data_t
{
    int                 home_node; /* location of map + entries */
    vm_addr_t           limit_start; /* 1st addr whose entry can be changed */
    vm_addr_t           limit_end; /* 1st addr whose entry can't be changed */
    vm_addr_t           anywhere_start; /* min default address for ANYWHERE */
    vm_queue_data_t	entries; /* list of vm_entries		*/
    vm_lock_data_t	lock;			
    vm_object_t         herd[VM_HERD_NHERDS]; /* Objects for newly 
						 privatized pages (COWs) */
    pmap_t		pmap;	/* MMU page tables etc.		*/
    int                 cid;	/* Cluster id for map, for borrow/scatter */
    long                counts[VM_CNT_COUNT]; /* vm_stat activity counters */
    vm_entry_t          bbtree_root; /* root of balanced binary search tree */
#ifdef VM_TEST_PROGRAM
    char                pad[8]; /* pad size to 128 bytes for testing */
#endif
} vm_map_data_t;


/* ----- functions over range of map addresses ---------------------------- */

#define vm_range_invalid(mp, start, end) \
    (!vm_isaligned(start) || !vm_isaligned(end) || (start) >= (end) || \
     (start) < (mp)->limit_start || (end) > (mp)->limit_end)

#if defined(__STDC__) || defined(c_plusplus)
vm_errno_t vm_range_access(vm_map_t mp, vm_addr_t start, vm_addr_t end,
			   vm_prot_t newprot);
vm_errno_t vm_range_find_hole(vm_map_t mp, vm_size_t size, int anywhere,
			    /*INOUT*/ vm_addr_t *vap);
vm_queue_t vm_range_first_qp(vm_map_t mp, vm_addr_t va);
vm_errno_t vm_range_inherit(vm_map_t mp, vm_addr_t start, vm_addr_t end,
			    vm_inherit_t newinherit);
vm_errno_t vm_range_iswired(vm_map_t mp, vm_addr_t start, vm_addr_t end);
vm_errno_t vm_range_protect(vm_map_t mp, vm_addr_t start, vm_addr_t end,
			    boolean_t setmax, vm_prot_t newprot);
vm_entry_t vm_range_remove(vm_map_t mp, vm_addr_t va, vm_addr_t eva,
			       vm_entry_t ep);
vm_errno_t vm_range_sync(vm_map_t mp,vm_addr_t start,vm_addr_t end,int flags);
vm_errno_t vm_range_wiring(vm_map_t mp, vm_addr_t start, vm_addr_t end,
			   boolean_t wiredflag);
#else
vm_errno_t vm_range_access(/*see above*/);
vm_errno_t vm_range_find_hole(/*see above*/);
vm_queue_t vm_range_first_qp(/*see above*/);
vm_errno_t vm_range_inherit(/*see above*/);
vm_errno_t vm_range_iswired(/*see above*/);
vm_errno_t vm_range_protect(/*see above*/);
vm_entry_t vm_range_remove(/*see above*/);
vm_errno_t vm_range_sync(/*see above*/);
vm_errno_t vm_range_wiring(/*see above*/);
#endif


/* ----- map entry data structure ----------------------------------------- */

typedef struct vm_entry_data_t
{
    vm_queue_data_t	entryq;	/* list of map entries		*/
    vm_addr_t		start;	/* first virtual address covered*/
    vm_addr_t		end;	/* first va NOT covered		*/
    vm_object_t		object;	/* object I point to 		*/
    vm_offset_t		offset;	/* offset into object 		*/
    vm_entry_t          bbtree[2]; /* b-tree link fields        */
    ushort_t            wirecnt; /* wiring count for entry      */
    ushort_t            zone_node; /* entry's alloc node        */
    char		flags;	/* see definitions below	*/
    char		curprot; /* current protection code 	*/
    char		maxprot; /* maximum protection code 	*/
    char		inherit; /* inheritance code 	        */
    char                caching; /* BFLY2 caching flags         */
    char                bbtree_bal; /* b-tree balance tag       */
#ifdef VM_TEST_PROGRAM
    char                pad[8];	/* pad size to 48 bytes for tests */
#endif
} vm_entry_data_t;

#define VM_ENTRY_MAXWIRE 65535
#define VM_ENTRY_COPY	0x01	/* copy on shared write or non-local ref */
#define VM_ENTRY_TEXT   0x02	/* copy on any write, is text segment */

/* bbtree_bal values */
#define VM_LEFT      0		/* Special: must be 0 */
#define VM_RIGHT     1		/* Special: must be 1 */
#define VM_BALANCED  2		/* Special: must not be 0 or 1 */
#define VM_NOBALANCE 3		/* Marks entry not in tree. */

/* macro for translating embedded queue entry to enclosing entry object */
#define vm_queue_entryq_entry(qp) vm_queue_coerce((qp),vm_entry_t,entryq)

#if defined(__STDC__) || defined(c_plusplus)
vm_entry_t vm_entry_create(vm_map_t mp, vm_addr_t start, vm_addr_t end,
			   vm_object_t object, vm_offset_t offset,
			   vm_entry_t ep);
void       vm_entry_remove(vm_map_t mp, vm_entry_t ep);
void       vm_entry_free(vm_map_t mp, vm_entry_t ep);
vm_entry_t vm_entry_insert(vm_map_t mp, vm_entry_t nep, vm_entry_t ep);
vm_entry_t vm_entry_lookup(vm_map_t mp, vm_addr_t va);
vm_entry_t vm_entry_merge(vm_map_t mp, vm_entry_t ep);
vm_entry_t vm_entry_split(vm_map_t mp, vm_entry_t ep, vm_addr_t split);
void       vm_entry_wire_decr(vm_map_t mp, vm_entry_t ep);
void       vm_entry_wire_incr(vm_map_t mp, vm_entry_t ep);
void       vm_bbtree_insert(vm_map_t mp, vm_entry_t ep);
void       vm_bbtree_delete(vm_map_t mp, vm_entry_t ep);
vm_entry_t vm_kentry_alloc();
#else
vm_entry_t vm_entry_create(/*see above*/);
void       vm_entry_remove(/*see above*/);
vm_entry_t vm_entry_insert(/*see above*/);
void       vm_entry_free(/*see above*/);
vm_entry_t vm_entry_lookup(/*see above*/);
vm_entry_t vm_entry_merge(/*see above*/);
vm_entry_t vm_entry_split(/*see above*/);
void       vm_entry_wire_decr(/*see above*/);
void       vm_entry_wire_incr(/*see above*/);
void       vm_bbtree_insert(/*see above*/);
void       vm_bbtree_delete(/*see above*/);
vm_entry_t vm_kentry_alloc();
#endif


/* ----- object data structure -------------------------------------------- */

typedef struct vm_object_data_t
{
    vm_queue_data_t	pages;	/* list of resident pages 	*/
    vm_lock_data_t	lock;	/* OFF FRONT, untouched in zone free list */
    vm_queue_data_t	empty_cacheq; /* Queue of empty cached text objects */
    vm_map_t            parmap;	/* parent map for object; may be NULL */
    vm_mapping_t        mapping; /* Optional text/mapmem file mapping */
    vm_addr_t           start_used; /* address of first page ever in object */
    vm_addr_t           end_used; /* address after last page ever in object */
    vm_offset_t         vlink;	/* link to pages on backing store */
    ushort_t		npages;	/* number of resident pages 	*/

    ushort_t            parcnt;	/* Number of parent map references */
    ushort_t		othcnt;	/* Number of other map references */
    ushort_t            outcnt;	/* Count of vnode pages not incore,512MB max */
    ushort_t		flags;	/* see definitions below	*/
    ushort_t		zone_node; /* node where object is allocated */
    short		page_node; /* node for pages; may be VM_BEST_NODE */
    char                page_status; /* active page status for object */
#ifdef VM_TEST_PROGRAM
    char                pad[11]; /* Padding to keep struct size 60 for tests */
#endif
} vm_object_data_t;

#define VM_OBJECT_MAXOUTBACK 65535
#define VM_OBJECT_VLINK_NIL 0xffffffff /* Nil link to vnode page hash table */

/* IMPORTANT:
   	The order of the first bits is very important for the memtype
   definition below.  This allows the herd to be selected very quickly.
   See the herd type definitions above.  If more memory types are added,
   we will probably need to move these bits into a separate object memory 
   type field.  This is because memtype flags 0x0008 would create a hole
   where 6 and 7 where unused.
 */

#define VM_OBJECT_NOBORROW    0x0001 /* pages in object can't be borrowed. */
#define VM_OBJECT_SCATTERED   0x0002 /* page-level interleaved	*/
#define VM_OBJECT_INTERLEAVED 0x0004 /* hardware interleaved	*/
#define VM_OBJECT_HERD        0x0008 /* herd object (my cows)	*/
#define VM_OBJECT_FOREIGN     0x0010 /* maps foreign (non-nX) memory */
#define VM_OBJECT_CACHED      0x0020 /* Object is part of the text cache */
#define VM_OBJECT_EMPTYCACHE  0x0040 /* Object is cached, but is empty */
#define VM_OBJECT_VME	      0x0080 /* Object maps I/O bus space */
#define VM_OBJECT_BUF	      0x0100 /* Object is part of buffer cache */

#define VM_OBJECT_MEMTYPE     (VM_OBJECT_NOBORROW|VM_OBJECT_INTERLEAVED \
			       |VM_OBJECT_SCATTERED)
#define VM_WHICH_HERD(oflags) (oflags & VM_OBJECT_MEMTYPE)

#if defined(__STDC__) || defined(c_plusplus)
vm_object_t  vm_object_create(vm_map_t mp, int pagenode, int page_status,
			      int flags, vm_object_t op);
void         vm_object_decr(vm_map_t mp, vm_object_t op);
void         vm_object_incr(vm_map_t mp, vm_object_t op);
char*        vm_object_label_text(int index, char* buf);
void         vm_object_print(vm_object_t op);
void         vm_object_print_once(vm_object_t op, int listlen, 
				  vm_object_t* oplist);
int          vm_object_try_switch(vm_map_t mp, vm_entry_t pep, vm_entry_t ep);
#else
vm_object_t  vm_object_create(/*see above*/);
void         vm_object_decr(/*see above*/);
void         vm_object_incr(/*see above*/);
char*        vm_object_label_text(/*see above*/);
void         vm_object_print(/*see above*/);
void         vm_object_print_once(/*see above*/);
int          vm_object_try_switch(/*see above*/);
#endif

/* ----- page data structure ---------------------------------------------- */

/*
 * blockmask_t is a type which should be large enough to hold a bitmap
 * of all DEV_BSIZE sized blocks that fit within a page
 */
typedef u_short blockmask_t;

typedef struct vm_page_data_t
{
    vm_queue_data_t	pageq;	/* object's resident page list	*/
    vm_queue_data_t	listq;	/* queue entry for vm_list	*/
    vm_queue_data_t	hashq;	/* (object,offset)->page hash	*/
    vm_object_t		object;	/* page's object; NULL if none  */
    vm_offset_t		offset;	/* offset into that object	*/
    vm_addr_t		physaddr; /* physical address of page 	*/
    vm_revpte           revptes; /* virtual mapping(s) of page  */
    ushort_t		wirecnt; /* number of wired mappings	*/
    ushort_t		status;	/* active, inactive, zeroed,etc	*/
    ushort_t		flags;	/* see below			*/
    ushort_t            locrefs; /* local reference count       */
    ushort_t            rmtrefs; /* remote reference count      */
    ushort_t		owner;	/* Proc which owns write lock on page */
    /* The following fields are used by the dynamic buffer cache.
     * Should they cause a problem with the size of the structure
     * they could be moved to a dynamically allocated structure
     * used only when a page actually maps a cached disk page. 
     * The cost is increased complexity in the buffer cache code.
     */
    vm_queue_data_t	bhash;	/* (device,offset)->page hash */
    struct vnode 	*devvp;	/* vnode for device file */
    vm_queue_data_t	devlist; /* device vnode cached page list */
    daddr_t		blkno;	/* 'disk address' of block 0 in page  */
    blockmask_t		valid_blocks;	/* bitmask to indicate which 
				         * 'DEV_BSIZE' blocks are valid
				         */
    blockmask_t		dirty_blocks;	/* bitmask to indicate which
				         * 'DEV_BSIZE' blocks are dirty
				         */
} vm_page_data_t;

/* macros on page object */

#define VM_PAGE_MAXWIRE  65535
#define VM_PAGE_NODE(pp) (log_node_map[paddr2pnode(pp->physaddr)])
 
/* list status codes, mutually exclusive - now live in vm.h */

/* a page with one of these status codes is not on any list */
#define VM_PAGE_REMOVED      VM_PAGE_NUMLISTS /* page in limbo */
#define VM_PAGE_REMOVED_ZEROED (VM_PAGE_NUMLISTS+1) /* zeroed page in limbo */

/* flags */
#define VM_PAGE_DIRTY        0x01 /* page contains dirty data	*/
#define VM_PAGE_BORROWED     0x02 /* page was borrowed in vm_page_alloc. */
#define VM_PAGE_OUTBACK      0x04 /* page has a copy on backing store. */
#ifdef BFLY2
#define	VM_PAGE_INTERLEAVED  0x08 /* page is interleaved (no home node) */
#endif BFLY2
#define VM_PAGE_BUSY	     0x10 /* Page is busy ([de]activating) */
#define VM_PAGE_BUSYWAIT     0x20 /* processes waiting on VM_PAGE_BUSY */
#define VM_PAGE_ERROR	     0x40 /* Parity (eg.) error in page */
#define VM_PAGE_BINVAL	     0x80 /* invalid in buffer cache */
#define VM_PAGE_BFLUSH	     0x100 /* flush needed on release */

/* macros for translating embedded queue entry to enclosing page object */
#define vm_queue_pageq_page(qp)   vm_queue_coerce((qp),vm_page_t,pageq)
#define vm_queue_listq_page(qp)   vm_queue_coerce((qp),vm_page_t,listq)
#define vm_queue_hashq_page(qp)   vm_queue_coerce((qp),vm_page_t,hashq)
#define vm_queue_bhash_page(qp)	  vm_queue_coerce((qp),vm_page_t,bhash)
#define vm_queue_devlist_page(qp) vm_queue_coerce((qp),vm_page_t,devlist)

/* 'function' to check whether a page is part of the buffer cache */
#define page_in_bufcache(pp) ((pp)->devvp != 0)

/* functions on vm_page objects - a few functions are public in ../h/vm.h */
#if defined(__STDC__) || defined(c_plusplus)
void       vm_page_init(vm_page_t pp, vm_addr_t physaddr);
vm_errno_t vm_page_clean(vm_object_t op, vm_offset_t offset, vm_page_t pp, int);
void       vm_page_insert(vm_object_t op, vm_offset_t offset, vm_page_t pp);
void       vm_page_print(vm_page_t pp);
vm_entry_t vm_page_privatize(vm_map_t mp, vm_entry_t ep, vm_addr_t va);
void       vm_page_remove(vm_object_t op, vm_offset_t offset, vm_page_t pp);
char*      vm_page_status_name(int status);
vm_page_t  PHYS_TO_VM_PAGE(vm_offset_t pa);
#else
void       vm_page_init(/*see above*/);
vm_errno_t vm_page_clean(/*see above*/);
void       vm_page_insert(/*see above*/);
void       vm_page_print(/*see above*/);
vm_entry_t vm_page_privatize(/*see above*/);
void       vm_page_remove(/*see above*/);
char*      vm_page_status_name(/*see above*/);
vm_page_t  PHYS_TO_VM_PAGE(/*see above*/);
#endif

/* ----- page allocation functions ---------------------------------------- */

#if defined(__STDC__) || defined(c_plusplus)
vm_page_t  vm_page_alloc(vm_map_t mp, vm_object_t op, vm_offset_t offset,
			 int flags);
vm_page_t  vm_page_alloc_easy(vm_map_t mp, vm_object_t op, vm_offset_t offset,
			      int zeroflag);
/* vm_page_t  vm_page_alloc_on_node(int node, int flags) in vm.h !! */
vm_page_t  vm_page_alloc_easy_on_node(int lognode, int zeroflag);
int        vm_page_alloc_lender(vm_map_t mp, int type);
vm_page_t  vm_page_alloc_borrow(vm_map_t mp, int home_node, int zeroflag);
#ifdef BFLY2
vm_page_t  vm_intrlv_page_alloc(vm_map_t mp, vm_object_t op, vm_offset_t offset,
			 int flags);
vm_page_t  vm_intrlv_page_alloc_from_node(int node, int flags);
vm_page_t  vm_intrlv_page_alloc_easy_from_node(int node, int flags)
vm_page_t  vm_intrlv_page_alloc_borrow(int stripe, int flags)
#endif BFLY2
#else
vm_page_t  vm_page_alloc(/*see above*/);
vm_page_t  vm_page_alloc_easy(/*see above*/);
vm_page_t  vm_page_alloc_easy_on_node(/*see above*/);
int        vm_page_alloc_lender(/*see above*/);
vm_page_t  vm_page_alloc_borrow(/*see above*/);
#ifdef BFLY2
vm_page_t  vm_intrlv_page_alloc(/*see above*/);
vm_page_t  vm_intrlv_page_alloc_from_node(/*see above*/);
vm_page_t  vm_intrlv_page_alloc_easy_from_node(/*see above*/);
vm_page_t  vm_intrlv_page_alloc_borrow(/*see above*/);
#endif BFLY2
#endif

/* ----- vm_list data structure ------------------------------------------- */

typedef struct vm_list_data_t
{
    simple_lock_data_t latch;	/* simple lock on list of pages */
    vm_queue_data_t    list;	/* Queue head for list of pages */
    int                count;	/* Number of pages in the list */
} vm_list_data_t;


/* Vectorized pointer to the indicated node's page queues */
extern vm_list_data_t  *my_vm_page_lists;
vm_list_data_t *VM_LIST();	/* used to be a macro... */

#define vm_list_peekcount(list, node) ((VM_LIST((list), (node)))->count)

#ifdef BFLY2
/* Interleaved Page vm_lists */

#define VM_INTRLV_NUMPOOLS	  8
#define VM_STRIPE_INTRLV	  512

/* Vectorized pointer to the indicated node's numpool intrlvd page queues */
extern vm_list_data_t  *my_vm_intrlv_lists;
vm_list_data_t *VM_INTRLV_LIST();	/* used to be a macro... */

#define vm_intrlv_list_peekcount(list, node, pool)  \
		       ((VM_INTRLV_LIST((list), (node), (pool)))->count)

/* for each pool, a list of nodes interleaved across */
short  vm_intrlv_phys_node_list[VM_INTRLV_NUMPOOLS][NCPUS];
short  vm_intrlv_node_count[VM_INTRLV_NUMPOOLS];

extern unsigned short stripeoffset2pnode[];
#define VM_INTRLV_PAGE_NODE(pp) (log_node_map[stripeoffset2pnode[          \
				 paddr2pnode(pp->physaddr)]])
#if defined(__STDC__) || defined(c_plusplus)
void      vm_intrlv_list_init(int list, int pool, int node);
vm_page_t vm_intrlv_list_getpage(int list, int pool, int node);
vm_page_t vm_intrlv_list_peekpage(int list, int pool, int node);
vm_page_t vm_intrlv_list_easypage(int list, int pool, int node);
void      vm_intrlv_list_putpage(int list, vm_page_t pp);
void      vm_intrlv_list_rempage(vm_page_t pp);
#else
void      vm_intrlv_list_init(/*see above*/);
vm_page_t vm_intrlv_list_getpage(/*see above*/);
vm_page_t vm_intrlv_list_peekpage(/*see above*/);
vm_page_t vm_intrlv_list_easypage(/*see above*/);
void      vm_intrlv_list_putpage(/*see above*/);
void      vm_intrlv_list_rempage(/*see above*/);
#endif
#endif BFLY2

/* Templates for functions of vm_list */
/* Note that vm_list_getpage will return NULL ptr if the list is empty. */

#if defined(__STDC__) || defined(c_plusplus)
void      vm_list_init(int list, int node);
vm_page_t vm_list_getpage(int list, int node);
vm_page_t vm_list_peekpage(int list, int node);
vm_page_t vm_list_easypage(int list, int node);
void      vm_list_putpage(int list, vm_page_t pp);
void      vm_list_rempage(vm_page_t pp);
#else
void      vm_list_init(/*see above*/);
vm_page_t vm_list_getpage(/*see above*/);
vm_page_t vm_list_peekpage(/*see above*/);
vm_page_t vm_list_easypage(/*see above*/);
void      vm_list_putpage(/*see above*/);
void      vm_list_rempage(/*see above*/);
#endif

/* ----- vm_ihash data structure ------------------------------------------- */

/* This is a framework used to implement several internal hash tables within */
/* the VM subsystem.  It provides a simple_lock which should be kept locked */
/* while a given queue is traversed or manipulated.  It implements a chained */
/* hash table in which a hash function on a multi-word key is used to select */
/* one of a set of queue heads, and all entries that hash to the same queue */
/* head are kept in that queue.  Each of the hash tables implements its own */
/* policy for order of entries within the queue, and where the key is stored */
/* within each entry.  This framework reduces the hash table management */
/* problem to a much simpler linear list management problem.  The raw hash */
/* number can be calculated before the hash table is locked, but the */
/* selection of the queue head and subsequent queue manipulations must be */
/* done while the hash table is locked.  In principle the hash arena could */
/* be dynamically resized and rehashed while it is locked. */

typedef struct vm_ihash_data_t
{
    simple_lock_data_t latch;	/* simple lock on hash table */
    int                nqueues; /* Number of queues in chained hash table */
    vm_queue_t         queues;	/* Ptr to array of queue heads */
} vm_ihash_data_t;

/* Macros for vm_ihash */

#define vm_ihash_headqp(hp,n) ((hp)->queues + ((n) % (hp)->nqueues))

/* Templates for functions of vm_ihash */

#if defined(__STDC__) || defined(c_plusplus)
void       vm_ihash_init(vm_ihash_t hp, int nqueues, int node);
int        vm_ihash_stat(vm_ihash_t hp, int *nqueuesp, int *nzqueuesp,
			 int *maxqueuep);
int        vm_ihash_num(int word1, int word2);
#else
void       vm_ihash_init(/*see above*/);
int        vm_ihash_stat(/*see above*/);
int        vm_ihash_num(/*see above*/);
#endif


/* ----- Internal table from (object,offset) to (pp) ---------------------- */

/* VECTORIZED: (object,offset)=>pp table. */
extern vm_ihash_data_t my_objoff_pp_ihash; 

#if defined(__STDC__) || defined(c_plusplus)
vm_page_t  vm_objoff_pp_lookup(vm_object_t object, vm_offset_t offset);
void       vm_objoff_pp_insert(vm_object_t object, vm_offset_t offset,
			     vm_page_t pp);
void       vm_objoff_pp_delete(vm_object_t object, vm_offset_t offset,
			     vm_page_t pp);
int        vm_objoff_pp_stat(int node, int *nqueuesp, int *nzqueuesp,
			   int *maxqueuep);
void       vm_objoff_pp_init(int pagespernode);
void       vm_objoff_pp_fixup(vm_map_t kmp);
#else
vm_page_t  vm_objoff_pp_lookup(/*see above*/);
void       vm_objoff_pp_insert(/*see above*/);
void       vm_objoff_pp_delete(/*see above*/);
int        vm_objoff_pp_stat(/*see above*/);
void       vm_objoff_pp_init(/*see above*/);
void       vm_objoff_pp_fixup(/*see above*/);
#endif

/* ----- External table from (object,offset) to (vnode,offset) ------------ */

/* Currently implemented as an internal table of status structures */

typedef struct vm_ovrec_data_t	/* Offset-vnode record */
{
    vm_object_t     object;	/* Object for page */
    vm_offset_t     offset;	/* Offset within object */
    vnode_t         vnode;	/* Vnode for page */
    vm_offset_t     voffset;	/* Offset within vnode */
    vm_offset_t     vlink;	/* offset within object for previous rec. */
    vm_queue_data_t ovrecq;	/* queue entry used in hashing records */
    vm_queue_data_t vpovrecq;	/* queue of ovrec entries headed by vnode */
    ushort_t        flags : 1;	/* state flags */
    ushort_t        refcnt : 15;/* reference count */
    ushort_t        zone_node;	/* node where ovrec is allocated */
} vm_ovrec_data_t;

/* ovrec state flags definitions */
#define	OVREC_WANTED	0x01	/* paging system wants to modify the ovrec */

/* macro for translating embedded queue entry to enclosing ovrec object */
#define vm_queue_ovrecq_ovrec(qp) vm_queue_coerce((qp),vm_ovrec_t,ovrecq)

/* macro for translating embedded vp queue entry to enclosing ovrec object */
#define vm_queue_vpovrecq_ovrec(qp) vm_queue_coerce((qp),vm_ovrec_t,vpovrecq)

vm_ihash_data_t vm_objoff_vnode_ihash; /* GLOBAL: (object,offset)=>vnode. */

#if defined(__STDC__) || defined(c_plusplus)
vnode_t     vm_objoff_vnode_lookup(vm_object_t object, vm_offset_t offset,
				   /*OUT*/ vm_offset_t *voffsetp,
				   /*OUT*/ vm_ovrec_t *ovrpp);
void	    vm_objoff_vnode_insert(vm_object_t object, vm_offset_t offset,
				   vnode_t vp, vm_offset_t voffset,
				   vm_offset_t vlink,
				   /*OUT*/ vm_ovrec_t *ovrpp);
void        vm_objoff_vnode_release(vm_ovrec_t ovrp);
void        vm_objoff_vnode_delete(vm_object_t object);
int         vm_objoff_vnode_stat(int *nqueuesp, int *nzqueuesp,
				 int *maxqueuep);
void        vm_objoff_vnode_init(int vpagespernode);
#else
vnode_t     vm_objoff_vnode_lookup(/*see above*/);
void	    vm_objoff_vnode_insert(/*see above*/);
void        vm_objoff_vnode_release(/*see above*/);
void        vm_objoff_vnode_delete(/*see above*/);
int         vm_objoff_vnode_stat(/*see above*/);
void        vm_objoff_vnode_init(/*see above*/);
#endif


/* ----- mapping data structure connecting vnode and object --------------- */

typedef struct vm_maptab_data_t
{
    vm_lock_data_t    lock;
    vm_ihash_data_t   ihash;
} vm_maptab_data_t;

typedef struct vm_mapping_data_t
{
    vm_queue_data_t     hashq;	/* Queue entry for mapping hash table */
    vm_queue_data_t	vnodeq;	/* Queue entry for vnode mapping list */
    vm_object_t		object;	/* Ptr to object side of mapping */
    vnode_t             vnode;	/* Ptr to vnode  side of mapping */
    vm_offset_t         vstart;	/* Starting offset in vnode */
    vm_offset_t         vend;	/* Ending offset in vnode; may expand */
    short               keynode; /* Mapping's node for table lookups */
    char                flags;	/* Set of flags */
#ifdef VM_TEST_PROGRAM
    char                pad[5];	/* Pad struct size to 32 bytes. */
#endif
} vm_mapping_data_t;

#define VM_MAPPING_TEXT     0x01 /* Mapping has text segment semantics */
#define VM_MAPPING_CACHED   0x02 /* Mapping points to a cached text object */
#define VM_MAPPING_ANON	    0x04 /* Mapping has "anonymous" semantics */

/* macro for translating embedded queue entry to enclosing mapping object */
#define vm_queue_hashq_mapping(qp) vm_queue_coerce((qp),vm_mapping_t,hashq)

/* GLOBAL: mapping tables, one for each node text plus one global for data */
/* $$$$ should be vectorized or distributed */
vm_maptab_data_t vm_mapping_maptab[NCPUS + 1];

#if defined(__STDC__) || defined(c_plusplus)
vm_errno_t   vm_mapping_create(vm_map_t mp, vnode_t vp, int keynode,
			       int pagenode, vm_offset_t vstart,
			       vm_offset_t vend, int page_status, int mapflags,
			       /*OUT*/ vm_mapping_t* mappingp);
void         vm_mapping_init(int mappingspernode);
void         vm_mapping_lock(int node);
vm_mapping_t vm_mapping_lookup(vnode_t vp, int node, vm_offset_t vstart,
			       vm_offset_t vend);
void         vm_mapping_print(vm_mapping_t mapping);
void         vm_mapping_remove(vm_mapping_t mapping);
vm_errno_t   vm_mapping_entry(vm_map_t mp, vm_addr_t start, vm_addr_t end,
			      vnode_t vp, vm_offset_t vstart,
			      vm_offset_t vend, int mapflags, int pagenode,
			      /*OUT*/vm_entry_t* epp);
void         vm_mapping_unlock(int node);
#else
vm_errno_t   vm_mapping_create(/*see above*/);
void         vm_mapping_init(/*see above*/);
void         vm_mapping_lock(/*see above*/);
vm_mapping_t vm_mapping_lookup(/*see above*/);
void         vm_mapping_print(/*see above*/);
void         vm_mapping_remove(/*see above*/);
vm_errno_t   vm_mapping_entry(/*see above*/);
void         vm_mapping_unlock(/*see above*/);
#endif

/* ----- VM error and status functions ----------------------------------- */

/* NOTE on using VM_ASSERT: This version of the assert macro takes three */
/* arguments: the obvious test condition, and a function name and short tag */
/* to differentiate assertions in the same function.  This fits into the */
/* panic_printf scheme already used in the kernel better than a __FILE__ and */
/* __LINE__ based scheme would, and provides a compact identification for */
/* the assertion when it fails. */

#if VM_ASSERT_DEBUG > 0
#define VM_ASSERT(e,fun,tag) (!(e) ? vm_assert_panic((fun),(tag)),0:0)
#else
#define VM_ASSERT(e,fun,tag)
#endif VM_ASSERT_DEBUG

#if defined(__STDC__) || defined(c_plusplus)
void         vm_assert_panic(char *fun, char *tag);
#else
void         vm_assert_panic(/*see above*/);
#endif

/* ----- vm_chunk_t data structures, used to support zones --------------- */

typedef struct vm_chunk_data_t
{
    simple_lock_data_t  latch;
    int    		flags;
    vm_offset_t         base_addr;
    vm_offset_t         limit_addr;
    vm_offset_t         firstfree;
    vm_offset_t         nextfree;
    vm_offset_t         nextlimit;
} vm_chunk_data_t;

#define VM_CHUNK_WIRED 0x00000001

/* vm_chunk_alloc[_nowait] function prototypes are in ../h/vm.h */


/* ----- vm_zone_t data structure ----------------------------------------- */

typedef struct vm_zone_data_t
{
    vm_queue_data_t freeq;	/* queue of free instances of structure */
    simple_lock_data_t latch;	/* zone lock */
    int             elsize;	/* size of an element */
    int             allocsize;	/* allocation quantum (page rounded) */
    int             cursize;	/* current size of zone */
    int             maxsize;	/* maximum size of zone */
    int             nused;      /* number of items allocated */
    int             nfree;	/* number of items free */
    /* START of debugging / size-printing info */
    vm_addr_t       page1;	/* address of first page (ONLY used to */
				/* dump zone_zones!) */
    /* END of debugging / size-printing info */
    char           *name;	/* name of zone (debugging) */
    int             lognode;
    int             flags;	/* pageable and expansion flags */
} vm_zone_data_t;

#define VM_ZONE_EXPANDING   0x00000001
#define VM_ZONE_EXPAND_WAIT 0x00000002
#define VM_ZONE_PAGEABLE    0x00000004

vm_zone_t       vm_mapping_zone[NCPUS+1]; /* GLOBAL: on master */
vm_zone_t       vm_map_zone[NCPUS]; /* GLOBAL: on master */
vm_zone_t       vm_entry_zone[NCPUS]; /* GLOBAL: on master */
vm_zone_t       vm_object_zone[NCPUS]; /* GLOBAL: on master */
vm_zone_t       vm_ovrec_zone[NCPUS]; /* GLOBAL: on master */
vm_zone_t       vm_current_ovrec_zone; /* GLOBAL: on master */
ushort_t        vm_current_ovrec_zone_node; /* GLOBAL: on master */
vm_zone_data_t  vm_zones[NCPUS]; /* GLOBAL: on master */
vm_chunk_data_t vm_chunks[NCPUS]; /* GLOBAL: on master */

/* These macros let us later vectorize these zones without changing the */
/* source code in lots of places; only the macros have to change. */

#define VM_MAPPING_ZONE(node) (vm_mapping_zone[node])
#define VM_MAP_ZONE(node)     (vm_map_zone[node])
#define VM_ENTRY_ZONE(node)   (vm_entry_zone[node])
#define VM_OBJECT_ZONE(node)  (vm_object_zone[node])

/* zone function prototypes are in ../h/vm.h, for public use... */

/* ----- Bootstrap definitions ------------------------------------------- */

#define VM_CHUNK_EXPAND_QUANTUM  (256 * 1024)
#define VM_CHUNK_MINFREE       (64 * 1024)

/* Vector of pointers to each node's vm_page_data_t's.  Note that */
/* the vm_page_data_t's actually live on the indicated nodes      */ 

vm_page_t   vm_page_array[NCPUS];
int         vm_first_page[NCPUS];
vm_offset_t vm_zone_bootstrap_vaddr;
#define     VM_ZONE_BOOTSTRAP_PAGES  3

/* ----- Used in garbage collection of empty cached text objects---------*/
simple_lock_data_t vm_empty_cache_lock;	/* Note simple lock, not vm lock */
vm_queue_data_t vm_empty_cache_queue;

/* ----- Hard-to-deal with allocation and expansion routines ------------*/
/* See vm_expand.c for details */

vm_ovrec_t vm_ovrec_alloc();

simple_lock_data_t vm_expand_lock;
char vm_ovrec_expand_flags;
char vm_chunks_need_to_expand;
char vm_kentries_need_to_expand;

/* flags for vm_ovrec_expand_flags */
#define VM_OVREC_EXPAND_NEEDED 0x1
#define VM_OVREC_EXPAND_WAITING 0x2

/* ----- vm_stat statistics gathering definitions ------------------------ */

/* These depend on public definitions in "../h/vm_stat.h" */

/* These macros let us later vectorize the  node stat structs without */
/* changing the source code in lots of places; only the macros have */
/* to change. */

#define VM_BUMP_COUNT(mp, node, event) (((mp)->counts[event])++, \
					((vm_nodecounts[node])[event])++)

#define VM_BUMP_ALLOC(node, event) (((vm_nodeallocs[node])[event])++)

int              vm_nodecounts[NCPUS][VM_CNT_COUNT];
int              vm_nodeallocs[NCPUS][VM_ALLO_COUNT];

/* Definitions of how many stat structures of various types we can */
/* safely fit on the kernel stack and still process an arbitrary page */
/* fault while we do the copyout... we try to keep the total bytes near */
/* 256, which should be quite safe.  -rwells, BBNACI, 4-Aug-89 */


#define VM_NSTAT_OLDREGION   4	/* sizeof(vm_mapstat_data_t)==52, total 208 */
#define VM_NSTAT_REGIONS     2	/* vm_regionstat_data_t <= 120, total 240 */
#define VM_NSTAT_PAGES      64	/* vm_pagestat_data_t == 4, total 256 */
#define VM_NSTAT_BYTES     256	/* total 256 */
#define VM_NSTAT_PAGEFRAMES  8	/* vm_pageframe_data_t <= 32, total 256 */

#if defined(__STDC__) || defined(c_plusplus)
int vm_stat_oldregion(vm_map_t mp, vm_addr_t start, vm_addr_t end, int maxcnt,
		      vm_mapstat_t msp /*OUT*/, vm_addr_t* nextp /*OUT*/);
int vm_stat_regions(vm_map_t mp, vm_addr_t start, vm_addr_t end, int maxcnt,
		    vm_regionstat_t rsp /*OUT*/, vm_addr_t* nextp /*OUT*/);
int vm_stat_pages(vm_map_t mp, vm_addr_t start, vm_addr_t end, int structsize,
		  int maxcnt, vm_pagestat_t psp /*OUT*/,
		  vm_addr_t* nextp /*OUT*/);
#else
int vm_stat_oldregion(/*see above*/);
int vm_stat_regions(/*see above*/);
int vm_stat_pages(/*see above*/);
#endif

/* ----- page victimization scanner definitions -------------------------- */

/* state variable definitions in vm.h */

#if defined(__STDC__) || defined(c_plusplus)
int vm_scanner ();
int vm_count_refs (pt_entry_t * pte);
int vm_scan (int * my_runq_count);
int vm_scanner_restart(int node);
int vm_move_to_inactive(int * my_runq_count);
#else
int vm_scanner ();
int vm_count_refs ();
int vm_scanner_restart();
int vm_scan();
int vm_move_to_inactive();
#endif


/* ----- end of the file... ---------------------------------------------- */

#endif KERNEL
#endif _VMI_H_

