/* $Header: /nfs/europa/u1/BFMACH.local/mk/vm/RCS/vnode_pager.h,v 1.13 91/01/09 12:29:37 lkaplan Exp $	*/

#ifndef	_VNODE_PAGER_H
#define	_VNODE_PAGER_H

/*
 * File: vnode_pager.h
 * "Swap" pager that pages to/from Unix vnodes. Also
 * handles demand paging from files.
 *
 * May 8, 1989 Thomas W. Uban
 *
 * Copyright 1989 by BBN Advanced Computers Inc.
 * ALL RIGHTS RESERVED
 *
 * No part of this material may be reproduced, stored in a retrieval
 * system, transmitted or used in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise, without the prior
 * written permission of, or express license from BBN Advanced Computers Inc.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
 * Technical Data and Computer Software clause at DFARS 52.227-7013.
 *
 * BBN Advanced Computers Inc.
 * 10 Fawcett Street
 * Cambridge, Ma. 02238
 * 617-873-6000
 */

#ifdef	KERNEL
/*
 * Routines used external to the vnode paging system.
 */
int vnode_pager_init();			/* initialize the vnode pager system */
vm_errno_t vnode_page_alloc();		/* allocate a page/vnode pair */
vm_errno_t vnode_page_out();		/* write a page out to a vnode */
vm_errno_t vnode_page_in();		/* read a page in from a vnode */
vm_errno_t vnode_page_free();		/* free a previously allocated page */
int vnode_pager_ctl();			/* paging system system call */

/*
 * Backing file definitions.
 */
#define	BF_FILENAME	"/.VnodePager"	/* paging file's name (1 per fs max) */
#define	BF_MAXINFO	16		/* maximum backing info structures */
#define	BF_FBLL		(DEV_BSIZE / sizeof(long))
					/* free blk list length, see bf_info */
					/* struct below for details */
#define	BF_FS_FBLL	64		/* UFS/NFS free block list length */
					/* BF_FS_FBLL must be <= BF_FBLL */

#endif	KERNEL

enum bf_type { BF_NULL, BF_RAW, BF_UFS, BF_NFS };

#ifdef	KERNEL

struct bf_free_blks {
    struct bf_free_blks *fb_next;	/* pointer to next free blks */
    struct bf_free_blks *fb_prev;	/* pointer to previous free blks */
    short fb_flags;			/* flags: contiguous, etc */
    short fb_count;			/* number of blocks in segment */
    long fb_low;			/* lowest free block in segment */
    long fb_high;			/* highest free block in segment */
};

/*
 * Fb_flags definitions.
 */
#define	FB_CONTIGUOUS	0x01		/* fb_blks list is contiguous */

/*
 * Free blocks maintenace structure definitions.
 */
#define	FB_MAXQUEUES	(VM_PAGESIZE / sizeof(struct bf_free_blks))

struct bf_info {
    struct bf_info *bf_next;		/* next backing file info struct */
    struct bf_info *bf_prev;		/* previous backing file info struct */
    struct vnode *bf_vp;		/* pointer to backing file's vnode */
    char *bf_file_name;			/* pointer to backing file name */
    enum bf_type bf_type;		/* backing file type */
    int bf_flags;			/* flags: locked, wanted, etc */
    struct ucred bf_cred;		/* credentials */
    u_short bf_preference;		/* preference */
    u_short bf_pcnt_full;		/* percent full before bf_alloc */
    long bf_pfree;			/* bf_choose_fs pcnt free */
    long bf_mfree;			/* bf_choose_fs min free */
    long bf_raw_blks;			/* blocks available in raw partition */
    long bf_blks_inuse;			/* blocks currently in use */
    long bf_high_blk;			/* highest block currently allocated */
    long bf_free_cnt;			/* number of blocks in free list */
    long bf_file_blks;			/* number of blocks in file */
    long bf_reserved;			/* reserved */
    long bf_raw_free_index;		/* index of last freed RAW block */
    struct bf_free_blks *bf_fbsq_head;	/* ptr to head of free block sort q */
    struct bf_free_blks *bf_fbsq_tail;	/* ptr to tail of free block sort q */
    simple_lock_data_t bf_lock;		/* multiprocessor lock */
    long bf_blks[BF_FBLL];		/* in-core free block list */
    /*
     * For raw paging, reads/writes want to be in DEV_BSIZE multiples
     * or they will be truncated. The total size of this structure wants
     * to fit nicely into the VM_PAGESIZE unit of allocation. The tradeoffs
     * are speed/frequency of freelist read/write to disk vs. kernel
     * memory waste.
     * The current model is to make bf_info less than 1024 in order to
     * fit at least 16 into 2 pages, and to make the free block list
     * length 1 sector in size.
     */
};

struct bf_info_qhead {
    struct bf_info *bf_next;		/* next backing file info struct */
    struct bf_info *bf_prev;		/* previous backing file info struct */
};

/*
 * Bf_flags definitions.
 */
#define	BF_LOCKED	0x01		/* backing file is locked */
#define	BF_WANTED	0x02		/* backing file lock is wanted */
#define	BF_CLOSED	0x04		/* backing file is no longer usable */
#define	BF_GCING	0x08		/* backing file is being GC'd */
#define	BF_ENOSPC	0x10		/* process waiting for backing file */
#define	BF_CLOSING	0x20		/* backing file is now being closed */

/*
 * Raw partition definitions.
 */
#define	BF_RAW_FULL	4		/* blks left in full raw partition */

#endif	KERNEL

/*
 * Vnode_pager_ctl system call definitions.
 */
#define	VNODE_PAGER_RAW		0	/* set raw paging partition device */
#define	VNODE_PAGER_CLOSE	1	/* close paging file, relocate pages */
#define	VNODE_PAGER_ENTRY_OLD	2	/* return next packet of entry info */
#define	VNODE_PAGER_ENTRY	3	/* return next packet of entry info */
#define	VNODE_PAGER_STATS	4	/* return paging statistics struct */
#define	VNODE_PAGER_STATS_OFF	5	/* turn off paging statistics */
#define	VNODE_PAGER_STATS_ON	6	/* turn on paging statistics */
#define	VNODE_PAGER_GC		7	/* perform garbage collection */
#define	VNODE_PAGER_SETPREF	8	/* set paging file preference */
#define	VNODE_PAGER_SETPCNT	9	/* set paging file %full */
#define	VNODE_PAGER_SETUID	10	/* set paging file UID (NFS only) */
#define	VNODE_PAGER_SETGID	11	/* set paging file GID (NFS only) */
#define	VNODE_PAGER_DUMP	12	/* dump internal paging structures */

#define	BF_NAMLEN		128	/* maximum file path/name length */
#define	BF_LAST_ENTRY		-1	/* VNODE_PAGER_ENTRY last record */

/*
 * VNODE_PAGER_ENTRY_OLD packet description.
 */
struct bf_user_info_old {
    long bf_last;			/* last entry cookie */
    enum bf_type bf_type;		/* backing file type */
    long bf_raw_blks;			/* blocks available in raw partition */
    long bf_blks_inuse;			/* blocks currently in use */
    long bf_free_cnt;			/* number of blocks in free list */
    char bf_file_name[BF_NAMLEN];	/* backing file name */
};

/*
 * VNODE_PAGER_ENTRY packet description.
 */
struct bf_user_info {
    long bf_last;			/* last entry cookie */
    enum bf_type bf_type;		/* backing file type */
    long bf_raw_blks;			/* blocks available in raw partition */
    long bf_blks_inuse;			/* blocks currently in use */
    long bf_free_cnt;			/* number of blocks in free list */
    char bf_file_name[BF_NAMLEN];	/* backing file name */
    u_short bf_preference;		/* preference */
    u_short bf_pcnt_full;		/* percent full before bf_alloc */
    uid_t bf_nfs_uid;			/* UID for NFS paging */
    gid_t bf_nfs_gid;			/* GID for NFS paging */
    long bf_gc_blks;			/* blocks shuffled by GC operation */
    long bf_spare[15];			/* reserved */
};

/*
 * VNODE_PAGER_STATS packet description.
 */
struct bf_stats {
    struct timeval bf_time;		/* time when statistics were enabled */
    u_long bf_tot_pageouts;		/* total pageouts */
    u_long bf_tot_pageins;		/* total pageins */
    u_long bf_raw_pageouts;		/* RAW pageouts */
    u_long bf_raw_pageins;		/* RAW pageins */
    u_long bf_ufs_pageouts;		/* UFS pageouts */
    u_long bf_ufs_pageins;		/* UFS pageins */
    u_long bf_nfs_pageouts;		/* NFS pageouts */
    u_long bf_nfs_pageins;		/* NFS pageins */
    u_long bf_map_pageouts;		/* mapped file pageouts */
    u_long bf_map_pageins;		/* mapped file pageins */
    u_long bf_txt_pageouts;		/* text pageouts */
    u_long bf_txt_pageins;		/* text pageins */
    u_long bf_pageout_errs;		/* errors during pageout */
    u_long bf_pagein_errs;		/* errors during pagein */
    u_long bf_gc_blks;			/* blocks shuffled by GC operation */
    u_long bf_spare[15];		/* reserved */
};

/*
 * Statistics gathering macro.
 */
#define	BF_BUMP_STAT(STAT) { \
    if (!paging_stats_disabled) \
	Atomic_add32(&(bfi_stats.STAT), 1); \
}

#endif	_VNODE_PAGER_H
